package org.modelbus.team.eclipse.changemodelnotification.commands;

import java.util.HashMap;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.IHandlerListener;
import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditorInput;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.HandlerUtil;
import org.modelbus.core.lib.notification.NotificationListenerManager;
import org.modelbus.team.eclipse.changemodelnotification.Activator;
import org.modelbus.team.eclipse.changemodelnotification.listener.ModelBusExcecutionListener;
import org.modelbus.team.eclipse.changemodelnotification.listener.ModelbusChangeModelNotificationListener;
import org.modelbus.team.eclipse.changemodelnotification.listener.ModelbusChangeModelResourceSetListener;
import org.modelbus.team.eclipse.changemodelnotification.preferences.PreferenceConstants;
import org.modelbus.team.eclipse.changemodelnotification.util.ChangeModelUtil;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;

public class InteractiveButtonHandler implements IHandler, IPropertyChangeListener, IPartListener {

	private ModelbusChangeModelResourceSetListener							modelbusChangeModelResourceSetListener;
	private ModelbusChangeModelNotificationListener							modelbusChangeModelNotificationListener;
	private HashMap<TransactionalEditingDomain, ModelBusExcecutionListener>	domainToExecutionListenerMap;
	private boolean															isActiveModeling	= false;
	private boolean															isCommitOnSave		= false;
	private boolean															buttonPushed		= false;
	private String															repositoryLocation;
	private NotificationListenerManager										notificationListenerManager;

	public InteractiveButtonHandler() {

		this.isActiveModeling = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.AUTOMATIC_UPDATE);
		this.isCommitOnSave = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.COMMIT_ON_SAVE);
		Activator.getDefault().getPreferenceStore().addPropertyChangeListener(this);
		this.modelbusChangeModelNotificationListener = new ModelbusChangeModelNotificationListener();
		this.modelbusChangeModelResourceSetListener = new ModelbusChangeModelResourceSetListener();
		this.domainToExecutionListenerMap = new HashMap<TransactionalEditingDomain, ModelBusExcecutionListener>();
		try {
			this.notificationListenerManager = NotificationListenerManager.getNotificationListenerManager();
		} catch (Throwable e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		final IPartListener partListener = this;
		final IWorkbench workbench = PlatformUI.getWorkbench();
		workbench.getDisplay().asyncExec(new Runnable() {
			public void run() {

				IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
				if (window != null) {
					window.getActivePage().addPartListener(partListener);
				}
			}
		});
	}

	@Override
	public boolean isEnabled() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		TransactionalEditingDomain transactionalEditingDomain = null;
		if (editor != null) {
			Object adapter = editor.getAdapter(IEditingDomainProvider.class);
			if (adapter instanceof IEditingDomainProvider) {
				IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
				EditingDomain domain = domainProvider.getEditingDomain();
				if (domain instanceof TransactionalEditingDomain) {
					transactionalEditingDomain = (TransactionalEditingDomain) domain;
				}
			}
		}
		if (transactionalEditingDomain != null && this.isActiveModeling) {
			ResourceSet resourceSet = transactionalEditingDomain.getResourceSet();
			for (Resource resource : resourceSet.getResources()) {
				String resourceUri = resource.getURI().toString();
				String remoteUri = ChangeModelUtil.getRemoteModelUri(resourceUri);
				if (remoteUri != null) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		if (notificationListenerManager == null) {
			return null;
		}

		Command command = event.getCommand();
		boolean oldValue = HandlerUtil.toggleCommandState(command);

		if (!oldValue) {

			this.buttonPushed = true;
			notificationListenerManager.addNotificationListener(modelbusChangeModelNotificationListener);

			prepareAllOpenEditorsForActiveModeling();

		} else {

			this.buttonPushed = false;
			removeAllListenersFromAllOpenEditors();

			notificationListenerManager.removeNotificationListener(modelbusChangeModelNotificationListener);
		}
		return null;
	}

	private void prepareAllOpenEditorsForActiveModeling() {
		IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
		for (IEditorReference editorReference : editorReferences) {
			IEditorPart editorPart = editorReference.getEditor(true);
			ResourceSet resourceSet = null;
			TransactionalEditingDomain transactionalEditingDomain = null;
			if (editorPart != null) {
				Object adapter = editorPart.getAdapter(IEditingDomainProvider.class);
				if (adapter instanceof IEditingDomainProvider) {
					IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
					EditingDomain domain = domainProvider.getEditingDomain();
					if (domain instanceof TransactionalEditingDomain) {
						transactionalEditingDomain = (TransactionalEditingDomain) domain;
						resourceSet = transactionalEditingDomain.getResourceSet();

						modelbusChangeModelResourceSetListener.setTarget(resourceSet);
						transactionalEditingDomain.addResourceSetListener(modelbusChangeModelResourceSetListener);

						if (isCommitOnSave) {
							prepareEditorForCommitOnSave(editorPart, transactionalEditingDomain); // prepare for listening commands(save ..)
						}
					}
				}
			}
		}

		if (!this.isCommitOnSave) {
			prepareActiveEditorForImmediateCommit();
		}
	}

	/*
	 * Sets this.modelbusChangeModelResourceSetListener.setFileEditorInputPath(remotePath) for all active editor
	 */
	private void prepareActiveEditorForImmediateCommit() {
		IEditorPart activeEditorPart = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (activeEditorPart != null) {
			Object adapter = activeEditorPart.getAdapter(IEditingDomainProvider.class);
			if (adapter instanceof IEditingDomainProvider) {
				IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
				EditingDomain domain = domainProvider.getEditingDomain();
				if (domain instanceof TransactionalEditingDomain) {
					IEditorInput input = activeEditorPart.getEditorInput();
					String remotePath = null;
					String localPath = null;
					if (input instanceof IPathEditorInput) {
						localPath = "file:/" + ((IPathEditorInput) input).getPath().toString();
					} else if (input instanceof IDiagramEditorInput) {
						IFile file = (IFile) ((IDiagramEditorInput) input).getAdapter(IFile.class);
						if (file != null) {
							localPath = "file:/" + file.getLocation().toString();
						}
					} else if (input instanceof URIEditorInput) { // XXX OBEO EXTENSION
						URIEditorInput uriEditorInput = (URIEditorInput) input;
						Object adapter2 = uriEditorInput.getAdapter(IFile.class);
						if (adapter2 != null) {
							IFile iFile = (IFile) adapter2;
							localPath = "file:/" + iFile.getLocation().toString();
						}
					}

					if (localPath != null) {
						remotePath = ChangeModelUtil.getRemoteModelUri(localPath);
						if (remotePath != null) {
							this.modelbusChangeModelResourceSetListener.setFileEditorInputPath(remotePath);
						}
					}
				}
			}
		}
	}

	/*
	 * Listening + reacting to global commands, such a save...
	 */
	private void prepareEditorForCommitOnSave(IEditorPart editorPart, TransactionalEditingDomain transactionalEditingDomain) {
		ICommandService service = (ICommandService) editorPart.getEditorSite().getService(ICommandService.class);
		ModelBusExcecutionListener modelBusExcecutionListener = getExecutionListener(transactionalEditingDomain);
		if (modelBusExcecutionListener == null) {
			modelBusExcecutionListener = new ModelBusExcecutionListener(modelbusChangeModelResourceSetListener, transactionalEditingDomain);
			getExecutionListenerMap().put(transactionalEditingDomain, modelBusExcecutionListener);
			service.addExecutionListener(modelBusExcecutionListener);
		}
	}

	private ModelBusExcecutionListener getExecutionListener(TransactionalEditingDomain domain) {
		HashMap<TransactionalEditingDomain, ModelBusExcecutionListener> domainToExecutionListenerMap = getExecutionListenerMap();

		return domainToExecutionListenerMap.get(domain);
	}

	private HashMap<TransactionalEditingDomain, ModelBusExcecutionListener> getExecutionListenerMap() {
		if (this.domainToExecutionListenerMap == null) {
			this.domainToExecutionListenerMap = new HashMap<TransactionalEditingDomain, ModelBusExcecutionListener>();
		}
		return this.domainToExecutionListenerMap;
	}

	private void removeExecutionListenerFromAllEditors() {

		IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
		for (IEditorReference editorReference : editorReferences) {
			IEditorPart editorPart = editorReference.getEditor(true);
			TransactionalEditingDomain transactionalEditingDomain = null;
			if (editorPart != null) {
				Object adapter = editorPart.getAdapter(IEditingDomainProvider.class);
				if (adapter instanceof IEditingDomainProvider) {
					IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
					EditingDomain domain = domainProvider.getEditingDomain();
					if (domain instanceof TransactionalEditingDomain) {
						transactionalEditingDomain = (TransactionalEditingDomain) domain;
						removeExecutionListenerFromEditor(editorPart, transactionalEditingDomain);
					}
				}
			}
		}
	}

	private void removeExecutionListenerFromEditor(IEditorPart editorPart, TransactionalEditingDomain transactionalEditingDomain) {

		ICommandService service = (ICommandService) editorPart.getEditorSite().getService(ICommandService.class);
		ModelBusExcecutionListener modelBusExcecutionListener = getExecutionListener(transactionalEditingDomain);
		if (modelBusExcecutionListener != null) {
			service.removeExecutionListener(modelBusExcecutionListener);
			getExecutionListenerMap().remove(transactionalEditingDomain);
		}

	}

	private void removeAllListenersFromAllOpenEditors() {
		IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
		for (IEditorReference editorReference : editorReferences) {
			IEditorPart editorPart = editorReference.getEditor(true);
			ResourceSet resourceSet = null;
			TransactionalEditingDomain transactionalEditingDomain = null;
			if (editorPart != null) {
				Object adapter = editorPart.getAdapter(IEditingDomainProvider.class);
				if (adapter instanceof IEditingDomainProvider) {
					IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
					EditingDomain domain = domainProvider.getEditingDomain();
					if (domain instanceof TransactionalEditingDomain) {
						transactionalEditingDomain = (TransactionalEditingDomain) domain;
						resourceSet = transactionalEditingDomain.getResourceSet();

						if (transactionalEditingDomain != null) {
							modelbusChangeModelResourceSetListener.unsetTarget(resourceSet);
							transactionalEditingDomain.removeResourceSetListener(modelbusChangeModelResourceSetListener);
						}

						if (isCommitOnSave) {
							removeExecutionListenerFromEditor(editorPart, transactionalEditingDomain);
						}
					}
				}
			}
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().equals(PreferenceConstants.AUTOMATIC_UPDATE)) {
			this.isActiveModeling = ((Boolean) event.getNewValue()).booleanValue();
			if (!this.isActiveModeling) {
				removeAllListenersFromAllOpenEditors();
			} else {
				prepareAllOpenEditorsForActiveModeling();
			}
		} else if (event.getProperty().equals(PreferenceConstants.COMMIT_ON_SAVE)) {
			this.isCommitOnSave = ((Boolean) event.getNewValue()).booleanValue();
			if (this.isCommitOnSave) {
				prepareAllEditorsForCommitOnSave();

			} else {
				removeExecutionListenerFromAllEditors();
			}
		}
	}

	private void prepareAllEditorsForCommitOnSave() {
		IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
		for (IEditorReference editorReference : editorReferences) {
			IEditorPart editorPart = editorReference.getEditor(true);
			// ResourceSet resourceSet = null;
			TransactionalEditingDomain transactionalEditingDomain = null;
			if (editorPart != null) {
				Object adapter = editorPart.getAdapter(IEditingDomainProvider.class);
				if (adapter instanceof IEditingDomainProvider) {
					IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
					EditingDomain domain = domainProvider.getEditingDomain();
					if (domain instanceof TransactionalEditingDomain) {
						transactionalEditingDomain = (TransactionalEditingDomain) domain;

						prepareEditorForCommitOnSave(editorPart, transactionalEditingDomain);
					}
				}
			}
		}
	}

	@Override
	public void partActivated(IWorkbenchPart part) {
		if (part instanceof IEditorPart && this.buttonPushed && !this.isCommitOnSave) {
			IEditorPart editorPart = (IEditorPart) part;
			Object adapter = editorPart.getAdapter(IEditingDomainProvider.class);
			if (adapter instanceof IEditingDomainProvider) {
				IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
				EditingDomain domain = domainProvider.getEditingDomain();
				if (domain instanceof TransactionalEditingDomain) {
					IEditorInput input = editorPart.getEditorInput();
					String remotePath = null;
					if (input instanceof IFileEditorInput) {
						String localPath = "file:/" + ((IFileEditorInput) input).getFile().getLocation().toString();
						remotePath = ChangeModelUtil.getRemoteModelUri(localPath);
						if (remotePath != null) {
							this.modelbusChangeModelResourceSetListener.setFileEditorInputPath(remotePath);
						}
					}
				}
			}
		}
	}

	@Override
	public void partClosed(IWorkbenchPart part) {
		if (part instanceof IEditorPart && this.buttonPushed) {
			TransactionalEditingDomain transactionalEditingDomain = null;
			ResourceSet resourceSet = null;
			Object adapter = part.getAdapter(IEditingDomainProvider.class);
			if (adapter instanceof IEditingDomainProvider) {
				IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
				EditingDomain domain = domainProvider.getEditingDomain();
				if (domain instanceof TransactionalEditingDomain) {
					transactionalEditingDomain = (TransactionalEditingDomain) domain;
					resourceSet = transactionalEditingDomain.getResourceSet();
					modelbusChangeModelResourceSetListener.unsetTarget(resourceSet);
					transactionalEditingDomain.removeResourceSetListener(modelbusChangeModelResourceSetListener);
				}
			}
			removeExecutionListenerFromEditor((IEditorPart) part, transactionalEditingDomain);
		}
	}

	@Override
	public void partOpened(IWorkbenchPart part) {
		if (part instanceof IEditorPart && this.buttonPushed) {
			TransactionalEditingDomain transactionalEditingDomain = null;
			ResourceSet resourceSet = null;
			Object adapter = part.getAdapter(IEditingDomainProvider.class);
			if (adapter instanceof IEditingDomainProvider) {
				IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
				EditingDomain domain = domainProvider.getEditingDomain();
				if (domain instanceof TransactionalEditingDomain) {
					transactionalEditingDomain = (TransactionalEditingDomain) domain;
					resourceSet = transactionalEditingDomain.getResourceSet();
					modelbusChangeModelResourceSetListener.setTarget(resourceSet);
					transactionalEditingDomain.addResourceSetListener(modelbusChangeModelResourceSetListener);
				}
			}

			if (isCommitOnSave) {
				prepareEditorForCommitOnSave((IEditorPart) part, transactionalEditingDomain);
			}
		}
	}

	protected String getRepositoryLocation() {
		if (this.repositoryLocation == null) {
			this.repositoryLocation = RepositoryLocationHelper.getPropertyRepositoryLocation();
		}

		return repositoryLocation;
	}

	// ----------------------------empty functions----------------------------

	@Override
	public void partDeactivated(IWorkbenchPart part) {

	}

	@Override
	public void partBroughtToTop(IWorkbenchPart part) {

	}

	@Override
	public boolean isHandled() {
		return true;
	}

	@Override
	public void removeHandlerListener(IHandlerListener handlerListener) {

	}

	@Override
	public void addHandlerListener(IHandlerListener handlerListener) {

	}

	@Override
	public void dispose() {

	}
}
