package org.modelbus.team.eclipse.changemodelnotification.listener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.change.ChangeDescription;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.transaction.NotificationFilter;
import org.eclipse.emf.transaction.ResourceSetChangeEvent;
import org.eclipse.emf.transaction.ResourceSetListener;
import org.eclipse.emf.transaction.RollbackException;
import org.eclipse.emf.transaction.Transaction;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.team.eclipse.changemodelnotification.Activator;
import org.modelbus.team.eclipse.changemodelnotification.commit.ModelBusChangeManager;
import org.modelbus.team.eclipse.changemodelnotification.commit.ModelBusChangeRecorderConverter;
import org.modelbus.team.eclipse.changemodelnotification.preferences.PreferenceConstants;
import org.modelbus.team.eclipse.changemodelnotification.util.ChangeModelUtil;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;
import org.modelbus.team.eclipse.repository.ui.LockHelper;

public class ModelbusChangeModelResourceSetListener implements Adapter.Internal, ResourceSetListener, IPropertyChangeListener {

	private boolean													commitOnSave		= false;
	private boolean													automaticLockUnlock	= false;
	private String													fileEditorInputPath;
	private List<EObject>											lockedElements;
	private Map<TransactionalEditingDomain, List<EObject>>			domainToLockedElementsMap;
	private Map<TransactionalEditingDomain, HashSet<Notification>>	resourceNotificationsToRecordMap;
	private HashSet<Notification>									resourceNotficationsToRecord;
	private TransactionalEditingDomain								domain;

	public ModelbusChangeModelResourceSetListener() {

		this.commitOnSave = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.COMMIT_ON_SAVE);
		this.automaticLockUnlock = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.AUTOMATIC_LOCK_UNLOCK);
		Activator.getDefault().getPreferenceStore().addPropertyChangeListener(this);

	}

	public void setFileEditorInputPath(String fileEditorInputPath) {
		this.fileEditorInputPath = fileEditorInputPath;
	}

	private synchronized List<EObject> getLockedElements(TransactionalEditingDomain domain) {

		Map<TransactionalEditingDomain, List<EObject>> domainToLockedElementsMap = getdomainToLockedElementsMap();

		if (!domainToLockedElementsMap.containsKey(domain)) {
			lockedElements = new ArrayList<EObject>();
			domainToLockedElementsMap.put(domain, lockedElements);
		}
		return domainToLockedElementsMap.get(domain);
	}

	private synchronized Map<TransactionalEditingDomain, List<EObject>> getdomainToLockedElementsMap() {

		if (domainToLockedElementsMap == null) {
			domainToLockedElementsMap = new HashMap<TransactionalEditingDomain, List<EObject>>();
		}
		return domainToLockedElementsMap;
	}

	private synchronized HashSet<Notification> getResourceNotificationsToRecord(TransactionalEditingDomain domain) {

		Map<TransactionalEditingDomain, HashSet<Notification>> resourceNotficationsToRecordMap = getResourceNotificationsToRecordMap();

		if (!resourceNotficationsToRecordMap.containsKey(domain)) {
			resourceNotficationsToRecord = new HashSet<Notification>();
			resourceNotficationsToRecordMap.put(domain, resourceNotficationsToRecord);
		}
		return resourceNotficationsToRecordMap.get(domain);
	}

	private Map<TransactionalEditingDomain, HashSet<Notification>> getResourceNotificationsToRecordMap() {

		if (resourceNotificationsToRecordMap == null) {
			resourceNotificationsToRecordMap = new HashMap<TransactionalEditingDomain, HashSet<Notification>>();
		}
		return resourceNotificationsToRecordMap;
	}

	@Override
	public NotificationFilter getFilter() {
		return null;
	}

	@Override
	public boolean isAggregatePrecommitListener() {
		return false;
	}

	@Override
	public boolean isPostcommitOnly() {
		return false;
	}

	@Override
	public boolean isPrecommitOnly() {
		return false;
	}

	@Override
	public void resourceSetChanged(ResourceSetChangeEvent event) {

		boolean update = false;
		Transaction transaction = event.getTransaction();
		if (transaction != null) {
			Map<?, ?> options = transaction.getOptions();
			Object option = options.get(ChangeModelUtil.OPTION_MODELBUS_UPDATE);
			if (option instanceof Boolean) {
				update = (Boolean) option;
			}
		}

		if (!update) {
			List<Notification> notifications = event.getNotifications();
			ModelBusChangeRecorderConverter changeManager = null;
			for (Notification notification : notifications) {
				final Object notifier = notification.getNotifier();
				if (notifier instanceof EObject) {
					switch (notification.getEventType()) {
						case Notification.ADD:
						case Notification.ADD_MANY:
						case Notification.CREATE:
						case Notification.MOVE:
						case Notification.REMOVE:
						case Notification.REMOVE_MANY:
						case Notification.SET:
						case Notification.UNSET: {
							final Resource resource = ((EObject) notifier).eResource();
							if (resource != null) {
								final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(resource);
								changeManager = ModelBusChangeManager.getModelBusChangeManager().getModelBusChangeRecorderConverter(domain);
								this.domain = domain;
								if (automaticLockUnlock) {

									Thread thread = new Thread(new Runnable() {

										@Override
										public void run() {

											List<EObject> lockEObjects = getLockedElements(domain);
											if (!lockEObjects.contains((EObject) notifier)
													&& LockHelper.getLockHelper().objectLockedBy((EObject) notifier) == null) {
												lockEObjects.add((EObject) notifier);
												getdomainToLockedElementsMap().put(domain, lockEObjects);
												IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
												try {
													URI eobjectUri = EcoreUtil.getURI((EObject) notifier);
													String remoteUri = ChangeModelUtil.getRemoteModelUri(eobjectUri.trimFragment().toString()) + "#"
															+ eobjectUri.fragment();
													repositoryHelper.lock(UserSessionHelper.getSession(), URI.createURI(remoteUri));
												} catch (Exception e) {

												}
											}

										}
									});
									thread.run();

								}
								changeManager.recordNotification(notification);
							}
							break;
						}

						default:
							break;
					}
				} else if (notifier instanceof Resource) {// && resourceNotificationsToRecord.contains(notification)){
					final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain((Resource) notifier);
					changeManager = ModelBusChangeManager.getModelBusChangeManager().getModelBusChangeRecorderConverter(domain);
					this.domain = domain;
					HashSet<Notification> resourceNotficationsToRecord = getResourceNotificationsToRecord(domain);
					if (resourceNotficationsToRecord.contains(notification)) {
						changeManager.recordNotification(notification);
						resourceNotficationsToRecord.remove(notification);
						getResourceNotificationsToRecordMap().put(domain, resourceNotficationsToRecord);
					}
				}
			} // end FOR

			if (!commitOnSave) {
				if (changeManager != null) {

					List<Notification> recordedNotifications = changeManager.getRecordedNotifications();
					if (!recordedNotifications.isEmpty()) {
						try {
							if (this.domain != null && this.fileEditorInputPath != null) {
								commitChanges(this.domain, this.fileEditorInputPath);
							}
							changeManager = null;

						} catch (Exception e) {
							throw new RuntimeException(e.getMessage(), e);
						}
					}
				}
			}
		}
	}

	protected void commitChanges(final TransactionalEditingDomain domain, String fileEditorInputPath) throws Exception {
		ModelBusChangeRecorderConverter changeManager = ModelBusChangeManager.getModelBusChangeManager().getModelBusChangeRecorderConverter(domain);
		List<Notification> notifications = changeManager.stopRecordNotifications();
		ChangeDescription changeDescription = changeManager.convertNotificationsToChangeDescription(notifications);

		if (!changeDescription.eContents().isEmpty()) {

			String changeDescriptionAsString = changeManager.eObjectToString(changeDescription);
			final IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			try {
				if (changeDescriptionAsString != null) {
					boolean commitChangeModelResult = repositoryHelper.commitChangeModel(UserSessionHelper.getSession(), Activator.MESSAGE_ID,
							URI.createURI(fileEditorInputPath), changeDescriptionAsString);
				}
			} catch (Exception exeptionAfterCommit) {
				exeptionAfterCommit.printStackTrace();
			} finally {
				Map<Resource, String> map = changeManager.getResource2LocalUriMap();

				for (Iterator<Map.Entry<Resource, String>> iterator = map.entrySet().iterator(); iterator.hasNext();) {
					Map.Entry<Resource, String> entry = iterator.next();
					Resource resource = entry.getKey();
					resource.setURI(URI.createURI(entry.getValue()));
				}

				if (automaticLockUnlock) {
					if (domain == null) {
						return;
					}
					Thread thread = new Thread(new Runnable() {

						@Override
						public void run() {

							List<EObject> lockedElements = getLockedElements(domain);
							for (EObject eObject : lockedElements) {
								URI eobjectUri = EcoreUtil.getURI((EObject) eObject);
								String remoteUri = ChangeModelUtil.getRemoteModelUri(eobjectUri.trimFragment().toString()) + "#"
										+ eobjectUri.fragment();
								try {
									repositoryHelper.unlock(UserSessionHelper.getSession(), URI.createURI(remoteUri));
								} catch (Exception e) {

									// throw new RuntimeException(e.getMessage(), e);
								}

							}
							lockedElements.clear();
							getdomainToLockedElementsMap().remove(domain);

						}
					});
					thread.run();
				}
			}
		}
	}

	@Override
	public Command transactionAboutToCommit(ResourceSetChangeEvent event) throws RollbackException {
		List<Notification> notifications = event.getNotifications();
		for (Notification notification : notifications) {

			switch (notification.getEventType()) {

				case Notification.ADD:
				case Notification.ADD_MANY:
				case Notification.CREATE:
				case Notification.MOVE:
				case Notification.REMOVE:
				case Notification.REMOVE_MANY:
				case Notification.SET:
				case Notification.UNSET: {

					if (notification.getNotifier() instanceof EObject) {
						Object eObject = notification.getNotifier();
						if (eObject != null) {
							String lockedBy = LockHelper.getLockHelper().objectLockedBy(eObject);
							if (lockedBy != null && !lockedBy.equals(UserSessionHelper.getPropertyUserName())) {
								throw new RollbackException(Status.CANCEL_STATUS);
							}

						}

					}
					break;
				}
				case Notification.REMOVING_ADAPTER: {
					break;
				}
				case Notification.RESOLVE: {
					break;
				}

				default: {
				}
			}
		}
		return null;
	}

	@Override
	public void notifyChanged(Notification notification) {
		Object notifier = notification.getNotifier();
		if (notifier instanceof Resource) {
			switch (notification.getEventType()) {
				case Notification.ADD:
				case Notification.ADD_MANY:
				case Notification.REMOVE:
				case Notification.REMOVE_MANY: {
					int featureID = notification.getFeatureID(Resource.class);
					if (featureID == Resource.RESOURCE__CONTENTS && !((Resource.Internal) notifier).isLoading()/*
																												 * &&
																												 * ResourceSetManager.getInstance().
																												 * isLoaded(( Resource) notifier) &&
																												 * ((Resource.Internal)notification
																												 * .getNotifier()).isModified()
																												 */) {
						TransactionalEditingDomain domain = TransactionUtil.getEditingDomain((Resource) notifier);
						HashSet<Notification> resourceNotificationsToRecord = getResourceNotificationsToRecord(domain);
						resourceNotificationsToRecord.add(notification);
						getResourceNotificationsToRecordMap().put(domain, resourceNotificationsToRecord);

					}

					break;
				}

				default:
					break;
			}

		}
	}

	@Override
	public void setTarget(Notifier newTarget) {

		Iterator<?> contents = null;
		if (newTarget instanceof ResourceSet) {
			contents = ((ResourceSet) newTarget).getResources().iterator();
		}

		if (contents != null) {
			while (contents.hasNext()) {
				Notifier notifier = (Notifier) contents.next();
				addAdapter(notifier);
			}

		}

	}

	@Override
	public boolean isAdapterForType(Object type) {

		return false;
	}

	@Override
	public Notifier getTarget() {

		return null;
	}

	@Override
	public void unsetTarget(Notifier oldTarget) {

		if (oldTarget instanceof ResourceSet) {
			ResourceSet resourceSet = (ResourceSet) oldTarget;
			for (Resource resource : resourceSet.getResources()) {
				removeAdapter(resource);
			}
		}

	}

	private void addAdapter(Notifier notifier) {
		if (notifier instanceof Resource) {
			EList<Adapter> eAdapters = notifier.eAdapters();
			if (!eAdapters.contains(this)) {
				eAdapters.add(this);
			}
		}
	}

	private void removeAdapter(Notifier notifier) {
		notifier.eAdapters().remove(this);
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {

		if (event.getProperty().equals(PreferenceConstants.COMMIT_ON_SAVE)) {
			this.commitOnSave = ((Boolean) event.getNewValue()).booleanValue();
		} else if (event.getProperty().equals(PreferenceConstants.AUTOMATIC_LOCK_UNLOCK)) {
			this.automaticLockUnlock = ((Boolean) event.getNewValue()).booleanValue();
		}
	}
}
