package org.modelbus.team.eclipse.changemodelnotification.update;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.change.ChangeDescription;
import org.eclipse.emf.ecore.change.ChangeKind;
import org.eclipse.emf.ecore.change.FeatureChange;
import org.eclipse.emf.ecore.change.ListChange;
import org.eclipse.emf.ecore.change.ResourceChange;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.transaction.Transaction;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.emf.workspace.AbstractEMFOperation;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.CanonicalEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.EditPolicyRoles;
import org.eclipse.gmf.runtime.diagram.ui.parts.DiagramEditor;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditorInput;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramGraphicalViewer;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart;
import org.eclipse.gmf.runtime.notation.Bendpoints;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.IdentityAnchor;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;
import org.modelbus.team.eclipse.changemodelnotification.util.ChangeModelUtil;
import org.modelbus.team.eclipse.core.ModelBusUriConverter;

public class UpdateManager {

	private TransactionalEditingDomain	domain;
	private static UpdateManager		updateManager;
	private IDiagramGraphicalViewer		viewer;

	private UpdateManager() {

	}

	public static UpdateManager getUpdateManager() {
		if (updateManager == null) {
			updateManager = new UpdateManager();
		}
		return updateManager;
	}

	public synchronized void applyChanges(String changeModelDescription, String fileEditorInputPath) {

		System.out.println("UpdateManager ChangeDescription: " + changeModelDescription);
		ModelBusUriConverter modelBusUriConverter = new ModelBusUriConverter();

		setEditingDomainOfResourceToChange(fileEditorInputPath);

		if (domain == null) {
			return;
		}

		ResourceSet domainResourceSet = domain.getResourceSet();
		domainResourceSet.setURIConverter(modelBusUriConverter);

		ChangeDescription changeDescription = ChangeModelUtil.stringToEObject(changeModelDescription, domainResourceSet);

		modifyModel(changeDescription);

	}

	private void executeSETCommand(TransactionalEditingDomain domain, final EObject eObject, final EStructuralFeature feature, final Object newValue) {

		Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(Transaction.OPTION_NO_UNDO, Boolean.TRUE);
		options.put(Transaction.OPTION_UNPROTECTED, Boolean.TRUE);
		options.put(ChangeModelUtil.OPTION_MODELBUS_UPDATE, Boolean.TRUE);
		AbstractEMFOperation operation = new AbstractEMFOperation(domain, "", options) {

			@Override
			protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
				// TODO Auto-generated method stub
				eObject.eSet(feature, newValue);
				return Status.OK_STATUS;
			}
		};
		try {
			operation.execute(new NullProgressMonitor(), null);
		} catch (ExecutionException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	private void executeADDListChangeCommand(TransactionalEditingDomain domain, final EList<Object> oldList, final int index,
			final EList<Object> newValue) {
		Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(Transaction.OPTION_NO_UNDO, Boolean.TRUE);
		options.put(Transaction.OPTION_UNPROTECTED, Boolean.TRUE);
		options.put(ChangeModelUtil.OPTION_MODELBUS_UPDATE, Boolean.TRUE);
		AbstractEMFOperation operation = new AbstractEMFOperation(domain, "", options) {

			@Override
			protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
				// TODO Auto-generated method stub
				if (index < 0 || index >= oldList.size()) {
					oldList.addAll(newValue);
				} else {
					oldList.addAll(index, newValue);
				}
				return Status.OK_STATUS;
			}
		};
		try {
			operation.execute(new NullProgressMonitor(), null);
		} catch (ExecutionException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	private void executeREMOVEListChangeCommand(TransactionalEditingDomain domain, final EList<Object> oldList, final int index/*
																																 * , final
																																 * EList<Object>
																																 * newValue
																																 */) {
		Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(Transaction.OPTION_NO_UNDO, Boolean.TRUE);
		options.put(Transaction.OPTION_UNPROTECTED, Boolean.TRUE);
		options.put(ChangeModelUtil.OPTION_MODELBUS_UPDATE, Boolean.TRUE);
		AbstractEMFOperation operation = new AbstractEMFOperation(domain, "", options) {

			@Override
			protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
				// TODO Auto-generated method stub
				if (!oldList.isEmpty()) {

					if (index > -1 && index < oldList.size()) {
						oldList.remove(index);
					} else {
						oldList.remove(oldList.size() - 1);
					}
				}
				return Status.OK_STATUS;
			}
		};
		try {
			operation.execute(new NullProgressMonitor(), null);
		} catch (ExecutionException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	private void executeMOVEListChangeCommand(TransactionalEditingDomain domain, final EList<Object> oldList, final int index, final int moveToIndex/*
																																					 * EList
																																					 * <
																																					 * Object
																																					 * >
																																					 * newValue
																																					 */) {

		Map<String, Boolean> options = new HashMap<String, Boolean>();
		options.put(Transaction.OPTION_NO_UNDO, Boolean.TRUE);
		options.put(Transaction.OPTION_UNPROTECTED, Boolean.TRUE);
		options.put(ChangeModelUtil.OPTION_MODELBUS_UPDATE, Boolean.TRUE);
		AbstractEMFOperation operation = new AbstractEMFOperation(domain, "", options) {

			@Override
			protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
				// TODO Auto-generated method stub
				if (moveToIndex < oldList.size()) {
					oldList.move(moveToIndex, index);
				}
				return Status.OK_STATUS;
			}
		};
		try {
			operation.execute(new NullProgressMonitor(), null);
		} catch (ExecutionException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	private void setEditingDomainOfResourceToChange(final String fileEditorInputPath) {
		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
			@Override
			public void run() {
				IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
				for (IEditorReference editorReference : editorReferences) {
					try {
						IEditorInput input = editorReference.getEditorInput();
						String remotePath = null;
						String localPath = null;
						if (input instanceof IPathEditorInput) {
							localPath = "file:/" + ((IPathEditorInput) input).getPath().toString();
						} else if (input instanceof IDiagramEditorInput) {
							IFile file = (IFile) ((IDiagramEditorInput) input).getAdapter(IFile.class);
							if (file != null) {
								localPath = "file:/" + file.getLocation().toString();
							}
						} else if (input instanceof URIEditorInput) {
							// XXX OBEO EXTENSION
							URIEditorInput uriEditorInput = (URIEditorInput) input;
							Object adapter2 = uriEditorInput.getAdapter(IFile.class);
							if (adapter2 != null) {
								IFile iFile = (IFile) adapter2;
								localPath = "file:/" + iFile.getLocation().toString();
							}
						}

						if (localPath != null) {
							remotePath = ChangeModelUtil.getRemoteModelUri(localPath);
							if (fileEditorInputPath.equals(remotePath)) {
								IEditorPart part = editorReference.getEditor(true);
								if (part != null) {
									viewer = ((IDiagramWorkbenchPart) part).getDiagramGraphicalViewer();
									Object adapter = part.getAdapter(IEditingDomainProvider.class);
									if (adapter instanceof IEditingDomainProvider) {
										IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
										EditingDomain editingDomain = domainProvider.getEditingDomain();
										if (editingDomain instanceof TransactionalEditingDomain) {
											domain = (TransactionalEditingDomain) editingDomain;
											break;

										}
									}
								}
							} else {
								// XXX OBEO CDIM-Service EXTENSION
								IEditorPart part = editorReference.getEditor(true);
								if (part != null) {
									viewer = ((IDiagramWorkbenchPart) part).getDiagramGraphicalViewer();
									Object adapter = part.getAdapter(IEditingDomainProvider.class);
									if (adapter instanceof IEditingDomainProvider) {
										IEditingDomainProvider domainProvider = (IEditingDomainProvider) adapter;
										EditingDomain editingDomain = domainProvider.getEditingDomain();
										if (editingDomain instanceof TransactionalEditingDomain) {
											domain = (TransactionalEditingDomain) editingDomain;
											break;

										}
									}
								}
							}
						}
					} catch (PartInitException e) {
						e.printStackTrace();
					}
				}
			}
		});
	}

	private void applyChangeEntries(TransactionalEditingDomain domain, EMap<EObject, EList<FeatureChange>> objectChanges) {
		EMap<EObject, EList<FeatureChange>> entriesToApplyLater = new BasicEMap<EObject, EList<FeatureChange>>();
		int objectChangesSize = objectChanges.size();

		for (Map.Entry<EObject, EList<FeatureChange>> entry : objectChanges) {
			EObject eObject = entry.getKey();
			URI eobjectUri = EcoreUtil.getURI(eObject);
			Resource resourceToChange = eObject.eResource();

			if (eObject.eIsProxy()) {
				entriesToApplyLater.add(entry);
				continue;
			}
			EList<FeatureChange> featureChanges = entry.getValue();
			for (FeatureChange featureChange : featureChanges) {
				EStructuralFeature feature = featureChange.getFeature();
				String featureName = feature.getName();
				EList<ListChange> listChanges = featureChange.getListChanges();
				if (!listChanges.isEmpty()) {
					for (ListChange listChange : listChanges) {
						ChangeKind kind = listChange.getKind();
						int position = listChange.getIndex();
						EList<Object> newObject = listChange.getValues(); // newList
						EList<Object> oldList = (EList<Object>) eObject.eGet(feature);
						if (kind == ChangeKind.ADD_LITERAL) {
							// if an eObject in the newObject list already exists in the resource to change we don't have
							// to add it again...

							EList<Object> newObjectCopy = new BasicEList<Object>(newObject);
							for (Object object : newObject) {
								int index = newObject.indexOf(object);
								if (object instanceof EObject) {
									Resource resource = ((EObject) object).eResource();
									if (resource != null) {
										String id = ((XMIResource) resource).getID((EObject) object);
										if (id != null) {
											EObject objectinResourceToChange = resourceToChange.getEObject(id);
											if (objectinResourceToChange != null) {
												newObjectCopy.set(index, objectinResourceToChange);
											}
										}
									}
								}
							}

							newObject = newObjectCopy;
							for (Object object : newObject) {
								if (object instanceof EObject) {
									EcoreUtil.resolveAll((EObject) object);
								}
							}
							Map<EObject, String> eObjectToIDMap = createEObjectToIDMap(newObject);
							executeADDListChangeCommand(domain, oldList, position, newObject);
							if (!eObjectToIDMap.isEmpty()) {
								setID(eObjectToIDMap);
							}

						} else if (kind == ChangeKind.REMOVE_LITERAL) {
							executeREMOVEListChangeCommand(domain, oldList, position);

						} else if (kind == ChangeKind.MOVE_LITERAL) {
							int moveToIndex = listChange.getMoveToIndex();
							executeMOVEListChangeCommand(domain, oldList, position, moveToIndex);
						}
					}
				} else {
					Object newValue = featureChange.getValue();
					Map<EObject, String> eObjectToIDMap = null;
					if (newValue instanceof EObject) {
						EcoreUtil.resolveAll((EObject) newValue);
						eObjectToIDMap = new HashMap<EObject, String>();
						createEObjectToIDMap((EObject) newValue, eObjectToIDMap);
					}
					executeSETCommand(domain, eObject, feature, newValue);
					if (eObjectToIDMap != null && !eObjectToIDMap.isEmpty()) {
						setID(eObjectToIDMap);
					}

				}
			}
		}

		if (!entriesToApplyLater.isEmpty()) {
			if (entriesToApplyLater.size() == objectChangesSize) {
				return;
			}
			applyChangeEntries(domain, entriesToApplyLater);
		}

	}

	private void refreshEditpart(EObject edge) {
		if (viewer != null) {
			EditPart editPart = viewer.getContents();
			// if (editPart.getParent() != null) {
			// editPart.getParent().refresh();
			// } else {
			editPart.activate();
			editPart.getViewer().flush();
			editPart.getViewer().getRootEditPart().refresh();
			editPart.getViewer().getRootEditPart().getViewer().flush();
			editPart.getViewer().getRootEditPart().getViewer().getContents().refresh();
			editPart.getViewer().getRootEditPart().getViewer().reveal(editPart.getViewer().getRootEditPart());
			editPart.getViewer().flush();
			editPart.refresh();

			// }
			Map registry = viewer.getEditPartRegistry();
			if (editPart instanceof IGraphicalEditPart) {
				View notationView = ((IGraphicalEditPart) editPart).getNotationView();
				if (notationView instanceof Diagram) {
					// ((EditPart)registry.get(notationView)).refresh();
					// // @SuppressWarnings("unchecked")
					// // EList<Edge> edges = ((Diagram)notationView).getEdges();
					// // for (Edge edge : edges) {
					Object sourceEditPart = registry.get(((Edge) edge).getSource());
					Object targetEditPart = registry.get(((Edge) edge).getTarget());
					Object edgeEditPart = registry.get(edge);
					System.out.println();
					// //// Object connection = registry.get(edge);
					// if(sourceEditPart != null){ //&& targetEditPart != null){
					// ((EditPart)sourceEditPart).refresh();
					// // ((EditPart)targetEditPart).refresh();
					//
					// }
					// ////
					// if(targetEditPart != null){
					// ((EditPart)targetEditPart).refresh();
					//
					// }
					//
					if (edgeEditPart != null) {
						// ((EditPart)edgeEditPart).refresh();
						EditPart source = ((ConnectionEditPart) edgeEditPart).getSource();
						EditPart target = ((ConnectionEditPart) edgeEditPart).getTarget();
						if (source != sourceEditPart) {
							// ((ConnectionEditPart)edgeEditPart).setSource((EditPart) sourceEditPart);
							// editPart.refresh();
						}
						if (target != targetEditPart) {
							// ((ConnectionEditPart)edgeEditPart).setTarget((EditPart) targetEditPart);
							// editPart.refresh();
						}
						// ((EditPart)edgeEditPart).refresh();
						// editPart.getViewer().flush();
						// editPart.refresh();
						System.out.println();
					}
					//
					// }
				}
			}
			// editPart.getViewer().flush();
			// editPart.refresh();
		}
	}

	private void refreshTargetEditpart(EObject edge) {
		if (viewer != null) {
			EditPart editPart = viewer.getContents();
			Map registry = viewer.getEditPartRegistry();
			if (editPart instanceof IGraphicalEditPart) {
				View notationView = ((IGraphicalEditPart) editPart).getNotationView();
				if (notationView instanceof Diagram) {
					// @SuppressWarnings("unchecked")
					// EList<Edge> edges = ((Diagram)notationView).getEdges();
					// for (Edge edge : edges) {
					// Object sourceEditPart = registry.get(((Edge)edge).getSource());
					Object targetEditPart = registry.get(((Edge) edge).getTarget());
					// Object connection = registry.get(edge);
					if (/* sourceEditPart != null){ */targetEditPart != null) {
						// ((EditPart)sourceEditPart).refresh();
						((EditPart) targetEditPart).refresh();

					}

					// }
				}
			}

		}
	}

	private void applyResourceChanges(TransactionalEditingDomain domain, EList<ResourceChange> resourceChanges) {
		for (ResourceChange resourceChange : resourceChanges) {
			Resource resourceToChange = resourceChange.getResource();
			if (resourceToChange == null) {
				return;
			}
			EList<ListChange> listChanges = resourceChange.getListChanges();
			for (ListChange listChange : listChanges) {
				ChangeKind kind = listChange.getKind();
				int index = listChange.getIndex();
				EList<Object> newObject = listChange.getValues();
				EList<Object> oldList = (EList) resourceToChange.getContents();

				if (kind == ChangeKind.ADD_LITERAL) {
					for (Object object : newObject) {
						if (object instanceof EObject) {
							EcoreUtil.resolveAll((EObject) object);
							if (((EObject) object).eIsProxy()) {
								// System.out.println("resourceChange ADD proxy: " + EcoreUtil.getURI((EObject) object));
							}
						}
					}
					Map<EObject, String> eObjectToIDMap = createEObjectToIDMap(newObject);
					executeADDListChangeCommand(domain, oldList, index, newObject);
					if (!eObjectToIDMap.isEmpty()) {
						setID(eObjectToIDMap);
					}
				} else if (kind == ChangeKind.REMOVE_LITERAL) {
					executeREMOVEListChangeCommand(domain, oldList, index);
				}
			}
		}

	}

	private void modifyModel(final ChangeDescription changeDescription) {

		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {

			@Override
			public void run() {

				ResourceSet resourceSet = domain.getResourceSet();

				EMap<EObject, EList<FeatureChange>> objectChanges = changeDescription.getObjectChanges();

				EList<ResourceChange> resourceChanges = changeDescription.getResourceChanges();

				resolveProxies(resourceSet, objectChanges/* , resourceChanges */);

				if (!resourceChanges.isEmpty()) {
					applyResourceChanges(domain, resourceChanges);
				}

				if (!objectChanges.isEmpty()) {
					applyChangeEntries(domain, objectChanges);
				}

				Resource tempResource = resourceSet.getResource(URI.createURI("tempResource"), false);
				if (tempResource != null) {
					resourceSet.getResources().remove(tempResource);
					TransactionUtil.disconnectFromEditingDomain(tempResource);
				}

				if (viewer != null) {
					EditPart editPart = viewer.getContents();
					Map registry = viewer.getEditPartRegistry();
					if (editPart instanceof IGraphicalEditPart) {
						View notationView = ((IGraphicalEditPart) editPart).getNotationView();
						if (notationView instanceof Diagram) {
							// XXX OBEO CDIM
							try {
								// dia res
								EditPart contents = viewer.getContents();
								Object model = contents.getModel();
								if (model instanceof EObject) {
									EObject eObject = (EObject) model;
									Resource eResource = eObject.eResource();
									eResource.setModified(true);
								}

								// active editor
								IWorkbench wb = PlatformUI.getWorkbench();
								IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
								if (win != null) {
									IWorkbenchPage page = win.getActivePage();
									if (page != null) {
										IEditorPart activeEditor = page.getActiveEditor();
										if (activeEditor != null) {
											activeEditor.setFocus();
										}
									}
								}

								//

								EditPart editPart2 = (EditPart) viewer.getRootEditPart().getChildren().get(0);
								CanonicalEditPolicy canonicalEditPolicy2 = (CanonicalEditPolicy) editPart2
										.getEditPolicy(EditPolicyRoles.CANONICAL_ROLE);
								if (canonicalEditPolicy2 != null) {
									canonicalEditPolicy2.refresh();
									viewer.getContents().refresh();
								}

							} catch (Exception e) {
								e.printStackTrace();
							}
							//
							// @SuppressWarnings("unchecked")
							EList<Edge> edges = ((Diagram) notationView).getEdges();
							for (Edge edge : edges) {
								Object sourceEditPart = registry.get(((Edge) edge).getSource());
								Object targetEditPart = registry.get(((Edge) edge).getTarget());
								if (sourceEditPart != null && targetEditPart != null) {
									((EditPart) sourceEditPart).refresh();
									((EditPart) targetEditPart).refresh();
									System.out.println();
								}

							}
						}
					}
				}
				updateManager = null;
			}
		});
	}

	private void resolveProxies(ResourceSet resourceSet, EMap<EObject, EList<FeatureChange>> objectChanges/* , EList<ResourceChange> resourceChanges */) {
		for (Map.Entry<EObject, EList<FeatureChange>> entry : objectChanges) {
			EObject key = entry.getKey();
			if (key.eIsProxy()) {
				EcoreUtil.resolve(key, resourceSet);
			}
		}
	}

	private void setID(Map<EObject, String> eObjectToIDMap) {
		for (Iterator it = eObjectToIDMap.entrySet().iterator(); it.hasNext();) {
			Map.Entry<EObject, String> entry = (Entry<EObject, String>) it.next();
			EObject eObject = entry.getKey();
			Resource eObjectResource = eObject.eResource();
			if (eObjectResource instanceof XMIResource) {
				String idToSet = entry.getValue();
				if (idToSet != null) {
					((XMIResource) eObjectResource).setID(eObject, idToSet);
				}
			}

		}
	}

	private Map<EObject, String> createEObjectToIDMap(EList<Object> objectsFromChangeDescription) {
		Map<EObject, String> eObjectToIDMap = new HashMap<EObject, String>();
		for (Object object : objectsFromChangeDescription) {
			if (object instanceof EObject) {
				createEObjectToIDMap((EObject) object, eObjectToIDMap);
			}
		}

		return eObjectToIDMap;

	}

	private void createEObjectToIDMap(EObject objectFromChangeDescription, Map<EObject, String> eObjectToIDMap) {
		XMIResource resource = (XMIResource) objectFromChangeDescription.eResource();
		if (resource != null) {

			String id = resource.getID(objectFromChangeDescription);
			if (id != null) {
				eObjectToIDMap.put(objectFromChangeDescription, id);
			}
			for (Iterator it = EcoreUtil.getAllContents(objectFromChangeDescription, true); it.hasNext();) {
				EObject contentObject = (EObject) it.next();
				if (contentObject.eIsProxy()) {
					// System.out.println("eobjecttoIDMap: " + EcoreUtil.getURI(contentObject));
				}
				id = resource.getID(contentObject);
				if (id != null) {
					eObjectToIDMap.put(contentObject, id);
				}
			}
		}
	}
}
