package org.modelbus.team.eclipse.changemodelnotification.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.Collections;
import java.util.Iterator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.change.ChangeDescription;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;

public class ChangeModelUtil {

	public static final String	OPTION_MODELBUS_UPDATE	= "modelbus_update";

	public static ChangeDescription stringToEObject(String changeDescriptionAsString, ResourceSet resourceSet) {
		System.out.println("ChangeDescription: " + changeDescriptionAsString);
		Resource tempResource = resourceSet.getResource(URI.createURI("tempResource"), false);
		if (tempResource == null) {
			// System.out.println("UpdateManger: tempResource has to be created");
			tempResource = resourceSet.createResource(URI.createURI("tempResource"));
		}
		// tempResource.getContents().clear();
		ByteArrayInputStream input = new ByteArrayInputStream(changeDescriptionAsString.getBytes());
		try {
			// System.out.println("tempResource start loading");
			tempResource.unload();
			tempResource.load(input, Collections.EMPTY_MAP);

		} catch (Exception e) {
			// TODO Auto-generated catch block
			for (Resource resource : resourceSet.getResources()) {
				if (!resource.getURI().toString().equals("tempResource")) {
					ChangeModelUtil.registerPackages(resource);
				}
			}
			try {
				input.close();
				input = new ByteArrayInputStream(changeDescriptionAsString.getBytes());
				tempResource.unload();
				tempResource.load(input, Collections.EMPTY_MAP);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				throw new RuntimeException(e1);
			}

		}

		// EcoreUtil.resolveAll(tempResource);

		EObject eObject = tempResource.getContents().get(0);
		// EcoreUtil.resolveAll(eObject);

		return (ChangeDescription) eObject;
	}

	public static String getRemoteModelUri(String modelUri) {
		URI resourceURI = URI.createURI(modelUri);
		URI projectURI = null;
		String remoteLocation = null;
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();

		IProject project = null;
		for (IProject currentProject : projects) {
			if (resourceURI.isPlatformResource()) {
				projectURI = URI.createPlatformResourceURI(currentProject.getName(), true);
			} else if (resourceURI.isFile()) {
				projectURI = URI.createFileURI(currentProject.getLocation().toString());
			}
			if (projectURI != null) { // TODO OBEO TO CORRECT BIG ERROR
				if (projectURI != null && modelUri.startsWith(projectURI.toString() + "/") || modelUri.equals(projectURI.toString())) {
					project = currentProject;
					break;
				}
			}

		}
		if (project != null) {
			IResource resource = project.getFile(URLDecoder.decode(modelUri.replace(projectURI.toString() + "/", "")));
			RepositoryProvider repositoryProvider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);

			if (repositoryProvider != null) {
				ModelBusRepositoryProvider modelBusRepositoryProvider = (ModelBusRepositoryProvider) repositoryProvider;

				remoteLocation = modelBusRepositoryProvider.getRepositoryPath(resource);

			}
		}
		return remoteLocation;
	}

	public static void registerPackages(Resource resource) {
		EPackage.Registry registry = resource.getResourceSet().getPackageRegistry();

		Iterator iterator = resource.getAllContents();
		while (iterator.hasNext()) {
			EObject currentObject = (EObject) iterator.next();
			if (currentObject instanceof EAnnotation) {
				EAnnotation annotation = (EAnnotation) currentObject;
				EList contents = annotation.getContents();
				for (Iterator it = contents.iterator(); it.hasNext();) {
					EObject object = (EObject) it.next();
					if (object instanceof EPackage) {
						EPackage currentPackage = (EPackage) object;
						registry.put(currentPackage.getNsURI(), currentPackage);

					}

				}
			} else if (currentObject instanceof EPackage) {
				EPackage currentPackage = (EPackage) currentObject;
				registry.put(currentPackage.getNsURI(), currentPackage);

			}
		}

	}

}
