/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.history;

import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.core.history.provider.FileHistory;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;

/**
 * IResource-based resource history model object
 * 
 * @author Alexander Gurov
 */
public class ModelBusResourceHistory extends FileHistory {
	protected static IFileRevision[]	EMPTY	= new IFileRevision[0];

	protected ILocalResource			local;
	protected IRepositoryResource		remote;
	protected ModelBusLogEntry[]		msgs;
	protected boolean					full;

	public ModelBusResourceHistory(ILocalResource local, IRepositoryResource remote, ModelBusLogEntry[] msgs, boolean full) {
		this.local = local;
		this.remote = remote;
		this.msgs = msgs;
		this.full = full;
	}

	public IFileRevision[] getContributors(IFileRevision revision) {
		int idx = this.getRevisionIdx(revision.getContentIdentifier());
		if (idx != -1 && idx < this.msgs.length - 1) {
			return new IFileRevision[] { new ModelBusRemoteResourceRevision(this.remote, this.msgs[idx + 1]) };
		}
		return ModelBusResourceHistory.EMPTY;
	}

	public IFileRevision getFileRevision(String id) {
		if (ModelBusRevision.WORKING.toString().equals(id)) {
			return new ModelBusLocalResourceRevision(this.local, ModelBusRevision.WORKING);
		} else if (ModelBusRevision.BASE.toString().equals(id)) {
			return new ModelBusLocalResourceRevision(this.local, ModelBusRevision.BASE);
		} else if (ModelBusRevision.HEAD.toString().equals(id)) {
			return this.full ? new ModelBusRemoteResourceRevision(this.remote, this.msgs[0]) : null;
		} else if (ModelBusRevision.START.toString().equals(id)) {
			return this.full ? new ModelBusRemoteResourceRevision(this.remote, this.msgs[this.msgs.length - 1]) : null;
		} else if (ModelBusRevision.PREVIOUS.toString().equals(id)) {
			int idx = this.getRevisionIdx(this.local.getRevision());
			return idx != -1 && idx < this.msgs.length - 1 ? new ModelBusRemoteResourceRevision(this.remote, this.msgs[idx + 1]) : null;
		} else if (ModelBusRevision.COMMITTED.toString().equals(id)) {
			int idx = this.getRevisionIdx(this.local.getRevision());
			return idx != -1 ? new ModelBusRemoteResourceRevision(this.remote, this.msgs[idx]) : null;
		}

		int idx = this.getRevisionIdx(id);
		if (idx != -1) {
			return new ModelBusRemoteResourceRevision(this.remote, this.msgs[idx]);
		}
		return null;
	}

	public IFileRevision[] getFileRevisions() {
		IFileRevision[] revs = new IFileRevision[this.msgs.length];
		for (int i = 0; i < this.msgs.length; i++) {
			revs[i] = new ModelBusRemoteResourceRevision(this.remote, this.msgs[i]);
		}
		return revs;
	}

	public IFileRevision[] getTargets(IFileRevision revision) {
		int idx = this.getRevisionIdx(revision.getContentIdentifier());
		if (idx > 0) {
			return new IFileRevision[] { new ModelBusRemoteResourceRevision(this.remote, this.msgs[idx - 1]) };
		}
		return ModelBusResourceHistory.EMPTY;
	}

//	protected int getRevisionIdx(String id) {
//		long revision;
//		try {
//			revision = Long.parseLong(id);
//		} catch (Exception ex) {
//			return -1;
//		}
//		return this.getRevisionIdx(revision);
//	}

	protected int getRevisionIdx(ModelBusRevision revision) {
		for (int i = 0; i < this.msgs.length; i++) {
			if (this.msgs[i].modelBusRevision.equals(revision)) {
				return i;
			}
		}
		return -1;
	}

	protected int getRevisionIdx(String revision) {
		for (int i = 0; i < this.msgs.length; i++) {
			if (this.msgs[i].modelBusRevision.equals(revision)) {
				return i;
			}
		}
		return -1;
	}
	
}
