/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;

/**
 * "Add to modelbus::ignore" operation implementation
 * 
 * @author Alexander Gurov
 */
public class AddToModelBusIgnoreOperation extends AbstractFileOperation {
	protected int		ignoreType;
	protected String	pattern;

	public AddToModelBusIgnoreOperation(File[] files, int ignoreType, String pattern) {
		super("Operation.AddToModelBusIgnoreFile", files);

		this.ignoreType = ignoreType;
		this.pattern = pattern;
	}

	public AddToModelBusIgnoreOperation(IFileProvider provider, int ignoreType, String pattern) {
		super("Operation.AddToModelBusIgnoreFile", provider);

		this.ignoreType = ignoreType;
		this.pattern = pattern;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					AddToModelBusIgnoreOperation.this.handleResource(current);
				}
			}, monitor, files.length);
		}
	}

	protected void handleResource(File current) throws Exception {
		File parent = current.getParentFile();
		if (parent == null) {
			return;
		}
		IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(parent, false);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			org.modelbus.team.eclipse.core.operation.local.AddToModelBusIgnoreOperation.changeIgnoreProperty(proxy, this.ignoreType, this.pattern,
					parent.getAbsolutePath(), current.getName());
		} finally {
		}
	}

}
