/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Add to version control operation
 * 
 * @author Alexander Gurov
 */
public class AddToModelBusOperation extends AbstractFileOperation {
	protected boolean	isRecursive;

	public AddToModelBusOperation(File[] files, boolean isRecursive) {
		super("Operation.AddToModelBusFile", files);
		this.isRecursive = isRecursive;
	}

	public AddToModelBusOperation(IFileProvider provider, boolean isRecursive) {
		super("Operation.AddToModelBusFile", provider);
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();
		if (this.isRecursive) {
			files = FileUtility.shrinkChildNodes(files, false);
		} else {
			FileUtility.reorder(files, true);
		}

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					AddToModelBusOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD,
							"modelbus add \"" + FileUtility.normalizePath(current.getAbsolutePath()) + "\""
									+ (AddToModelBusOperation.this.isRecursive ? "" : " -N") + "\n");

					File parent = current.getParentFile();
					if (parent != null) {
						org.modelbus.team.eclipse.core.operation.local.AddToModelBusOperation.removeFromParentIgnore(proxy, parent.getAbsolutePath(),
								current.getName());
					}

					proxy.add(current.getAbsolutePath(), Depth.infinityOrEmpty(AddToModelBusOperation.this.isRecursive),
							IModelBusConnector.Options.FORCE | IModelBusConnector.Options.INCLUDE_PARENTS, new ModelBusProgressMonitor(
									AddToModelBusOperation.this, monitor, null));
				}
			}, monitor, files.length);
		}
	}

}
