/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorUnresolvedConflictException;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Commit operation implementation
 * 
 * @author Alexander Gurov
 */
public class CommitOperation extends AbstractFileConflictDetectionOperation implements IRevisionProvider {
	protected boolean	recursive;
	protected boolean	keepLocks;
	protected String	message;
	protected ArrayList	revisionsPairs;

	protected String[]	paths;

	public CommitOperation(File[] files, String message, boolean recursive, boolean keepLocks) {
		super("Operation.CommitFile", files);
		this.message = message;
		this.recursive = recursive;
		this.keepLocks = keepLocks;
	}

	public CommitOperation(IFileProvider provider, String message, boolean recursive, boolean keepLocks) {
		super("Operation.CommitFile", provider);
		this.message = message;
		this.recursive = recursive;
		this.keepLocks = keepLocks;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionsPairs == null ? null : (RevisionPair[]) this.revisionsPairs.toArray(new RevisionPair[this.revisionsPairs.size()]);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.revisionsPairs = new ArrayList();
		File[] files = this.operableData();

		this.defineInitialResourceSet(files);

		if (this.recursive) {
			files = FileUtility.shrinkChildNodes(files, false);
		} else {
			FileUtility.reorder(files, true);
		}
		// CHECKMWA
		// TODOMWA
		// if ((CoreExtensionsManager.instance().getModelBusConnectorFactory().getSupportedFeatures() &
		// IModelBusConnectorFactory.OptionalFeatures.ATOMIC_X_COMMIT) != 0) {
		// if (ModelBusConnector.isSupportAtomicCommit()) {
		// Map proxy2Resources = ModelBusUtility.splitRepositoryLocations(files);
		// for (Iterator it = proxy2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled(); ) {
		// Map.Entry entry = (Map.Entry)it.next();
		// this.performCommit((List)entry.getValue(), monitor, proxy2Resources.size());
		// }
		// }
		// else {
		// Map project2Resources = ModelBusUtility.splitWorkingCopies(files);
		// for (Iterator it = project2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled(); ) {
		// Map.Entry entry = (Map.Entry)it.next();
		// this.performCommit((List)entry.getValue(), monitor, project2Resources.size());
		// }
		// }
		throw new RuntimeException("TODOMWA");
	}

	protected void performCommit(List files, final IProgressMonitor monitor, int total) {
		this.paths = FileUtility.asPathArray((File[]) files.toArray(new File[files.size()]));

		this.complexWriteToConsole(new Runnable() {
			public void run() {
				CommitOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus commit");
				for (int i = 0; i < CommitOperation.this.paths.length && !monitor.isCanceled(); i++) {
					CommitOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + CommitOperation.this.paths[i] + "\"");
				}
				CommitOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD,
						(CommitOperation.this.recursive ? "" : " -N") + (CommitOperation.this.keepLocks ? " --no-unlock" : "") + " -m \""
								+ CommitOperation.this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			}
		});

		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				ModelBusRevision revisionNumbers[] = proxy.commit(CommitOperation.this.paths, CommitOperation.this.message, null, Depth
						.infinityOrEmpty(CommitOperation.this.recursive), CommitOperation.this.keepLocks ? IModelBusConnector.Options.KEEP_LOCKS
						: IModelBusConnector.Options.NONE, null, new ModelBusProgressMonitor(CommitOperation.this, monitor, null));
				if (revisionNumbers.length > 0) {
					CommitOperation.this.revisionsPairs.add(new RevisionPair(revisionNumbers[0], CommitOperation.this.paths));
					String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
							new String[] { String.valueOf(revisionNumbers[0]) });
					CommitOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
				}
			}
		}, monitor, total);
	}

	protected void reportError(Throwable t) {
		if (t instanceof ModelBusConnectorUnresolvedConflictException) {
			this.hasUnresolvedConflict = true;
			this.conflictMessage = t.getMessage();
			for (int i = 0; i < this.paths.length; i++) {
				for (Iterator it = this.processed.iterator(); it.hasNext();) {
					File res = (File) it.next();
					if (new Path(res.getAbsolutePath()).equals(new Path(this.paths[i]))) {
						it.remove();
						this.unprocessed.add(res);
						break;
					}
				}
			}
		} else {
			super.reportError(t);
		}
	}

}
