/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;

/**
 * Create patch operation implementation
 * 
 * @author Alexander Gurov
 */
public class CreatePatchOperation extends AbstractFileOperation {
	protected String	fileName;
	protected boolean	recurse;
	protected boolean	ignoreDeleted;
	protected boolean	processBinary;
	protected boolean	useRelativePath;

	public CreatePatchOperation(File file, String fileName, boolean recurse, boolean ignoreDeleted, boolean processBinary, boolean useRelativePath) {
		super("Operation.CreatePatchFile", new File[] { file });
		this.fileName = fileName;
		this.recurse = recurse;
		this.ignoreDeleted = ignoreDeleted;
		this.processBinary = processBinary;
		this.useRelativePath = useRelativePath;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File file = this.operableData()[0];
		IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(file, false);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus diff " + (this.recurse ? "" : " -N")
					+ (this.ignoreDeleted ? " --no-diff-deleted" : "") + "\n");
			String path = file.getAbsolutePath();
			long options = IModelBusConnector.Options.IGNORE_ANCESTRY;
			options |= this.ignoreDeleted ? IModelBusConnector.Options.SKIP_DELETED : IModelBusConnector.Options.NONE;
			options |= this.processBinary ? IModelBusConnector.Options.FORCE : IModelBusConnector.Options.NONE;
			proxy.diff(new ModelBusEntryRevisionReference(path, null, ModelBusRevision.BASE), new ModelBusEntryRevisionReference(path, null,
					ModelBusRevision.WORKING), this.useRelativePath ? path : null, this.fileName, Depth.infinityOrFiles(this.recurse), options, null,
					new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

}
