/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * JavaHL-mode merge implementation
 * 
 * @author Alexander Gurov
 */
public class JavaHLMergeOperation extends AbstractFileOperation {
	protected IRepositoryResource			from1;
	protected IRepositoryResource			from2;
	protected boolean						dryRun;
	protected IModelBusNotificationCallback	notify;

	public JavaHLMergeOperation(File localTo, IRepositoryResource from1, IRepositoryResource from2, boolean dryRun,
			IModelBusNotificationCallback notify) {
		super("Operation.JavaHLMergeFile", new File[] { localTo });
		this.from1 = from1;
		this.from2 = from2;
		this.dryRun = dryRun;
		this.notify = notify;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File file = this.operableData()[0];

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		proxy.setTouchUnresolved(true);
		if (this.notify != null) {
			ModelBusUtility.addModelBusNotifyListener(proxy, this.notify);
		}
		try {
			proxy.merge(ModelBusUtility.getEntryRevisionReference(this.from1), ModelBusUtility.getEntryRevisionReference(this.from2),
					file.getAbsolutePath(), Depth.INFINITY, this.dryRun ? IModelBusConnector.Options.SIMULATE : IModelBusConnector.Options.NONE,
					new ModelBusProgressMonitor(this, monitor, null));
		} finally {
			if (this.notify != null) {
				ModelBusUtility.removeModelBusNotifyListener(proxy, this.notify);
			}
			proxy.setTouchUnresolved(false);
		}
	}

}
