/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConflictResolution;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Mark conflicts as resolved
 * 
 * @author Alexander Gurov
 */
public class MarkResolvedOperation extends AbstractFileOperation {
	protected boolean	recursive;

	public MarkResolvedOperation(File[] files, boolean recursive) {
		super("Operation.MarkResolvedFile", files);
		this.recursive = recursive;
	}

	public MarkResolvedOperation(IFileProvider provider, boolean recursive) {
		super("Operation.MarkResolvedFile", provider);
		this.recursive = recursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		if (this.recursive) {
			files = FileUtility.shrinkChildNodes(files, false);
		} else {
			FileUtility.reorder(files, true);
		}

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.resolve(current.getAbsolutePath(), ModelBusConflictResolution.CHOOSE_MERGED, Depth
							.infinityOrEmpty(MarkResolvedOperation.this.recursive), new ModelBusProgressMonitor(MarkResolvedOperation.this, monitor,
							null));
				}
			}, monitor, files.length);
		}
	}

}
