/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IPath;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.modelbusstorage.AbstractModelBusStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * ModelBus storage provider based on java.io.File
 * 
 * @author Alexander Gurov
 */
public class ModelBusFileStorage extends AbstractModelBusStorage implements IFileStorage {
	/**
	 * The name of the preferences node in the Subversive Core preferences that contains the known repositories as its children (for integrations).
	 */
	public static final String			PREF_REPOSITORIES_NODE	= "externalRepositories";

	/**
	 * The name of file containing the ModelBus repository locations information (for integration). Deprecated since Subversive 0.7.0 v20080404 - must
	 * not be used. The valid information is stored in preferences.
	 * 
	 * @see ModelBusFileStorage.PREF_REPOSITORIES_NODE
	 */
	public static final String			STATE_INFO_FILE_NAME	= ".externalModelBusRepositories";

	private static ModelBusFileStorage	instance				= new ModelBusFileStorage();

	public static ModelBusFileStorage instance() {
		return ModelBusFileStorage.instance;
	}

	public void initialize(IPath stateInfoLocation) throws Exception {
		this.setStateInfoFile(stateInfoLocation, ModelBusFileStorage.STATE_INFO_FILE_NAME);
		this.initializeImpl(ModelBusFileStorage.PREF_REPOSITORIES_NODE);
	}

	public IRepositoryResource asRepositoryResource(File file, boolean allowsNull) {
		// check if this resource is placed in working copy
		File wcRoot = file;
		ModelBusEntryInfo info = null;
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// CoreExtensionsManager.instance().getModelBusConnectorFactory().newInstance();
		try {
			while (info == null) {
				if (wcRoot == null) {
					// no WC found
					if (allowsNull) {
						return null;
					}
					throw new RuntimeException(file.getAbsolutePath() + " is not under version control");
				}
				info = ModelBusUtility.getModelBusInfo(wcRoot, proxy);
				if (info == null) {
					wcRoot = wcRoot.getParentFile();
				}
			}
		} finally {
			proxy.dispose();
		}

		String wcUrl = ModelBusUtility.decodeURL(info.url);
		String rootUrl = ModelBusUtility.decodeURL(info.reposRootUrl);

		if (wcRoot != file) {
			wcUrl += file.getAbsolutePath().substring(wcRoot.getAbsolutePath().length());
		}

		return file.isFile() ? (IRepositoryResource) ModelBusConnector.asRepositoryFile(wcUrl, allowsNull) : ModelBusConnector.asRepositoryContainer(
				wcUrl, allowsNull);
	}

	private ModelBusFileStorage() {
		super();
	}

}
