/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.management;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Disconnect files from source control by ModelBus meta-information deletion. Only folders could be disconnected.
 * 
 * @author Alexander Gurov
 */
public class DisconnectOperation extends AbstractFileOperation {
	public DisconnectOperation(File[] files) {
		super("Operation.DisconnectFile", files);
	}

	public DisconnectOperation(IFileProvider provider) {
		super("Operation.DisconnectFile", provider);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.disconnect(this.operableData(), monitor);
	}

	protected void disconnect(File[] files, IProgressMonitor monitor) {
		files = FileUtility.shrinkChildNodes(files, true);
		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			File meta = new File(files[i].getAbsolutePath() + "/" + ModelBusUtility.getModelBusFolderName());
			FileUtility.deleteRecursive(meta, monitor);
			if (files[i].isDirectory()) {
				this.disconnect(files[i].listFiles(), monitor);
			}
		}
	}

}
