/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.management;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Relocate specified folders to the new url.
 * 
 * @author Alexander Gurov
 */
public class RelocateOperation extends AbstractFileOperation {
	protected String	toUrl;

	public RelocateOperation(File[] folders, String toUrl) {
		super("Operation.RelocateFile", folders);
		this.toUrl = toUrl;
	}

	public RelocateOperation(IFileProvider provider, String toUrl) {
		super("Operation.RelocateFile", provider);
		this.toUrl = toUrl;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = FileUtility.shrinkChildNodes(this.operableData(), true);

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			final IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(current, false);
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					String path = current.getAbsolutePath();
					RelocateOperation.this.writeToConsole(
							IConsoleStream.LEVEL_CMD,
							"modelbus switch --relocate \"" + remote.getUrl() + "\" \"" + RelocateOperation.this.toUrl + "\" \""
									+ FileUtility.normalizePath(path) + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
					proxy.relocate(ModelBusUtility.encodeURL(remote.getUrl()), ModelBusUtility.encodeURL(RelocateOperation.this.toUrl), path,
							Depth.INFINITY, new ModelBusProgressMonitor(RelocateOperation.this, monitor, null));
				}
			}, monitor, files.length);
		}
	}

}
