/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.property;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Get resource properties operation
 * 
 * @author Alexander Gurov
 */
public class GetPropertiesOperation extends AbstractFileOperation {
	protected ModelBusProperty[]	properties;
	protected ModelBusRevision		revision;

	public GetPropertiesOperation(File file) {
		this(file, ModelBusRevision.WORKING);
	}

	public GetPropertiesOperation(File file, ModelBusRevision revision) {
		super("Operation.GetPropertiesFile", new File[] { file });
		this.revision = revision;
	}

	public ModelBusProperty[] getProperties() {
		return this.properties;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File file = this.operableData()[0];
		IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(file, false);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			this.properties = ModelBusUtility.properties(proxy, new ModelBusEntryRevisionReference(file.getAbsolutePath(), null, this.revision),
					new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

}
