/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.property;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Remove resource property operation
 * 
 * @author Alexander Gurov
 */
public class RemovePropertyOperation extends AbstractFileOperation {
	protected boolean	isRecursive;
	protected String[]	names;

	public RemovePropertyOperation(File[] files, String[] names, boolean isRecursive) {
		super("Operation.RemovePropertiesFile", files);
		this.names = names;
		this.isRecursive = isRecursive;
	}

	public RemovePropertyOperation(IFileProvider provider, String[] names, boolean isRecursive) {
		super("Operation.RemovePropertiesFile", provider);
		this.names = names;
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();
		if (this.isRecursive) {
			files = FileUtility.shrinkChildNodes(files, false);
		}

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(files[i], false);
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					for (int i = 0; i < RemovePropertyOperation.this.names.length && !monitor.isCanceled(); i++) {
						final String name = RemovePropertyOperation.this.names[i];
						RemovePropertyOperation.this.protectStep(new IUnprotectedOperation() {
							public void run(IProgressMonitor monitor) throws Exception {
								proxy.removeProperty(current.getAbsolutePath(), name, RemovePropertyOperation.this.isRecursive ? Depth.INFINITY
										: Depth.EMPTY, null, new ModelBusProgressMonitor(RemovePropertyOperation.this, monitor, null));
							}
						}, monitor, RemovePropertyOperation.this.names.length);
					}
				}
			}, monitor, files.length);
		}
	}

	protected ISchedulingRule getSchedulingRule(File file) {
		return file.isDirectory() ? new LockingRule(file) : super.getSchedulingRule(file);
	}

}
