/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.property;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Set resource property operation
 * 
 * @author Alexander Gurov
 */
public class SetPropertyOperation extends AbstractFileOperation {
	protected ModelBusProperty[]	propertyData;
	protected boolean				isRecursive;

	public SetPropertyOperation(File[] files, String name, byte[] data, boolean isRecursive) {
		this(files, new ModelBusProperty[] { new ModelBusProperty(name, new String(data)) }, isRecursive);
	}

	public SetPropertyOperation(File[] files, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.SetPropertiesFile", files);
		this.propertyData = data;
		this.isRecursive = isRecursive;
	}

	public SetPropertyOperation(IFileProvider provider, String name, byte[] data, boolean isRecursive) {
		this(provider, new ModelBusProperty[] { new ModelBusProperty(name, new String(data)) }, isRecursive);
	}

	public SetPropertyOperation(IFileProvider provider, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.SetPropertiesFile", provider);
		this.propertyData = data;
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();
		if (this.isRecursive) {
			files = FileUtility.shrinkChildNodes(files, false);
		}

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(files[i], false);
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					for (int i = 0; i < SetPropertyOperation.this.propertyData.length && !monitor.isCanceled(); i++) {
						final ModelBusProperty property = SetPropertyOperation.this.propertyData[i];
						SetPropertyOperation.this.protectStep(new IUnprotectedOperation() {
							public void run(IProgressMonitor monitor) throws Exception {
								proxy.setProperty(current.getAbsolutePath(), property.name, property.value,
										Depth.infinityOrEmpty(SetPropertyOperation.this.isRecursive), IModelBusConnector.Options.NONE, null,
										new ModelBusProgressMonitor(SetPropertyOperation.this, monitor, null));
							}
						}, monitor, SetPropertyOperation.this.propertyData.length);
					}
				}
			}, monitor, files.length);
		}
	}

	protected ISchedulingRule getSchedulingRule(File file) {
		return file.isDirectory() ? new LockingRule(file) : super.getSchedulingRule(file);
	}

}
