/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.refactor;

import java.io.File;
import java.io.FileFilter;
import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Copy files between working copies
 * 
 * @author Alexander Gurov
 */
public class CopyOperation extends AbstractFileOperation {
	protected File		localTo;
	protected boolean	forceNonModelBus;

	public CopyOperation(File[] files, File localTo, boolean forceNonModelBus) {
		super("Operation.CopyFile", files);
		this.localTo = localTo;
		this.forceNonModelBus = forceNonModelBus;
	}

	public CopyOperation(IFileProvider provider, File localTo, boolean forceNonModelBus) {
		super("Operation.CopyFile", provider);
		this.localTo = localTo;
		this.forceNonModelBus = forceNonModelBus;
	}

	public ISchedulingRule getSchedulingRule() {
		ISchedulingRule parentRule = super.getSchedulingRule();
		return MultiRule.combine(new LockingRule(this.localTo), parentRule);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		IRepositoryResource remoteTo = ModelBusFileStorage.instance().asRepositoryResource(this.localTo, true);
		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			final File current = files[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(current, true);
					File checked = CopyOperation.this.getCopyTo(current);
					if (remote == null || proxy == null || CopyOperation.this.forceNonModelBus) {
						CopyOperation.this.nonModelBusCopy(current, monitor);
					} else {
						proxy.copy(new String[] { current.getAbsolutePath() }, checked.getAbsolutePath(), ModelBusRevision.WORKING,
								new ModelBusProgressMonitor(CopyOperation.this, monitor, null));
					}
				}
			}, monitor, files.length);
		}
	}

	protected File getCopyTo(File what) {
		File checked = new File(this.localTo.getAbsolutePath() + "/" + what.getName());
		if (checked.exists()) {
			String message = this.getNationalizedString("Error.AlreadyExists");
			throw new UnreportableException(MessageFormat.format(message, new Object[] { checked.getAbsolutePath() }));
		}
		return checked;
	}

	protected void nonModelBusCopy(File what, IProgressMonitor monitor) throws Exception {
		FileUtility.copyAll(this.localTo, what, FileUtility.COPY_NO_OPTIONS, new FileFilter() {
			public boolean accept(File pathname) {
				return !pathname.getName().equals(ModelBusUtility.getModelBusFolderName());
			}
		}, monitor);
	}

}
