/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Options;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * This operation provide Info2 information for local resource
 * 
 * @author Alexander Gurov
 */
public class InfoOperation extends AbstractActionOperation {
	protected IResource			resource;
	protected ILocalResource	local;
	protected ModelBusEntryInfo	info;

	public InfoOperation(IResource resource) {
		super("Operation.Info");
		this.resource = resource;
	}

	public ModelBusEntryInfo getInfo() {
		return this.info;
	}

	public ILocalResource getLocal() {
		return this.local;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.info = null;
		this.local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(this.resource);

		if (IStateFilter.SF_ONREPOSITORY.accept(this.local)) {
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus info \"" + this.local.getWorkingCopyPath() + "\"\n");
				ModelBusChangeStatus[] statuses = ModelBusUtility.status(proxy, FileUtility.getWorkingCopyPath(this.resource), Depth.EMPTY,
						Options.INCLUDE_UNCHANGED, new ModelBusProgressMonitor(this, monitor, null));
				if (statuses != null && statuses.length > 0) {
					ModelBusUtility.reorder(statuses, true);
					this.info = new ModelBusEntryInfo(statuses[0].path, statuses[0].url, statuses[0].revision, statuses[0].nodeKind, null, null,
							statuses[0].lastChangedRevision, statuses[0].lastChangedDate, statuses[0].lastCommitAuthor, null, true, 0,
							statuses[0].urlCopiedFrom, statuses[0].revisionCopiedFrom, statuses[0].textStatus, statuses[0].propStatus, null,
							statuses[0].conflictOld, statuses[0].conflictNew, statuses[0].conflictWorking);
					ModelBusEntryInfo[] infos = ModelBusUtility.info(proxy, new ModelBusEntryRevisionReference(statuses[0].url, null,
							statuses[0].revision), Depth.EMPTY, new ModelBusProgressMonitor(this, monitor, null));

					if (infos != null && infos.length > 0) {
						this.info = infos[0];
					}
				}
			} finally {
			}
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.resource.getName() });
	}

}
