/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Revert operation implementation
 * 
 * @author Alexander Gurov
 */
public class RevertOperation extends AbstractWorkingCopyOperation {
	protected boolean	doRecursiveRevert;

	public RevertOperation(IResource[] resources, boolean doRecursiveRevert) {
		super("Operation.Revert", resources);
		this.doRecursiveRevert = doRecursiveRevert;
	}

	public RevertOperation(IResourceProvider provider, boolean doRecursiveRevert) {
		super("Operation.Revert", provider);
		this.doRecursiveRevert = doRecursiveRevert;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		if (this.doRecursiveRevert) {
			resources = FileUtility.shrinkChildNodes(resources);
		} else {
			FileUtility.reorder(resources, false);
		}

		IRemoteStorage storage = ModelBusRemoteStorage.instance();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final String wcPath = FileUtility.getWorkingCopyPath(resources[i]);
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus revert \"" + FileUtility.normalizePath(wcPath) + "\""
					+ (this.doRecursiveRevert ? " -R" : "") + "\n");
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.revert(wcPath, Depth.infinityOrEmpty(RevertOperation.this.doRecursiveRevert), null, new ModelBusProgressMonitor(
							RevertOperation.this, monitor, null));
				}
			}, monitor, resources.length);
		}
	}

}
