/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.change;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.resource.ILocalFile;
import org.modelbus.team.eclipse.core.resource.ILocalFolder;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Local resource change store
 * 
 * @author Alexander Gurov
 */
public abstract class ResourceChange {
	protected ILocalResource		local;
	protected File					tmp;
	protected ModelBusProperty[]	properties;

	public ResourceChange(ResourceChange parent, ILocalResource local, boolean needsTemporary) throws Exception {
		this.local = local;
		if (needsTemporary) {
			this.tmp = File.createTempFile("merge", ".tmp",
					parent == null ? ModelBusTeamPlugin.instance().getStateLocation().toFile() : parent.getTemporary());
			this.tmp.delete();
		}
		this.properties = null;
	}

	public ILocalResource getLocal() {
		return this.local;
	}

	public ModelBusProperty[] getProperties() {
		return this.properties;
	}

	public void setProperties(ModelBusProperty[] properties) {
		this.properties = properties;
	}

	public File getTemporary() {
		return this.tmp;
	}

	public void disposeChangeModel(IProgressMonitor monitor) throws Exception {
		if (this.tmp != null) {
			FileUtility.deleteRecursive(this.tmp, monitor);
		}
	}

	public void traverse(IResourceChangeVisitor visitor, int depth, IActionOperationProcessor processor, IProgressMonitor monitor) throws Exception {
		this.preTraverse(visitor, depth, processor, monitor);
		this.postTraverse(visitor, depth, processor, monitor);
	}

	protected abstract void preTraverse(IResourceChangeVisitor visitor, int depth, IActionOperationProcessor processor, IProgressMonitor monitor)
			throws Exception;

	protected abstract void postTraverse(IResourceChangeVisitor visitor, int depth, IActionOperationProcessor processor, IProgressMonitor monitor)
			throws Exception;

	public static ResourceChange wrapLocalResource(ResourceChange parent, ILocalResource local, boolean needsTemporary) throws Exception {
		if (local == null) {
			return null;
		}
		return local instanceof ILocalFile ? new FileChange(parent, (ILocalFile) local, needsTemporary) : (ResourceChange) new FolderChange(parent,
				(ILocalFolder) local, needsTemporary);
	}

}
