/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.management;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.ModelBusTeamProjectMapper;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.UnreportableException;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Reconnect project that has ModelBus metainformation
 * 
 * @author Alexander Gurov
 */
public class ReconnectProjectOperation extends AbstractWorkingCopyOperation {

	public ReconnectProjectOperation(IProject[] projects) {
		super("Operation.Reconnect", projects);
	}

	public ReconnectProjectOperation(IResourceProvider provider) {
		super("Operation.Reconnect", provider);
	}

	public ISchedulingRule getSchedulingRule() {
		// reconnect always requires root as scheduling rule
		return ResourcesPlugin.getWorkspace().getRoot();
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IProject project = (IProject) resources[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					ModelBusChangeStatus st = ModelBusUtility.getModelBusInfoForNotConnected(project);
					if (st == null) {
						throw new UnreportableException(ModelBusTeamPlugin.instance().getResource("Error.NonModelBusPath"));
					}
					IRepositoryContainer remote = ModelBusConnector.asRepositoryContainer(ModelBusUtility.decodeURL(st.url), false);
					ModelBusTeamProjectMapper.map(project, remote);
				}
			}, monitor, resources.length);
		}
	}

}
