/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.refactor;

import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusResourceRuleFactory;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Copy only work files without ModelBus metainformation operation implementation
 * 
 * @author Alexander Gurov
 */
public class CopyResourceOperation extends AbstractActionOperation {
	protected IResource	source;
	protected IResource	destination;
	protected boolean	skipModelBusMeta;

	public CopyResourceOperation(IResource source, IResource destination) {
		this(source, destination, true);
	}

	public CopyResourceOperation(IResource source, IResource destination, boolean skipModelBusMeta) {
		super("Operation.CopyLocal");
		this.source = source;
		this.destination = destination;
		this.skipModelBusMeta = skipModelBusMeta;
	}

	public ISchedulingRule getSchedulingRule() {
		return ModelBusResourceRuleFactory.INSTANCE.copyRule(this.source, this.destination);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.source.copy(this.destination.getFullPath(), true, monitor);
		if (this.skipModelBusMeta) {
			FileUtility.removeModelBusMetaInformation(this.destination, monitor);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.source.getName(), this.destination.toString() });
	}

}
