/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.refactor;

import java.text.MessageFormat;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Copies versioned resource with the history preserved
 * 
 * @author Sergiy Logvin
 */
public class CopyResourceWithHistoryOperation extends AbstractActionOperation {
	protected IResource	source;
	protected IResource	destination;

	public CopyResourceWithHistoryOperation(IResource source, IResource destination) {
		super("Operation.CopyLocalH");
		this.source = source;
		this.destination = destination;
	}

	public ISchedulingRule getSchedulingRule() {
		return this.destination instanceof IProject ? this.destination : this.destination.getParent();
	}

	public boolean isAllowed() {
		IRemoteStorage storage = ModelBusRemoteStorage.instance();
		ILocalResource localSource = storage.asLocalResource(this.source);

		return IStateFilter.SF_ONREPOSITORY.accept(localSource);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRemoteStorage storage = ModelBusRemoteStorage.instance();
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus copy \"" + FileUtility.normalizePath(FileUtility.getWorkingCopyPath(this.source))
					+ "\" \"" + FileUtility.getWorkingCopyPath(this.destination) + "\"\n");
			proxy.copy(new String[] { FileUtility.getWorkingCopyPath(this.source) }, FileUtility.getWorkingCopyPath(this.destination),
					ModelBusRevision.WORKING, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.source.getName(), this.destination.toString() });
	}

}
