/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.refactor;

import java.text.MessageFormat;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Move Eclipse resource in the working copy
 * 
 * @author Alexander Gurov
 */
public class MoveResourceOperation extends AbstractActionOperation {
	protected IResource	source;
	protected IResource	destination;

	public MoveResourceOperation(IResource source, IResource destination) {
		super("Operation.MoveLocal");
		this.source = source;
		this.destination = destination;
	}

	public ISchedulingRule getSchedulingRule() {
		return MultiRule.combine(this.source instanceof IProject ? this.source : this.source.getParent(),
				this.destination instanceof IProject ? this.destination : this.destination.getParent());
	}

	public boolean isAllowed() {
		IRemoteStorage storage = ModelBusRemoteStorage.instance();
		return true;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRemoteStorage storage = ModelBusRemoteStorage.instance();

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

		String srcPath = FileUtility.getWorkingCopyPath(this.source);
		String dstPath = FileUtility.getWorkingCopyPath(this.destination);
		try {
			// FIXME JavaModelBus does not provide any statuses in time of extended API move
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus move \"" + FileUtility.normalizePath(localSource.getWorkingCopyPath()) +
			// "\" \"" +
			// FileUtility.normalizePath(localDestination.getWorkingCopyPath()) + "\" --force\n");
			proxy.move(new String[] { srcPath }, dstPath, IModelBusConnector.Options.FORCE, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.source.getName(),
				this.destination.getParent().getFullPath().toString() });
	}

}
