/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.util.ArrayList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Branch and Tag operation implementation
 * 
 * @author Alexander Gurov
 */
public class BranchTagOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String					destinationUrl;
	protected String					message;
	protected ArrayList<RevisionPair>	revisionsPairs;

	public BranchTagOperation(String operationName, IRepositoryResource[] resources, IRepositoryResource destination, String message) {
		super("Operation." + operationName, resources);
		this.destinationUrl = destination.getUrl();
		this.message = message;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionsPairs == null ? null : this.revisionsPairs.toArray(new RevisionPair[this.revisionsPairs.size()]);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.revisionsPairs = new ArrayList<RevisionPair>();
		IRepositoryResource[] resources = this.operableData();

		ProgressMonitorUtility.setTaskInfo(monitor, this, FileUtility.getNamesListAsString(resources));
		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
				final IRepositoryResource current = resources[i];
				final String url2 = ModelBusUtility.encodeURL(BranchTagOperation.this.destinationUrl);

				IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
					public void notify(ModelBusNotification info) {
						BranchTagOperation.this.revisionsPairs.add(new RevisionPair(info.revision, new String[] { url2 }));
						String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
								new String[] { String.valueOf(info.revision) });
						BranchTagOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
					}
				};
				ModelBusUtility.addModelBusNotifyListener(proxy, notify);

				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						BranchTagOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus copy \"" + current.getUrl() + "\" \""
								+ BranchTagOperation.this.destinationUrl + "\" -r " + current.getSelectedRevision() + " -m \""
								+ BranchTagOperation.this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
						ModelBusEntryRevisionReference[] src = new ModelBusEntryRevisionReference[] { new ModelBusEntryRevisionReference(
								ModelBusUtility.encodeURL(current.getUrl()), current.getPegRevision(), current.getSelectedRevision()) };
						proxy.copy(src, url2, BranchTagOperation.this.message, IModelBusConnector.Options.INTERPRET_AS_CHILD, null,
								new ModelBusProgressMonitor(BranchTagOperation.this, monitor, null));
					}
				}, monitor, resources.length);

				ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
			}
		} finally {
		}
	}

}
