/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Create diff between two repository resources. Will be usefull, for example, in order to collect differences between trunk and branch.
 * 
 * @author Alexander Gurov
 */
public class CreatePatchOperation extends AbstractRepositoryOperation {
	protected String	fileName;
	protected boolean	recurse;
	protected boolean	ignoreDeleted;
	protected boolean	processBinary;
	protected boolean	ignoreAncestry;

	public CreatePatchOperation(IRepositoryResource first, IRepositoryResource second, String fileName, boolean recurse, boolean ignoreDeleted,
			boolean processBinary) {
		this(first, second, fileName, recurse, ignoreDeleted, processBinary, true);
	}

	public CreatePatchOperation(IRepositoryResource first, IRepositoryResource second, String fileName, boolean recurse, boolean ignoreDeleted,
			boolean processBinary, boolean ignoreAncestry) {
		super("Operation.CreatePatchRemote", new IRepositoryResource[] { first, second });
		this.fileName = fileName;
		this.recurse = recurse;
		this.ignoreDeleted = ignoreDeleted;
		this.processBinary = processBinary;
		this.ignoreAncestry = ignoreAncestry;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource first = this.operableData()[0];
		IRepositoryResource second = this.operableData()[1];
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			ModelBusEntryRevisionReference ref1 = ModelBusUtility.getEntryRevisionReference(first);
			ModelBusEntryRevisionReference ref2 = ModelBusUtility.getEntryRevisionReference(second);
			long options = this.ignoreAncestry ? IModelBusConnector.Options.IGNORE_ANCESTRY : IModelBusConnector.Options.NONE;
			options |= this.ignoreDeleted ? IModelBusConnector.Options.SKIP_DELETED : IModelBusConnector.Options.NONE;
			options |= this.processBinary ? IModelBusConnector.Options.FORCE : IModelBusConnector.Options.NONE;
			if (ModelBusUtility.useSingleReferenceSignature(ref1, ref2)) {
				this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus diff -r " + ref1.revision + ":" + ref2.revision + " \"" + first.getUrl()
						+ "@" + ref1.pegRevision + "\"" + (this.recurse ? "" : " -N") + (this.ignoreDeleted ? " --no-diff-deleted" : "")
						+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
				proxy.diff(ref1, ref1.revision, ref2.revision, null, this.fileName, this.recurse ? Depth.INFINITY : Depth.IMMEDIATES, options, null,
						new ModelBusProgressMonitor(this, monitor, null));
			} else {
				this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus diff \"" + first.getUrl() + "@" + first.getSelectedRevision() + "\" \""
						+ second.getUrl() + "@" + second.getSelectedRevision() + "\"" + (this.recurse ? "" : " -N")
						+ (this.ignoreDeleted ? " --no-diff-deleted" : "") + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
				proxy.diff(ref1, ref2, null, this.fileName, this.recurse ? Depth.INFINITY : Depth.IMMEDIATES, options, null,
						new ModelBusProgressMonitor(this, monitor, null));
			}
		} finally {
		}
	}

}
