/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Delete remote resources operation
 * 
 * @author Alexander Gurov
 */
public class DeleteResourcesOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String					message;
	protected ArrayList<RevisionPair>	revisionsPairs;

	public DeleteResourcesOperation(IRepositoryResource[] resources, String message) {
		super("Operation.DeleteRemote", resources);
		this.message = message;
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		this.revisionsPairs = new ArrayList<RevisionPair>(); // TODOMES: Check what this is all about
		IRepositoryResource[] resources = ModelBusUtility.shrinkChildNodes(this.operableData());

		// Map<?, ?> repository2Resources = ModelBusUtility.splitRepositoryLocations(resources);
		Map<?, ?> repository2Resources = ModelBusUtility.fillRepository2ResourcesMap(resources);

		for (Iterator<?> it = repository2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
			Map.Entry entry = (Map.Entry) it.next();
			final String[] paths = ModelBusUtility.asURLArray(((List<?>) entry.getValue()).toArray(new IRepositoryResource[0]), true);

			this.complexWriteToConsole(new Runnable() {
				public void run() {
					DeleteResourcesOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus delete");
					for (int i = 0; i < paths.length && !monitor.isCanceled(); i++) {
						DeleteResourcesOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + ModelBusUtility.decodeURL(paths[i]) + "\"");
					}
					DeleteResourcesOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " -m \"" + DeleteResourcesOperation.this.message + "\""
							+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
				}
			});

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
				public void notify(ModelBusNotification info) {
					DeleteResourcesOperation.this.revisionsPairs.add(new RevisionPair(info.revision, paths));
					String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
							new String[] { String.valueOf(info.revision) });
					DeleteResourcesOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
				}
			};
			ModelBusUtility.addModelBusNotifyListener(proxy, notify);
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.remove(paths, DeleteResourcesOperation.this.message, IModelBusConnector.Options.FORCE, null, new ModelBusProgressMonitor(
							DeleteResourcesOperation.this, monitor, null));
				}
			}, monitor, repository2Resources.size());
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);

		}
	}

	public RevisionPair[] getRevisions() {
		return this.revisionsPairs == null ? null : this.revisionsPairs.toArray(new RevisionPair[this.revisionsPairs.size()]);
	}

}
