/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Operation organize export local folder to repository
 * 
 * @author Sergiy Logvin
 */
public class ExportOperation extends AbstractRepositoryOperation {
	protected String	path;
	protected int		depth;

	public ExportOperation(IRepositoryResource[] resources, String path, int depth) {
		super("Operation.ExportRevision", resources);
		this.path = path;
		this.depth = depth;
	}

	public ExportOperation(IRepositoryResourceProvider provider, String path, int depth) {
		super("Operation.ExportRevision", provider);
		this.path = path;
		this.depth = depth;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource[] resources = this.operableData();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			final String path = this.path + "/" + resources[i].getName();
			final ModelBusEntryRevisionReference entryRef = ModelBusUtility.getEntryRevisionReference(resources[i]);
			this.writeToConsole(
					IConsoleStream.LEVEL_CMD,
					"modelbus export \"" + resources[i].getUrl() + "@" + resources[i].getPegRevision() + "\" -r "
							+ resources[i].getSelectedRevision() + " \"" + FileUtility.normalizePath(path) + "\""
							+ ModelBusUtility.getDepthArg(ExportOperation.this.depth) + " --force"
							+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.doExport(entryRef, path, null, ExportOperation.this.depth, IModelBusConnector.Options.FORCE, new ModelBusProgressMonitor(
							ExportOperation.this, monitor, null));
				}
			}, monitor, resources.length);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getUrl() });
	}

}
