/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusAnnotationCallback;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusAnnotationData;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusErrorCodes;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Resource annotation operation implementation
 * 
 * @author Alexander Gurov
 */
public class GetResourceAnnotationOperation extends AbstractRepositoryOperation {
	protected ModelBusAnnotationData[]	annotatedLines;
	protected byte[]					content;
	protected boolean					includeMerged;

	public GetResourceAnnotationOperation(IRepositoryResource resource) {
		super("Operation.GetAnnotation", new IRepositoryResource[] { resource });
	}

	public boolean getIncludeMerged() {
		return this.includeMerged;
	}

	public void setIncludeMerged(boolean includeMerged) {
		this.includeMerged = includeMerged;
	}

	public IRepositoryResource getRepositoryResource() {
		return this.operableData()[0];
	}

	public ModelBusAnnotationData[] getAnnotatedLines() {
		return this.annotatedLines;
	}

	public byte[] getContent() {
		return this.content;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final ByteArrayOutputStream stream = new ByteArrayOutputStream();
		final ArrayList<ModelBusAnnotationData> lines = new ArrayList<ModelBusAnnotationData>();
		IRepositoryResource resource = this.operableData()[0];
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus blame " + url + "@" + resource.getPegRevision() + " -r 0:" +
			// resource.getSelectedRevision() + " --username \"" + location.getUsername() + "\"\n");
			long options = IModelBusConnector.Options.IGNORE_MIME_TYPE;
			options |= this.includeMerged ? IModelBusConnector.Options.INCLUDE_MERGED_REVISIONS : IModelBusConnector.Options.NONE;
			proxy.annotate(ModelBusUtility.getEntryReference(resource), ModelBusRevision.fromString("0"), resource.getSelectedRevision(), options,
					new IModelBusAnnotationCallback() {
						public void annotate(String line, ModelBusAnnotationData data) {
							lines.add(data);
							try {
								stream.write((line + "\n").getBytes());
							} catch (IOException e) {
								throw new RuntimeException(e);
							}
						}
					}, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
		this.annotatedLines = lines.toArray(new ModelBusAnnotationData[lines.size()]);
		this.content = stream.toByteArray();
	}

	protected String getShortErrorMessage(Throwable t) {
		if (t instanceof ModelBusConnectorException && ((ModelBusConnectorException) t).getErrorId() == ModelBusErrorCodes.clientIsBinaryFile) {
			return this.getOperationResource("Error.IsBinary");
		}
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getName() });
	}

}
