/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.property.IRevisionPropertiesProvider;

/**
 * Get revision properties operation implementation
 * 
 * @author Alexei Goncharov
 */
public class GetRevisionPropertiesOperation extends AbstractActionOperation implements IRevisionPropertiesProvider {
	protected ModelBusRevision		revision;
	protected ModelBusProperty[]	revProperties;

	/**
	 * Creates an instance of GetRevisionPropertiesOperation depending on the repository location.
	 * 
	 * @param location
	 *            - repository location to get properties for
	 * @param revision
	 *            - revision to get properties for
	 * 
	 * @author Alexei Goncharov
	 */
	public GetRevisionPropertiesOperation(ModelBusRevision revision) {
		super("Operation.GetRevisionProperties");
		this.revision = revision;
	}

	public ModelBusProperty[] getRevisionProperties() {
		if (this.revProperties == null) {
			return new ModelBusProperty[0];
		}
		return this.revProperties;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		ModelBusEntryRevisionReference reference = new ModelBusEntryRevisionReference(ModelBusConnector.getRepositoryRootUrl(), this.revision,
				this.revision);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			this.revProperties = proxy.getRevisionProperties(reference, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}
}
