/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Rename remote resource operation implementation
 * 
 * @author Alexander Gurov
 */
public class RenameResourceOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String			message;
	protected String			newName;
	protected RevisionPair[]	revisionPair;

	public RenameResourceOperation(IRepositoryResource resource, String newName, String message) {
		super("Operation.Rename", new IRepositoryResource[] { resource });
		this.message = message;
		this.newName = newName;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionPair;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource resource = this.operableData()[0];
		this.revisionPair = new RevisionPair[1];
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		final String newUrl = resource.getParent().getUrl() + "/" + this.newName;
		IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
			public void notify(ModelBusNotification info) {
				String[] path = new String[] { newUrl };
				RenameResourceOperation.this.revisionPair[0] = new RevisionPair(info.revision, path);
				String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
						new String[] { String.valueOf(info.revision) });
				RenameResourceOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
			}
		};
		try {
			ModelBusUtility.addModelBusNotifyListener(proxy, notify);
			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus move \"" + resource.getUrl() + "\" \"" + newUrl + "\" -m \"" + this.message
					+ "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			proxy.move(new ModelBusEntryReference[] { ModelBusUtility.getEntryReference(resource) }, ModelBusUtility.encodeURL(newUrl), this.message,
					IModelBusConnector.Options.INTERPRET_AS_CHILD, null, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getName(), this.newName });
	}

}
