/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.resource;

import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusLock;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;

/**
 * Abstract repository resource
 * 
 * @author Alexander Gurov
 */
public interface IRepositoryResource extends IRepositoryBase, IRepositoryResourceFactory {

	public static class Information {
		public final ModelBusLock	lock;
		public final long			fileSize;
		public final String			lastAuthor;
		public final long			lastChangedDate;
		public final boolean		hasProperties;

		public Information(ModelBusLock lock, long fileSize, String lastAuthor, long lastChangedDate, boolean hasProperties) {
			this.lock = lock;
			this.fileSize = fileSize;
			this.lastAuthor = lastAuthor;
			this.hasProperties = hasProperties;
			this.lastChangedDate = lastChangedDate;
		}
	}

	public ModelBusRevision getSelectedRevision();

	public void setSelectedRevision(ModelBusRevision revision);

	public ModelBusRevision getPegRevision();

	public void setPegRevision(ModelBusRevision pegRevision);

	public boolean isInfoCached();

	public void refresh();

	public boolean exists() throws ModelBusConnectorException;

	public String getName();

	public String getUrl();

	public ModelBusRevision getRevision() throws ModelBusConnectorException;

	public IRepositoryResource getParent();

	public IRepositoryResource getRoot();

	public Information getInfo();

}
