package org.modelbus.team.eclipse.core.utility;

import java.io.IOException;
import java.util.List;

import org.eclipse.compare.IStreamContentAccessor;
import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.ICompareInput;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.compare.ui.team.AbstractResolvingURIConverter;
import org.eclipse.emf.compare.ui.team.AbstractTeamHandler;
import org.eclipse.emf.compare.util.EclipseModelUtils;
import org.eclipse.emf.compare.util.ModelUtils;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.internal.ui.history.FileRevisionTypedElement;
import org.modelbus.core.lib.eclipse.util.ModelBusCoreLibUtil;
import org.modelbus.core.lib.util.ModelBusCoreUtil.URIUtil;

public class ModelBusCompareTeamHandler extends AbstractTeamHandler {

	private boolean	leftIsRemote;

	@Override
	public boolean isLeftRemote() {
		// TODO Auto-generated method stub
		return this.leftIsRemote;
	}

	@Override
	public boolean loadResources(ICompareInput input) throws IOException, CoreException {
		// TODO Auto-generated method stub
		final ITypedElement left = input.getLeft();
		final ITypedElement right = input.getRight();
		final ITypedElement ancestor = input.getAncestor();

		if (right instanceof FileRevisionTypedElement) {
			final IFileRevision rightRevision = ((FileRevisionTypedElement) right).getFileRevision();

			final ResourceSet leftResourceSet = new ResourceSetImpl();
			final ResourceSet rightResourceSet = new ResourceSetImpl();

			if (left instanceof FileRevisionTypedElement) {
				try {
					leftResourceSet.setURIConverter(new ModelBusRevisionedURIConverter(((FileRevisionTypedElement) left).getFileRevision()));
					leftResource = ModelUtils.load(((IStreamContentAccessor) left).getContents(), left.getName(), leftResourceSet).eResource();
				} catch (final IOException e) {
					// We couldn't load the resource. Considers it has been deleted
					leftResource = ModelUtils.createResource(URI.createURI(left.getName()));
					// Set the left as remote to disable merge facilities
					leftIsRemote = true;
				}
			} else if (((ResourceNode) left).getResource().isAccessible()) {
				leftResource = EclipseModelUtils.load(((ResourceNode) left).getResource().getFullPath(), leftResourceSet).eResource();
			} else {
				leftResource = ModelUtils.createResource(URI.createPlatformResourceURI(
						((ResourceNode) left).getResource().getFullPath().toOSString(), true));
				// resource has been deleted. We set it as "remote" to disable merge facilities
				leftIsRemote = true;
			}

			try {
				rightResourceSet.setURIConverter(new ModelBusRevisionedURIConverter(rightRevision));
				rightResource = ModelUtils.load(((IStreamContentAccessor) right).getContents(), right.getName(), rightResourceSet).eResource();
			} catch (final IOException e) {
				// We couldn't load the remote resource. Considers it has been added to the repository
				rightResource = ModelUtils.createResource(URI.createURI(right.getName()));
				// Set the left as remote to disable merge facilities
				leftIsRemote = true;
			}

			if (ancestor != null) {
				final IFileRevision ancestorRevision = ((FileRevisionTypedElement) ancestor).getFileRevision();
				final ResourceSet ancestorResourceSet = new ResourceSetImpl();
				try {
					ancestorResourceSet.setURIConverter(new ModelBusRevisionedURIConverter(ancestorRevision));
					ancestorResource = ModelUtils.load(((IStreamContentAccessor) ancestor).getContents(), ancestor.getName(), ancestorResourceSet)
							.eResource();
				} catch (final IOException e) {
					// Couldn't load ancestor resource, create an empty one
					ancestorResource = ModelUtils.createResource(URI.createURI(ancestor.getName()));
				}
			}
			return true;
		}
		return false;
	}

	private class ModelBusRevisionedURIConverter extends AbstractResolvingURIConverter {

		private final IFileRevision	baseRevision;

		public ModelBusRevisionedURIConverter(IFileRevision revision) {
			super();
			baseRevision = revision;
		}

		@Override
		protected URI resolve(URI uri) throws CoreException {
			// TODO Auto-generated method stub
			URI deresolvedURI = uri;
			// We'll have to change the EMF URI to find the IFile it points to
			final IStorage storage = baseRevision.getStorage(null);
			if (uri.isRelative()) {
				// Current revision, yet the proxy could point to a file that has changed since.
				if (storage instanceof IFile) {
					final IFile file = (IFile) storage;
					deresolvedURI = uri.resolve(URI.createURI(file.getLocationURI().toString()));
				} else {
					String pathString = storage.getFullPath().toString();
					if (!URIUtil.isHTTPURI(pathString)) {
						pathString = "http://" + pathString;
					}
					List<String> localPaths = ModelBusCoreLibUtil.getLocalPaths(/* "http://" + */pathString);

					for (int i = 0; i < localPaths.size(); i++) {
						IPath path = new Path(localPaths.get(i));
						IPath workspacePath = EcorePlugin.getWorkspaceRoot().getLocation();
						path = path.makeRelativeTo(workspacePath);
						final IResource stateFile = EcorePlugin.getWorkspaceRoot().findMember(path);
						if (stateFile != null) {
							deresolvedURI = uri.resolve(URI.createURI(stateFile.getLocationURI().toString()));
							break;
						}
					}
				}
			}
			deresolvedURI = URI.createPlatformResourceURI(deresolvedURI.deresolve(
					URI.createURI(EcorePlugin.getWorkspaceRoot().getLocationURI().toString() + '/')).toString());
			return deresolvedURI;
		}

	}

}
