/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.utility;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;

/**
 * Notify composite listener
 * 
 * @author Alexander Gurov
 */
public class Notify2Composite implements IModelBusNotificationCallback {
	protected IModelBusNotificationCallback[]	listeners;

	public Notify2Composite() {
		this.listeners = new IModelBusNotificationCallback[0];
	}

	public void add(IModelBusNotificationCallback listener) {
		List<IModelBusNotificationCallback> tmp = new ArrayList<IModelBusNotificationCallback>(Arrays.asList(this.listeners));
		if (!tmp.contains(listener)) {
			tmp.add(listener);
		}
		this.listeners = tmp.toArray(new IModelBusNotificationCallback[tmp.size()]);
	}

	public void remove(IModelBusNotificationCallback listener) {
		List<IModelBusNotificationCallback> tmp = new ArrayList<IModelBusNotificationCallback>(Arrays.asList(this.listeners));
		tmp.remove(listener);
		this.listeners = tmp.toArray(new IModelBusNotificationCallback[tmp.size()]);
	}

	public void notify(ModelBusNotification info) {
		// thread safe...
		IModelBusNotificationCallback[] tmp = this.listeners;
		for (int i = 0; i < tmp.length; i++) {
			tmp[i].notify(info);
		}
	}

}
