package org.modelbus.team.eclipse.dependencies.view;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.IncomingReferencesInfo;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.dependencies.Activator;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.repository.UserSessionHelper;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;

public class DependenciesView extends ViewPart {// implements ICellModifier{// implements ISelectionListener{

	public static final int	COLUMN_REFERENCING_OBJECTS	= 0;
	public static final int	COLUMN_OBJECT_TYPE_URI		= 2;
	public static final int	COLUMN_NAME					= 1;
	// public static final int COLUMN_DESCRIPTION = 1;

	private Object			selectedElement;
	private TableViewer		viewer;

	private Action			refreshAction;
	private Action			doubleClickAction;

	public DependenciesView() {
		// TODO Auto-generated constructor stub
	}

	private Object[] getDependencies(Object inputObject) throws IOException, RepositoryAuthentificationException, NonExistingResourceException {
		// TODO Auto-generated method stub
		List<String[]> output = new ArrayList<String[]>();
		URI inputUri = null;

		if (inputObject instanceof RepositoryFile) {
			inputUri = URI.createURI(((RepositoryFile) inputObject).getRepositoryResource().getUrl());
		} else if (inputObject instanceof IFile) {
			inputUri = getRemoteLocation(inputObject);
		} else if (inputObject instanceof EObject) {
			inputUri = getRemoteLocation(inputObject);
		}

		URI remoteLocation = inputUri;

		if (remoteLocation != null) {
			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			IncomingReferencesInfo[] incomingReferencesInfos = repositoryHelper.getIncomingReferences(UserSessionHelper.getSession(), remoteLocation);
			if (incomingReferencesInfos.length == 0) {
				output.add(new String[] { "no incoming references", "", "" });
				return output.toArray();
			}
			for (int i = 0; i < incomingReferencesInfos.length; i++) {
				String[] row = new String[3];
				row[COLUMN_REFERENCING_OBJECTS] = incomingReferencesInfos[i].getReferencingObjectUri();
				row[COLUMN_NAME] = incomingReferencesInfos[i].getObjectName();
				row[COLUMN_OBJECT_TYPE_URI] = incomingReferencesInfos[i].getObjectTypeUri();
				output.add(row);
			}
		} else {
			output.add(new String[] { "not connected", "", "" });
		}

		return output.toArray();
	}

	private URI getRemoteLocation(Object localObject) {
		// TODO Auto-generated method stub
		URI uri = null;

		if (localObject instanceof IFile) {

			IFile file = (IFile) localObject;
			IProject project = file.getProject();
			RepositoryProvider provider = RepositoryProvider.getProvider(file.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);

			if (provider != null) {
				ModelBusRepositoryProvider modelBusRepositoryProvider = (ModelBusRepositoryProvider) provider;
				String remoteFileLocation = modelBusRepositoryProvider.getRepositoryPath(file);
				uri = URI.createURI(remoteFileLocation);
			}
		} else if (localObject instanceof EObject) {

			EObject eObject = (EObject) localObject;
			Resource res = eObject.eResource();
			URI eObjectUri = eObject.eResource().getURI();
			String fragment = res.getURIFragment(eObject);
			URI eObjectUriWithFragment = eObjectUri.appendFragment(fragment);
			if (!eObjectUriWithFragment.scheme().equals("http")) {
				IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(eObjectUriWithFragment.segment(1));
				IResource resource = project.getFile(eObjectUriWithFragment.toString().substring(
						eObjectUriWithFragment.toString().indexOf(eObjectUriWithFragment.segment(2))));
				RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);

				if (provider != null) {
					ModelBusRepositoryProvider modelBusRepositoryProvider = (ModelBusRepositoryProvider) provider;
					String remoteFileLocation = modelBusRepositoryProvider.getRepositoryPath(resource);
					uri = URI.createURI(remoteFileLocation);
				}

			} else {
				uri = eObjectUriWithFragment;
			}

		}

		return uri;
	}

	class ViewContentProvider implements IStructuredContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {

		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {

			Object[] objects = null;
			// repositoryFile = (RepositoryFile) parent;
			try {
				objects = getDependencies(parent);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NonExistingResourceException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return objects;

		}

	}

	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			return getText(((String[]) obj)[index]);
		}

		public Image getColumnImage(Object obj, int index) {
			return null;// getImage(obj);
		}

		public Image getImage(Object obj) {
			return null;// PlatformUI.getWorkbench().
			// getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
		}
	}

	@Override
	public void createPartControl(Composite parent) {
		// TODO Auto-generated method stub

		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 4;
		table.setLayoutData(gridData);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableColumn column = new TableColumn(table, SWT.LEFT, COLUMN_REFERENCING_OBJECTS);
		column.setText("Referencing Objects");
		column.setWidth(400);

		column = new TableColumn(table, SWT.LEFT, COLUMN_NAME);
		column.setText("Name");
		column.setWidth(200);

		column = new TableColumn(table, SWT.LEFT, COLUMN_OBJECT_TYPE_URI);
		column.setText("Type");
		column.setWidth(400);

		// column = new TableColumn(table, SWT.CENTER, COLUMN_OCCURENCE);
		// column.setText("Occurence");
		// column.setWidth(80);
		// column = new TableColumn(table, SWT.LEFT, COLUMN_REFERENCING_OBJECTS);
		// column.setText("Referencing Objects");
		// column.setWidth(400);
		//
		// column = new TableColumn(table, SWT.LEFT, COLUMN_DESCRIPTION);
		// column.setText("Description");
		// column.setWidth(100);

		viewer = new TableViewer(table);
		viewer.setUseHashlookup(true);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		// viewer.setCellModifier(this);
		createActions();
		hookDoubleClickAction();
		contributeToActionBars();
		// add myself as a global selection listener
		// getSite().getPage().addSelectionListener(this);

		// prime the selection
		// selectionChanged(null, getSite().getPage().getSelection());

	}

	private void contributeToActionBars() {
		// TODO Auto-generated method stub
		IActionBars bars = getViewSite().getActionBars();
		bars.getToolBarManager().add(refreshAction);
		bars.getToolBarManager().update(true);

	}

	private void createActions() {
		// TODO Auto-generated method stub
		refreshAction = new Action("Refresh", IAction.AS_PUSH_BUTTON) {
			public void run() {
				viewer.refresh();
			}
		};

		refreshAction.setImageDescriptor(Activator.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/refresh.gif"));

		doubleClickAction = new Action() {
			public void run() {
				TableItem[] items = viewer.getTable().getSelection();
				for (TableItem tableItem : items) {
					String text = tableItem.getText();
					if (!text.isEmpty()) {
						IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
						File file = new File(text);
						IEditorRegistry registry = PlatformUI.getWorkbench().getEditorRegistry();
						IEditorDescriptor[] desc = registry.getEditors(file.getName());
						// page.openEditor(new FileE, desc.getId());
						showMessage("Double-click detected on ");
					}
				}
				// Object obj = ((IStructuredSelection)selection).getFirstElement();

			}
		};

	}

	private void showMessage(String message) {
		MessageDialog.openInformation(viewer.getControl().getShell(), "Dependencies View", message);
	}

	private void hookDoubleClickAction() {

		viewer.addDoubleClickListener(new IDoubleClickListener() {

			@Override
			public void doubleClick(DoubleClickEvent event) {
				// TODO Auto-generated method stub
				ISelection selection = event.getSelection();
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection structured = (IStructuredSelection) selection;
					if (structured.size() == 1) {
						doubleClickAction.run();
					}
				}
			}
		});
	}

	@Override
	public void dispose() {
		super.dispose();
		// getSite().getPage().removeSelectionListener(this);
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
		viewer.getControl().setFocus();

	}

	public TableViewer getViewer() {

		return viewer;
	}

	// @Override
	// public void selectionChanged(IWorkbenchPart part, ISelection selection) {
	// // TODO Auto-generated method stub
	// //if the selection is a readme file, get its sections.
	// Object input = null;
	// if(selection instanceof IStructuredSelection){
	// Object object = ((IStructuredSelection)selection).getFirstElement();
	// if(object instanceof RepositoryFile || object instanceof IFile || object instanceof EObject){
	// input = object;
	// }
	// }
	//
	// viewer.setInput(input);
	//
	//
	// }

}
