/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.localhistory;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.team.core.subscribers.Subscriber;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.ui.TeamUI;
import org.eclipse.team.ui.synchronize.ISynchronizeModelElement;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.ISynchronizeParticipantDescriptor;
import org.eclipse.team.ui.synchronize.SubscriberParticipant;
import org.eclipse.team.ui.synchronize.SynchronizeModelAction;
import org.eclipse.team.ui.synchronize.SynchronizeModelOperation;
import org.eclipse.team.ui.synchronize.SynchronizePageActionGroup;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public class LocalHistoryParticipant extends SubscriberParticipant {

	public static final String	ID								= "org.eclipse.team.synchronize.example";	//$NON-NLS-1$
	public static final String	CONTEXT_MENU_CONTRIBUTION_GROUP	= "context_group_1";						//$NON-NLS-1$

	private class LocalHistoryActionContribution extends SynchronizePageActionGroup {
		public void initialize(ISynchronizePageConfiguration configuration) {
			super.initialize(configuration);
			appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, CONTEXT_MENU_CONTRIBUTION_GROUP, new SynchronizeModelAction(
					"Revert to latest in local history", configuration) { //$NON-NLS-1$
						protected SynchronizeModelOperation getSubscriberOperation(ISynchronizePageConfiguration configuration,
								IDiffElement[] elements) {
							return new RevertAllOperation(configuration, elements);
						}
					});
		}
	}

	private class LocalHistoryDecorator extends LabelProvider implements ILabelDecorator {
		public String decorateText(String text, Object element) {
			if (element instanceof ISynchronizeModelElement) {
				ISynchronizeModelElement node = (ISynchronizeModelElement) element;
				if (node instanceof IAdaptable) {
					SyncInfo info = (SyncInfo) ((IAdaptable) node).getAdapter(SyncInfo.class);
					if (info != null) {
						LocalHistoryVariant state = (LocalHistoryVariant) info.getRemote();
						return text + " (" + state.getContentIdentifier() + ")";
					}
				}
			}
			return text;
		}

		public Image decorateImage(Image image, Object element) {
			return null;
		}
	}

	public LocalHistoryParticipant() {
		setSubscriber(new LocalHistorySubscriber());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.synchronize.subscriber.SubscriberParticipant#setSubscriber(org.eclipse.team.core.subscribers.Subscriber)
	 */
	protected void setSubscriber(Subscriber subscriber) {
		super.setSubscriber(subscriber);
		try {
			ISynchronizeParticipantDescriptor descriptor = TeamUI.getSynchronizeManager().getParticipantDescriptor(ID);
			setInitializationData(descriptor);
			setSecondaryId(Long.toString(System.currentTimeMillis()));
		} catch (CoreException e) {
		}
	}

	protected void initializeConfiguration(ISynchronizePageConfiguration configuration) {
		super.initializeConfiguration(configuration);
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, CONTEXT_MENU_CONTRIBUTION_GROUP);
		configuration.addActionContribution(new LocalHistoryActionContribution());
		configuration.addLabelDecorator(new LocalHistoryDecorator());
	}
}
