/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.localhistory;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.SynchronizeModelOperation;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public class RevertAllOperation extends SynchronizeModelOperation {

	protected RevertAllOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		super(configuration, elements);
	}

	protected boolean canRunAsJob() {
		return true;
	}

	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		final SyncInfo infos[] = getSyncInfoSet().getSyncInfos();
		if (infos.length == 0)
			return;

		WorkspaceModifyOperation operation = new WorkspaceModifyOperation() {
			public void execute(IProgressMonitor pm) throws InvocationTargetException {
				try {
					pm.beginTask("Reverting from local history", 100 * infos.length); //$NON-NLS-1$
					for (int i = 0; i < infos.length; i++) {
						SyncInfo info = infos[i];
						LocalHistoryVariant state = (LocalHistoryVariant) info.getRemote();
						IFile file = (IFile) info.getLocal();
						if (file.exists()) {
							file.setContents(state.getFileState(), false, true, new SubProgressMonitor(pm, 100));
						} else {
							// TODO: have to pre-create parents if they dont exist
							file.create(state.getFileState().getContents(), false, new SubProgressMonitor(pm, 100));
						}
					}
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					pm.done();
				}
			}
		};
		operation.run(monitor);
	}
}
