/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public abstract class ModelContainer extends ModelResource {

	protected ModelContainer(IContainer container) {
		super(container);
	}

	protected IContainer getContainer() {
		return (IContainer) getResource();
	}

	public ModelObject[] getChildren() throws CoreException {
		IResource[] members = getContainer().members();
		List result = new ArrayList();
		for (int i = 0; i < members.length; i++) {
			IResource resource = members[i];
			if (resource instanceof IFolder) {
				result.add(new ModelFolder((IFolder) resource));
			} else if (ModelObjectDefinitionFile.isModFile(resource)) {
				result.add(new ModelObjectDefinitionFile((IFile) resource));
			} else if (resource instanceof IProject && ModelProject.isModProject((IProject) resource)) {
				result.add(new ModelProject((IProject) resource));
			}
		}
		return (ModelObject[]) result.toArray(new ModelObject[result.size()]);
	}

}
