/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.model.mapping;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.core.resources.mapping.ResourceMappingContext;
import org.eclipse.core.resources.mapping.ResourceTraversal;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.model.ModelContainer;
import org.modelbus.team.eclipse.model.ModelObject;
import org.modelbus.team.eclipse.model.ModelResource;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public class ModelContainerResourceMapping extends ModelResourceMapping {

	public ModelContainerResourceMapping(ModelContainer container) {
		super(container);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.mapping.ResourceMapping#getTraversals(org.eclipse.core.resources.mapping.ResourceMappingContext,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	public ResourceTraversal[] getTraversals(ResourceMappingContext context, IProgressMonitor monitor) throws CoreException {
		return new ResourceTraversal[] { new ResourceTraversal(new IResource[] { getResource() }, IResource.DEPTH_INFINITE, IResource.NONE) };
	}

	private IResource getResource() {
		return ((ModelContainer) getModelObject()).getResource();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.mapping.ResourceMapping#contains(org.eclipse.core.resources.mapping.ResourceMapping)
	 */
	public boolean contains(ResourceMapping mapping) {
		if (mapping instanceof ModelResourceMapping) {
			ModelObject object = (ModelObject) mapping.getModelObject();
			if (object instanceof ModelResource) {
				IResource resource = ((ModelResource) object).getResource();
				return getResource().getFullPath().isPrefixOf(resource.getFullPath());
			}
		}
		return false;
	}

}
