/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.model.ui;

import java.net.URL;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.modelbus.team.eclipse.model.ModelFolder;
import org.modelbus.team.eclipse.model.ModelObject;
import org.modelbus.team.eclipse.model.ModelObjectDefinitionFile;
import org.modelbus.team.eclipse.model.ModelObjectElementFile;
import org.modelbus.team.eclipse.model.ModelProject;
import org.modelbus.team.eclipse.model.ModelWorkspace;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;

/**
 * The workbench adapter for ModelObjects. TODO comment
 * 
 * @author Michael Wagner
 * 
 */
public class ModelWorkbenchAdapter implements IWorkbenchAdapter {

	// image path
	private final String	ICON_PATH	= "$nl$/icons/full/";	//$NON-NLS-1$

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object o) {
		if (o instanceof ModelObject) {
			ModelObject mo = (ModelObject) o;
			try {
				return mo.getChildren();
			} catch (CoreException e) {
				ModelBusRepositoryPlugin.log(e.getStatus());
			}
		}
		return new Object[0];
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getImageDescriptor(java.lang.Object)
	 */
	public ImageDescriptor getImageDescriptor(Object object) {
		if (object instanceof ModelProject) {
			return createImageDescriptor("obj/prj_obj.gif");
		}
		if (object instanceof ModelWorkspace) {
			return createImageDescriptor("obj/root_obj.gif");
		}
		if (object instanceof ModelFolder) {
			return createImageDescriptor("obj/fldr_obj.gif");
		}
		if (object instanceof ModelObjectDefinitionFile) {
			return createImageDescriptor("obj/mod_obj.gif");
		}
		if (object instanceof ModelObjectElementFile) {
			return createImageDescriptor("obj/moe_obj.gif");
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getLabel(java.lang.Object)
	 */
	public String getLabel(Object o) {
		if (o instanceof ModelObject) {
			ModelObject mo = (ModelObject) o;
			return mo.getName();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getParent(java.lang.Object)
	 */
	public Object getParent(Object o) {
		if (o instanceof ModelObject) {
			ModelObject mo = (ModelObject) o;
			return mo.getParent();
		}
		return null;
	}

	/**
	 * Creates an image descriptor.
	 */
	private ImageDescriptor createImageDescriptor(String id) {
		URL url = FileLocator.find(ModelBusRepositoryPlugin.getPlugin().getBundle(), new Path(ICON_PATH + id), null);
		return ImageDescriptor.createFromURL(url);
	}

}
