/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.modelbus.team.eclipse.notification.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.modelbus.team.eclipse.notification.Activator;

/**
 * This class represents a preference page that is contributed to the Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows us to create a page that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the preference store that belongs to the main plug-in class. That way, preferences
 * can be accessed directly via the preference store.
 */

public class NotificationPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	public NotificationPreferencePage() {
		super(GRID);
		this.setPreferenceStore(Activator.getDefault().getPreferenceStore());
		this.setDescription("ModelBus Notification Configuration");
	}

	/**
	 * Creates the field editors. Field editors are abstractions of the common GUI blocks needed to manipulate various types of preferences. Each
	 * field editor knows how to save and restore itself.
	 */
	public void createFieldEditors() {
		this.addField(new BooleanFieldEditor(PreferenceConstants.FILTER_BY_OPEN_PROJECTS, "&Notification only on open models", this.getFieldEditorParent()));

		this.addField(new BooleanFieldEditor(PreferenceConstants.POPUP_NOTIFICATION, "&Open popup window for each notification", this.getFieldEditorParent()));

		this.addField(new StringFieldEditor(PreferenceConstants.REGEXP_NOTIFICATION, "&Regex filter:", this.getFieldEditorParent()));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

}