/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.modelbus.team.eclipse.notification.views;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.modelbus.team.eclipse.notification.Activator;
import org.modelbus.team.eclipse.notification.preferences.PreferenceConstants;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;

public class FilterHelper {

	public static boolean filter(String url) {
		String regex = Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.REGEXP_NOTIFICATION);

		if (!url.matches(regex)) {
			return false;
		}

		if (!isFilterByProject()) {
			return true;
		}
		try {
			IWorkspace root = ResourcesPlugin.getWorkspace();
			IProject[] projects = root.getRoot().getProjects();
			for (int i = 0; i < projects.length; i++) {
				if (projects[i].exists() && projects[i].isOpen()) {
					String projectUrl = projects[i].getPersistentProperty(ModelBusRepositoryProvider.MODELBUS_REPO_LOC);
					if (projectUrl != null && projectUrl != "") {
						if (url.startsWith(projectUrl)) {
							return true;
						}
					}
				}

			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return false;
	}

	private static boolean isFilterByProject() {
		return Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.FILTER_BY_OPEN_PROJECTS);
	}

}
