package org.modelbus.team.eclipse.repository;

import java.util.HashMap;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.core.lib.ModelBusCoreLib;

public class ModelBusRepositoryHelper implements IResourceChangeListener, IPropertyChangeListener {

	private ModelBusRepositoryHelper() {

	}

	private static ModelBusRepositoryHelper	modelBusRepositoryHelper;
	private static IRepositoryHelper		repositoryHelper;
	private static boolean					oldCheckDependencies	= false;
	private static String[]					oldExtensions			= new String[0];
	private static HashMap<String, String>	oldlocal2remoteMap;

	public static ModelBusRepositoryHelper getModelBusRepositoryHelper() {
		if (modelBusRepositoryHelper == null) {
			modelBusRepositoryHelper = new ModelBusRepositoryHelper();
		}
		return modelBusRepositoryHelper;
	}

	public static IRepositoryHelper getRepositoryHelper() {

		if (repositoryHelper == null) { // || !oldRepositoryLocation.equals(newRepositoryLocation)){
			repositoryHelper = ModelBusCoreLib.getRepositoryHelper();
			// repositoryHelper = ModelBusRepositoryPlugin.getPlugin().getRepositoryHelper();
			if (repositoryHelper != null) { // May happen during initialization. RepositoryHelper takes its time to get registered.
				// oldRepositoryLocation = _getRepositoryLocation(); //newRepositoryLocation;
				oldlocal2remoteMap = _getLocal2RemoteMap();
				repositoryHelper.setProjectPathMap(oldlocal2remoteMap);
				oldCheckDependencies = _getCheckDependencies();
				repositoryHelper.setEnableDependencies(oldCheckDependencies);
				oldExtensions = _getExtensions();
				repositoryHelper.setModelExtensions(oldExtensions);
			}
		}
		return repositoryHelper;
	}

	// private static String _getRepositoryLocation(){
	//
	// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin
	// .getDefault()
	// .getPreferenceStore()
	// .getString(org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
	//
	// return repositoryLocation;
	// }

	private static boolean _getCheckDependencies() {
		boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
				.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		return checkDependencies;
	}

	private static String[] _getExtensions() {
		String[] extensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();
		return extensions;
	}

	private static HashMap<String, String> _getLocal2RemoteMap() {
		HashMap<String, String> local2remoteMap = new HashMap<String, String>();

		IProject[] localProjects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < localProjects.length; i++) {
			org.eclipse.team.core.RepositoryProvider provider = org.eclipse.team.core.RepositoryProvider.getProvider(localProjects[i],
					ModelBusRepositoryPlugin.PROVIDER_ID);
			if (provider != null) {
				ModelBusRepositoryProvider modelbusProvider = (ModelBusRepositoryProvider) provider;
				String remoteLocation = modelbusProvider.getRepositoryPath(localProjects[i]);
				String localLocation = localProjects[i].getLocationURI().toString();
				local2remoteMap.put(localLocation, remoteLocation);
			}
		}
		return local2remoteMap;
	}

	public static HashMap<String, String> getLocal2RemoteMap() {
		if (oldlocal2remoteMap == null) {
			oldlocal2remoteMap = new HashMap<String, String>();
		}
		return oldlocal2remoteMap;
	}

	@Override
	public void resourceChanged(IResourceChangeEvent event) {

		if (event.getType() == IResourceChangeEvent.POST_CHANGE) {
			IRepositoryHelper repositoryHelper = getRepositoryHelper();
			HashMap<String, String> local2RemoteMap = getLocal2RemoteMap();
			if (repositoryHelper != null) {
				IResourceDelta delta = event.getDelta();
				IResourceDelta[] resource = delta.getAffectedChildren();
				for (IResourceDelta iResourceDelta : resource) {
					IResource iResource = iResourceDelta.getResource();
					if (iResource.getType() == IResource.PROJECT) {
						if (iResourceDelta.getKind() == IResourceDelta.ADDED) {
							org.eclipse.team.core.RepositoryProvider provider = org.eclipse.team.core.RepositoryProvider.getProvider(
									(IProject) iResource, ModelBusRepositoryPlugin.PROVIDER_ID);
							if (provider != null) {
								ModelBusRepositoryProvider modelbusProvider = (ModelBusRepositoryProvider) provider;
								String remoteLocation = modelbusProvider.getRepositoryPath((IProject) iResource);
								String localLocation = iResource.getLocationURI().toString();
								local2RemoteMap.put(localLocation, remoteLocation);
								repositoryHelper.setProjectPathMap(local2RemoteMap);
							}
						} else if (iResourceDelta.getKind() == IResourceDelta.REMOVED) {
							org.eclipse.team.core.RepositoryProvider provider = org.eclipse.team.core.RepositoryProvider.getProvider(
									(IProject) iResource, ModelBusRepositoryPlugin.PROVIDER_ID);
							if (provider != null) {
								String localLocation = iResource.getLocationURI().toString();
								local2RemoteMap.remove(localLocation);
								repositoryHelper.setProjectPathMap(local2RemoteMap);
							} else {
								String localLocation = "file:/" + ResourcesPlugin.getWorkspace().getRoot().getLocation().toString()
										+ iResource.getFullPath().toString();
								local2RemoteMap.remove(localLocation);
								repositoryHelper.setProjectPathMap(local2RemoteMap);
							}
						} else if (iResourceDelta.getKind() == IResourceDelta.CHANGED) {
							org.eclipse.team.core.RepositoryProvider provider = org.eclipse.team.core.RepositoryProvider.getProvider(
									(IProject) iResource, ModelBusRepositoryPlugin.PROVIDER_ID);
							if (provider != null) {
								ModelBusRepositoryProvider modelbusProvider = (ModelBusRepositoryProvider) provider;
								String remoteLocation = modelbusProvider.getRepositoryPath((IProject) iResource);
								String localLocation = iResource.getLocationURI().toString();
								local2RemoteMap.put(localLocation, remoteLocation);
								repositoryHelper.setProjectPathMap(local2RemoteMap);
							}
						}
					}
				}
			}

		}

	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {

		IRepositoryHelper repositoryHelper = getRepositoryHelper();
		if (repositoryHelper != null) {
			if (event.getProperty().equals(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES)) {
				oldCheckDependencies = ((Boolean) event.getNewValue()).booleanValue();
				repositoryHelper.setEnableDependencies(oldCheckDependencies);

			} else if (event.getProperty().equals(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.EXTENSIONS_PREFERENCE)) {
				String newValue = event.getNewValue().toString();
				oldExtensions = convert(newValue);
				repositoryHelper.setModelExtensions(oldExtensions);
			}
		}
		// }else if(event.getProperty().equals(org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING)){
		// repositoryHelper = null;
		//
		// }

	}

	private String[] convert(String preferenceValue) {
		StringTokenizer tokenizer = new StringTokenizer(preferenceValue, ";");
		int tokenCount = tokenizer.countTokens();
		String[] elements = new String[tokenCount];

		for (int i = 0; i < tokenCount; i++) {
			elements[i] = tokenizer.nextToken();
		}

		return elements;
	}

}
