/*******************************************************************************
 * ongoing che
 *******************************************************************************/
package org.modelbus.team.eclipse.repository;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.mapping.ResourceTraversal;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.IResourceVariantComparator;
import org.eclipse.team.core.variants.ThreeWaySynchronizer;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.ConstraintViolationException;
import org.modelbus.dosgi.repository.descriptor.InvalidRevisionException;
import org.modelbus.dosgi.repository.descriptor.InvalidValueException;
import org.modelbus.dosgi.repository.descriptor.LockedException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.PathFileContentMapEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.RepositoryDirEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryRuntimeException;
import org.modelbus.dosgi.repository.descriptor.UnresolvedReferencesException;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositoryResourceVariant;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositorySubscriber;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusThreeWaySynchronizer;
import org.modelbus.team.eclipse.repository.ui.LockHelper;

/**
 * The get and put operations for the file system provider.
 */
public class ModelBusRepositoryOperations {

	// A reference to the provider
	private ModelBusRepositoryProvider					provider;
	private List<PathFileContentMapEntry>				createFileMap;
	private List<PathFileContentMapEntry>				checkInFileMap;
	private List<String>								folderPaths;
	private List<String>								deletionPaths;
	private List<IContainer>							containerToSetBaseBytes;
	private List<IFile>									filesToSetBaseBytes;
	private Map<IFile, org.eclipse.emf.common.util.URI>	modelsToCheckinMap;
	private List<IResource>								resourcesToDelete;

	// private RepositoryHelper repositoryHelper;

	ModelBusRepositoryOperations(ModelBusRepositoryProvider provider) {
		this.provider = provider;
		// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getString(org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// this.repositoryHelper = new RepositoryHelper(repLocation,checkDependencies,modelExtensions);
		// this.setLocal2RemoteMap();
	}

	/**
	 * Make the local state of the project match the remote state by getting any out-of-sync resources. The overrideOutgoing flag is used to indicate
	 * whether locally modified files should also be replaced or left alone.
	 * 
	 * @param resources
	 *            the resources to get
	 * @param depth
	 *            the depth of the operation
	 * @param overrideOutgoing
	 *            whether locally modified resources should be replaced
	 * @param progress
	 *            a progress monitor
	 * @throws TeamException
	 * @throws InvalidRevisionException
	 * @throws IOException
	 */
	public void get(IResource[] resources, int depth, boolean overrideOutgoing, IProgressMonitor progress) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidRevisionException {
		try {
			// ensure the progress monitor is not null
			progress = Policy.monitorFor(progress);
			progress.beginTask(Policy.bind("GetAction.working"), 100); //$NON-NLS-1$
			// Refresh the subscriber so we have the latest remote state
			ModelBusRepositorySubscriber.getInstance().refresh(resources, depth, new SubProgressMonitor(progress, 30));
			internalGet(resources, depth, overrideOutgoing, new SubProgressMonitor(progress, 70));
		} finally {
			progress.done();
		}
	}

	/**
	 * Make the local state of the traversals match the remote state by getting any out-of-sync resources. The overrideOutgoing flag is used to
	 * indicate whether locally modified files should also be replaced or left alone.
	 * 
	 * @param traversals
	 *            the traversals that cover the resources to get
	 * @param overrideOutgoing
	 *            whether locally modified resources should be replaced
	 * @param progress
	 *            a progress monitor
	 * @throws TeamException
	 * @throws InvalidRevisionException
	 * @throws IOException
	 */
	public void get(ResourceTraversal[] traversals, boolean overrideOutgoing, IProgressMonitor monitor) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidRevisionException {
		try {
			// ensure the progress monitor is not null
			monitor = Policy.monitorFor(monitor);
			monitor.beginTask(null, 100 * traversals.length);
			for (int i = 0; i < traversals.length; i++) {
				ResourceTraversal traversal = traversals[i];
				get(traversal.getResources(), traversal.getDepth(), overrideOutgoing, new SubProgressMonitor(monitor, 100));
			}
		} finally {
			monitor.done();
		}
	}

	/**
	 * Checkout the given resources to the given depth by setting any files to writable (i.e set read-only to <code>false</code>.
	 * 
	 * @param resources
	 *            the resources to be checked out
	 * @param depth
	 *            the depth of the checkout
	 * @param progress
	 *            a progress monitor
	 * @throws TeamException
	 */
	public void checkout(IResource[] resources, int depth, IProgressMonitor progress) throws TeamException {
		try {
			progress = Policy.monitorFor(progress);
			progress.beginTask(Policy.bind("FileSystemSimpleAccessOperations.1"), resources.length); //$NON-NLS-1$
			for (int i = 0; i < resources.length; i++) {
				Policy.checkCanceled(progress);
				resources[i].accept(new IResourceVisitor() {
					public boolean visit(IResource resource) throws CoreException {
						if (resource.getType() == IResource.FILE) {
							// TODO: lock the file on the 'server'.
							resource.setReadOnly(false);
						}
						return true;
					}
				}, depth, false /* include phantoms */);
				progress.worked(1);
			}
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		} finally {
			progress.done();
		}
	}

	/**
	 * Check-in the given resources to the given depth by replacing the remote (i.e. file system) contents with the local workspace contents.
	 * 
	 * @param resources
	 *            the resources
	 * @param depth
	 *            the depth of the operation
	 * @param overrideIncoming
	 *            indicate whether incoming remote changes should be replaced
	 * @param progress
	 *            a progress monitor
	 * @throws TeamException
	 * @throws UnresolvedReferencesException
	 * @throws ConstraintViolationException
	 * @throws InvalidValueException
	 * @throws LockedException
	 * @throws IOException
	 * @throws InvalidRevisionException
	 */
	public void checkin(IResource[] resources, int depth, boolean overrideIncoming, IProgressMonitor progress, String commitMessage)
			throws TeamException, RepositoryAuthentificationException, NonExistingResourceException, LockedException, InvalidValueException,
			ConstraintViolationException, UnresolvedReferencesException, IOException, InvalidRevisionException {
		try {
			// ensure the progress monitor is not null
			progress = Policy.monitorFor(progress);
			progress.beginTask(Policy.bind("PutAction.working"), 100); //$NON-NLS-1$
			// Refresh the subscriber so we have the latest remote state
			ModelBusRepositorySubscriber.getInstance().refresh(resources, depth, new SubProgressMonitor(progress, 5));

			createFileMap = new ArrayList<PathFileContentMapEntry>();
			checkInFileMap = new ArrayList<PathFileContentMapEntry>();
			folderPaths = new ArrayList<String>();
			deletionPaths = new ArrayList<String>();
			containerToSetBaseBytes = new ArrayList<IContainer>();
			filesToSetBaseBytes = new ArrayList<IFile>();
			modelsToCheckinMap = new HashMap<IFile, org.eclipse.emf.common.util.URI>();
			resourcesToDelete = new ArrayList<IResource>();

			internalPut(resources, depth, overrideIncoming, new SubProgressMonitor(progress, 5), commitMessage);

			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			if (!(createFileMap.isEmpty() && checkInFileMap.isEmpty() && folderPaths.isEmpty() && deletionPaths.isEmpty())) {
				commitFilesInBatch(repositoryHelper, commitMessage, new SubProgressMonitor(progress, 20));
			}

			ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();

			if (!resourcesToDelete.isEmpty()) {
				flushSynchronizerForDeletedResources(synchronizer);
			}

			if (!containerToSetBaseBytes.isEmpty()) {
				setBaseBytesForContainers(synchronizer);
			}

			if (!filesToSetBaseBytes.isEmpty()) {
				setBaseBytesForCommittedFiles(synchronizer);
			}

			if (!modelsToCheckinMap.isEmpty()) {
				commitModels(repositoryHelper, synchronizer, commitMessage, new SubProgressMonitor(progress, 70));
			}

		} finally {
			progress.done();
		}
	}

	private void commitFilesInBatch(IRepositoryHelper repositoryHelper, String commitMessage, IProgressMonitor progress)
			throws RepositoryRuntimeException, InvalidValueException, RepositoryAuthentificationException, LockedException {

		try {
			progress = Policy.monitorFor(progress);
			progress.beginTask(Policy.bind("PutAction.working"), IProgressMonitor.UNKNOWN);
			Policy.checkCanceled(progress);
			progress.subTask("committing files...");
			repositoryHelper.commit(UserSessionHelper.getSession(), createFileMap, checkInFileMap, folderPaths, deletionPaths, commitMessage);

			progress.worked(1);
		} finally {
			progress.done();
		}
	}

	private void flushSynchronizerForDeletedResources(ThreeWaySynchronizer synchronizer) throws TeamException {
		for (IResource resource : resourcesToDelete) {
			if (resource instanceof IFolder) {
				synchronizer.flush(resource, IResource.DEPTH_INFINITE);
			} else {
				synchronizer.flush(resource, IResource.DEPTH_ZERO);
			}
		}
	}

	private void setBaseBytesForContainers(ThreeWaySynchronizer synchronizer) throws TeamException, RepositoryAuthentificationException,
			NonExistingResourceException, IOException, InvalidRevisionException {

		for (IContainer container : containerToSetBaseBytes) {
			if (container.exists()) {
				ModelBusRepositoryResourceVariant remote = (ModelBusRepositoryResourceVariant) provider.getResourceVariant(container);
				synchronizer.setBaseBytes(container, remote.asBytes());
				((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(container, remote.getRevision());

			}
		}

	}

	private void setBaseBytesForCommittedFiles(ThreeWaySynchronizer synchronizer) throws RepositoryAuthentificationException,
			NonExistingResourceException, TeamException, IOException, InvalidRevisionException {

		for (IFile file : filesToSetBaseBytes) {
			IResourceVariant remote = getResourceVariant(file); // MES: Slow but
			// precise!
			// // TODO
			// // TODOMWA
			// // remove adaptive hack
			if (remote != null) {
				synchronizer.setBaseBytes(file, remote.asBytes());
				((ModelBusThreeWaySynchronizer) synchronizer)
						.setWorkingCopyRevision(file, ((ModelBusRepositoryResourceVariant) remote).getRevision());
			} else {

				try {
					// TODO optimize use DataSource
					ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
					FileInputStream fileInputStream = new FileInputStream(file.getLocation().toFile());
					byte[] buffer = new byte[16384];
					for (int len = fileInputStream.read(buffer); len > 0; len = fileInputStream.read(buffer)) {
						byteArrayOutputStream.write(buffer, 0, len);
					}
					fileInputStream.close();

					synchronizer.setBaseBytes(file, byteArrayOutputStream.toByteArray());
					((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(file, IRepositoryHelper.REVISION_HEAD);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					throw ModelBusRepositoryPlugin.wrapException(e);
				}
			}
		}

	}

	private void commitModels(IRepositoryHelper repositoryHelper, ThreeWaySynchronizer synchronizer, String commitMessage, IProgressMonitor progress)
			throws TeamException, RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidValueException,
			ConstraintViolationException, LockedException, UnresolvedReferencesException, InvalidRevisionException {

		try {
			progress = Policy.monitorFor(progress);
			progress.beginTask(Policy.bind("PutAction.working"), IProgressMonitor.UNKNOWN);
			progress.subTask("committing models...");
			for (Iterator<Entry<IFile, org.eclipse.emf.common.util.URI>> iterator = modelsToCheckinMap.entrySet().iterator(); iterator.hasNext();) {
				Policy.checkCanceled(progress);

				Map.Entry<IFile, org.eclipse.emf.common.util.URI> entry = iterator.next();

				List<IContainer> parentContainers = new ArrayList<IContainer>();
				// Resource resource = entry.getKey();
				IFile localFile = entry.getKey();

				progress.subTask("committing " + localFile.getName() + "...");

				ResourceSet resourceSet = new ResourceSetImpl();
				org.eclipse.emf.common.util.URI localUri = org.eclipse.emf.common.util.URI.createFileURI(localFile.getLocation().toString());
				Resource resource = resourceSet.createResource(localUri);
				resource.load(Collections.EMPTY_MAP);
				org.eclipse.emf.common.util.URI remoteUri = entry.getValue();
				resource.setURI(remoteUri);
				repositoryHelper.checkInModel(UserSessionHelper.getSession(), resource, localUri, Collections.EMPTY_MAP, commitMessage);

				IResourceVariant remote = getResourceVariant(localFile); // MES: Slow but
				// precise!
				// // TODO
				// // TODOMWA
				// // remove adaptive hack
				if (remote != null) {
					synchronizer.setBaseBytes(localFile, remote.asBytes());
					((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(localFile,
							((ModelBusRepositoryResourceVariant) remote).getRevision());
				} else {

					try {
						// TODO optimize use DataSource
						ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
						FileInputStream fileInputStream = new FileInputStream(localFile.getLocation().toFile());
						byte[] buffer = new byte[16384];
						for (int len = fileInputStream.read(buffer); len > 0; len = fileInputStream.read(buffer)) {
							byteArrayOutputStream.write(buffer, 0, len);
						}
						fileInputStream.close();

						synchronizer.setBaseBytes(localFile, byteArrayOutputStream.toByteArray());
						RepositoryDirEntry dirEntry = repositoryHelper.getInfo(UserSessionHelper.getSession(), remoteUri, IRepositoryHelper.REVISION_HEAD);
						((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(localFile, dirEntry.getRevision());
					} catch (IOException e) {
						// TODO Auto-generated catch block
						throw ModelBusRepositoryPlugin.wrapException(e);
					}
				}

				List<String> filesToCheckin = repositoryHelper.getFilesToCheckin();
				for (String filePath : filesToCheckin) {
					URI fileLocation = URI.create(filePath);
					URI rooturi = ResourcesPlugin.getWorkspace().getRoot().getLocationURI();
					if (!localFile.getLocationURI().equals(fileLocation)) {
						fileLocation = rooturi.relativize(fileLocation);
						IPath path = new Path(fileLocation.toString());
						IFile ifile = ResourcesPlugin.getWorkspace().getRoot().getFile(path);
						ModelBusRepositoryResourceVariant variant = getResourceVariant(ifile);
						if (variant != null) {
							synchronizer.setBaseBytes(ifile, variant.asBytes());
							((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(ifile, variant.getRevision());
							getAllParents(ifile, parentContainers);
						}
					}
				}

				for (Object ctr : parentContainers) {
					if (synchronizer.isLocallyModified((IResource) ctr)) {
						synchronizer.setBaseBytes((IResource) ctr, new byte[0]);
						((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision((IResource) ctr, IRepositoryHelper.REVISION_HEAD);
					}
				}

				progress.worked(1);
			}
		} finally {
			progress.done();
		}

	}

	/**
	 * Check-in the given resources to the given depth by replacing the remote (i.e. file system) contents with the local workspace contents.
	 * 
	 * @param traversals
	 *            the traversals that cover the resources to check in
	 * @param overrideIncoming
	 *            indicate whether incoming remote changes should be replaced
	 * @param progress
	 *            a progress monitor
	 * @throws TeamException
	 * @throws UnresolvedReferencesException
	 * @throws ConstraintViolationException
	 * @throws InvalidValueException
	 * @throws LockedException
	 * @throws IOException
	 * @throws InvalidRevisionException
	 */
	public void checkin(ResourceTraversal[] traversals, boolean overrideIncoming, IProgressMonitor monitor, String commitMessage)
			throws TeamException, RepositoryAuthentificationException, NonExistingResourceException, LockedException, InvalidValueException,
			ConstraintViolationException, UnresolvedReferencesException, IOException, InvalidRevisionException {
		try {
			// ensure the progress monitor is not null
			monitor = Policy.monitorFor(monitor);
			monitor.beginTask(null, 100 * traversals.length);
			for (int i = 0; i < traversals.length; i++) {
				ResourceTraversal traversal = traversals[i];
				checkin(traversal.getResources(), traversal.getDepth(), overrideIncoming, new SubProgressMonitor(monitor, 100), commitMessage);
			}
		} finally {
			monitor.done();
		}
	}

	/**
	 * Return whether the local resource is checked out. A resource is checked out if it is a file that is not read-only. Folders are always checked
	 * out.
	 * 
	 * @param resource
	 *            the resource
	 * @return whether the resource is checked out and can be modified
	 */
	public boolean isCheckedOut(IResource resource) {
		if (resource.getType() != IResource.FILE)
			return true;
		return !resource.isReadOnly();
	}

	/*
	 * Get the resource variant for the given resource.
	 */
	private ModelBusRepositoryResourceVariant getResourceVariant(IResource resource) throws RepositoryAuthentificationException,
			NonExistingResourceException, IOException, InvalidRevisionException {
		return (ModelBusRepositoryResourceVariant) provider.getResourceVariant(resource);
	}

	private void internalGet(IResource[] resources, int depth, boolean overrideOutgoing, IProgressMonitor progress) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidRevisionException {
		// Traverse the resources and get any that are out-of-sync
		progress.beginTask(Policy.bind("GetAction.working"), IProgressMonitor.UNKNOWN); //$NON-NLS-1$
		for (int i = 0; i < resources.length; i++) {
			Policy.checkCanceled(progress);
			if (resources[i].getType() == IResource.FILE) {
				internalGet((IFile) resources[i], overrideOutgoing, progress);
			} else if (depth != IResource.DEPTH_ZERO) {
				internalGet((IContainer) resources[i], depth, overrideOutgoing, progress);
			}
			progress.worked(1);
		}
	}

	/*
	 * Get the folder and its children to the depth specified.
	 */
	private void internalGet(IContainer container, int depth, boolean overrideOutgoing, IProgressMonitor progress) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidRevisionException {
		try {
			ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();
			// Make the local folder state match the remote folder state
			List toDelete = new ArrayList();
			if (container.getType() == IResource.FOLDER) {
				IFolder folder = (IFolder) container;
				ModelBusRepositoryResourceVariant remote = getResourceVariant(container);
				if (!folder.exists() && remote != null) {
					// Create the local folder
					folder.create(false, true, progress);
					synchronizer.setBaseBytes(folder, remote.asBytes());
					((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(folder, remote.getRevision());
				} else if (folder.exists() && remote == null) {
					// Schedule the folder for removal but delay in
					// case the folder contains outgoing changes
					toDelete.add(folder);
				}
			}

			// Get the children
			IResource[] children = synchronizer.members(container);
			if (children.length > 0) {
				internalGet(children, depth == IResource.DEPTH_INFINITE ? IResource.DEPTH_INFINITE : IResource.DEPTH_ZERO, overrideOutgoing, progress);
			}

			// Remove any empty folders
			for (Iterator iter = toDelete.iterator(); iter.hasNext();) {
				IFolder folder = (IFolder) iter.next();
				if (folder.members().length == 0) {
					folder.delete(false, true, progress);
					synchronizer.flush(folder, IResource.DEPTH_INFINITE);
				}
			}
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		}
	}

	/*
	 * Get the file if it is out-of-sync.
	 */
	private void internalGet(IFile localFile, boolean overrideOutgoing, IProgressMonitor progress) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, IOException, InvalidRevisionException {
		ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();
		IResourceVariantComparator comparator = ModelBusRepositorySubscriber.getInstance().getResourceComparator();
		ModelBusRepositoryResourceVariant remote = getResourceVariant(localFile);
		byte[] baseBytes = synchronizer.getBaseBytes(localFile);
		IResourceVariant base = provider.getLocalResourceVariant(localFile, baseBytes);
		if (!synchronizer.hasSyncBytes(localFile) || (isLocallyModified(localFile) && !overrideOutgoing)) {
			// Do not overwrite the local modification
			return;
		}
		if (base != null && remote == null) {
			// The remote no longer exists so remove the local
			try {
				synchronizer.flush(localFile, IResource.DEPTH_ZERO);// Needed at
				// all?
				localFile.delete(false, true, progress);
			} catch (CoreException e) {
				throw TeamException.asTeamException(e);
			}
			return; // MES
		}
		if (!synchronizer.isLocallyModified(localFile) && base != null && remote != null && comparator.compare(base, remote)) {
			// The base and remote are the same and there's no local changes
			// so nothing needs to be done
			return; // MES
		}
		try {
			// Copy from the remote file to the local file:
			InputStream source = null;
			try {
				// Get the remote file content.
				source = remote.getContents();
				// Set the local file content to be the same as the remote file.
				if (localFile.exists())
					localFile.setContents(source, false, false, progress);
				else
					localFile.create(source, false, progress);
			} finally {
				if (source != null)
					source.close();
			}
			// Mark as read-only to force a checkout before editing
			// localFile.setReadOnly(true); // MES: Why should anything be
			// read-only here.
			synchronizer.setBaseBytes(localFile, remote.asBytes());
			((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision(localFile, remote.getRevision());
		} catch (IOException e) {
			throw ModelBusRepositoryPlugin.wrapException(e);
		} catch (CoreException e) {
			throw ModelBusRepositoryPlugin.wrapException(e);
		}
	}

	private void internalPut(IResource[] resources, int depth, boolean overrideIncoming, IProgressMonitor progress, String commitMessage)
			throws TeamException, RepositoryAuthentificationException, NonExistingResourceException, LockedException, InvalidValueException,
			ConstraintViolationException, UnresolvedReferencesException, IOException, InvalidRevisionException {
		// ensure the progress monitor is not null
		progress = Policy.monitorFor(progress);
		progress.beginTask(Policy.bind("PutAction.working"), IProgressMonitor.UNKNOWN); //$NON-NLS-1$

		for (int i = 0; i < resources.length; i++) {
			Policy.checkCanceled(progress);
			if (resources[i].getType() == IResource.FILE) {
				internalPut((IFile) resources[i], overrideIncoming, progress, commitMessage);
			} else if (depth > 0) { // Assume that resources are either files or
				// containers.
				internalPut((IContainer) resources[i], depth, overrideIncoming, progress, commitMessage);
			}
			progress.worked(1);
		}

		progress.done();
	}

	public void lock(IFile localFile, IProgressMonitor progress) throws RepositoryAuthentificationException, InvalidValueException,
			NonExistingResourceException, LockedException {

		String remoteFileLocation = provider.getRepositoryPath(localFile);
		final org.eclipse.emf.common.util.URI remoteFileURI = org.eclipse.emf.common.util.URI.createURI(remoteFileLocation);
		// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(repLocation,checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		repositoryHelper.lock(UserSessionHelper.getSession(), remoteFileURI);

	}

	public void unlock(IFile localFile, IProgressMonitor progress) throws RepositoryAuthentificationException, LockedException,
			NonExistingResourceException {

		String remoteFileLocation = provider.getRepositoryPath(localFile);
		final org.eclipse.emf.common.util.URI remoteFileURI = org.eclipse.emf.common.util.URI.createURI(remoteFileLocation);
		// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getString(
		// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// repLocation,checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		repositoryHelper.unlock(UserSessionHelper.getSession(), remoteFileURI);
	}

	private void getAllParents(IResource iResource, List folders) throws TeamException {
		IContainer parent = iResource.getParent();
		while (parent != null) {
			if (!folders.contains(parent)) {
				folders.add(parent);

			}
			parent = parent.getParent();
		}
	}

	/**
	 * Put the file if the sync state allows it.
	 * 
	 * @param localFile
	 *            the local file
	 * @param overrideIncoming
	 *            whether incoming changs shoudl be overwritten
	 * @param progress
	 *            a progress monitor
	 * @return whether the put succeeded (i.e. the local matches the remote)
	 * @throws TeamException
	 * @throws LockedException
	 * @throws UnresolvedReferencesException
	 * @throws ConstraintViolationException
	 * @throws InvalidValueException
	 * @throws InvalidRevisionException
	 * @throws IOException
	 */
	private boolean internalPut(IFile localFile, boolean overrideIncoming, IProgressMonitor progress, String commitMessage) throws TeamException,
			RepositoryAuthentificationException, NonExistingResourceException, LockedException, InvalidValueException, ConstraintViolationException,
			UnresolvedReferencesException, IOException, InvalidRevisionException {

		boolean commitModels = false;
		String lockUser = LockHelper.getLockHelper().objectLockedBy(localFile);
		if (lockUser != null && !lockUser.equals(UserSessionHelper.getPropertyUserName())) {
			return false;
		}
		ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();
		IResourceVariantComparator comparator = ModelBusRepositorySubscriber.getInstance().getResourceComparator();
		ModelBusRepositoryResourceVariant remote = getResourceVariant(localFile);
		byte[] baseBytes = synchronizer.getBaseBytes(localFile);
		// hgh
		// System.out.println("Test");
		IResourceVariant base = provider.getLocalResourceVariant(localFile, baseBytes);

		// Check whether we are overriding a remote change
		if (base == null && remote != null && !overrideIncoming) {
			// The remote is an incoming (or conflicting) addition.
			// Do not replace unless we are overriding
			return false;
		} else if (base != null && remote == null) {
			// The remote is an incoming deletion
			if (!localFile.exists()) {
				// Conflicting deletion. Clear the synchronizer.
				synchronizer.flush(localFile, IResource.DEPTH_ZERO);
			} else if (!overrideIncoming) {
				// Do not override the incoming deletion
				return false;
			}
		} else if (base != null && remote != null) {
			boolean same = comparator.compare(base, remote);
			if (!isLocallyModified(localFile) && same) {
				// The base and remote are the same and there's no local changes
				// so nothing needs to be done
				return true;
			}
			if (!same && !overrideIncoming) {
				// The remote has changed. Only override if specified
				return false;
			}
		}

		// Handle an outgoing deletion
		String remoteFileLocation = provider.getRepositoryPath(localFile);
		if (!localFile.exists()) {
			// Copy from the local file to the remote file:
			// try {
			// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
			// .getDefault()
			// .getPreferenceStore()
			// .getString(
			// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
			// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
			// .getDefault()
			// .getPreferenceStore()
			// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
			// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
			// .getDefault().getExtensionPreference();
			// RepositoryHelper repositoryHelper = new RepositoryHelper(
			// repLocation,checkDependencies,modelExtensions);
			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			// ---- GDU: for committing all resources in a batch
			// ----------------------------------------------------------------------------------------------
			// repositoryHelper.delete(UserSessionHelper.getSession(),
			// remoteFileLocation,commitMessage);
			deletionPaths.add(remoteFileLocation);
			resourcesToDelete.add(localFile);
			// synchronizer.flush(localFile, IResource.DEPTH_ZERO);
			// ------------------------------------------------------------------------------------------------
			// } catch (CoreException e) {
			// throw ModelBusRepositoryPlugin.wrapException(e);
			// }

		} else {
			// Otherwise, this is an outgoing addition. Upload the contents.
			// Collect all parent folders. Creating files remotely silently
			// mirrors
			// the whole local folder hierarchy, if not yet committed.
			List<IContainer> parentContainers = new ArrayList<IContainer>();
			// try {
			IContainer parent = localFile.getParent();
			while (parent != null) {
				parentContainers.add(parent);
				parent = parent.getParent();
			}
			// Copy from the local file to the remote file:
			// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
			// .getDefault()
			// .getPreferenceStore()
			// .getString(
			// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
			boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getPreferenceStore()
					.getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
			// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
			// .getDefault().getExtensionPreference();
			// RepositoryHelper repositoryHelper = new RepositoryHelper(
			// repLocation,checkDependencies,modelExtensions);
			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			// repositoryHelper.setEnableDependencies(checkDependencies);
			// repositoryHelper.setModelExtensions(modelExtensions);
			// this.setLocal2RemoteMap();
			// File file = localFile.getLocation().toFile();
			// FileDataSource fileDataSource = new FileDataSource(file);
			// DataHandler dataHandler = new DataHandler(fileDataSource);

			// ---- GDU: for committing all resources in a batch
			// ----------------------------------------------------------------------------------------------------------
			// repositoryHelper.checkInFile(UserSessionHelper.getSession(),
			// remoteFileLocation, file, commitMessage);
			final org.eclipse.emf.common.util.URI remoteFileURI = org.eclipse.emf.common.util.URI.createURI(remoteFileLocation);

			if (checkDependencies && isExtensionForDependenciesSupport(localFile.getFileExtension())) {
				modelsToCheckinMap.put(localFile, remoteFileURI);
				// resource.setURI(org.eclipse.emf.common.util.URI.createURI(remoteFileLocation));
				// repositoryHelper.checkInModel(UserSessionHelper.getSession(), resource, localUri, Collections.EMPTY_MAP, commitMessage);
				// commitModels = true;
			} else {
				PathFileContentMapEntry mapEntry = new PathFileContentMapEntry();
				File file = localFile.getLocation().toFile();
				FileDataSource fileDataSource = new FileDataSource(file);
				DataHandler dataHandler = new DataHandler(fileDataSource);
				mapEntry.setPath(remoteFileLocation);
				mapEntry.setFileContent(dataHandler);

				if (repositoryHelper.exists(UserSessionHelper.getSession(), remoteFileURI, IRepositoryHelper.REVISION_HEAD)) {
					checkInFileMap.add(mapEntry);
				} else {
					createFileMap.add(mapEntry);
				}

				filesToSetBaseBytes.add(localFile);
			}
			// -----------------------------------------------------------------------------------------------------------

			// ResourceSet resourceSet = new ResourceSetImpl();
			//
			// //
			// Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xml",
			// new
			// XMLResourceFactoryImpl());
			//
			//
			// Resource resource =
			// resourceSet.createResource(URI.createURI(remoteFileLocation));
			// resource.load(new
			// FileInputStream(localFile.getLocation().toFile()), null);
			// repositoryHelper.checkInModel(UserSessionHelper
			// .getSession(), resource);

			// --- GDU: this is done in checkin(IResource[]...) after committing all resources
			// --------------------------------------------------------------------------------

			// remote = getResourceVariant(localFile); // MES: Slow but
			// // precise!
			// // // TODO
			// // // TODOMWA
			// // // remove adaptive hack
			// if (remote != null) {
			// synchronizer.setBaseBytes(localFile, remote.asBytes());
			// } else {
			// // TODO optimize use DataSource
			// ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			// FileInputStream fileInputStream = new FileInputStream(
			// localFile.getLocation().toFile());
			// byte[] buffer = new byte[16384];
			// for (int len = fileInputStream.read(buffer); len > 0; len = fileInputStream
			// .read(buffer)) {
			// byteArrayOutputStream.write(buffer, 0, len);
			// }
			// fileInputStream.close();
			// synchronizer.setBaseBytes(localFile, byteArrayOutputStream
			// .toByteArray());
			// }
			//
			// // we have to set the base bytes for the automatically committed models
			// if(checkDependencies && isExtensionForDependenciesSupport(localFile.getFileExtension())
			// /*Arrays.asList(modelExtensions).contains(localFile.getFileExtension())*/){
			// List<String> filesToCheckin = repositoryHelper.getFilesToCheckin();
			// for (Object object : filesToCheckin) {
			// URI fileLocation = URI.create((String) object);
			// URI rooturi= ResourcesPlugin.getWorkspace().getRoot().getLocationURI();
			// if(!localFile.getLocationURI().equals(fileLocation)){
			// fileLocation = rooturi.relativize(fileLocation);
			// IPath path = new Path(fileLocation.toString());
			// IFile ifile = ResourcesPlugin.getWorkspace().getRoot().getFile(path);
			// ModelBusRepositoryResourceVariant variant = getResourceVariant(ifile);
			// if(variant != null){
			// synchronizer.setBaseBytes(ifile, variant.asBytes());
			// getAllParents(ifile, parentContainers);
			// }
			// }
			// }
			//
			// }
			// -----------------------------------------------------------------------------------------------------------

			// } catch (IOException e) {
			// throw ModelBusRepositoryPlugin.wrapException(e);
			// } catch (CoreException e) {
			// throw ModelBusRepositoryPlugin.wrapException(e);
			// }

			// When committing single files out of a complex file system tree to
			// be added to version control
			// all the parent folders must be marked as in-sync to avoid
			// undesired pseudo-conflicts upon subsequent
			// synchronize requests.
			for (Object ctr : parentContainers) {
				if (synchronizer.isLocallyModified((IResource) ctr)) {
					synchronizer.setBaseBytes((IResource) ctr, new byte[0]);
					((ModelBusThreeWaySynchronizer) synchronizer).setWorkingCopyRevision((IResource) ctr, IRepositoryHelper.REVISION_HEAD);
				}
			}
		}
		return true;
	}

	// private void setLocal2RemoteMap(){
	// HashMap local2remoteMap = new HashMap<String, String>();
	// IProject[] localProjects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
	// for (int i = 0; i < localProjects.length; i++) {
	// String remoteLocation = provider.getRepositoryPath(localProjects[i]);
	// String localLocation = localProjects[i].getLocationURI().toString();
	// local2remoteMap.put(localLocation, remoteLocation);
	// }
	// ModelBusRepositoryHelper.getRepositoryHelper().setProjectPathMap(local2remoteMap);
	// }

	private boolean isLocallyModified(IFile localFile) throws TeamException {
		ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();
		if (!localFile.exists()) {
			// Extra check for bug 141415
			return synchronizer.getBaseBytes(localFile) != null;
		}
		return synchronizer.isLocallyModified(localFile);
	}

	private boolean isExtensionForDependenciesSupport(String extension) {
		final String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin.getDefault().getExtensionPreference();

		for (String modelExtension : modelExtensions) {
			String planeExtension = modelExtension;
			if (modelExtension.startsWith("*.")) {
				planeExtension = modelExtension.substring(2);
			} else if (modelExtension.startsWith(".")) {
				planeExtension = modelExtension.substring(1);
			}
			if (planeExtension.equals(extension)) {
				return true;
			}
		}
		return false;
	}

	/*
	 * Get the folder and its children to the depth specified.
	 */
	private void internalPut(IContainer container, int depth, boolean overrideIncoming, IProgressMonitor progress, String commitMessage)
			throws TeamException, NonExistingResourceException, RepositoryAuthentificationException, LockedException, InvalidValueException,
			ConstraintViolationException, UnresolvedReferencesException, IOException, InvalidRevisionException {
		try {
			ThreeWaySynchronizer synchronizer = ModelBusRepositorySubscriber.getInstance().getSynchronizer();
			// Make the local folder state match the remote folder state
			List<IFolder> toDelete = new ArrayList<IFolder>();
			if (container.getType() == IResource.FOLDER) {
				IFolder folder = (IFolder) container;
				// File diskFile = provider.getFile(container);
				ModelBusRepositoryResourceVariant remote = getResourceVariant(container);
				if (!folder.exists() && remote != null) {
					// Schedule the folder for removal but delay in
					// case the folder contains incoming changes ????? see
					// PutOperation#execute()!
					toDelete.add(folder); // ALWAYS DELETE BOTTOM UP! Will mess
					// up the synchronize view
					// otherwise.
				} else if (folder.exists() && remote == null) {
					// Create the remote directory and sync up the local
					// diskFile.mkdir();
					// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
					// .getDefault()
					// .getPreferenceStore()
					// .getString(
					// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
					// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
					// .getDefault()
					// .getPreferenceStore()
					// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
					// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
					// .getDefault().getExtensionPreference();
					// RepositoryHelper repositoryHelper = new RepositoryHelper(
					// repLocation,checkDependencies,modelExtensions);
					IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();

					try {
						// --- GDU: for committing all resources in a batch
						// ----------------------------------------------------------------------------------------------
						// base bytes will be set below.
						// repositoryHelper.createDir(UserSessionHelper
						// .getSession(), provider
						// .getRepositoryPath(folder), commitMessage);

						folderPaths.add(provider.getRepositoryPath(folder));
						// -----------------------------------------------------------------------------------------------

					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

			// Get the children and descend. Container may have already been
			// deleted locally at this point.
			IResource[] children = synchronizer.members(container);
			if (children.length > 0) {
				internalPut(children, depth == IResource.DEPTH_INFINITE ? IResource.DEPTH_INFINITE : IResource.DEPTH_ZERO, overrideIncoming,
						progress, commitMessage);
			}

			// Remove remote folders collected above.
			for (Iterator<IFolder> iter = toDelete.iterator(); iter.hasNext();) {
				// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin
				// .getDefault()
				// .getPreferenceStore()
				// .getString(
				// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
				// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault()
				// .getPreferenceStore()
				// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
				// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault().getExtensionPreference();
				// RepositoryHelper repositoryHelper = new RepositoryHelper(
				// repLocation,checkDependencies,modelExtensions);
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				try {
					// --- GDU: for committing all resources in a batch
					// ---------------------------------------------------------------------------------------------------
					IFolder folderToDelete = iter.next();
					// repositoryHelper.delete(UserSessionHelper.getSession(),
					// provider.getRepositoryPath(folderToDelete),commitMessage);

					deletionPaths.add(provider.getRepositoryPath(folderToDelete));
					this.resourcesToDelete.add(folderToDelete);
					// synchronizer
					// .flush(folderToDelete, IResource.DEPTH_INFINITE);
					// -----------------------------------------------------------------------------------------------------
				} catch (Exception e) {
					e.printStackTrace();
					// File diskFile = (File) iter.next();
					// if (diskFile.listFiles().length == 0) {
					// diskFile.delete();
					// synchronizer.flush(container, IResource.DEPTH_INFINITE);
				}

			}
			if (container.exists()) {
				// For projects and folders that were not deleted in the last
				// step! May already have been done by one of the children.
				// Just to be on the safe side :)
				// ---- GDU: this is done in checkin(IResource[]...) after committing all resources
				// synchronizer.setBaseBytes(container, provider.getResourceVariant(container).asBytes());
				containerToSetBaseBytes.add(container);
			}
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		}
	}
}
