/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.subscriber;

import java.io.IOException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.ThreeWayRemoteTree;
import org.modelbus.dosgi.repository.descriptor.InvalidRevisionException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;

/**
 * The file system three-way remote resource variant tree that provides the ability to traverse the file system for the creation of resource variants.
 */
public class ModelBusRepositoryRemoteTree extends ThreeWayRemoteTree {

	/**
	 * Create the file system remote resource variant tree
	 * 
	 * @param subscriber
	 *            the file system subscriber
	 */
	public ModelBusRepositoryRemoteTree(ModelBusRepositorySubscriber subscriber) {
		super(subscriber);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.AbstractResourceVariantTree#fetchMembers(org.eclipse.team.core.variants.IResourceVariant,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IResourceVariant[] fetchMembers(IResourceVariant variant, IProgressMonitor progress) throws TeamException {
		return ((ModelBusRepositoryResourceVariant) variant).members();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.AbstractResourceVariantTree#fetchVariant(org.eclipse.core.resources.IResource, int,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IResourceVariant fetchVariant(IResource resource, int depth, IProgressMonitor monitor) throws TeamException {
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider != null) {
			IResourceVariant iVariant = null;
			try {
				iVariant = ((ModelBusRepositoryProvider) provider).getResourceVariant(resource);
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				throw new RuntimeException(e);
			} catch (NonExistingResourceException e) {
				// TODO Auto-generated catch block
				throw new RuntimeException(e);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new RuntimeException(e);
			} catch (InvalidRevisionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new RuntimeException(e);
			}
			return iVariant;
		}
		return null;
	}

	public IResourceVariant getResourceVariant(IResource resource) throws TeamException {
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider != null) {
			IResourceVariant iVariant = null;
			try {
				iVariant = ((ModelBusRepositoryProvider) provider).getResourceVariant(resource);
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				throw new RuntimeException(e);
			} catch (NonExistingResourceException e) {
				// TODO Auto-generated catch block
				throw new RuntimeException(e);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new RuntimeException(e);
			} catch (InvalidRevisionException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new RuntimeException(e);
			}

			return iVariant;
		}
		return null;
	}
}
