/*******************************************************************************
 * ongoing check MWA
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.subscriber;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.IEncodedStorage;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.osgi.util.NLS;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.CachedResourceVariant;
import org.eclipse.team.internal.core.Messages;
import org.eclipse.team.internal.core.TeamPlugin;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.core.lib.types.IModelBusDataSource;
import org.modelbus.core.lib.util.ModelBusCoreUtil.URIUtil;
import org.modelbus.dosgi.repository.descriptor.RepositoryDirEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryNodeKind;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

/**
 * A file system resource variant. Although not strictly necessary, this class extends <code>CachedResourceVariant</code> which will cache the
 * contents of the resource variant.
 */
public class ModelBusRepositoryResourceVariant extends CachedResourceVariant
// extends PlatformObject implements IResourceVariant
{

	private RepositoryDirEntry	repositoryDirEntry; // contains revision number :)
	private String				repositoryPath;
	private byte[]				bytes;
	private String				revision	= IRepositoryHelper.REVISION_HEAD;

	private IRepositoryHelper	repositoryHelper;

	private boolean				isContainer;

	/**
	 * Create a resource variant for the given file. The bytes will be calculated when they are accessed.
	 * 
	 * @param file
	 *            the file
	 */
	public ModelBusRepositoryResourceVariant(RepositoryDirEntry repositoryPath, IRepositoryHelper repositoryHelper) {
		this.repositoryHelper = repositoryHelper;
		this.repositoryDirEntry = repositoryPath;
		if (repositoryDirEntry == null) {
			RuntimeException e = new RuntimeException();
			e.printStackTrace();
			throw e;
		}
		this.revision = repositoryPath.getRevision();
	}

	public ModelBusRepositoryResourceVariant(String repositoryPath, IRepositoryHelper repositoryHelper, String revision) {
		this.revision = revision;
		this.repositoryPath = repositoryPath;
		this.repositoryHelper = repositoryHelper;
		if (repositoryPath == null) {
			RuntimeException e = new RuntimeException();
			e.printStackTrace();
			throw e;
		}
	}

	public ModelBusRepositoryResourceVariant(String repositoryPath, IRepositoryHelper repositoryHelper) {
		this.repositoryHelper = repositoryHelper;
		this.repositoryPath = repositoryPath;
		if (repositoryPath == null) {
			RuntimeException e = new RuntimeException();
			e.printStackTrace();
			throw e;
		}
	}

	public ModelBusRepositoryResourceVariant(String repositoryPath, IRepositoryHelper repositoryHelper, byte[] bytes) {
		this.repositoryHelper = repositoryHelper;
		this.repositoryPath = repositoryPath;
		this.bytes = bytes;
		if (repositoryPath == null) {
			RuntimeException e = new RuntimeException();
			e.printStackTrace();
			throw e;
		}
	}

	public RepositoryDirEntry getRepositoryDirEntry() {
		if (repositoryDirEntry == null) {
			try {// repositoryPath is fucking null! How can this be!
				repositoryDirEntry = repositoryHelper.getInfo(UserSessionHelper.getSession(), URI.createURI(repositoryPath), this.revision); // revision
				revision = repositoryDirEntry.getRevision(); // will
				// usually
				// be
				// HEAD
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
		return repositoryDirEntry;
	}

	/**
	 * Create a resource variant for the given file and sync bytes.
	 * 
	 * @param file
	 *            the file
	 * @param bytes
	 *            the timestamp bytes
	 */
	public ModelBusRepositoryResourceVariant(RepositoryDirEntry repositoryPath, byte[] bytes) {
		this.repositoryDirEntry = repositoryPath;
		this.bytes = bytes;
		if (repositoryPath == null) {
			RuntimeException e = new RuntimeException();
			e.printStackTrace();
			throw e;
		}
		this.revision = repositoryPath.getRevision();
	}

	// // private RepositoryDirEntry getRepositoryDirEntry() {
	// // return null;
	// // }
	//
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.CachedResourceVariant#fetchContents(org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void fetchContents(IProgressMonitor monitor) throws TeamException {
		setContents(getContents(), monitor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.CachedResourceVariant#getCachePath()
	 */
	protected String getCachePath() {
		// append the timestamp to the file path to give each variant a unique
		// path

		return org.eclipse.emf.common.util.URI.createURI(getRepositoryDirEntry().getUri()).path() + " " + getRepositoryDirEntry().getRevision();// +
		// ioFile.lastModified();
		// //$NON-NLS-1$
	}

	//
	// public String getFilePath() {
	// // try {
	// // return null; // ioFile.getCanonicalPath();
	// // } catch (Exception e) {
	// // // // Failed for some reason. Try the absolute path.
	// // // ModelBusRepositoryPlugin.log(new Status(IStatus.ERROR,
	// // // ModelBusRepositoryPlugin.ID, 0,
	// // // "Failed to obtain canonical path for " +
	// // // ioFile.getAbsolutePath(), e)); //$NON-NLS-1$
	// // // return ioFile.getAbsolutePath();
	// // return null;
	// // }
	// throw new RuntimeException("NYI");
	// // return repositoryPath.getUrl().toString();
	// }

	// public boolean isContentsCached() {
	// return false;
	// }
	//
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.CachedResourceVariant#getCacheId()
	 */
	protected String getCacheId() {
		return ModelBusRepositoryPlugin.ID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.IResourceVariant#getName()
	 */
	public String getName() {
		return getRepositoryDirEntry().getName();
		// return null; // ioFile.getName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.IResourceVariant#isContainer()
	 */
	public boolean isContainer() {

		if (bytes != null && bytes.length > 0) { // MES: Containers never have any bytes stored. Avoids contacting the
			// server.
			return false;
		}
		RepositoryDirEntry dirEntry = null;
		try {
			dirEntry = getRepositoryDirEntry();
		} catch (RuntimeException e) {
			/* Remote node may have been deleted. Return false. */
		}
		if (dirEntry != null) {
			return (RepositoryNodeKind.DIR.equals(dirEntry.getKind()));
		}
		/*
		 * MES: else { In this case a dirEntry could not be obtained, either because an error occurred along the way or the remote object does no
		 * longer exist. What the remote object looked like cannot easily be determined. Even if we had quick direct access to the local IResource
		 * handle here, the remote object might already have been replaced. Maybe in this case both methods isFile() and isContainer() should return
		 * true to give the sync logic the opportunity to cover both cases. Recall ThreeWayResourceComparator#compare(IResource, IResourceVariant) in
		 * case of folder sync. }
		 */
		return false; // ioFile.isDirectory();
	}

	/* MES: See isContainer() */
	public boolean isFile() {
		if (bytes != null && bytes.length == 0) {
			return false;
		}
		RepositoryDirEntry dirEntry = getRepositoryDirEntry();
		if (dirEntry != null) {
			return (RepositoryNodeKind.FILE.equals(dirEntry.getKind()));
		}

		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.IResourceVariant#getContentIdentifier()
	 */
	public String getContentIdentifier() {
		return getRepositoryDirEntry().getUri().toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.IResourceVariant#asBytes()
	 */
	public byte[] asBytes() {
		if (bytes == null) {
			// For simplicity, convert the timestamp to it's string
			// representation.
			// A more optimal storage format would be the 8 bytes that make up
			// the long
			try {
				// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
				// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
				// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault()
				// .getPreferenceStore()
				// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
				// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault().getExtensionPreference();
				//
				// RepositoryHelper repositoryHelper = new RepositoryHelper(repLocation,checkDependencies,modelExtensions);
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				// RepositoryStub stub = new RepositoryStub(repLocation);
				//
				// ServiceClient serviceClient = stub._getServiceClient();
				// // ServiceClient serviceClient = new ServiceClient ();
				// Options options = new Options();
				// options.setTo(new EndpointReference(repLocation));
				// options.setProperty(Constants.Configuration.ENABLE_MTOM,
				// Constants.VALUE_TRUE);
				// // options.setProperty(Constants.Configuration.ENABLE_SWA,
				// // Constants.VALUE_TRUE);
				// options.setProperty(Constants.Configuration.CACHE_ATTACHMENTS,
				// Constants.VALUE_TRUE);
				// options.setProperty(Constants.Configuration.ATTACHMENT_TEMP_DIR,
				// "/tmp");
				// options.setProperty(Constants.Configuration.FILE_SIZE_THRESHOLD,
				// "4096");
				// options.setTransportInfo(Constants.TRANSPORT_HTTP,
				// Constants.TRANSPORT_HTTP, false);
				// options.setSoapVersionURI(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI);
				//
				// options.setProperty(org.apache.axiom.om.OMConstants.IS_BINARY,
				// Constants.VALUE_TRUE);
				// serviceClient.get

				if (isContainer()) {
					bytes = new byte[0];
					return bytes;
					// return null;
				}

				InputStream stream = repositoryHelper.checkOutFile(UserSessionHelper.getSession(), URI.createURI(getRepositoryDirEntry().getUri()),
						getRepositoryDirEntry().getRevision());

				if (stream != null) {

					ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
					byte[] buffer = new byte[1024];
					int len;

					while ((len = stream.read(buffer)) >= 0) {
						out.write(buffer, 0, len);
					}

					stream.close();
					out.close();
					bytes = out.toByteArray();
					return bytes;
				} else { /*
						 * Remote file exists but is empty. MUST NOT RETURN NULL. Messes up ThreewayResourceComparator#equals( ... ) used to calculate
						 * the SyncInfo.
						 */
					bytes = new byte[0];
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return null;
			}

			// OMElement payload
			// OMElement result = serviceClient.sendReceive(payload);

		}
		return bytes;
	}

	/**
	 * Return the files contained by the file of this resource variant.
	 * 
	 * @return the files contained by the file of this resource variant.
	 */

	ModelBusRepositoryResourceVariant[]	members	= null;

	public ModelBusRepositoryResourceVariant[] members() {
		if (isContainer()) {
			if (members == null) {
				// java.io.File[] members = ioFile.listFiles();
				try {
					// String repositoryLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
					// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
					// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
					// .getDefault()
					// .getPreferenceStore()
					// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
					// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
					// .getDefault().getExtensionPreference();
					// RepositoryHelper repositoryHelper = new RepositoryHelper(repositoryLocation,checkDependencies,modelExtensions);
					IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
					RepositoryDirEntry[] entries = repositoryHelper.getDirEntries(UserSessionHelper.getSession(),
							URI.createURI(getRepositoryDirEntry().getUri()), getRepositoryDirEntry().getRevision());

					ModelBusRepositoryResourceVariant[] result = new ModelBusRepositoryResourceVariant[entries.length];

					for (int i = 0; i < entries.length; i++) {
						result[i] = new ModelBusRepositoryResourceVariant(entries[i], repositoryHelper);
					}
					members = result;
				} catch (Exception e) {
					members = new ModelBusRepositoryResourceVariant[0];
				}
			}
			return members;
		}
		return new ModelBusRepositoryResourceVariant[0];
	}

	/**
	 * @return
	 */

	IModelBusDataSource	contents	= null;

	// TODO IStorage#getContents() MUST return an open InputStream in any case or throw an exception. In other words:
	// NEVER RETURN NULL!
	public InputStream getContents() throws TeamException {
		// try {
		// return new BufferedInputStream(new FileInputStream(ioFile));
		// } catch (FileNotFoundException e) {
		// throw new TeamException("Failed to fetch contents for " +
		// getFilePath(), e); //$NON-NLS-1$
		// }
		if (contents == null) {
			try {
				// String repLocation = org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(
				// org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
				// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault()
				// .getPreferenceStore()
				// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
				// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
				// .getDefault().getExtensionPreference();
				//
				// RepositoryHelper repositoryHelper = new RepositoryHelper(repLocation,checkDependencies,modelExtensions);
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				contents = repositoryHelper.checkOutFileAsDataSource(UserSessionHelper.getSession(), URI.createURI(getRepositoryDirEntry().getUri()),
						getRepositoryDirEntry().getRevision());
			} catch (Exception e) {
				e.printStackTrace();
				return null;
			}
		}
		if (contents != null) { // valid remote resource found
			try {
				return contents.getInputStream();
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
				return null; // Something went really wrong. Don't care if the return value causes further errors. At
				// least it gets noticed.
			}
		} else { // remote resource does not exist or is empty. DO NOT RETURN NULL. Error scenario: all bytes deleted
			// remotely from a shared file.
			// will cause NPE-trouble on update. See ModelBusMergeContext#merge() ... .
			return new ByteArrayInputStream(new byte[0]);
		}
	}

	public RepositoryDirEntry getFile() {
		return getRepositoryDirEntry();
	}

	public String getRevision() {

		if (IRepositoryHelper.REVISION_HEAD.equals(revision)) {
			revision = getRepositoryDirEntry().getRevision();
		}
		return revision;
	}

	/*
	 * Internal class which provides access to the cached contents of this resource variant
	 */
	class ModelBusResourceVariantStorage implements IEncodedStorage {
		public InputStream getContents() throws CoreException {
			return ModelBusRepositoryResourceVariant.this.getContents();
		}

		public IPath getFullPath() {

			// gdu: path is not enough...

			String uri = getRepositoryDirEntry().getUri();
			String path = null;
			if (URIUtil.isHTTPURI(uri)) {
				path = uri.substring("http://".length());
			} else {
				path = uri;
			}

			return new Path(null, path/* org.eclipse.emf.common.util.URI.createURI(getRepositoryDirEntry().getUri()).path() */);
		}

		public String getName() {
			return ModelBusRepositoryResourceVariant.this.getName();
		}

		public boolean isReadOnly() {
			return true;
		}

		public Object getAdapter(Class adapter) {
			return ModelBusRepositoryResourceVariant.this.getAdapter(adapter);
		}

		public String getCharset() throws CoreException {
			InputStream contents = getContents();
			try {
				String charSet = TeamPlugin.getCharset(getName(), contents);
				return charSet;
			} catch (IOException e) {
				throw new TeamException(new Status(IStatus.ERROR, TeamPlugin.ID, IResourceStatus.FAILED_DESCRIBING_CONTENTS, NLS.bind(
						Messages.CachedResourceVariant_1, new String[] { getFullPath().toString() }), e));
			} finally {
				try {
					contents.close();
				} catch (IOException e1) {
					// Ignore
				}
			}
		}
	}

	ModelBusResourceVariantStorage	storage	= null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.variants.IResourceVariant#getStorage(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public IStorage getStorage(IProgressMonitor monitor) throws TeamException {
		if (isContainer())
			return null;
		if (storage == null) {
			storage = new ModelBusResourceVariantStorage();
		}
		return storage;
	}
}
