package org.modelbus.team.eclipse.repository.ui;

import java.net.InetAddress;

import org.eclipse.emf.common.util.URI;
import org.modelbus.core.lib.IServicesHelper;
import org.modelbus.core.lib.ModelBusCoreLib;
import org.modelbus.core.lib.types.Status;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

public class ModelBusRepositoryDecoratorPing extends Thread {

	private final long		interval;
	private boolean			stop			= false;

	private IServicesHelper	servicesHelper	= null;

	public ModelBusRepositoryDecoratorPing(long interval) {
		super();

		if (interval < 0) {
			throw new IllegalArgumentException("Value for ping interval has to be non negative.");
		}

		this.interval = interval;
	}

	@Override
	public void run() {

		ModelBusRepositoryStatus modelBusRepositoryStatus = ModelBusRepositoryPlugin.getPlugin().getRepositoryStatus();
		while (!this.stop) {
			try {
				if (servicesHelper == null) {
					servicesHelper = ModelBusCoreLib.getServicesHelper();

				}

				// final Status status = servicesHelper.getStatus(UserSessionHelper.getSession(), false);
				//
				// final String repoLocation = RepositoryLocationHelper.getPropertyRepositoryLocation();
				//
				// if(repoLocation != null && repoLocation.equals(status.getLocation())){
				// modelBusRepositoryStatus.setAvailable(true);
				// }else{
				// modelBusRepositoryStatus.setAvailable(false);
				// }

				final Status status = servicesHelper.getStatus(UserSessionHelper.getSession(), false);
				String statusLocation = status.getLocation();
				final URI statusLocationURI = URI.createURI(statusLocation);
				String statusLocationAuthority = statusLocationURI.authority();
				final String statusLocationPort = statusLocationURI.port();

				if (statusLocationAuthority != null) {
					if (statusLocationPort != null) {
						statusLocationAuthority = statusLocationAuthority.split(":")[0];
					}

					if (statusLocationAuthority.equals("0.0.0.0")) {
						statusLocationAuthority = InetAddress.getLocalHost().getHostAddress();
					} else {
						statusLocationAuthority = InetAddress.getByName(statusLocationAuthority).getHostAddress();
					}

				}

				String repoLocation = RepositoryLocationHelper.getPropertyRepositoryLocation();
				final URI repoLocationURI = URI.createURI(repoLocation);
				String authority = repoLocationURI.authority();
				final String port = repoLocationURI.port();
				if (authority != null) {
					if (port != null) {
						authority = authority.split(":")[0];
					}

					if (authority.equals("0.0.0.0")) {
						authority = InetAddress.getLocalHost().getHostAddress();
					} else {
						authority = InetAddress.getByName(authority).getHostAddress();
					}

				}

				if (repoLocation != null && (repoLocation.equals(statusLocation) || statusLocationAuthority.equals(authority))) {
					modelBusRepositoryStatus.setAvailable(true);
				} else {
					modelBusRepositoryStatus.setAvailable(false);
				}

			} catch (RepositoryAuthentificationException e) {
				modelBusRepositoryStatus.setAvailable(false);
			} catch (Exception e) {
				modelBusRepositoryStatus.setAvailable(false);
			}

			try {
				Thread.sleep(this.interval);
			} catch (InterruptedException exc) {
				throw new RuntimeException(exc);
			}

		}
	}

	public void kill() {
		this.stop = true;
	}

}
