package org.modelbus.team.eclipse.repository.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.internal.preferences.Base64;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ResourceUtil;

public class MultiLineInputDialog extends Dialog {

	private class UserInputHistory {
		protected static final String	HISTORY_NAME_BASE	= "history.";

		protected String				name;
		protected int					depth;
		protected List<String>			history;

		public UserInputHistory(String name, int depth) {
			this.name = name;
			this.depth = depth;

			this.loadHistoryLines();
		}

		public String[] getHistory() {
			return (String[]) this.history.toArray(new String[this.history.size()]);
		}

		public void addLine(String line) {
			if (line == null || line.trim().length() == 0) {
				return;
			}
			this.history.remove(line);
			this.history.add(0, line);
			if (this.history.size() > this.depth) {
				this.history.remove(this.history.size() - 1);
			}
			this.saveHistoryLines();
		}

		protected void loadHistoryLines() {
			this.history = new ArrayList<String>();
			String historyData = ModelBusRepositoryPlugin.getPlugin().getPreferenceStore().getString(UserInputHistory.HISTORY_NAME_BASE + this.name);
			if (historyData != null && historyData.length() > 0) {
				String[] historyArray = historyData.split(";");
				for (int i = 0; i < historyArray.length; i++) {
					historyArray[i] = new String(Base64.decode(historyArray[i].getBytes()));
				}
				this.history.addAll(Arrays.asList(historyArray));
			}
		}

		protected void saveHistoryLines() {
			String result = "";
			for (Iterator<String> it = this.history.iterator(); it.hasNext();) {
				String str = it.next();
				str = new String(Base64.encode(str.getBytes()));
				result += result.length() == 0 ? str : (";" + str);
			}
			ModelBusRepositoryPlugin.getPlugin().getPreferenceStore().setValue(UserInputHistory.HISTORY_NAME_BASE + this.name, result);
		}
	}

	private static final String	COMMENT_HISTORY_NAME					= "comment";

	private static String		PREVIOUS_COMMENTS_HEADER;

	private static String		PREVIOUS_COMMENTS_HINT;

	private static final int	COMMENT_SAVED_COMMENTS_COUNT_DEFAULT	= 10;

	/**
	 * The title of the dialog.
	 */
	private String				title;

	/**
	 * The input value; the empty string by default.
	 */
	private String				value									= "";			//$NON-NLS-1$

	/**
	 * Ok button widget.
	 */
	private Button				okButton;

	/**
	 * Input text widget.
	 */
	private Text				text;

	private UserInputHistory	history;

	private Set<String>			ignoredStrings;

	public MultiLineInputDialog(Shell parentShell, String dialogTitle) {
		super(parentShell);
		this.title = dialogTitle;
		MultiLineInputDialog.PREVIOUS_COMMENTS_HEADER = ModelBusRepositoryPlugin.getPlugin().getResource("InputDialog.Previous");
		MultiLineInputDialog.PREVIOUS_COMMENTS_HINT = "    " + ModelBusRepositoryPlugin.getPlugin().getResource("InputDialog.Previous.Hint");
		this.ignoredStrings = new HashSet<String>();
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	/*
	 * (non-Javadoc) Method declared on Dialog.
	 */
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			value = text.getText();
			this.history.addLine(this.value);
		} else {
			value = null;
		}
		super.buttonPressed(buttonId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		if (title != null) {
			shell.setText(title);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		okButton = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
		// do this here because setting the text will set enablement on the ok
		// button
		text.setFocus();
		if (value != null) {
			text.setText(value);
			text.selectAll();
		}
	}

	/*
	 * (non-Javadoc) Method declared on Dialog.
	 */
	protected Control createDialogArea(Composite parent) {
		// create composite
		GridLayout layout = null;
		GridData data = null;
		Color bgColor = new Color(null, 255, 255, 255);

		Composite composite = new Composite(parent, SWT.NONE);

		layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.verticalSpacing = 0;
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);
		Dialog.applyDialogFont(composite);

		Composite infoPanel = new Composite(composite, SWT.NONE);
		layout = new GridLayout();
		layout.marginHeight = 1;
		layout.marginWidth = 1;
		layout.numColumns = 2;
		infoPanel.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		Point pt = new Point(510, SWT.DEFAULT);
		data.widthHint = pt.x;
		infoPanel.setLayoutData(data);
		infoPanel.setBackground(bgColor);

		Composite leftSide = new Composite(infoPanel, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		layout.horizontalSpacing = 1;
		leftSide.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		leftSide.setLayoutData(data);
		leftSide.setBackground(bgColor);

		Font defaultFont = JFaceResources.getBannerFont();
		FontData[] fData = defaultFont.getFontData();
		Display display = Display.getCurrent();
		if (display == null) {
			display = PlatformUI.getWorkbench().getDisplay();
		}
		Font mainLabelFont = new Font(display, fData);

		// create description
		Label description = new Label(leftSide, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		description.setLayoutData(data);
		String descText = ModelBusRepositoryPlugin.getPlugin().getResource("InputDialog.Description");
		description.setText(descText != null ? descText : "");
		description.setFont(mainLabelFont);
		description.setBackground(bgColor);

		// create dialog message
		Label message = new Label(leftSide, SWT.WRAP);
		message.setText(ModelBusRepositoryPlugin.getPlugin().getResource("InputDialog.Message"));
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalIndent = 3;
		message.setLayoutData(data);
		message.setFont(parent.getFont());
		message.setBackground(bgColor);

		Composite fullSizePanel = new Composite(composite, SWT.NONE);
		layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		fullSizePanel.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		fullSizePanel.setLayoutData(data);

		Label separator = new Label(fullSizePanel, SWT.HORIZONTAL | SWT.SEPARATOR);
		separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Composite customPanel = new Composite(fullSizePanel, SWT.NONE);
		layout = new GridLayout();
		layout.marginHeight = 7;
		layout.marginWidth = 7;
		customPanel.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		pt = new Point(510, SWT.DEFAULT);
		data.widthHint = pt.x;
		customPanel.setLayoutData(data);

		separator = new Label(fullSizePanel, SWT.HORIZONTAL | SWT.SEPARATOR);
		separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		text = createText(customPanel);

		Label label = new Label(customPanel, SWT.NULL);
		label.setLayoutData(new GridData());
		label.setText(ModelBusRepositoryPlugin.getPlugin().getResource("InputDialog.ChooseComment"));

		this.history = new UserInputHistory(MultiLineInputDialog.COMMENT_HISTORY_NAME, MultiLineInputDialog.COMMENT_SAVED_COMMENTS_COUNT_DEFAULT);

		final Combo previousCommentsCombo = new Combo(customPanel, SWT.READ_ONLY);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = IDialogConstants.ENTRY_FIELD_WIDTH;
		previousCommentsCombo.setLayoutData(data);

		final List<String> commentsList = this.getCommentsList();

		List<String> flattenCommentsList = new ArrayList<String>();
		for (Iterator<String> iter = commentsList.iterator(); iter.hasNext();) {
			flattenCommentsList.add(ResourceUtil.flattenText(iter.next()));
		}
		previousCommentsCombo.setVisibleItemCount(flattenCommentsList.size());
		previousCommentsCombo.setItems((String[]) flattenCommentsList.toArray(new String[flattenCommentsList.size()]));
		previousCommentsCombo.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				int idx = previousCommentsCombo.getSelectionIndex();
				if (idx != -1) {
					String comboText = (String) commentsList.get(idx);
					MultiLineInputDialog.this.text.setText(MultiLineInputDialog.this.ignoredStrings.contains(comboText) ? MultiLineInputDialog.this.text
							.getText() : comboText);
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		return composite;
	}

	/**
	 * Returns the ok button.
	 * 
	 * @return the ok button
	 */
	protected Button getOkButton() {
		return okButton;
	}

	/**
	 * Returns the text area.
	 * 
	 * @return the text area
	 */
	protected Text getText() {
		return text;
	}

	/**
	 * Returns the string typed into this input dialog.
	 * 
	 * @return the input string
	 */
	public String getValue() {
		return value;
	}

	/**
	 * Returns the style bits that should be used for the input text field. Defaults to a single line entry. Subclasses may override.
	 * 
	 * @return the integer style bits that should be used when creating the input text
	 * 
	 * @since 3.4
	 */
	protected int getInputTextStyle() {
		return SWT.SINGLE | SWT.BORDER;
	}

	protected Text createText(Composite composite) {
		Text text = new Text(composite, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.heightHint = 5 * text.getLineHeight();
		data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.ENTRY_FIELD_WIDTH);
		text.setLayoutData(data);
		return text;
	}

	protected List<String> getCommentsList() {
		List<String> commentsList = new ArrayList<String>();

		commentsList.add(MultiLineInputDialog.PREVIOUS_COMMENTS_HEADER);
		this.ignoredStrings.add(MultiLineInputDialog.PREVIOUS_COMMENTS_HEADER);
		if (this.history.getHistory().length > 0) {
			commentsList.addAll(Arrays.asList(this.history.getHistory()));
		} else {
			commentsList.add(MultiLineInputDialog.PREVIOUS_COMMENTS_HINT);
			this.ignoredStrings.add(MultiLineInputDialog.PREVIOUS_COMMENTS_HINT);
		}

		return commentsList;
	}

}
