/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui;

import org.eclipse.core.resources.team.FileModificationValidator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.modelbus.team.eclipse.core.connector.IModelBusCredentialsPrompt;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.extension.options.IOptionProvider;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.utility.ILoggedOperationFactory;
import org.modelbus.team.eclipse.core.utility.StringMatcher;
import org.modelbus.team.eclipse.ui.panel.callback.PromptCredentialsPanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPropsPreferencePage;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * UI Plugin option provider. Implements extension point "coreoptions".
 * 
 * @author Alexander Gurov
 */
public class UIOptionProvider implements IOptionProvider {
	private ModelBusTeamModificationValidator	modificationValidator	= new ModelBusTeamModificationValidator();

	public IModelBusCredentialsPrompt getCredentialsPrompt() {
		return PromptCredentialsPanel.DEFAULT_PROMPT;
	}

	public ILoggedOperationFactory getLoggedOperationFactory() {
		return UIMonitorUtility.DEFAULT_FACTORY;
	}

	public void addProjectSetCapabilityProcessing(CompositeOperation op) {
		System.out.println("CHECKMWA15");

		// op.add(new RefreshRepositoryLocationsOperation(false));
	}

	public boolean isAutomaticProjectShareEnabled() {
		return ModelBusTeamPreferences.getBehaviourBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_NAME);
	}

	public FileModificationValidator getFileModificationValidator() {
		return this.modificationValidator;
	}

	public String getModelBusConnectorId() {
		return ModelBusTeamPreferences.getCoreString(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_NAME);
	}

	public String getDefaultBranchesName() {
		String retVal = ModelBusTeamPreferences.getRepositoryString(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.REPOSITORY_BRANCHES_NAME);
		if (retVal == null || retVal.length() == 0) {
			retVal = ModelBusTeamPreferences.REPOSITORY_BRANCHES_DEFAULT;
		}
		return retVal;
	}

	public String getDefaultTagsName() {
		String retVal = ModelBusTeamPreferences.getRepositoryString(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.REPOSITORY_TAGS_NAME);
		if (retVal == null || retVal.length() == 0) {
			retVal = ModelBusTeamPreferences.REPOSITORY_TAGS_DEFAULT;
		}
		return retVal;
	}

	public String getDefaultTrunkName() {
		String retVal = ModelBusTeamPreferences.getRepositoryString(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.REPOSITORY_HEAD_NAME);
		if (retVal == null || retVal.length() == 0) {
			retVal = ModelBusTeamPreferences.REPOSITORY_HEAD_DEFAULT;
		}
		return retVal;
	}

	public boolean isModelBusCacheEnabled() {
		return ModelBusTeamPreferences.getDecorationBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_NAME);
	}

	public ModelBusProperty[] getAutomaticProperties(String template) {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		Object[] autoProperties = ModelBusTeamPropsPreferencePage.loadAutoProperties(ModelBusTeamPreferences.getAutoPropertiesList(store,
				ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_NAME));
		for (int i = 0; i < autoProperties.length; i++) {
			ModelBusTeamPropsPreferencePage.AutoProperty autoProperty = (ModelBusTeamPropsPreferencePage.AutoProperty) autoProperties[i];
			if (!autoProperty.enabled) {
				continue;
			}

			StringMatcher matcher = new StringMatcher(autoProperty.fileName);
			if (matcher.match(template)) {
				if (autoProperty.properties.length() == 0) {
					return new ModelBusProperty[0];
				}
				String[] props = autoProperty.properties.split(";");
				ModelBusProperty[] propertyData = new ModelBusProperty[props.length];
				for (int j = 0; j < props.length; j++) {
					String[] propsNameValue = props[j].split("=");
					propertyData[j] = new ModelBusProperty(propsNameValue[0], propsNameValue.length == 1 ? "" : propsNameValue[1]);
				}
				return propertyData;
			}
		}
		return new ModelBusProperty[0];
	}

	public String getResource(String key) {
		return ModelBusTeamUIPlugin.instance().getResource(key);
	}

}
