/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.internal.CompareEditor;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.ui.action.AbstractWorkingCopyAction;
import org.modelbus.team.eclipse.ui.compare.ModelBusCompareEditorInput;

/**
 * Compare menu "compare with base revision" action implementation
 * 
 * @author Alexander Gurov
 */
public class CompareWithWorkingCopyAction extends AbstractWorkingCopyAction {

	public CompareWithWorkingCopyAction() {
		super();
	}

	public void runImpl(IAction action) {
		IResource resource = this.getSelectedResources()[0];
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);

		CompareConfiguration configuration = new CompareConfiguration();
		configuration.setProperty(CompareEditor.CONFIRM_SAVE_PROPERTY, Boolean.TRUE);
		ModelBusCompareEditorInput compare = new ModelBusCompareEditorInput(configuration, provider, resource, ModelBusRevision.INVALID_REVISION_STRING_2);
		CompareUI.openCompareEditor(compare);
	}

	public boolean isEnabled() {
		if (this.getSelectedResources().length != 1) {
			return false;
		}
		RepositoryProvider provider = RepositoryProvider.getProvider(this.getSelectedResources()[0].getProject(),
				ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider instanceof ModelBusRepositoryProvider) {
			return true;
		}
		return false;
	}

	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

	public static final IStateFilter	COMPARE_FILTER	= new IStateFilter.AbstractStateFilter() {
															protected boolean acceptImpl(ILocalResource local, IResource resource, String state,
																	int mask) {
																return IStateFilter.SF_EXCLUDE_DELETED.accept(resource, state, mask)
																		| (mask & ILocalResource.IS_COPIED) != 0;
															}

															protected boolean allowsRecursionImpl(ILocalResource local, IResource resource,
																	String state, int mask) {
																return IStateFilter.SF_EXCLUDE_DELETED.accept(resource, state, mask);
															}
														};

}
