/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.modelbusstorage.ResourcesParentsProvider;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.JavaHLMergeOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.remote.LocateResourceURLInHistoryOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractNonRecursiveTeamAction;
import org.modelbus.team.eclipse.ui.dialog.AdvancedDialog;
import org.modelbus.team.eclipse.ui.dialog.OperationErrorDialog;
import org.modelbus.team.eclipse.ui.operation.ShowMergeViewOperation;
import org.modelbus.team.eclipse.ui.panel.local.MergePanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;

/**
 * Merge action implementation
 * 
 * @author Alexander Gurov
 */
public class MergeAction extends AbstractNonRecursiveTeamAction {

	public MergeAction() {
		super();
	}

	public void runImpl(IAction action) {
		IResource[] resources = this.getSelectedResources(IStateFilter.SF_EXCLUDE_DELETED);

		if (!OperationErrorDialog.isAcceptableAtOnce(resources, ModelBusTeamUIPlugin.instance().getResource("MergeAction.MergeError"),
				this.getShell())) {
			return;
		}

		IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resources[0]);
		ModelBusRevision revision = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resources[0]).getRevision();

		if (resources.length > 1) {
			revision = ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
			remote = remote.getRoot();
		}

		MergePanel panel = new MergePanel(resources, remote, revision);
		AdvancedDialog dialog = new AdvancedDialog(this.getShell(), panel);
		if (dialog.open() == 0) {
			// 2URL mode requires peg as revision
			LocateResourceURLInHistoryOperation locateFirst = new LocateResourceURLInHistoryOperation(panel.getFirstSelection());
			LocateResourceURLInHistoryOperation locateSecond = null;
			IRepositoryResourceProvider firstSet = locateFirst;
			IRepositoryResourceProvider secondSet = null;
			if (panel.getMode() == MergePanel.MODE_1URL) {
				firstSet = new IRepositoryResourceProvider.DefaultRepositoryResourceProvider(panel.getFirstSelection());
			} else if (panel.getMode() == MergePanel.MODE_2URL) {
				secondSet = locateSecond = new LocateResourceURLInHistoryOperation(panel.getSecondSelection());
			}

			IActionOperation mergeOp = null;
			if (ModelBusTeamPreferences.getMergeBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
					ModelBusTeamPreferences.MERGE_USE_JAVAHL_NAME) /*
																	 * || panel.getMode() == MergePanel. MODE_REINTEGRATE
																	 */) {
				JavaHLMergeOperation mainOp = null;
				if (panel.getMode() == MergePanel.MODE_2URL) {
					mainOp = new JavaHLMergeOperation(resources, firstSet, secondSet, false, panel.getIgnoreAncestry(), panel.getDepth());
				} else if (panel.getMode() == MergePanel.MODE_1URL) {
					mainOp = new JavaHLMergeOperation(resources, firstSet, panel.getSelectedRevisions(), false, panel.getIgnoreAncestry(),
							panel.getDepth());
				} else {
					mainOp = new JavaHLMergeOperation(resources, firstSet, false);
				}
				CompositeOperation op = new CompositeOperation(mainOp.getId());
				SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
				op.add(saveOp);
				op.add(mainOp);
				op.add(new RestoreProjectMetaOperation(saveOp));
				op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(resources)));
				mergeOp = op;
			} else if (panel.getMode() == MergePanel.MODE_2URL) {
				mergeOp = new ShowMergeViewOperation(resources, firstSet, secondSet, panel.getIgnoreAncestry(), panel.getDepth(),
						this.getTargetPart());
			} else if (panel.getMode() == MergePanel.MODE_1URL) {
				mergeOp = new ShowMergeViewOperation(resources, firstSet, panel.getSelectedRevisions(), panel.getIgnoreAncestry(), panel.getDepth(),
						this.getTargetPart());
			} else {
				mergeOp = new ShowMergeViewOperation(resources, firstSet, this.getTargetPart());
			}
			if (panel.getMode() != MergePanel.MODE_1URL) {
				CompositeOperation op = new CompositeOperation(mergeOp.getId());
				op.add(locateFirst);
				if (panel.getMode() == MergePanel.MODE_2URL) {
					op.add(locateSecond);
				}
				op.add(mergeOp);
				this.runScheduled(op);
			} else {
				this.runScheduled(mergeOp);
			}
		}
	}

	public boolean isEnabled() {
		return this.checkForResourcesPresence(IStateFilter.SF_EXCLUDE_DELETED);
	}

	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

}
