/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote;

import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.remote.AbstractCopyMoveResourcesOperation;
import org.modelbus.team.eclipse.core.operation.remote.MoveResourcesOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;

/**
 * Move repository resource action implementation
 * 
 * @author Sergiy Logvin
 */
public class MoveToAction extends AbstractCopyMoveAction {
	public MoveToAction() {
		super("MoveToAction");
	}

	protected AbstractCopyMoveResourcesOperation makeCopyOperation(IRepositoryResource destination, IRepositoryResource[] selected, String message,
			String name) {
		return new MoveResourcesOperation(destination, selected, message, name);
	}

	protected RefreshRemoteResourcesOperation makeRefreshOperation(IRepositoryResource destination, IRepositoryResource[] selected) {
		IRepositoryResource[] toRefresh = new IRepositoryResource[selected.length + 1];
		System.arraycopy(selected, 0, toRefresh, 0, selected.length);
		toRefresh[selected.length] = destination;
		return new RefreshRemoteResourcesOperation(ModelBusUtility.getCommonParents(toRefresh));
	}

	public boolean isEnabled() {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		if (resources.length == 0) {
			return false;
		}
		// disable transfer between different repositories
		// IRepositoryLocation first = resources[0].getRepositoryLocation();
		for (int i = 0; i < resources.length; i++) {
			// IRepositoryLocation location = resources[i].getRepositoryLocation();
			if (
			// first != location ||
			resources[i].getSelectedRevision().getKind() != Kind.HEAD
					|| resources[i] instanceof IRepositoryRoot
					&& (((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_ROOT || ((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_LOCATION_ROOT)) {
				return false;
			}
		}
		return true;
	}

}
