/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote;

import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Options;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.remote.RenameResourceOperation;
import org.modelbus.team.eclipse.core.operation.remote.SetRevisionAuthorNameOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;
import org.modelbus.team.eclipse.ui.panel.remote.RenameResourcePanel;

/**
 * Rename remote resource action implementation
 * 
 * @author Alexander Gurov
 */
public class RenameAction extends AbstractRepositoryTeamAction {

	public RenameAction() {
		super();
	}

	public void runImpl(IAction action) {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		RenameResourcePanel panel = new RenameResourcePanel(resources[0].getName());
		DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);

		if (dialog.open() == 0) {
			RenameResourceOperation mainOp = new RenameResourceOperation(resources[0], panel.getResourceName(), panel.getMessage());

			CompositeOperation op = new CompositeOperation(mainOp.getId());

			op.add(mainOp);
			op.add(new RefreshRemoteResourcesOperation(ModelBusUtility.getCommonParents(resources)));
			op.add(new SetRevisionAuthorNameOperation(mainOp, Options.FORCE));

			this.runScheduled(op);
		}
	}

	public boolean isEnabled() {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		for (int i = 0; i < resources.length; i++) {
			// IRepositoryLocation location = resources[i].getRepositoryLocation();
			if (
			// resources[i].getUrl().equals(location.getRoot().getUrl()) ||
			resources[i].getSelectedRevision().getKind() != Kind.HEAD
					|| resources[i] instanceof IRepositoryRoot
					&& (((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_ROOT || ((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_LOCATION_ROOT)) {
				return false;
			}
		}
		return resources.length == 1;
	}

}
