/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote;

import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorCancelException;
import org.modelbus.team.eclipse.core.operation.ActivityCancelledException;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.operation.UILoggedOperation;
import org.modelbus.team.eclipse.ui.operation.UILoggedOperation.OperationErrorInfo;
import org.modelbus.team.eclipse.ui.panel.reporting.ErrorCancelPanel;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryError;

/**
 * Show repository error action implementation
 * 
 * @author Sergiy Logvin
 */
public class ShowBrowsingErrorAction extends AbstractRepositoryTeamAction {

	public ShowBrowsingErrorAction() {
		super();
	}

	public void runImpl(IAction action) {
		Object selectedElement = this.getSelection().getFirstElement();
		OperationErrorInfo errorInfo = UILoggedOperation.formatMessage(((RepositoryError) selectedElement).getErrorStatus(), true);
		ErrorCancelPanel panel;
		if (errorInfo.exception instanceof ModelBusConnectorCancelException || errorInfo.exception instanceof ActivityCancelledException
				|| errorInfo.exception instanceof OperationCanceledException) {
			panel = new ErrorCancelPanel(ModelBusTeamUIPlugin.instance().getResource("ShowBrowsingErrorAction.Dialog.Title"),
					errorInfo.simpleMessage, errorInfo.advancedMessage, false, null);
		} else {
			panel = new ErrorCancelPanel(ModelBusTeamUIPlugin.instance().getResource("ShowBrowsingErrorAction.Dialog.Title"),
					errorInfo.numberOfErrors, errorInfo.simpleMessage, errorInfo.advancedMessage, false, false, null);
		}
		DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);
		dialog.open();
	}

	public boolean isEnabled() {
		return true;
	}

}
