/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.composite;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.operation.GetRemoteFolderChildrenOperation;
import org.modelbus.team.eclipse.ui.panel.common.RepositoryTreePanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;
import org.modelbus.team.eclipse.ui.utility.UserInputHistory;
import org.modelbus.team.eclipse.ui.verifier.CompositeVerifier;
import org.modelbus.team.eclipse.ui.verifier.IValidationManager;
import org.modelbus.team.eclipse.ui.verifier.NonEmptyFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.ResourceNameVerifier;

/**
 * Composite for branch/tag selection with appropriate revision
 * 
 * @author Alexei Goncharov
 */
public class BranchTagSelectionComposite extends Composite {
	public static final int			BRANCH_OPERATED	= 0;
	public static final int			TAG_OPERATED	= 1;

	protected Combo					urlText;
	protected Button				browse;
	protected UserInputHistory		inputHistory;
	protected RevisionComposite		revisionComposite;
	protected RevisionComposite		secondRevisionComposite;
	protected IValidationManager	validationManager;
	protected IRepositoryResource	baseResource;
	protected boolean				considerStructure;
	protected int					type;
	protected String				url;
	protected CompositeVerifier		verifier;
	protected String				selectionTitle;
	protected String				selectionDescription;
	protected IRepositoryRoot		root;

	protected String				ignored;

	public BranchTagSelectionComposite(Composite parent, int style, IRepositoryResource baseResource, String historyKey,
			IValidationManager validationManager, int type) {
		super(parent, style);
		this.baseResource = baseResource;
		this.inputHistory = new UserInputHistory(historyKey);
		this.validationManager = validationManager;
		this.type = type;
		this.ignored = ModelBusTeamUIPlugin.instance().getResource(
				this.type == BranchTagSelectionComposite.BRANCH_OPERATED ? "Branch.Read.Separator" : "Tag.Read.Separator");
		this.considerStructure =
		// baseResource.getRepositoryLocation().isStructureEnabled() &&
		ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);
		this.root = (this.type == BranchTagSelectionComposite.BRANCH_OPERATED) ? ModelBusUtility.getBranchesLocation(this.baseResource)
				: ModelBusUtility.getTagsLocation(this.baseResource);
		this.createControls(parent);
		// throw new RuntimeException("TODOMWA29");
	}

	protected void createControls(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = layout.marginWidth = 0;
		this.setLayout(layout);

		Label resourceLabel = new Label(this, SWT.NONE);
		resourceLabel.setLayoutData(new GridData());
		if (this.type == BranchTagSelectionComposite.BRANCH_OPERATED) {
			resourceLabel.setText(ModelBusTeamUIPlugin.instance().getResource("Select.Branch.Label"));
		} else {
			resourceLabel.setText(ModelBusTeamUIPlugin.instance().getResource("Select.Tag.Label"));
		}

		Composite select = new Composite(this, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		select.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		select.setLayoutData(data);

		// final IRepositoryLocation location = this.baseResource.getRepositoryLocation();

		this.urlText = new Combo(select, SWT.NULL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = IDialogConstants.ENTRY_FIELD_WIDTH;
		this.urlText.setLayoutData(data);
		this.urlText.setVisibleItemCount(this.inputHistory.getDepth() * 2);
		this.urlText.setItems(this.inputHistory.getHistory());

		IRepositoryResource[] children = new IRepositoryResource[0];
		if (this.considerStructure) {
			GetRemoteFolderChildrenOperation op = new GetRemoteFolderChildrenOperation(this.root, true);
			UIMonitorUtility.doTaskNowDefault(op, false);
			if (op.getChildren() != null) {
				children = op.getChildren();
			}
		}
		if (children.length > 0) {
			this.urlText.add(this.ignored);
		}
		for (int i = 0; i < children.length; i++) {
			this.urlText.add(children[i].getName());
		}
		this.urlText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				BranchTagSelectionComposite.this.url = ((Combo) e.widget).getText();
				BranchTagSelectionComposite.this.revisionComposite.setSelectedResource(BranchTagSelectionComposite.this.getSelectedResource());
			}
		});
		this.urlText.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int idx = BranchTagSelectionComposite.this.urlText.getSelectionIndex();
				if (idx != -1) {
					String comboText = BranchTagSelectionComposite.this.urlText.getItem(idx);
					if (comboText.equals(BranchTagSelectionComposite.this.ignored)) {
						BranchTagSelectionComposite.this.urlText.setText("");
					}
				}
			}
		});
		this.validationManager.attachTo(this.urlText, new ResourceNameVerifier(resourceLabel.getText(), true));
		this.validationManager.attachTo(this.urlText, new NonEmptyFieldVerifier(resourceLabel.getText()));

		this.browse = new Button(select, SWT.PUSH);
		this.browse.setText(ModelBusTeamUIPlugin.instance().getResource("Button.Browse"));
		data = new GridData();
		data.widthHint = DefaultDialog.computeButtonWidth(this.browse);
		this.browse.setLayoutData(data);
		this.browse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				RepositoryTreePanel panel;
				String part = (BranchTagSelectionComposite.this.type == BranchTagSelectionComposite.BRANCH_OPERATED) ? "Branch" : "Tag";
				if (BranchTagSelectionComposite.this.considerStructure) {
					panel = new RepositoryTreePanel(ModelBusTeamUIPlugin.instance().getResource("Select." + part + ".Title"), ModelBusTeamUIPlugin
							.instance().getResource("RepositoryBrowsingPanel.Description"), ModelBusTeamUIPlugin.instance().getResource(
							"RepositoryBrowsingPanel.Message"), null, true, BranchTagSelectionComposite.this.root);
				} else {
					panel = new RepositoryTreePanel(ModelBusTeamUIPlugin.instance().getResource("Select." + part + ".Title"), ModelBusTeamUIPlugin
							.instance().getResource("RepositoryBrowsingPanel.Description"), ModelBusTeamUIPlugin.instance().getResource(
							"RepositoryBrowsingPanel.Message"), null, true);
				}
				DefaultDialog browser = new DefaultDialog(BranchTagSelectionComposite.this.getShell(), panel);
				if (browser.open() == 0) {
					IRepositoryResource selectedResource = panel.getSelectedResource();
					boolean samePeg = selectedResource.getPegRevision().equals(BranchTagSelectionComposite.this.baseResource.getPegRevision());
					if (BranchTagSelectionComposite.this.considerStructure) {
						String toTrim = BranchTagSelectionComposite.this.root.getUrl();
						BranchTagSelectionComposite.this.urlText.setText(samePeg ? selectedResource.getUrl().substring(toTrim.length() + 1)
								: ModelBusUtility.getEntryReference(selectedResource).toString());
					} else {
						BranchTagSelectionComposite.this.urlText.setText(samePeg ? selectedResource.getUrl() : ModelBusUtility.getEntryReference(
								selectedResource).toString());
					}
					BranchTagSelectionComposite.this.revisionComposite.setSelectedResource(selectedResource);
					BranchTagSelectionComposite.this.validationManager.validateContent();
				}
			}
		});

		Composite revisions = new Composite(this, SWT.NONE);
		layout = new GridLayout();
		layout.marginHeight = layout.marginWidth = 0;
		layout.numColumns = 2;
		revisions.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		revisions.setLayoutData(data);
		this.revisionComposite = new RevisionComposite(revisions, this.validationManager, true, new String[] {
				ModelBusTeamUIPlugin.instance().getResource("RevisionComposite.Revision"),
				ModelBusTeamUIPlugin.instance().getResource("RepositoryResourceSelectionComposite.HeadRevision") }, ModelBusRevision.HEAD, false) {
			public void additionalValidation() {
				BranchTagSelectionComposite.this.validationManager.validateContent();
			}
		};
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		this.revisionComposite.setLayoutData(data);
		this.revisionComposite.setSelectedResource(this.baseResource);
	}

	public void saveChanges() {
		this.inputHistory.addLine(this.url);
	}

	public void setCurrentRevision(ModelBusRevision currentRevision) {
		this.revisionComposite.setCurrentRevision(currentRevision);
	}

	public IRepositoryResource getSelectedResource() {
		String url = this.considerStructure ? (this.root.getUrl() + "/" + this.url) : this.url;
		IRepositoryResource resource = this.getDestination(ModelBusUtility.asEntryReference(url), false);
		resource.setSelectedRevision(this.revisionComposite.getSelectedRevision());
		return resource;
	}

	protected IRepositoryResource getDestination(ModelBusEntryReference ref, boolean allowsNull) {
		if (ref == null) {
			return ModelBusUtility.copyOf(this.baseResource);
		}
		String url = ModelBusUtility.normalizeURL(ref.path);
		try {
			IRepositoryResource resource = this.baseResource instanceof IRepositoryContainer ? (IRepositoryResource) ModelBusConnector
					.asRepositoryContainer(url, false) : this.baseResource.asRepositoryFile(url, false);
			if (ref.pegRevision != null) {
				resource.setPegRevision(ref.pegRevision);
			}
			return resource;
		} catch (IllegalArgumentException ex) {
			return allowsNull ? null : ModelBusUtility.copyOf(this.baseResource);
		}
	}

}
