/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.composite;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.internal.ide.misc.ContainerSelectionGroup;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.verifier.AbstractFormattedVerifier;

/**
 * This control works with Eclispe IDE 3.0 - 3.2
 * 
 * @author Alexander Gurov
 */
public class ModelBusContainerSelectionGroup extends ContainerSelectionGroup {
	public ModelBusContainerSelectionGroup(Composite parent, Listener listener) {
		super(parent, listener, false, "", false);
	}

	public void createContents(String message, int heightHint, int widthHint) {
		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		setLayout(layout);
		setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		createTreeViewer(heightHint);
		Dialog.applyDialogFont(this);
	}

	public void createContents(String message, int heightHint) {
		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		setLayout(layout);
		setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		createTreeViewer(heightHint);
		Dialog.applyDialogFont(this);
	}

	public static class ModelBusContainerSelectionVerifier extends AbstractFormattedVerifier {
		protected static String	ERROR_MESSAGE;
		protected static String	DESTINATION_IS_DETACHED_FROM_ModelBus;
		protected static String	DESTINATION_DIRECTORY_IS_DELETED;
		protected static String	DESTINATION_DIRECTORY_IS_OBSTRUCTED;

		public ModelBusContainerSelectionVerifier() {
			super("");

			ModelBusContainerSelectionVerifier.ERROR_MESSAGE = ModelBusTeamUIPlugin.instance().getResource(
					"ModelBusContainerSelectionGroup.Verifier.NotSelected");
			ModelBusContainerSelectionVerifier.DESTINATION_IS_DETACHED_FROM_ModelBus = ModelBusTeamUIPlugin.instance().getResource(
					"ModelBusContainerSelectionGroup.Verifier.NonModelBus");
			ModelBusContainerSelectionVerifier.DESTINATION_DIRECTORY_IS_DELETED = ModelBusTeamUIPlugin.instance().getResource(
					"ModelBusContainerSelectionGroup.Verifier.Deleted");
			ModelBusContainerSelectionVerifier.DESTINATION_DIRECTORY_IS_OBSTRUCTED = ModelBusTeamUIPlugin.instance().getResource(
					"ModelBusContainerSelectionGroup.Verifier.Obstructed");
		}

		protected String getErrorMessageImpl(Control input) {
			ModelBusContainerSelectionGroup control = (ModelBusContainerSelectionGroup) input;
			if (control.getContainerFullPath() == null) {
				return ModelBusContainerSelectionVerifier.ERROR_MESSAGE;
			}
			IResource destinationRoot = ResourcesPlugin.getWorkspace().getRoot().findMember(control.getContainerFullPath());
			ILocalResource localDest = ModelBusRemoteStorage.instance().asLocalResource(destinationRoot);
			if (IStateFilter.SF_INTERNAL_INVALID.accept(localDest)) {
				return this.isNonModelBusCheckDisabled() ? ModelBusContainerSelectionVerifier.DESTINATION_IS_DETACHED_FROM_ModelBus : null;
			}
			if (IStateFilter.SF_DELETED.accept(localDest)) {
				return ModelBusContainerSelectionVerifier.DESTINATION_DIRECTORY_IS_DELETED;
			}
			if (IStateFilter.SF_OBSTRUCTED.accept(localDest)) {
				return ModelBusContainerSelectionVerifier.DESTINATION_DIRECTORY_IS_OBSTRUCTED;
			}
			return null;
		}

		protected String getWarningMessageImpl(Control input) {
			return null;
		}

		protected boolean isNonModelBusCheckDisabled() {
			return true;
		}
	}

	public static class ModelBusContainerCheckOutSelectionVerifier extends ModelBusContainerSelectionVerifier {

		protected static String	WARNING_MESSAGE;

		public ModelBusContainerCheckOutSelectionVerifier() {
			super();
			ModelBusContainerCheckOutSelectionVerifier.WARNING_MESSAGE = ModelBusTeamUIPlugin.instance().getResource(
					"ModelBusContainerSelectionGroup.Verifier.NonModelBusWarning");
		}

		protected String getWarningMessageImpl(Control input) {
			ModelBusContainerSelectionGroup control = (ModelBusContainerSelectionGroup) input;
			IResource destinationRoot = ResourcesPlugin.getWorkspace().getRoot().findMember(control.getContainerFullPath());
			if (!FileUtility.isConnected(destinationRoot)) {
				return ModelBusContainerCheckOutSelectionVerifier.WARNING_MESSAGE;
			}
			return null;
		}

		protected boolean isNonModelBusCheckDisabled() {
			return false;
		}
	}

}
