/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexey Mikoyan - Initial implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.ui.console;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.console.IPatternMatchListenerDelegate;
import org.eclipse.ui.console.PatternMatchEvent;
import org.eclipse.ui.console.TextConsole;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Listen to ModelBus console. Finds lines that contains local resources paths. Adds hyperlinks to local resources.
 * 
 * @author Alexey Mikoyan
 * 
 */

public class LocalPathMatcher implements IPatternMatchListenerDelegate, IPropertyChangeListener {

	protected Pattern		pattern;

	protected TextConsole	console;
	protected boolean		enabled;

	public void createPattern() {
		String regExp = "(?:\\s|\")(?:[A-Z]\\:)?(?:[\\\\/][^\\\\/\\:\\?\\*\r\n\"]+)+";
		this.pattern = Pattern.compile(regExp);
	}

	public void connect(TextConsole console) {
		this.console = console;
		this.createPattern();
		this.loadPreferences();
		ModelBusTeamUIPlugin.instance().getPreferenceStore().addPropertyChangeListener(this);
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().startsWith(ModelBusTeamPreferences.CONSOLE_BASE)) {
			this.loadPreferences();
		}
	}

	public void disconnect() {
		this.console = null;
		ModelBusTeamUIPlugin.instance().getPreferenceStore().removePropertyChangeListener(this);
	}

	public void matchFound(PatternMatchEvent event) {
		if (this.console == null || !this.enabled) {
			return;
		}
		UIMonitorUtility.doTaskBusyDefault(new AddConsoleHyperlinkOperation(event));
	}

	protected void loadPreferences() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		this.enabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_NAME);
	}

	protected class AddConsoleHyperlinkOperation extends AbstractActionOperation {
		protected PatternMatchEvent	event;

		public AddConsoleHyperlinkOperation(PatternMatchEvent event) {
			super("Operation.AddConsoleHyperlink");
			this.event = event;
		}

		protected void runImpl(IProgressMonitor monitor) throws Exception {
			int offset = this.event.getOffset();
			int length = this.event.getLength();
			String path = LocalPathMatcher.this.console.getDocument().get(offset, length);
			if (path == null) {
				return;
			}

			Matcher matcher = LocalPathMatcher.this.pattern.matcher(path);
			if (matcher.find(0)) {
				String link = matcher.group(matcher.groupCount()).trim();
				LocalPathMatcher.this.console.addHyperlink(new LocalFileHyperlink(link), offset + matcher.start(matcher.groupCount()) + 1, matcher
						.group(matcher.groupCount()).length() - 1);
			}
		}

	}

}
