/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.extension.impl.synchronize;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.synchronize.AbstractSynchronizeActionGroup;

/**
 * Synchronize options action set
 * 
 * @author Alexander Gurov
 */
public class OptionsActionGroup extends AbstractSynchronizeActionGroup {
	public static final String	GROUP_SYNCH_OPTIONS	= "synchronizeViewOptions";

	protected IAction			contiguousOptionAction;

	public void configureMenuGroups(ISynchronizePageConfiguration configuration) {
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_VIEW_MENU, OptionsActionGroup.GROUP_SYNCH_OPTIONS);
	}

	protected void configureActions(ISynchronizePageConfiguration configuration) {
		this.contiguousOptionAction = new Action(ModelBusTeamUIPlugin.instance().getResource("OptionsActionGroup.ShowInfoContiguous"),
				IAction.AS_CHECK_BOX) {
			public void run() {
				IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
				boolean fastReportMode = ModelBusTeamPreferences.getSynchronizeBoolean(store,
						ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME);
				ModelBusTeamPreferences
						.setSynchronizeBoolean(store, ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME, !fastReportMode);
				ModelBusTeamUIPlugin.instance().savePluginPreferences();
			}
		};
		this.refreshOptionButtons();
		this.appendToGroup(ISynchronizePageConfiguration.P_VIEW_MENU, OptionsActionGroup.GROUP_SYNCH_OPTIONS, this.contiguousOptionAction);
	}

	protected void refreshOptionButtons() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		this.contiguousOptionAction.setChecked(ModelBusTeamPreferences.getSynchronizeBoolean(store,
				ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME));
	}
}
