/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *    Igor Burilo - Bug 211415: Export History log
 *    Igor Burilo - Bug 245509: Improve extract log
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareUI;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.history.provider.FileRevision;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.internal.ui.actions.CompareRevisionAction;
import org.eclipse.team.ui.history.HistoryPage;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.editors.text.EditorsUI;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusDiffStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Kind;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusLogPath;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.history.ModelBusRemoteResourceRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.FiniExtractLogOperation;
import org.modelbus.team.eclipse.core.operation.local.GetRemoteContentsOperation;
import org.modelbus.team.eclipse.core.operation.local.InitExtractLogOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.UpdateOperation;
import org.modelbus.team.eclipse.core.operation.remote.ExportOperation;
import org.modelbus.team.eclipse.core.operation.remote.ExtractToOperationRemote;
import org.modelbus.team.eclipse.core.operation.remote.GetRemotePropertiesOperation;
import org.modelbus.team.eclipse.core.operation.remote.LocateResourceURLInHistoryOperation;
import org.modelbus.team.eclipse.core.operation.remote.PreparedBranchTagOperation;
import org.modelbus.team.eclipse.core.operation.remote.management.AddRevisionLinkOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceWithStatusProvider;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.action.remote.BranchTagAction;
import org.modelbus.team.eclipse.ui.action.remote.CreatePatchAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileWithAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileWithExternalAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileWithInplaceAction;
import org.modelbus.team.eclipse.ui.compare.TwoWayPropertyCompareInput;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.dialog.ReplaceWarningDialog;
import org.modelbus.team.eclipse.ui.history.data.AffectedPathsNode;
import org.modelbus.team.eclipse.ui.history.data.ModelBusChangedPathData;
import org.modelbus.team.eclipse.ui.history.data.ModelBusLocalFileRevision;
import org.modelbus.team.eclipse.ui.history.model.ILogNode;
import org.modelbus.team.eclipse.ui.history.model.ModelBusLogNode;
import org.modelbus.team.eclipse.ui.operation.CompareRepositoryResourcesOperation;
import org.modelbus.team.eclipse.ui.operation.CompareResourcesOperation;
import org.modelbus.team.eclipse.ui.operation.OpenRemoteFileOperation;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;
import org.modelbus.team.eclipse.ui.operation.RemoteShowAnnotationOperation;
import org.modelbus.team.eclipse.ui.operation.ShowHistoryViewOperation;
import org.modelbus.team.eclipse.ui.operation.ShowPropertiesOperation;
import org.modelbus.team.eclipse.ui.operation.ShowRevisionPropertiesOperation;
import org.modelbus.team.eclipse.ui.operation.UILoggedOperation;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFolder;
import org.modelbus.team.eclipse.ui.utility.LockProposeUtility;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;
import org.modelbus.team.eclipse.ui.wizard.CreatePatchWizard;

/**
 * Provides actions for affected paths window
 * 
 * @author Alexander Gurov
 */
public class HistoryActionManager {
	public interface IControlActionManager {
		public void installKeyBindings(StructuredViewer viewer);

		public void installDefaultAction(StructuredViewer viewer);

		public void installMenuActions(StructuredViewer viewer, IWorkbenchPartSite site);
	}

	public static class HistoryAction extends Action {
		protected HistoryAction(String text) {
			super(ModelBusTeamUIPlugin.instance().getResource(text));
		}

		protected HistoryAction(String text, Object[] args) {
			this(text, args, null);
		}

		protected HistoryAction(String text, Object[] args, String imageDescriptor) {
			super(ModelBusTeamUIPlugin.instance().getResource(text, args));
			this.setHoverImageDescriptor(imageDescriptor == null ? null : ModelBusTeamUIPlugin.instance().getImageDescriptor(imageDescriptor));
		}

		protected HistoryAction(String text, String imageDescriptor) {
			this(text);
			this.setHoverImageDescriptor(imageDescriptor == null ? null : ModelBusTeamUIPlugin.instance().getImageDescriptor(imageDescriptor));
		}

		protected HistoryAction(String text, String imageDescriptor, int style) {
			super(ModelBusTeamUIPlugin.instance().getResource(text), style);
			this.setHoverImageDescriptor(imageDescriptor == null ? null : ModelBusTeamUIPlugin.instance().getImageDescriptor(imageDescriptor));
		}

	}

	protected IModelBusHistoryView		view;
	protected ModelBusRevision		selectedRevision;

	public final IControlActionManager	affectedTableManager;
	public final IControlActionManager	affectedTreeManager;
	public final IControlActionManager	logMessagesManager;

	public HistoryActionManager(IModelBusHistoryView info) {
		this.view = info;
		this.affectedTableManager = new AffectedTableActionManager();
		this.affectedTreeManager = new AffectedTreeActionManager();
		this.logMessagesManager = new LogMessagesActionManager();
	}

	public ModelBusRevision getSelectedRevision() {
		return this.selectedRevision;
	}

	public void setSelectedRevision(ModelBusRevision selectedRevision) {
		this.selectedRevision = selectedRevision;
	}

	protected class LogMessagesActionManager implements IControlActionManager {
		public void installKeyBindings(final StructuredViewer viewer) {
			viewer.getControl().addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent event) {
					if (event.keyCode == SWT.F5) {
						HistoryActionManager.this.view.refresh(IModelBusHistoryView.REFRESH_ALL);
					}
					if (event.stateMask == SWT.CTRL && (event.keyCode == 'c' || event.keyCode == 'C')) {
						ILogNode[] selection = LogMessagesActionManager.this.getSelection(viewer);
						if (selection.length > 0) {
							HistoryActionManager.this.handleCopy(selection);
						}
					}
				}
			});
		}

		public void installDefaultAction(final StructuredViewer viewer) {
			viewer.addDoubleClickListener(new IDoubleClickListener() {
				public void doubleClick(DoubleClickEvent e) {
					ISelection selection = e.getSelection();
					if (selection instanceof IStructuredSelection) {
						IStructuredSelection structured = (IStructuredSelection) selection;
						if (structured.size() == 1) {
							HistoryActionManager.this.handleDoubleClick((TreeViewer) viewer, (ILogNode) structured.getFirstElement(), true);
						}
					}
				}
			});
		}

		public void installMenuActions(final StructuredViewer viewer, IWorkbenchPartSite site) {
			MenuManager menuMgr = new MenuManager();
			Menu menu = menuMgr.createContextMenu(viewer.getControl());
			menuMgr.addMenuListener(new IMenuListener() {
				public void menuAboutToShow(IMenuManager manager) {
					manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

					HistoryAction refreshAction = new HistoryAction("HistoryView.Refresh", "icons/common/refresh.gif") {
						public void run() {
							HistoryActionManager.this.view.refresh(IModelBusHistoryView.REFRESH_ALL);
						}
					};

					ILogNode[] selection = LogMessagesActionManager.this.getSelection(viewer);
					if (selection.length == 0 || selection[0].getType() == ILogNode.TYPE_NONE) {
						if (!HistoryActionManager.this.view.isPending() && HistoryActionManager.this.view.isFilterEnabled()
								&& HistoryActionManager.this.view.getMode() != ModelBusHistoryPage.MODE_LOCAL) {
							LogMessagesActionManager.this.addFilterPart(viewer, manager);
						}
						manager.add(new Separator());
						manager.add(refreshAction);
						return;
					}

					boolean onlyLogEntries = true;
					boolean onlyLocalEntries = true;
					boolean containsCategory = false;
					boolean containsMergeHistory = false;
					for (ILogNode node : selection) {
						int type = node.getType();
						if (type != ILogNode.TYPE_ModelBus) {
							onlyLogEntries = false;
						} else if (node.getParent() instanceof ModelBusLogNode) {
							containsMergeHistory = true;
						}
						if (type != ILogNode.TYPE_LOCAL) {
							onlyLocalEntries = false;
						}
						if (type == ILogNode.TYPE_CATEGORY) {
							onlyLogEntries = false;
							onlyLocalEntries = false;
							containsCategory = true;
						}
					}

					if (onlyLogEntries) {
						LogMessagesActionManager.this.addRemotePart(viewer, manager, selection, containsMergeHistory);
						LogMessagesActionManager.this.addFilterPart(viewer, manager);
						manager.add(new Separator());
					}
					if (onlyLocalEntries) {
						LogMessagesActionManager.this.addLocalPart(viewer, manager, selection);

						manager.add(new Separator());
					}
					if (!onlyLogEntries && !onlyLocalEntries && !containsCategory) {
						LogMessagesActionManager.this.addLocalOrRemotePart(viewer, manager, selection, containsMergeHistory);

						manager.add(new Separator());
					}
					if (!onlyLogEntries) {
						LogMessagesActionManager.this.addCommonPart(viewer, manager, selection, containsMergeHistory);

						manager.add(new Separator());
					}

					manager.add(refreshAction);
				}
			});
			menuMgr.setRemoveAllWhenShown(true);
			viewer.getControl().setMenu(menu);
			site.registerContextMenu(menuMgr, viewer);
		}

		protected void addRemotePart(final StructuredViewer viewer, IMenuManager manager, final ILogNode[] selection, boolean containsMergeHistory) {
			Action tAction = null;
			ModelBusLogEntry[] entries = HistoryActionManager.this.view.getFullRemoteHistory();
			boolean existsInPrevious = entries[entries.length - 1] != selection[0].getEntity()
					|| !HistoryActionManager.this.view.isAllRemoteHistoryFetched();
			final ModelBusLogEntry current = (ModelBusLogEntry) selection[0].getEntity();
			String revision = HistoryActionManager.this.view.getResource() != null ? String.valueOf(current.revision) : ModelBusTeamUIPlugin
					.instance().getResource("HistoryView.HEAD");
			if (!containsMergeHistory) {
				if (HistoryActionManager.this.view.getRepositoryResource() instanceof IRepositoryFile) {
					String name = HistoryActionManager.this.view.getRepositoryResource().getName();
					manager.add(tAction = new HistoryAction("HistoryView.Open") {
						public void run() {
							HistoryActionManager.this.handleDoubleClick((TreeViewer) viewer, selection[0], false);
						}
					});
					tAction.setEnabled(selection.length == 1);
					tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getWorkbench().getEditorRegistry().getImageDescriptor(name));

					// FIXME: "Open with" submenu shouldn't be hardcoded after reworking of
					// the HistoryView. Should be made like the RepositoriesView menu.
					MenuManager sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("HistoryView.OpenWith"), "historyOpenWithMenu");
					sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

					sub.add(new Separator("nonDefaultTextEditors"));
					IEditorDescriptor[] editors = ModelBusTeamUIPlugin.instance().getWorkbench().getEditorRegistry().getEditors(name);
					for (int i = 0; i < editors.length; i++) {
						if (!editors[i].getId().equals(EditorsUI.DEFAULT_TEXT_EDITOR_ID)) {
							HistoryActionManager.this.addMenuItem(viewer, sub, editors[i].getLabel(), new OpenFileWithAction(editors[i].getId(),
									false));
						}
					}

					sub.add(new Separator("variousEditors"));
					HistoryActionManager.this.addMenuItem(viewer, sub, ModelBusTeamUIPlugin.instance().getResource("HistoryView.TextEditor"),
							new OpenFileWithAction());
					HistoryActionManager.this.addMenuItem(viewer, sub, ModelBusTeamUIPlugin.instance().getResource("HistoryView.SystemEditor"),
							new OpenFileWithExternalAction());
					HistoryActionManager.this.addMenuItem(viewer, sub, ModelBusTeamUIPlugin.instance().getResource("HistoryView.InplaceEditor"),
							new OpenFileWithInplaceAction());
					HistoryActionManager.this.addMenuItem(viewer, sub, ModelBusTeamUIPlugin.instance().getResource("HistoryView.DefaultEditor"),
							new OpenFileAction());

					manager.add(sub);

					manager.add(new Separator());
				}

				boolean isCompareAllowed =
				// CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion() ==
				// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x
				true // CHECKMWA
				|| HistoryActionManager.this.view.getRepositoryResource() instanceof IRepositoryFile;

				manager.add(tAction = new HistoryAction("HistoryView.CompareEachOther") {
					public void run() {
						ModelBusLogEntry msg0 = (ModelBusLogEntry) selection[0].getEntity();
						ModelBusLogEntry msg1 = (ModelBusLogEntry) selection[1].getEntity();
						IRepositoryResource next = HistoryActionManager.this.getResourceForSelectedRevision(msg0);
						IRepositoryResource prev = HistoryActionManager.this.getResourceForSelectedRevision(msg1);
						if (msg0.modelBusRevision.isLowerThan(msg1.modelBusRevision)) {
							IRepositoryResource tmp = prev;
							prev = next;
							next = tmp;
						}
						CompareRepositoryResourcesOperation op = new CompareRepositoryResourcesOperation(prev, next);
						op.setForceId(HistoryActionManager.this.getCompareForceId());
						UIMonitorUtility.doTaskScheduledActive(op);
					}
				});
				tAction.setEnabled(selection.length == 2 && isCompareAllowed);

				manager.add(tAction = new HistoryAction("HistoryView.CompareCurrentWith", new String[] { revision }) {
					public void run() {
						HistoryActionManager.this.compareWithCurrent(current);
					}
				});
				tAction.setEnabled(selection.length == 1 && isCompareAllowed);

				manager.add(tAction = new HistoryAction("HistoryView.CompareWithPrevious") {
					public void run() {
						HistoryActionManager.this.compareWithPreviousRevision(null, new IRepositoryResourceProvider() {
							public IRepositoryResource[] getRepositoryResources() {
								ModelBusLogEntry current = (ModelBusLogEntry) selection[0].getEntity();
								return new IRepositoryResource[] { HistoryActionManager.this.getResourceForSelectedRevision(current) };
							}
						});
					}
				});
				tAction.setEnabled(selection.length == 1 && isCompareAllowed && existsInPrevious);

				manager.add(new Separator());

				manager.add(tAction = new HistoryAction("ShowPropertiesAction.label", "icons/views/propertiesedit.gif") {
					public void run() {
						ModelBusLogEntry current = (ModelBusLogEntry) selection[0].getEntity();
						IRepositoryResource resource = HistoryActionManager.this.getResourceForSelectedRevision(current);
						IResourcePropertyProvider provider = new GetRemotePropertiesOperation(resource);
						ShowPropertiesOperation op = new ShowPropertiesOperation(
								PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), resource, provider);
						UIMonitorUtility.doTaskScheduledActive(op);
					}
				});
				tAction.setEnabled(selection.length == 1);

				manager.add(tAction = new HistoryAction("SynchronizeActionGroup.CompareProperties") {
					public void run() {
						ModelBusLogEntry first = (ModelBusLogEntry) selection[0].getEntity();
						ModelBusLogEntry second = (ModelBusLogEntry) selection[1].getEntity();
						if (first.modelBusRevision.isLowerThan(second.modelBusRevision)) {
							ModelBusLogEntry tmp = second;
							second = first;
							first = tmp;
						}
						IRepositoryResource remote = HistoryActionManager.this.view.getRepositoryResource();
						if (remote == null) {
							remote = ModelBusRemoteStorage.instance().asRepositoryResource(HistoryActionManager.this.view.getResource());
						}
						TwoWayPropertyCompareInput input = new TwoWayPropertyCompareInput(new CompareConfiguration(),
								new ModelBusEntryRevisionReference(remote.getUrl(), HistoryActionManager.this.view
										.getCurrentRevision(), ModelBusRevision.fromString(first.revision)), new ModelBusEntryRevisionReference(
										remote.getUrl(), HistoryActionManager.this.view.getCurrentRevision(),
										ModelBusRevision.fromString(second.revision)));
						CompareUI.openCompareEditor(input);
					}
				});
				tAction.setEnabled(selection.length == 2);
			}

			manager.add(tAction = new HistoryAction("ShowRevisionProperties.label") {
				public void run() {
					ModelBusRevision selectedRevision = HistoryActionManager.this.getSelectedRevision();
					ShowRevisionPropertiesOperation op = new ShowRevisionPropertiesOperation(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
							.getActivePage(), selectedRevision);
					UIMonitorUtility.doTaskScheduledActive(op);
				}
			});
			tAction.setEnabled(selection.length == 1);

			if (!containsMergeHistory) {
				if (HistoryActionManager.this.view.getRepositoryResource() instanceof IRepositoryFile) {
					manager.add(tAction = new HistoryAction("ShowAnnotationCommand.label") {
						public void run() {
							IRepositoryResource remote = HistoryActionManager.this.getResourceForSelectedRevision((ModelBusLogEntry) selection[0]
									.getEntity());
							UIMonitorUtility.doTaskScheduledDefault(new RemoteShowAnnotationOperation(remote));
						}
					});
					tAction.setEnabled(selection.length == 1);
				}

				manager.add(new Separator());

				manager.add(tAction = new HistoryAction("ExportCommand.label", "icons/common/export.gif") {
					public void run() {
						HistoryActionManager.this.doExport(selection[0]);
					}
				});
				tAction.setEnabled(selection.length == 1);
				manager.add(tAction = new HistoryAction("CreatePatchCommand.label") {
					public void run() {
						HistoryActionManager.this.createPatch(selection);
					}
				});
				tAction.setEnabled(selection.length == 1 && existsInPrevious || selection.length == 2);
				if (HistoryActionManager.this.view.getResource() != null) {
					manager.add(tAction = new HistoryAction("HistoryView.GetContents") {
						public void run() {
							if (HistoryActionManager.this.confirmReplacement()) {
								HistoryActionManager.this.getRevisionContents((ModelBusLogEntry) selection[0].getEntity());
							}
						}
					});
					tAction.setEnabled(selection.length == 1);
				}
				if (HistoryActionManager.this.view.getResource() != null) {
					manager.add(tAction = new HistoryAction("HistoryView.UpdateTo") {
						public void run() {
							if (HistoryActionManager.this.confirmReplacement()) {
								HistoryActionManager.this.updateTo((ModelBusLogEntry) selection[0].getEntity());
							}
						}
					});
					tAction.setEnabled(selection.length == 1);
				}

				manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("ExtractToAction.Label")) {
					public void run() {
						HistoryActionManager.this.runExtractTo(selection);
					}
				});
				tAction.setEnabled((selection.length == 1 && existsInPrevious) || selection.length == 2);

				manager.add(new Separator());

				String branchFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.BranchFromRevision");
				String tagFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.TagFromRevision");
				if (selection.length == 1) {
					revision = String.valueOf(((ModelBusLogEntry) selection[0].getEntity()).revision);
					branchFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.BranchFrom", new String[] { revision });
					tagFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.TagFrom", new String[] { revision });
				}
				manager.add(tAction = new HistoryAction(branchFrom, "icons/common/actions/branch.gif") {
					public void run() {
						PreparedBranchTagOperation op = BranchTagAction.getBranchTagOperation(new IRepositoryResource[] { HistoryActionManager.this
								.getResourceForSelectedRevision((ModelBusLogEntry) selection[0].getEntity()) }, UIMonitorUtility.getShell(),
								BranchTagAction.BRANCH_ACTION);
						if (op != null) {
							UIMonitorUtility.doTaskScheduledActive(op);
						}
					}
				});
				tAction.setEnabled(selection.length == 1);
				manager.add(tAction = new HistoryAction(tagFrom, "icons/common/actions/tag.gif") {
					public void run() {
						PreparedBranchTagOperation op = BranchTagAction.getBranchTagOperation(new IRepositoryResource[] { HistoryActionManager.this
								.getResourceForSelectedRevision((ModelBusLogEntry) selection[0].getEntity()) }, UIMonitorUtility.getShell(),
								BranchTagAction.TAG_ACTION);
						if (op != null) {
							UIMonitorUtility.doTaskScheduledActive(op);
						}
					}
				});
				tAction.setEnabled(selection.length == 1);
				manager.add(tAction = new HistoryAction("AddRevisionLinkAction.label") {
					public void run() {
						HistoryActionManager.this.addRevisionLinks(selection);
					}
				});
				tAction.setEnabled(selection.length > 0);
			}

			manager.add(new Separator());

			manager.add(tAction = new HistoryAction("HistoryView.CopyHistory", "icons/common/copy.gif") {
				public void run() {
					HistoryActionManager.this.handleCopy(selection);
				}
			});
			tAction.setEnabled(selection.length > 0);

		}

		protected void addFilterPart(final StructuredViewer viewer, IMenuManager manager) {
			manager.add(new Separator());
			Action tAction = null;
			manager.add(tAction = new HistoryAction("HistoryView.QuickFilter", "icons/views/history/filter.gif") {
				public void run() {
					HistoryActionManager.this.view.setFilter();
				}
			});
			manager.add(tAction = new HistoryAction("HistoryView.ClearFilter", "icons/views/history/clear_filter.gif") {
				public void run() {
					HistoryActionManager.this.view.clearFilter();
				}
			});
			tAction.setEnabled(HistoryActionManager.this.view.isFilterEnabled());
		}

		protected void addLocalPart(final StructuredViewer viewer, IMenuManager manager, final ILogNode[] selection) {
			Action tAction = null;
			manager.add(tAction = new HistoryAction("HistoryView.Open") {
				public void run() {
					HistoryActionManager.this.handleDoubleClick((TreeViewer) viewer, selection[0], false);
				}
			});
			tAction.setEnabled(selection.length == 1);
			tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getWorkbench().getEditorRegistry()
					.getImageDescriptor(((ModelBusLocalFileRevision) selection[0].getEntity()).getName()));

			manager.add(new Separator());

			manager.add(tAction = new HistoryAction("HistoryView.CompareEachOther") {
				public void run() {
					HistoryActionManager.this.runCompareForLocal(selection);
				}
			});
			tAction.setEnabled(selection.length == 2);
			manager.add(tAction = new HistoryAction("HistoryView.CompareCurrentWith", new String[] { ModelBusTeamUIPlugin.instance().getResource(
					"HistoryView.RevisionLocal") }) {
				public void run() {
					HistoryActionManager.this.runCompareForLocal(selection);
				}
			});
			tAction.setEnabled(selection.length == 1);
			manager.add(tAction = new HistoryAction("HistoryView.CompareWithPrevious") {
				public void run() {
					ModelBusLocalFileRevision[] localHistory = HistoryActionManager.this.view.getLocalHistory();
					ModelBusLocalFileRevision currentSelected = (ModelBusLocalFileRevision) selection[0].getEntity();
					ArrayList<ModelBusLocalFileRevision> toOperate = new ArrayList<ModelBusLocalFileRevision>();
					toOperate.add(currentSelected);
					for (int i = 0; i < localHistory.length - 1; i++) {
						if (currentSelected.equals(localHistory[i])) {
							toOperate.add(localHistory[i + 1]);
							break;
						}
					}
					HistoryActionManager.this.runCompareForLocal(toOperate.toArray());
				}
			});
			tAction.setEnabled(selection.length == 1
					&& !selection[0].getEntity().equals(
							HistoryActionManager.this.view.getLocalHistory()[HistoryActionManager.this.view.getLocalHistory().length - 1]));

			manager.add(new Separator());

			manager.add(tAction = new HistoryAction("ExportCommand.label", "icons/common/export.gif") {
				public void run() {
					HistoryActionManager.this.doExport(selection[0]);
				}
			});
			tAction.setEnabled(selection.length == 1);
			manager.add(tAction = new HistoryAction("HistoryView.GetContents") {
				public void run() {
					try {
						((IFile) HistoryActionManager.this.view.getResource()).setContents(
								((ModelBusLocalFileRevision) selection[0].getEntity()).getState(), true, true, new NullProgressMonitor());
					} catch (CoreException ex) {
						UILoggedOperation.reportError(this.getText(), ex);
					}
				}
			});
			tAction.setEnabled(selection.length == 1 && (!((ModelBusLocalFileRevision) selection[0].getEntity()).isCurrentState()));
		}

		protected void addLocalOrRemotePart(StructuredViewer viewer, IMenuManager manager, final ILogNode[] selection, boolean containsMergeHistory) {
			Action tAction = null;
			if (!containsMergeHistory) {
				manager.add(tAction = new HistoryAction("HistoryView.CompareEachOther") {
					public void run() {
						ArrayList<Object> selected = new ArrayList<Object>();
						for (ILogNode item : selection) {
							if (item.getType() == ILogNode.TYPE_LOCAL) {
								selected.add(item);
							} else {
								selected.add(new ModelBusRemoteResourceRevision(HistoryActionManager.this
										.getResourceForSelectedRevision((ModelBusLogEntry) item.getEntity()), (ModelBusLogEntry) item.getEntity()));
							}
						}
						HistoryActionManager.this.runCompareForLocal(selected.toArray());
					}
				});
				tAction.setEnabled(selection.length == 2);
			}
		}

		protected void addCommonPart(StructuredViewer viewer, IMenuManager manager, final ILogNode[] selection, boolean containsMergeHistory) {
			Action tAction = null;
			manager.add(tAction = new HistoryAction("HistoryView.CopyHistory", "icons/common/copy.gif") {
				public void run() {
					HistoryActionManager.this.handleCopy(selection);
				}
			});
			tAction.setEnabled(selection.length > 0);
		}

		protected ILogNode[] getSelection(StructuredViewer viewer) {
			List<?> selection = ((IStructuredSelection) viewer.getSelection()).toList();
			return selection.toArray(new ILogNode[selection.size()]);
		}

	}

	protected Action addMenuItem(StructuredViewer viewer, MenuManager menuManager, String label, final AbstractRepositoryTeamAction action) {
		IStructuredSelection tSelection = (IStructuredSelection) viewer.getSelection();
		Action wrapper = new Action(label) {
			public void run() {
				action.run(this);
			}
		};
		IStructuredSelection resourceSelection;
		if (tSelection.size() == 1 && (((ILogNode) tSelection.getFirstElement()).getType() == ILogNode.TYPE_ModelBus)) {
			resourceSelection = new StructuredSelection(new RepositoryFile(null,
					this.getResourceForSelectedRevision((ModelBusLogEntry) ((ILogNode) tSelection.getFirstElement()).getEntity())));
		} else {
			resourceSelection = new StructuredSelection(StructuredSelection.EMPTY);
		}
		action.selectionChanged(wrapper, resourceSelection);
		menuManager.add(wrapper);
		return wrapper;
	}

	protected void getRevisionContents(ModelBusLogEntry item) {
		IRepositoryResource remote = this.getResourceForSelectedRevision(item);

		// propose user to lock the file if it needs lock
		boolean canWrite = true;

		if (this.view.getResource() instanceof IFile) {
			ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(this.view.getResource());
			if (!local.isLocked() && IStateFilter.SF_NEEDS_LOCK.accept(local)) {
				canWrite = LockProposeUtility.proposeLock(new IResource[] { this.view.getResource() });
			}
		}
		if (canWrite) {
			HashMap<String, String> remote2local = new HashMap<String, String>();
			remote2local.put(ModelBusUtility.encodeURL(remote.getUrl()), FileUtility.getWorkingCopyPath(this.view.getResource()));
			GetRemoteContentsOperation mainOp = new GetRemoteContentsOperation(new IResource[] { this.view.getResource() },
					new IRepositoryResource[] { remote }, remote2local);

			CompositeOperation op = new CompositeOperation(mainOp.getId());
			op.add(mainOp);
			op.add(new RefreshResourcesOperation(new IResource[] { this.view.getResource() }));

			UIMonitorUtility.doTaskScheduledWorkspaceModify(op);
		}
	}

	protected void doExport(ILogNode item) {
		if (item.getType() == ILogNode.TYPE_LOCAL) {
			final ModelBusLocalFileRevision revision = (ModelBusLocalFileRevision) item.getEntity();
			FileDialog dlg = new FileDialog(UIMonitorUtility.getShell(), SWT.PRIMARY_MODAL | SWT.SAVE);
			dlg.setText(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.Title"));
			dlg.setFileName(revision.getName());
			dlg.setFilterExtensions(new String[] { "*.*" });
			final String file = dlg.open();
			if (file != null) {
				IActionOperation op = new AbstractActionOperation("Operation.ExportLocalHistory") {
					protected void runImpl(IProgressMonitor monitor) throws Exception {
						FileOutputStream output = new FileOutputStream(file);
						InputStream input = null;
						try {
							IStorage storage = revision.getStorage(monitor);
							input = storage.getContents();
							byte[] data = new byte[2048];
							int len = 0;
							while ((len = input.read(data)) > 0) {
								output.write(data, 0, len);
							}
						} finally {
							output.close();
							if (input != null) {
								input.close();
							}
						}
					}
				};
				UIMonitorUtility.doTaskScheduledDefault(op);
			}
		} else {
			DirectoryDialog fileDialog = new DirectoryDialog(UIMonitorUtility.getShell());
			fileDialog.setText(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder"));
			fileDialog.setMessage(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder.Msg"));
			String path = fileDialog.open();
			if (path != null) {
				IRepositoryResource resource = this.traceResourceToRevision((ModelBusLogEntry) item.getEntity());
				UIMonitorUtility.doTaskScheduledDefault(new ExportOperation(new IRepositoryResource[] { resource }, path, Depth.INFINITY));
			}
		}
	}

	protected IRepositoryResource traceResourceToRevision(ModelBusLogEntry to) {
		IRepositoryResource resource = this.getResourceForSelectedRevision(to);
		String rootUrl = ""; // TODO CHECKMWA resource.getRepositoryLocation().getRepositoryRootUrl();
		String url = this.traceUrlToRevision(rootUrl, resource.getUrl().substring(rootUrl.length()), this.view.getCurrentRevision(), to.modelBusRevision);
		IRepositoryResource retVal = resource instanceof IRepositoryFile ? resource.asRepositoryFile(url, false) : resource.asRepositoryContainer(
				url, false);
		retVal.setPegRevision(ModelBusRevision.fromString(to.revision));
		return retVal;
	}

	protected void addRevisionLinks(ILogNode[] tSelection) {

		CompositeOperation op = new CompositeOperation("Operation.HAddSelectedRevision");
		for (ILogNode node : tSelection) {
			ModelBusLogEntry item = (ModelBusLogEntry) node.getEntity();
			IRepositoryResource resource = ModelBusUtility.copyOf(this.view.getRepositoryResource());
			resource.setSelectedRevision(ModelBusRevision.fromString(item.revision));
			LocateResourceURLInHistoryOperation locateOp = new LocateResourceURLInHistoryOperation(new IRepositoryResource[] { resource });
			op.add(locateOp);
			op.add(new AddRevisionLinkOperation(locateOp, item.revision), new IActionOperation[] { locateOp });
		}
		UIMonitorUtility.doTaskScheduledDefault(op);
	}

	protected void updateTo(final ModelBusLogEntry item) {
		IResource[] resources = new IResource[] { this.view.getResource() };
		CompositeOperation op = new CompositeOperation("Operation.HUpdateTo");
		SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
		op.add(saveOp);
		op.add(new UpdateOperation(resources, ModelBusRevision.fromString(item.revision), true));
		op.add(new RestoreProjectMetaOperation(saveOp));
		op.add(new RefreshResourcesOperation(resources));
		UIMonitorUtility.doTaskScheduledWorkspaceModify(op);
	}

	protected void createPatch(ILogNode[] selected) {
		CreatePatchWizard wizard = new CreatePatchWizard(this.view.getRepositoryResource().getName());
		WizardDialog dialog = new WizardDialog(UIMonitorUtility.getShell(), wizard);
		if (dialog.open() == DefaultDialog.OK) {
			ModelBusLogEntry msg0 = (ModelBusLogEntry) selected[0].getEntity();
			IRepositoryResource next = this.getResourceForSelectedRevision(msg0);
			IRepositoryResource prev = null;
			if (selected.length == 1) {
				// FIXME peg revisions for renamed resources: (rev - 1) works only if the revision really exists in repository for current resource
				// use LocateUrlInHistory
				prev = this.getResourceForSelectedRevision(new ModelBusLogEntry(msg0.modelBusRevision.getPreviousRevision().getRevision(), 0, null, null, null, false));
			} else {
				ModelBusLogEntry msg1 = (ModelBusLogEntry) selected[1].getEntity();
				prev = this.getResourceForSelectedRevision(msg1);
				if (msg0.date < msg1.date) {
					IRepositoryResource tmp = next;
					next = prev;
					prev = tmp;
				}
			}
			UIMonitorUtility.doTaskScheduledDefault(UIMonitorUtility.getActivePart(), CreatePatchAction.getCreatePatchOperation(prev, next, wizard));
		}
	}

	protected void handleCopy(ILogNode[] selection) {
		String historyText = this.getSelectedMessagesAsString(selection);
		Clipboard clipboard = new Clipboard(UIMonitorUtility.getDisplay());
		try {
			clipboard.setContents(new Object[] { historyText }, new Transfer[] { TextTransfer.getInstance() });
		} finally {
			clipboard.dispose();
		}
	}

	public String getSelectedMessagesAsString(ILogNode[] selection) {
		String[] historyText = { "" };
		HashSet<ILogNode> processed = new HashSet<ILogNode>();
		ModelBusRevision revision = this.view.getCurrentRevision();
		for (ILogNode node : selection) {
			this.toString(processed, node, revision, historyText);
		}
		return historyText[0];
	}

	protected void toString(HashSet<ILogNode> processed, ILogNode node, ModelBusRevision revision, String[] retStr) {
		if (processed.contains(node)) {
			return;
		}
		processed.add(node);
		String toAdd = "";
		toAdd += node.getLabel(0, ILogNode.LABEL_FLAT, revision);
		for (int i = 1; i < ILogNode.NUM_OF_COLUMNS; i++) {
			toAdd += "\t" + node.getLabel(i, ILogNode.LABEL_FLAT, revision);
		}
		toAdd = toAdd.trim();
		if (!toAdd.equals("")) {
			retStr[0] += toAdd + System.getProperty("line.separator");
		}
		if (node.hasChildren()) {
			ILogNode[] children = node.getChildren();
			for (int j = 0; j < children.length; j++) {
				this.toString(processed, children[j], revision, retStr);
			}
		}
	}

	protected String toString(HashSet<ILogNode> processed, ILogNode node, ModelBusRevision revision) {
		if (processed.contains(node)) {
			return "";
		}
		processed.add(node);
		String historyText = node.getLabel(0, ILogNode.LABEL_FLAT, revision);
		for (int i = 1; i < ILogNode.NUM_OF_COLUMNS; i++) {
			historyText += "\t" + node.getLabel(i, ILogNode.LABEL_FLAT, revision);
		}
		return historyText + System.getProperty("line.separator");
	}

	protected void handleDoubleClick(TreeViewer viewer, ILogNode item, boolean doubleClick) {
		int type = item.getType();
		if (type == ILogNode.TYPE_CATEGORY) {
			if (viewer.getExpandedState(item)) {
				viewer.collapseToLevel(item, TreeViewer.ALL_LEVELS);
			} else {
				viewer.expandToLevel(item, TreeViewer.ALL_LEVELS);
			}
			return;
		}
		boolean isCompareAllowed =

		// CHECKMWA CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion() ==
		// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x
		true || this.view.getRepositoryResource() instanceof IRepositoryFile;
		if ((this.view.getOptions() & IModelBusHistoryView.COMPARE_MODE) != 0 && doubleClick && isCompareAllowed) {
			if (type == ILogNode.TYPE_ModelBus && !(item.getParent() instanceof ModelBusLogNode)) {
				this.compareWithCurrent((ModelBusLogEntry) item.getEntity());
			} else if (type == ILogNode.TYPE_LOCAL) {
				this.runCompareForLocal(new Object[] { item });
			}
		} else if (type == ILogNode.TYPE_LOCAL) {
			try {
				ModelBusLocalFileRevision selected = (ModelBusLocalFileRevision) item.getEntity();
				Utils.openEditor(UIMonitorUtility.getActivePage(), selected, new NullProgressMonitor());
			} catch (CoreException ex) {
				UILoggedOperation.reportError("Open Editor", ex);
			}
		} else if (!(this.view.getRepositoryResource() instanceof IRepositoryContainer) && type == ILogNode.TYPE_ModelBus
				&& !(item.getParent() instanceof ModelBusLogNode)) {
			UIMonitorUtility.doTaskScheduledActive(new OpenRemoteFileOperation(new IRepositoryFile[] { (IRepositoryFile) this
					.getResourceForSelectedRevision((ModelBusLogEntry) item.getEntity()) }, OpenRemoteFileOperation.OPEN_DEFAULT));
		}
	}

	protected void compareWithCurrent(ModelBusLogEntry item) {
		IRepositoryResource resource = this.getResourceForSelectedRevision(item);
		if (this.view.getResource() != null || this.view.getCompareWith() != null) {
			ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(this.view.getCompareWith());
			CompareResourcesOperation op = new CompareResourcesOperation(local, resource);
			op.setForceId(this.getCompareForceId());
			UIMonitorUtility.doTaskScheduledActive(op);
		} else {
			CompareRepositoryResourcesOperation op = new CompareRepositoryResourcesOperation(this.getResourceForHeadRevision(), resource);
			op.setForceId(this.getCompareForceId());
			UIMonitorUtility.doTaskScheduledActive(op);
		}
	}

	protected void runExtractTo(ILogNode[] selection) {
		String path = null;
		DirectoryDialog fileDialog = new DirectoryDialog(UIMonitorUtility.getShell());
		fileDialog.setText(ModelBusTeamUIPlugin.instance().getResource("ExtractToAction.Select.Title"));
		fileDialog.setMessage(ModelBusTeamUIPlugin.instance().getResource("ExtractToAction.Select.Description"));
		path = fileDialog.open();
		if (path == null) {
			return;
		}
		ModelBusLogEntry[] selectedLogs = new ModelBusLogEntry[2];
		selectedLogs[0] = (ModelBusLogEntry) selection[0].getEntity();
		if (selection.length == 2) {
			selectedLogs[1] = (ModelBusLogEntry) selection[1].getEntity();
			if (selectedLogs[0].date < selectedLogs[1].date) {
				ModelBusLogEntry tmp = selectedLogs[0];
				selectedLogs[0] = selectedLogs[1];
				selectedLogs[1] = tmp;
			}
		} else {
			selectedLogs[1] = new ModelBusLogEntry(selectedLogs[0].modelBusRevision.getPreviousRevision().getRevision(), 0, null, null, null, false);
		}
		HashMap<String, String> resource2project = new HashMap<String, String>();
		IResource local = this.view.getResource();
		if (local != null) {
			IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(local.getProject());
			resource2project.put(remote.getUrl(), local.getProject().getName());
		} else {
			IRepositoryResource remote = this.view.getRepositoryResource();
			resource2project.put(remote.getUrl(), remote.getName());
		}
		CompositeOperation op = new CompositeOperation(ModelBusTeamPlugin.instance().getResource("Operation.ExtractTo"));
		InitExtractLogOperation logger = new InitExtractLogOperation(path);
		FromDifferenceRepositoryResourceProvider provider = new FromDifferenceRepositoryResourceProvider(selectedLogs);
		op.add(provider);
		op.add(logger);
		op.add(new ExtractToOperationRemote(provider, provider.getDeletionsProvider(), path, resource2project, logger, true),
				new IActionOperation[] { provider });
		op.add(new FiniExtractLogOperation(logger));
		UIMonitorUtility.doTaskScheduledActive(op);
	}

	protected void runCompareForLocal(Object[] selection) {
		// FIXME reimplement without internals usage
		ArrayList<FileRevision> newSelection = new ArrayList<FileRevision>();
		for (Object item : selection) {
			if (item instanceof ILogNode) {
				newSelection.add((ModelBusLocalFileRevision) ((ILogNode) item).getEntity());
			} else {
				newSelection.add((FileRevision) item);
			}
		}
		CompareRevisionAction compare = null;
		try {
			compare = CompareRevisionAction.class.getConstructor((Class[]) null).newInstance((Object[]) null);
			compare.getClass().getMethod("setPage", new Class[] { HistoryPage.class }).invoke(compare, this.view.getHistoryPage());
		} catch (Exception ex) {
			try {
				compare = CompareRevisionAction.class.getConstructor(new Class[] { String.class, HistoryPage.class }).newInstance(
						new Object[] { "", this.view.getHistoryPage() });
			} catch (RuntimeException ex1) {
				throw ex1;
			} catch (Exception ex1) {
				throw new RuntimeException(ex1);
			}
		}
		compare.selectionChanged(new StructuredSelection(newSelection));
		compare.setCurrentFileRevision(new ModelBusLocalFileRevision((IFile) this.view.getResource()));
		compare.run();
	}

	protected IRepositoryResource getResourceForHeadRevision() {
		IRepositoryResource res = this.view.getRepositoryResource() instanceof IRepositoryFile ? (IRepositoryResource) ((IRepositoryRoot) this.view
				.getRepositoryResource().getRoot()).asRepositoryFile(this.view.getRepositoryResource().getUrl(), false)
				: ((IRepositoryRoot) this.view.getRepositoryResource().getRoot()).asRepositoryContainer(this.view.getRepositoryResource().getUrl(),
						false);
		res.setSelectedRevision(ModelBusRevision.HEAD);
		res.setPegRevision(ModelBusRevision.HEAD);
		return res;
	}

	protected String getCompareForceId() {
		return this.toString();
	}

	protected IRepositoryResource getResourceForSelectedRevision(ModelBusLogEntry item) {
		String revNum = item.revision;
		IRepositoryResource res = ModelBusUtility.copyOf(this.view.getRepositoryResource());
		res.setSelectedRevision(ModelBusRevision.fromString(revNum));
		res.setPegRevision(this.view.getRepositoryResource().getPegRevision());
		return res;
	}

	protected boolean confirmReplacement() {
		ReplaceWarningDialog dialog = new ReplaceWarningDialog(UIMonitorUtility.getShell());
		return dialog.open() == 0;
	}

	protected class AffectedTableActionManager implements IControlActionManager {
		public void installKeyBindings(StructuredViewer viewer) {
		}

		public void installDefaultAction(StructuredViewer viewer) {
			// add double click listener for the table viewer
			viewer.addDoubleClickListener(new IDoubleClickListener() {
				public void doubleClick(DoubleClickEvent e) {
					if (e.getSelection() instanceof IStructuredSelection) {
						IStructuredSelection selection = (IStructuredSelection) e.getSelection();
						if (selection.size() == 1) {
							ModelBusChangedPathData data = (ModelBusChangedPathData) selection.getFirstElement();
							if ((HistoryActionManager.this.view.getOptions() & IModelBusHistoryView.COMPARE_MODE) != 0) {
								boolean isPreviousExists = data.action == ModelBusLogPath.ChangeType.MODIFIED
										|| data.action == ModelBusLogPath.ChangeType.REPLACED || data.copiedFromPath != null
										&& !data.copiedFromPath.equals("");
								if (isPreviousExists) {
									FromChangedPathDataProvider provider = new FromChangedPathDataProvider(data, false);
									HistoryActionManager.this.compareWithPreviousRevision(provider, provider);
								} else {
									MessageDialog dialog = new MessageDialog(e.getViewer().getControl().getShell(), ModelBusTeamUIPlugin.instance()
											.getResource("AffectedPathsComposite.NoPreviousRevision.Title"), null, ModelBusTeamUIPlugin.instance()
											.getResource("AffectedPathsComposite.NoPreviousRevision.Message"), MessageDialog.INFORMATION,
											new String[] { IDialogConstants.OK_LABEL }, 0);
									dialog.open();
								}
							} else {
								HistoryActionManager.this.openRemoteResource(data, OpenRemoteFileOperation.OPEN_DEFAULT, null);
							}
						}
					}
				}
			});
		}

		public void installMenuActions(final StructuredViewer viewer, IWorkbenchPartSite site) {
			MenuManager menuMgr = new MenuManager();
			Menu menu = menuMgr.createContextMenu(viewer.getControl());
			menuMgr.addMenuListener(new IMenuListener() {
				public void menuAboutToShow(IMenuManager manager) {
					manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
					final IStructuredSelection affectedTableSelection = (IStructuredSelection) viewer.getSelection();
					if (affectedTableSelection.size() == 0) {
						return;
					}
					final ModelBusChangedPathData firstData = (ModelBusChangedPathData) affectedTableSelection.getFirstElement();
					Action tAction = null;

					IEditorRegistry editorRegistry = ModelBusTeamUIPlugin.instance().getWorkbench().getEditorRegistry();
					manager.add(tAction = new HistoryAction("HistoryView.Open") {
						public void run() {
							HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_DEFAULT, null);
						}
					});
					String name = firstData.resourceName;
					tAction.setImageDescriptor(editorRegistry.getImageDescriptor(name));
					tAction.setEnabled(affectedTableSelection.size() == 1);

					// FIXME: "Open with" submenu shouldn't be hardcoded after reworking of
					// the HistoryView. Should be made like the RepositoriesView menu.
					MenuManager sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("HistoryView.OpenWith"), "historyOpenWithMenu");
					sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

					sub.add(new Separator("nonDefaultTextEditors"));
					IEditorDescriptor[] editors = editorRegistry.getEditors(name);
					for (int i = 0; i < editors.length; i++) {
						final String id = editors[i].getId();
						if (!id.equals(EditorsUI.DEFAULT_TEXT_EDITOR_ID)) {
							sub.add(tAction = new HistoryAction(editors[i].getLabel()) {
								public void run() {
									HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_SPECIFIED, id);
								}
							});
							tAction.setImageDescriptor(editors[i].getImageDescriptor());
							tAction.setEnabled(affectedTableSelection.size() == 1);
						}
					}

					sub.add(new Separator("variousEditors"));
					IEditorDescriptor descriptor = null;
					sub.add(tAction = new HistoryAction("HistoryView.TextEditor") {
						public void run() {
							HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_SPECIFIED,
									EditorsUI.DEFAULT_TEXT_EDITOR_ID);
						}
					});
					descriptor = editorRegistry.findEditor(EditorsUI.DEFAULT_TEXT_EDITOR_ID);
					tAction.setImageDescriptor(descriptor.getImageDescriptor());
					tAction.setEnabled(affectedTableSelection.size() == 1);
					sub.add(tAction = new HistoryAction("HistoryView.SystemEditor") {
						public void run() {
							HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_EXTERNAL, null);
						}
					});
					if (editorRegistry.isSystemExternalEditorAvailable(name)) {
						tAction.setImageDescriptor(editorRegistry.getSystemExternalEditorImageDescriptor(name));
						tAction.setEnabled(affectedTableSelection.size() == 1);
					} else {
						tAction.setEnabled(false);
					}
					sub.add(tAction = new HistoryAction("HistoryView.InplaceEditor") {
						public void run() {
							HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_INPLACE, null);
						}
					});
					if (editorRegistry.isSystemInPlaceEditorAvailable(name)) {
						tAction.setImageDescriptor(editorRegistry.getSystemExternalEditorImageDescriptor(name));
						tAction.setEnabled(affectedTableSelection.size() == 1);
					} else {
						tAction.setEnabled(false);
					}
					sub.add(tAction = new HistoryAction("HistoryView.DefaultEditor") {
						public void run() {
							HistoryActionManager.this.openRemoteResource(firstData, OpenRemoteFileOperation.OPEN_DEFAULT, null);
						}
					});
					tAction.setImageDescriptor(editorRegistry.getImageDescriptor(name));
					tAction.setEnabled(affectedTableSelection.size() == 1);

					manager.add(sub);

					manager.add(new Separator());

					boolean isPreviousExists = false;
					if (affectedTableSelection.size() > 0) {
						isPreviousExists = HistoryActionManager.this.checkSelectionForExistanceInPrev(affectedTableSelection);
					}
					manager.add(tAction = new HistoryAction("HistoryView.CompareWithPrevious") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.compareWithPreviousRevision(provider, provider);
						}
					});
					tAction.setEnabled(isPreviousExists);

					manager.add(new Separator());

					manager.add(tAction = new HistoryAction("ShowPropertiesAction.label", "icons/views/propertiesedit.gif") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.showProperties(provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() == 1);
					manager.add(tAction = new HistoryAction("ShowResourceHistoryCommand.label", "icons/views/history.gif") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.showHistory(provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() == 1);
					manager.add(tAction = new HistoryAction("ShowAnnotationCommand.label") {
						public void run() {
							HistoryActionManager.this.showAnnotation(firstData);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() == 1);

					manager.add(new Separator());

					manager.add(tAction = new HistoryAction("ExportCommand.label", "icons/common/export.gif") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.doExport(viewer.getControl().getShell(), provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && firstData.action != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction("CreatePatchCommand.label") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.createPatchToPrevious(viewer.getControl().getShell(), provider, provider);
						}
					});
					tAction.setEnabled(isPreviousExists);

					if (HistoryActionManager.this.view.getResource() != null) {
						manager.add(tAction = new HistoryAction("HistoryView.GetContents") {
							public void run() {
								FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
								HistoryActionManager.this.getContentAffected(provider, provider, "/" + firstData.getFullResourcePath());
							}
						});
						tAction.setEnabled(affectedTableSelection.size() > 0 && firstData.action != ModelBusLogPath.ChangeType.DELETED);
					}
					manager.add(new Separator());

					String branchFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.BranchFrom",
							new String[] { String.valueOf(HistoryActionManager.this.selectedRevision) });
					String tagFrom = ModelBusTeamUIPlugin.instance().getResource("HistoryView.TagFrom",
							new String[] { String.valueOf(HistoryActionManager.this.selectedRevision) });
					manager.add(tAction = new HistoryAction(branchFrom, "icons/common/actions/branch.gif") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.createBranchTag(viewer.getControl().getShell(), provider, provider,
									BranchTagAction.BRANCH_ACTION);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && firstData.action != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction(tagFrom, "icons/common/actions/tag.gif") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.createBranchTag(viewer.getControl().getShell(), provider, provider, BranchTagAction.TAG_ACTION);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && firstData.action != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction("AddRevisionLinkAction.label") {
						public void run() {
							FromChangedPathDataProvider provider = new FromChangedPathDataProvider(firstData, false);
							HistoryActionManager.this.addRevisionLink(provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && firstData.action != ModelBusLogPath.ChangeType.DELETED);
				}
			});
			menuMgr.setRemoveAllWhenShown(true);
			viewer.getControl().setMenu(menu);
			site.registerContextMenu(menuMgr, viewer);
		}
	}

	protected class AffectedTreeActionManager implements IControlActionManager {
		public void installKeyBindings(StructuredViewer viewer) {
		}

		public void installDefaultAction(StructuredViewer viewer) {
		}

		public void installMenuActions(final StructuredViewer viewer, IWorkbenchPartSite site) {
			MenuManager menuMgr = new MenuManager();
			Menu menu = menuMgr.createContextMenu(viewer.getControl());
			menuMgr.addMenuListener(new IMenuListener() {
				public void menuAboutToShow(IMenuManager manager) {
					manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

					final IStructuredSelection affectedTableSelection = (IStructuredSelection) viewer.getSelection();
					if (affectedTableSelection.size() == 0) {
						return;
					}
					final AffectedPathsNode node = (AffectedPathsNode) affectedTableSelection.getFirstElement();

					Action tAction = null;
					manager.add(tAction = new HistoryAction("HistoryView.CompareWithPrevious") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.compareWithPreviousRevision(provider, provider);
						}
					});
					boolean isCompareFoldersAllowed = true; // CHECKMWA
					// CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion() ==
					// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x;
					tAction.setEnabled(isCompareFoldersAllowed
							&& !"0".equals(HistoryActionManager.this.selectedRevision)
							&& affectedTableSelection.size() == 1
							&& (node.getStatus() == '\0' || node.getStatus() == ModelBusLogPath.ChangeType.MODIFIED || node.getStatus() == ModelBusLogPath.ChangeType.REPLACED));

					manager.add(new Separator());

					manager.add(tAction = new HistoryAction("ShowPropertiesAction.label", "icons/views/propertiesedit.gif") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.showProperties(provider, provider);
						}
					});
					tAction.setEnabled(!"0".equals(HistoryActionManager.this.selectedRevision) && affectedTableSelection.size() == 1 /*
																															 * && (node.getStatus() ==
																															 * null ||
																															 * node.getStatus()
																															 * .charAt(0) == 'M')
																															 */);
					manager.add(tAction = new HistoryAction("ShowResourceHistoryCommand.label", "icons/views/history.gif") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.showHistory(provider, provider);
						}
					});
					tAction.setEnabled(!"0".equals(HistoryActionManager.this.selectedRevision) && affectedTableSelection.size() == 1);

					manager.add(new Separator());

					manager.add(tAction = new HistoryAction("ExportCommand.label", "icons/common/export.gif") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.doExport(viewer.getControl().getShell(), provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && node.getStatus() != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction("CreatePatchCommand.label") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.createPatchToPrevious(viewer.getControl().getShell(), provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() == 1 && !"0".equals(HistoryActionManager.this.selectedRevision)
							&& affectedTableSelection.size() == 1
							&& (node.getStatus() == '\0' || node.getStatus() == ModelBusLogPath.ChangeType.MODIFIED));

					if (HistoryActionManager.this.view.getResource() != null) {
						manager.add(tAction = new HistoryAction("HistoryView.GetContents") {
							public void run() {
								FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
								HistoryActionManager.this.getContentAffected(provider, provider, node.getFullPath());
							}
						});
						tAction.setEnabled(affectedTableSelection.size() > 0 && node.getStatus() != ModelBusLogPath.ChangeType.DELETED);
					}

					manager.add(new Separator());

					manager.add(tAction = new HistoryAction("HistoryView.BranchFrom", new String[] { String
							.valueOf(HistoryActionManager.this.selectedRevision) }, "icons/common/actions/branch.gif") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.createBranchTag(viewer.getControl().getShell(), provider, provider,
									BranchTagAction.BRANCH_ACTION);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && node.getStatus() != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction("HistoryView.TagFrom", new String[] { String
							.valueOf(HistoryActionManager.this.selectedRevision) }, "icons/common/actions/tag.gif") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.createBranchTag(viewer.getControl().getShell(), provider, provider, BranchTagAction.TAG_ACTION);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && node.getStatus() != ModelBusLogPath.ChangeType.DELETED);
					manager.add(tAction = new HistoryAction("AddRevisionLinkAction.label") {
						public void run() {
							FromAffectedPathsNodeProvider provider = new FromAffectedPathsNodeProvider(node);
							HistoryActionManager.this.addRevisionLink(provider, provider);
						}
					});
					tAction.setEnabled(affectedTableSelection.size() > 0 && node.getStatus() != ModelBusLogPath.ChangeType.DELETED);
				}
			});
			menuMgr.setRemoveAllWhenShown(true);
			viewer.getControl().setMenu(menu);
			site.registerContextMenu(menuMgr, viewer);
		}

	}

	protected boolean checkSelectionForExistanceInPrev(IStructuredSelection selection) {
		for (Iterator<?> it = selection.iterator(); it.hasNext();) {
			Object next = it.next();
			if (next instanceof ModelBusChangedPathData) {
				ModelBusChangedPathData current = (ModelBusChangedPathData) next;
				if (!(current.action == ModelBusLogPath.ChangeType.MODIFIED || current.action == ModelBusLogPath.ChangeType.REPLACED || (current.copiedFromPath != null && !current.copiedFromPath
						.equals("")))) {
					return false;
				}
			}
		}
		return true;
	}

	protected void createPatchToPrevious(Shell shell, IActionOperation preOp, IRepositoryResourceProvider provider) {
		if (UIMonitorUtility.doTaskNowDefault(shell, preOp, true).isCancelled()) {
			return;
		}

		IRepositoryResource current = provider.getRepositoryResources()[0];
		CreatePatchWizard wizard = new CreatePatchWizard(current.getName());
		WizardDialog dialog = new WizardDialog(shell, wizard);
		if (dialog.open() == DefaultDialog.OK) {
			IRepositoryResource previous = (current instanceof RepositoryFolder) ? current.asRepositoryContainer(current.getUrl(), false) : current
					.asRepositoryFile(current.getUrl(), false);
			previous.setSelectedRevision(ModelBusRevision.fromString(HistoryActionManager.this.selectedRevision.getPreviousRevision().getRevision()));
			previous.setPegRevision(HistoryActionManager.this.selectedRevision);
			UIMonitorUtility.doTaskScheduledDefault(CreatePatchAction.getCreatePatchOperation(previous, current, wizard));
		}
	}

	protected void createBranchTag(Shell shell, IActionOperation preOp, IRepositoryResourceProvider provider, int type) {
		if (UIMonitorUtility.doTaskNowDefault(shell, preOp, true).isCancelled()) {
			return;
		}

		IRepositoryResource[] resources = provider.getRepositoryResources();
		PreparedBranchTagOperation op = BranchTagAction.getBranchTagOperation(resources, shell, type);

		if (op != null) {
			CompositeOperation composite = new CompositeOperation(op.getId());
			composite.add(op);
			RefreshRemoteResourcesOperation refreshOp = new RefreshRemoteResourcesOperation(new IRepositoryResource[] { op.getDestination()
					.getParent() });
			composite.add(refreshOp, new IActionOperation[] { op });
			UIMonitorUtility.doTaskScheduledDefault(op);
		}
	}

	protected void doExport(Shell shell, IActionOperation preOp, IRepositoryResourceProvider provider) {
		DirectoryDialog fileDialog = new DirectoryDialog(shell);
		fileDialog.setText(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder"));
		fileDialog.setMessage(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder.Msg"));
		String path = fileDialog.open();
		if (path != null) {
			ExportOperation mainOp = new ExportOperation(provider, path, Depth.INFINITY);
			CompositeOperation op = new CompositeOperation(mainOp.getId());
			op.add(preOp);
			op.add(mainOp, new IActionOperation[] { preOp });
			UIMonitorUtility.doTaskScheduledDefault(op);
		}
	}

	protected void openRemoteResource(ModelBusChangedPathData selectedPath, int openType, String openWith) {
		FromChangedPathDataProvider provider = new FromChangedPathDataProvider(selectedPath, true);
		OpenRemoteFileOperation openOp = new OpenRemoteFileOperation(provider, openType, openWith);

		CompositeOperation composite = new CompositeOperation(openOp.getId(), true);
		composite.add(provider);
		composite.add(openOp, new IActionOperation[] { provider });
		UIMonitorUtility.doTaskScheduledActive(composite);
	}

	protected void showAnnotation(ModelBusChangedPathData selectedPath) {
		FromChangedPathDataProvider provider = new FromChangedPathDataProvider(selectedPath, true);
		RemoteShowAnnotationOperation mainOp = new RemoteShowAnnotationOperation(provider);
		CompositeOperation op = new CompositeOperation(mainOp.getId(), true);
		op.add(provider);
		op.add(mainOp, new IActionOperation[] { provider });
		UIMonitorUtility.doTaskScheduledActive(op);
	}

	protected void showHistory(IActionOperation preOp, IRepositoryResourceProvider provider) {
		ShowHistoryViewOperation mainOp = new ShowHistoryViewOperation(provider, 0, 0);
		CompositeOperation op = new CompositeOperation(mainOp.getId());
		op.add(preOp);
		op.add(mainOp, new IActionOperation[] { preOp });
		UIMonitorUtility.doTaskScheduledActive(op);
	}

	protected void showProperties(IActionOperation preOp, IRepositoryResourceProvider provider) {
		IResourcePropertyProvider propertyProvider = new GetRemotePropertiesOperation(provider);
		ShowPropertiesOperation mainOp = new ShowPropertiesOperation(UIMonitorUtility.getActivePage(), provider, propertyProvider);
		CompositeOperation op = new CompositeOperation(mainOp.getId());
		op.add(preOp);
		op.add(mainOp, new IActionOperation[] { preOp });
		UIMonitorUtility.doTaskScheduledActive(op);
	}

	protected void getContentAffected(IActionOperation preOp, IRepositoryResourceProvider provider, String remotePath) {
		String rootUrl = ""; // CHECKMWA repository root
								// ModelBusRemoteStorage.instance().asRepositoryResource(this.view.getResource()).getRepositoryLocation().getRepositoryRootUrl();
		String remoteViewedResourceUrl = ModelBusRemoteStorage.instance().asRepositoryResource(this.view.getResource()).getUrl();
		String remoteFoundPath = this.traceUrlToRevision(rootUrl, remotePath, this.view.getCurrentRevision(), this.selectedRevision);
		if (!remoteFoundPath.startsWith(remoteViewedResourceUrl)) {
			MessageDialog dialog = new MessageDialog(UIMonitorUtility.getShell(), ModelBusTeamUIPlugin.instance().getResource(
					"AffectedPathsActions.CantGetContent.Title"), null, ModelBusTeamUIPlugin.instance().getResource(
					"AffectedPathsActions.CantGetContent.Message"), MessageDialog.INFORMATION, new String[] { IDialogConstants.OK_LABEL }, 0);
			dialog.open();
			return;
		}
		if (!this.confirmReplacement()) {
			return;
		}
		IPath resourcePath = new Path(remoteFoundPath.substring(remoteViewedResourceUrl.length()));
		IResource resourceToLock;
		HashMap<String, String> remote2local = new HashMap<String, String>();
		if (this.view.getResource() instanceof IContainer) {
			IContainer viewedResource = (IContainer) this.view.getResource();
			remote2local.put(ModelBusUtility.encodeURL(rootUrl + remotePath),
					FileUtility.getWorkingCopyPath(viewedResource).concat(resourcePath.toString()));
			resourceToLock = viewedResource.findMember(resourcePath);
			while (resourceToLock == null) {
				resourcePath = resourcePath.removeLastSegments(1);
				resourceToLock = viewedResource.findMember(resourcePath);
			}
		} else {
			IFile viewedResource = (IFile) this.view.getResource();
			remote2local.put(ModelBusUtility.encodeURL(rootUrl + remotePath),
					FileUtility.getWorkingCopyPath(viewedResource).concat(resourcePath.toString()));
			resourceToLock = viewedResource.getParent();
		}
		GetRemoteContentsOperation mainOp = new GetRemoteContentsOperation(new IResource[] { resourceToLock }, provider, remote2local);
		CompositeOperation op = new CompositeOperation(mainOp.getId());
		op.add(preOp);
		op.add(mainOp, new IActionOperation[] { preOp });
		op.add(new RefreshResourcesOperation(new IResource[] { resourceToLock }));
		UIMonitorUtility.doTaskScheduledActive(op);
	}

	protected String traceUrlToRevision(String rootUrl, String resourcePath, ModelBusRevision currentRevision, ModelBusRevision selectedRevision) {
		String url = rootUrl + resourcePath;
		ModelBusLogEntry[] entries = this.view.getFullRemoteHistory();

		if (currentRevision.equals(selectedRevision) || entries[entries.length - 1].modelBusRevision.isGreaterThan(currentRevision)) {
			return url;
		}
		if (currentRevision.isGreaterThan(selectedRevision)) {
			for (int i = entries.length - 1; i > -1; i--) {
				ModelBusLogEntry entry = entries[i];
				if (entry.modelBusRevision.isLowerThan(selectedRevision)) {
					continue;
				}
				if (entry.modelBusRevision.isGreaterThan(currentRevision)) {
					break;
				}
				if (entry.changedPaths == null) {
					return url;
				}
				for (ModelBusLogPath path : entry.changedPaths) {
					if (path.copiedFromPath != null && url.endsWith(path.copiedFromPath)) {
						url = rootUrl + path.path;
						break;
					}
				}
			}
		} else {
			for (ModelBusLogEntry entry : entries) {
				if (entry.date > selectedRevision.getDate()) {
					continue;
				}
				if (entry.date < currentRevision.getDate()) {
					break;
				}
				if (entry.changedPaths == null) {
					return url;
				}
				for (ModelBusLogPath path : entry.changedPaths) {
					if (path.copiedFromPath != null && url.endsWith(path.path)) {
						url = rootUrl + path.copiedFromPath;
						break;
					}
				}
			}
		}
		return url;
	}

	protected void addRevisionLink(IActionOperation preOp, IRepositoryResourceProvider provider) {
		CompositeOperation op = new CompositeOperation("Operation.HAddSelectedRevision");
		op.add(preOp);
		IActionOperation[] condition = new IActionOperation[] { preOp };
		op.add(new AddRevisionLinkOperation(provider, this.selectedRevision.getRevision()), condition);
		UIMonitorUtility.doTaskScheduledDefault(op);
	}

	protected void compareWithPreviousRevision(IActionOperation preOp, final IRepositoryResourceProvider provider) {
		CompareRepositoryResourcesOperation mainOp = new CompareRepositoryResourcesOperation(new IRepositoryResourceProvider() {
			public IRepositoryResource[] getRepositoryResources() {
				IRepositoryResource next = provider.getRepositoryResources()[0];
				IRepositoryResource prev = ModelBusUtility.copyOf(next);
				prev.setSelectedRevision(ModelBusRevision.fromString(next.getSelectedRevision().getPreviousRevision().getRevision()));
				return new IRepositoryResource[] { prev, next };
			}
		});
		mainOp.setForceId(this.getCompareForceId());
		if (preOp != null) {
			CompositeOperation op = new CompositeOperation(mainOp.getId());
			op.add(preOp);
			op.add(mainOp, new IActionOperation[] { preOp });
			UIMonitorUtility.doTaskScheduledActive(op);
		} else {
			UIMonitorUtility.doTaskScheduledActive(mainOp);
		}
	}

	protected class FromAffectedPathsNodeProvider extends AbstractActionOperation implements IRepositoryResourceProvider {
		protected AffectedPathsNode		affectedPathsItem;
		protected IRepositoryResource	returnResource;

		public FromAffectedPathsNodeProvider(AffectedPathsNode affectedPathsItem) {
			super("Operation.GetRepositoryResource");
			this.affectedPathsItem = affectedPathsItem;
		}

		protected void runImpl(IProgressMonitor monitor) throws Exception {
			String rootUrl = ModelBusConnector.getRepositoryRootUrl();
			// HistoryActionManager.this.view.getRepositoryResource().getRepositoryLocation().getRepositoryRootUrl();
			String path = this.affectedPathsItem.getFullPath();

			String resourceUrl = rootUrl + (path.startsWith("/") ? "" : "/") + path;
			ModelBusRevision revision = HistoryActionManager.this.selectedRevision;

			// IRepositoryLocation location = HistoryActionManager.this.view.getRepositoryResource().getRepositoryLocation();
			this.returnResource = ModelBusConnector.asRepositoryContainer(resourceUrl, false);
			this.returnResource.setSelectedRevision(revision);
			this.returnResource.setPegRevision(revision);
		}

		public IRepositoryResource[] getRepositoryResources() {
			return new IRepositoryResource[] { this.returnResource };
		}
	}

	protected class FromChangedPathDataProvider extends AbstractActionOperation implements IRepositoryResourceProvider {
		protected IRepositoryResource[]		repositoryResources;
		protected ModelBusChangedPathData	affectedPathsItem;
		protected boolean					filesOnly;

		public FromChangedPathDataProvider(ModelBusChangedPathData affectedPathsItem, boolean filesOnly) {
			super("Operation.GetRepositoryResource");
			this.affectedPathsItem = affectedPathsItem;
			this.filesOnly = filesOnly;
		}

		protected void runImpl(IProgressMonitor monitor) throws Exception {
			String affectedPath = this.affectedPathsItem.getFullResourcePath();
			String rootUrl = ModelBusConnector.getRepositoryRootUrl();
			// HistoryActionManager.this.view.getRepositoryResource().getRepositoryLocation().getRepositoryRootUrl();
			String resourceUrl = rootUrl + "/" + affectedPath;
			ModelBusRevision revision = this.affectedPathsItem.action == ModelBusLogPath.ChangeType.DELETED ? HistoryActionManager.this.selectedRevision.getPreviousRevision()
							: HistoryActionManager.this.selectedRevision;

			ModelBusEntryInfo info = null;
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				ModelBusEntryInfo[] infos = ModelBusUtility.info(proxy, new ModelBusEntryRevisionReference(ModelBusUtility.encodeURL(resourceUrl),
						revision, revision), Depth.EMPTY, new ModelBusProgressMonitor(this, monitor, null));
				if (infos == null || infos.length == 0) {
					return;
				}
				info = infos[0];
			} finally {
			}

			if (info.kind == Kind.DIR && this.filesOnly) {
				final String message = ModelBusTeamUIPlugin.instance().getResource("AffectedPathsComposite.Open.Message",
						new String[] { ModelBusUtility.decodeURL(info.url) });
				UIMonitorUtility.getDisplay().syncExec(new Runnable() {
					public void run() {
						MessageDialog dialog = new MessageDialog(UIMonitorUtility.getDisplay().getActiveShell(), ModelBusTeamUIPlugin.instance()
								.getResource("AffectedPathsComposite.Open.Title"), null, message, MessageDialog.INFORMATION,
								new String[] { IDialogConstants.OK_LABEL }, 0);
						dialog.open();
					}
				});
				this.reportStatus(new Status(IStatus.WARNING, ModelBusTeamPlugin.NATURE_ID, IStatus.OK, message, null));
				return;
			}
			this.repositoryResources = new IRepositoryResource[1];
			this.repositoryResources[0] = info.kind == Kind.FILE ? (IRepositoryResource) ModelBusConnector.asRepositoryFile(resourceUrl, false)
					: ModelBusConnector.asRepositoryContainer(resourceUrl, false);
			this.repositoryResources[0].setSelectedRevision(revision);
			this.repositoryResources[0].setPegRevision(revision);
		}

		public IRepositoryResource[] getRepositoryResources() {
			return this.repositoryResources;
		}

	}

	protected class FromDifferenceRepositoryResourceProvider extends AbstractActionOperation implements IRepositoryResourceWithStatusProvider {
		protected IRepositoryResource[]		repositoryResources;
		protected IRepositoryResource[]		repositoryResourcesToDelete;
		protected IRepositoryResource		newer;
		protected IRepositoryResource		older;
		protected HashMap<String, String>	url2status;

		public FromDifferenceRepositoryResourceProvider(ModelBusLogEntry[] logEntries) {// (HashMap<ModelBusLogPath, Long> paths, ModelBusLogEntry
																						// selectedLogEntry) {
			super("Operation.GetRepositoryResource");
			this.newer = HistoryActionManager.this.getResourceForSelectedRevision(logEntries[0]);
			this.older = HistoryActionManager.this.getResourceForSelectedRevision(logEntries[1]);
			this.url2status = new HashMap<String, String>();
		}

		public IRepositoryResourceProvider getDeletionsProvider() {
			return new IRepositoryResourceProvider() {
				public IRepositoryResource[] getRepositoryResources() {
					return FromDifferenceRepositoryResourceProvider.this.repositoryResourcesToDelete;
				}
			};
		}

		protected IRepositoryResource createResourceFor(int kind, String url) {
			IRepositoryResource retVal = null;
			if (kind == ModelBusEntry.Kind.FILE) {
				retVal = ModelBusConnector.asRepositoryFile(url, false);
			} else if (kind == ModelBusEntry.Kind.DIR) {
				retVal = ModelBusConnector.asRepositoryContainer(url, false);
			}
			if (retVal == null) {
				throw new RuntimeException(ModelBusTeamUIPlugin.instance().getResource("Error.CompareUnknownNodeKind"));
			}
			return retVal;
		}

		protected IRepositoryResource getResourceForStatus(ModelBusDiffStatus status) {
			String url = ModelBusUtility.decodeURL(status.pathNext);
			return this.createResourceFor(ModelBusUtility.getNodeKind(status.pathPrev, status.nodeKind, false), url);
		}

		protected void runImpl(IProgressMonitor monitor) throws Exception {
			HashSet<IRepositoryResource> resourcesToReturn = new HashSet<IRepositoryResource>();
			HashSet<IRepositoryResource> resourcesToDelete = new HashSet<IRepositoryResource>();
			ArrayList<ModelBusDiffStatus> statusesList = new ArrayList<ModelBusDiffStatus>();
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			final LocateResourceURLInHistoryOperation op = new LocateResourceURLInHistoryOperation(
					new IRepositoryResource[] { this.newer, this.older });
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					ProgressMonitorUtility.doTaskExternal(op, monitor);
				}
			}, monitor, 3);
			this.newer = op.getRepositoryResources()[0];
			this.older = op.getRepositoryResources()[1];
			ModelBusEntryRevisionReference refPrev = ModelBusUtility.getEntryRevisionReference(this.older);
			ModelBusEntryRevisionReference refNext = ModelBusUtility.getEntryRevisionReference(this.newer);
			ProgressMonitorUtility.setTaskInfo(monitor, this, ModelBusTeamPlugin.instance().getResource("Progress.Running"));
			try {
				if (ModelBusUtility.useSingleReferenceSignature(refPrev, refNext)) {
					ModelBusUtility.diffStatus(proxy, statusesList, refPrev, refPrev.revision, refNext.revision, Depth.INFINITY,
							IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(this, monitor, null, false));
				} else {
					ModelBusUtility.diffStatus(proxy, statusesList, refPrev, refNext, Depth.INFINITY, IModelBusConnector.Options.NONE,
							new ModelBusProgressMonitor(this, monitor, null, false));
				}
			} finally {
			}

			for (ModelBusDiffStatus status : statusesList) {
				IRepositoryResource resourceToAdd = this.getResourceForStatus(status);
				resourceToAdd.setSelectedRevision(this.newer.getSelectedRevision());
				resourceToAdd.setPegRevision(this.newer.getPegRevision());
				resourcesToReturn.add(resourceToAdd);
				this.url2status.put(resourceToAdd.getUrl(),
						ModelBusRemoteStorage.instance().getStatusString(status.propStatus, status.textStatus, true));
				if (status.textStatus == ModelBusEntryStatus.Kind.DELETED) {
					resourcesToDelete.add(resourceToAdd);
				}
			}
			this.repositoryResources = resourcesToReturn.toArray(new IRepositoryResource[0]);
			this.repositoryResourcesToDelete = resourcesToDelete.toArray(new IRepositoryResource[0]);
		}

		public IRepositoryResource[] getRepositoryResources() {
			return this.repositoryResources;
		}

		public Map<String, String> getStatusesMap() {
			return this.url2status;
		}

	}

}
