/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history.data;

import java.io.ByteArrayInputStream;

import org.eclipse.compare.IEditableContent;
import org.eclipse.compare.ITypedElement;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFileState;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.team.internal.core.history.LocalFileRevision;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.operation.UILoggedOperation;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;

/**
 * Local file revision representation. Wrapper for both IFileState and IFile.
 * 
 * @author Alexei Goncharov
 */
public class ModelBusLocalFileRevision extends LocalFileRevision implements IEditableContent {
	public ModelBusLocalFileRevision(IFile file) {
		super(file);
	}

	public ModelBusLocalFileRevision(IFileState fileState) {
		super(fileState);
	}

	public String getComment() {
		if (this.getFile() != null) {
			return ModelBusTeamUIPlugin.instance().getResource("ModelBusLocalFileRevision.CurrentVersion");
		}
		return "";
	}

	public String getContentIdentifier() {
		if (this.getFile() != null) {
			return "";
		}
		return "[" + DateFormatter.formatDate(this.getTimestamp()) + "]";
	}

	public boolean isEditable() {
		return this.isCurrentState();
	}

	public ITypedElement replace(ITypedElement dest, ITypedElement src) {
		return dest;
	}

	public void setContent(byte[] newContent) {
		if (this.isEditable()) {
			try {
				this.getFile().setContents(new ByteArrayInputStream(newContent), true, true, new NullProgressMonitor());
			} catch (CoreException ex) {
				UILoggedOperation.reportError(ModelBusTeamUIPlugin.instance().getResource("Operation.GetFileContent.SetContent"), ex);
			}
		}
	}

}
