/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history.filter;

import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;

/**
 * Log entry filter to remove revisions from view
 * 
 * @author Alexei Goncharov
 */
public class RevisionLogEntryFilter implements ILogEntryFilter {

	protected ModelBusRevision	from;
	protected ModelBusRevision	to;

	public boolean accept(ModelBusLogEntry logEntry) {
		ModelBusRevision current = logEntry.modelBusRevision;
		
		return (current.isGreaterThan(this.to) || current.isLowerThan(this.from));
	}

	/**
	 * Sets revisions range to exclude
	 * 
	 * @param from
	 *            - lower revision
	 * @param to
	 *            - higher revision
	 */
	public void setRevisionstoHide(ModelBusRevision from, ModelBusRevision to) {
		this.from = from;
		this.to = to;
	}

}
