/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history.model;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.team.core.history.IFileRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.history.data.ModelBusLocalFileRevision;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;

/**
 * Local log UI node
 * 
 * @author Alexander Gurov
 */
public class LocalLogNode extends AbstractLogNode {
	protected ModelBusLocalFileRevision	entry;

	public LocalLogNode(ModelBusLocalFileRevision entry, ILogNode parent) {
		super(parent);
		this.entry = entry;
	}

	public Object getAdapter(Class adapter) {
		if (adapter.equals(IFileRevision.class)) {
			return this.entry;
		}
		return null;
	}

	public ILogNode[] getChildren() {
		return null;
	}

	public Object getEntity() {
		return this.entry;
	}

	public boolean requiresBoldFont(ModelBusRevision currentRevision) {
		return this.entry.isCurrentState();
	}

	public ImageDescriptor getImageDescriptor() {
		return ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/history/local_rev.gif");
	}

	public String getLabel(int columnIndex, int labelType, ModelBusRevision currentRevision) {
		switch (columnIndex) {
			case ILogNode.COLUMN_DATE: {
				return DateFormatter.formatDate(this.entry.getTimestamp());
			}
			case ILogNode.COLUMN_REVISION: {
				if (this.entry.isCurrentState()) {
					if (!ModelBusRevision.INVALID_REVISION_STRING.equals(currentRevision)) {
						return ModelBusTeamUIPlugin.instance().getResource("LogMessagesComposite.CurrentRevision",
								new String[] { String.valueOf(currentRevision) });
					}
					return "*";
				}
				return "";
			}
			case ILogNode.COLUMN_COMMENT: {
				String retVal = this.entry.getComment();
				if (labelType == ILogNode.LABEL_TRIM) {
					return FileUtility.formatMultilineText(retVal);
				} else if (labelType == ILogNode.LABEL_FLAT) {
					return AbstractLogNode.flattenMultiLineText(retVal, " ");
				}
				return retVal;
			}
		}
		return "";
	}

	public int getType() {
		return ILogNode.TYPE_LOCAL;
	}

	public boolean hasChildren() {
		return false;
	}

	public String getAuthor() {
		return "";
	}

	public int getChangesCount() {
		return 0;
	}

	public String getComment() {
		return this.entry.getComment();
	}

	public ModelBusRevision getRevision() {
		return ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
	}

	public long getTimeStamp() {
		return this.entry.getTimestamp();
	}

	public int hashCode() {
		return this.entry.hashCode();
	}

	public boolean equals(Object obj) {
		if (obj instanceof LocalLogNode) {
			return this.entry.equals(((LocalLogNode) obj).entry);
		}
		return false;
	}

}
