/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history.model;

import org.eclipse.jface.resource.ImageDescriptor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;

/**
 * ModelBus log entry UI node
 * 
 * @author Alexander Gurov
 */
public class ModelBusLogNode extends AbstractLogNode {
	protected ModelBusLogEntry	entry;

	public ModelBusLogNode(ModelBusLogEntry entry, ILogNode parent) {
		super(parent);
		this.entry = entry;
	}

	public Object getAdapter(Class adapter) {
		if (adapter.equals(ModelBusLogEntry.class)) {
			return this.entry;
		}
		return null;
	}

	public ILogNode[] getChildren() {
		ModelBusLogEntry[] entries = this.entry.getChildren();
		ILogNode[] children = new ILogNode[entries.length];
		for (int i = 0; i < entries.length; i++) {
			children[i] = new ModelBusLogNode(entries[i], this);
		}
		return children;
	}

	public ImageDescriptor getImageDescriptor() {
		return ModelBusTeamUIPlugin.instance().getImageDescriptor(
				this.parent instanceof ModelBusLogNode ? "icons/objects/repository-gray.gif" : "icons/objects/repository.gif");
	}

	public boolean requiresBoldFont(ModelBusRevision currentRevision) {
		return !ModelBusRevision.INVALID_REVISION_STRING.equals(currentRevision) && this.entry.revision.equals(currentRevision);
	}

	public String getLabel(int columnIndex, int labelType, ModelBusRevision currentRevision) {
		switch (columnIndex) {
			case ILogNode.COLUMN_REVISION: {
				String retVal = String.valueOf(this.entry.revision);
				if (currentRevision.equals(this.entry.revision)) {
					retVal = "*" + retVal;
				}
				return retVal;
			}
			case ILogNode.COLUMN_DATE: {
				return this.entry.date == 0 ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoDate") : DateFormatter
						.formatDate(this.entry.date);
			}
			case ILogNode.COLUMN_CHANGES: {
				return String.valueOf(this.entry.changedPaths != null ? this.entry.changedPaths.length : 0);
			}
			case ILogNode.COLUMN_AUTHOR: {
				return this.entry.author == null || this.entry.author.length() == 0 ? ModelBusTeamPlugin.instance().getResource(
						"ModelBusInfo.NoAuthor") : this.entry.author;
			}
			case ILogNode.COLUMN_COMMENT: {
				String retVal = this.entry.message;
				if (retVal == null || retVal.length() == 0) {
					return ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoComment");
				}
				if (labelType == ILogNode.LABEL_TRIM) {
					return FileUtility.formatMultilineText(retVal);
				} else if (labelType == ILogNode.LABEL_FLAT) {
					return AbstractLogNode.flattenMultiLineText(retVal, " ");
				}
				return retVal;
			}
		}
		return "";
	}

	public Object getEntity() {
		return this.entry;
	}

	public int getType() {
		return ILogNode.TYPE_ModelBus;
	}

	public boolean hasChildren() {
		return this.entry.hasChildren();
	}

	public String getAuthor() {
		return this.entry.author;
	}

	public int getChangesCount() {
		return this.entry.changedPaths == null ? 0 : this.entry.changedPaths.length;
	}

	public String getComment() {
		return this.entry.message;
	}

	public ModelBusRevision getRevision() {
		return this.entry.modelBusRevision;
	}

	public long getTimeStamp() {
		return this.entry.date;
	}

	public int hashCode() {
		return this.entry.revision.hashCode();
	}

	public boolean equals(Object obj) {
		if (obj instanceof ModelBusLogNode) {
			return this.entry.revision == ((ModelBusLogNode) obj).entry.revision;
		}
		return false;
	}

}
