/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.info;

import org.eclipse.core.resources.IResource;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.internal.util.Util;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo;
import org.modelbus.team.eclipse.core.connector.ModelBusLock;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.local.InfoOperation;
import org.modelbus.team.eclipse.core.operation.local.property.GetPropertiesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.PropertiesComposite;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * This page allows to view working copy information for local resource
 * 
 * @author Alexander Gurov
 */
public class LocalInfoPage extends PropertyPage {
	protected PropertiesComposite	properties;

	public LocalInfoPage() {
		super();
	}

	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = layout.marginWidth = 0;
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		composite.setLayoutData(data);

		this.noDefaultAndApplyButton();

		IResource resource = (IResource) Util.getAdapter(this.getElement(), IResource.class);
		InfoOperation op = new InfoOperation(resource);
		UIMonitorUtility.doTaskBusyDefault(op);

		ILocalResource local = op.getLocal();

		Label description = new Label(composite, SWT.WRAP);
		description.setLayoutData(new GridData());
		description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LocalPath"));

		Text content = new Text(composite, SWT.WRAP);
		data = new GridData();
		data.widthHint = 300;
		content.setLayoutData(data);
		content.setEditable(false);
		content.setText(resource.getFullPath().toString());

		description = new Label(composite, SWT.WRAP);
		description.setLayoutData(new GridData());
		description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.State"));

		content = new Text(composite, SWT.SINGLE);
		content.setLayoutData(new GridData());
		content.setEditable(false);
		content.setText(ModelBusUtility.getStatusText(local.getStatus()));

		ModelBusEntryInfo info = op.getInfo();
		if (IStateFilter.SF_ONREPOSITORY.accept(local) && info != null) {
			// add space
			new Label(composite, SWT.WRAP);
			new Label(composite, SWT.WRAP);

			description = new Label(composite, SWT.WRAP);
			description.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
			description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.ResourceURL"));

			content = new Text(composite, SWT.WRAP);
			data = new GridData();
			data.widthHint = 300;
			content.setLayoutData(data);
			content.setEditable(false);
			content.setText(ModelBusUtility.decodeURL(info.url));

			description = new Label(composite, SWT.WRAP);
			description.setLayoutData(new GridData());
			description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.Revision"));

			content = new Text(composite, SWT.SINGLE);
			content.setLayoutData(new GridData());
			content.setEditable(false);
			content.setText(String.valueOf(info.revision));

			description = new Label(composite, SWT.WRAP);
			description.setLayoutData(new GridData());
			description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LastChangedAuthor"));

			content = new Text(composite, SWT.SINGLE);
			content.setLayoutData(new GridData());
			content.setEditable(false);
			content.setText(info.lastChangedAuthor == null ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor")
					: info.lastChangedAuthor);

			description = new Label(composite, SWT.WRAP);
			description.setLayoutData(new GridData());
			description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LastChangedDate"));

			content = new Text(composite, SWT.SINGLE);
			content.setLayoutData(new GridData());
			content.setEditable(false);
			content.setText(info.lastChangedDate == 0 ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoDate") : DateFormatter
					.formatDate(info.lastChangedDate));

			description = new Label(composite, SWT.WRAP);
			description.setLayoutData(new GridData());
			description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LastChangedRevision"));

			content = new Text(composite, SWT.SINGLE);
			content.setLayoutData(new GridData());
			content.setEditable(false);
			content.setText(String.valueOf(info.lastChangedRevision));

			ModelBusLock lock = info.lock;
			if (lock != null) {
				// add space
				new Label(composite, SWT.WRAP);
				new Label(composite, SWT.WRAP);

				description = new Label(composite, SWT.WRAP);
				description.setLayoutData(new GridData());
				description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LockOwner"));

				content = new Text(composite, SWT.SINGLE);
				content.setLayoutData(new GridData());
				content.setEditable(false);
				content.setText(lock.owner == null ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor") : lock.owner);

				description = new Label(composite, SWT.WRAP);
				description.setLayoutData(new GridData());
				description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LockComment"));

				content = new Text(composite, SWT.SINGLE);
				content.setLayoutData(new GridData());
				content.setEditable(false);
				content.setText(lock.comment == null ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoComment") : lock.comment);

				description = new Label(composite, SWT.WRAP);
				description.setLayoutData(new GridData());
				description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LockCreationDate"));

				content = new Text(composite, SWT.SINGLE);
				content.setLayoutData(new GridData());
				content.setEditable(false);
				content.setText(lock.creationDate == 0 ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor") : DateFormatter
						.formatDate(lock.creationDate));
				if (lock.expirationDate != 0) {
					description = new Label(composite, SWT.WRAP);
					description.setLayoutData(new GridData());
					description.setText(ModelBusTeamUIPlugin.instance().getResource("LocalInfoPage.LockExpirationDate"));

					content = new Text(composite, SWT.SINGLE);
					content.setLayoutData(new GridData());
					content.setEditable(false);
					content.setText(lock.expirationDate == 0 ? ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoDate") : DateFormatter
							.formatDate(lock.expirationDate));
				}
			}
		}
		if (IStateFilter.SF_VERSIONED.accept(local)) {
			// add space
			new Label(composite, SWT.WRAP);
			new Label(composite, SWT.WRAP);

			Composite group = new Composite(composite, SWT.BORDER);
			data = new GridData(GridData.FILL_BOTH);
			data.horizontalSpan = 2;
			group.setLayoutData(data);
			layout = new GridLayout();
			layout.marginHeight = layout.marginWidth = 0;
			group.setLayout(layout);
			this.properties = new PropertiesComposite(group);
			IResourcePropertyProvider propertyProvider = new GetPropertiesOperation(resource);
			UIMonitorUtility.doTaskBusyDefault(propertyProvider);
			this.properties.setResource(resource, propertyProvider);
			UIMonitorUtility.doTaskBusyDefault(this.properties.getRefreshViewOperation());
			this.properties.setLayoutData(new GridData(GridData.FILL_BOTH));
		}

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.ModelBusInfoContext");

		return composite;
	}

}
