/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetLogMessagesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Corrects Log Message revision
 * 
 * @author Alexander Gurov
 */
public class CorrectRevisionOperation extends AbstractActionOperation {
	protected IRepositoryResource[]		repositoryResources;
	protected String[]					knownRevisions;
	protected GetLogMessagesOperation[]	msgsOps;
	protected IResource[]				resources;

	public CorrectRevisionOperation(GetLogMessagesOperation msgsOp, IRepositoryResource repositoryResource, ModelBusRevision knownRevision, IResource resource) {
		this(msgsOp == null ? null : new GetLogMessagesOperation[] { msgsOp }, new IRepositoryResource[] { repositoryResource },
				new String[] { knownRevision.getRevision() }, new IResource[] { resource });
	}

	public CorrectRevisionOperation(GetLogMessagesOperation[] msgsOps, IRepositoryResource[] repositoryResources, String[] knownRevisions,
			IResource[] resources) {
		super("Operation.CorrectRevision");
		this.repositoryResources = repositoryResources;
		this.knownRevisions = knownRevisions;
		this.msgsOps = msgsOps;
		this.resources = resources;
	}

	public int getOperationWeight() {
		if (this.msgsOps == null) {
			return 0;
		}
		return super.getOperationWeight();
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		boolean hasWarning = false;
		for (int i = 0; i < this.repositoryResources.length; i++) {
			if (!this.repositoryResources[i].exists() && this.resources != null && this.resources[i] != null
					&& this.resources[i].getType() != IResource.PROJECT) {
				// calculate peg revision for the repository resource
				ILocalResource parent = ModelBusRemoteStorage.instance().asLocalResourceAccessible(this.resources[i].getParent());
				ILocalResource self = ModelBusRemoteStorage.instance().asLocalResourceAccessible(this.resources[i]);
				boolean switchedStateEquals = (parent.getChangeMask() & ILocalResource.IS_SWITCHED) == (self.getChangeMask() & ILocalResource.IS_SWITCHED);
				if (switchedStateEquals) {
					ModelBusRevision parentRevision = parent.getRevision();
					ModelBusRevision selfRevision = self.getRevision();
					ModelBusRevision revision = parentRevision.isGreaterThan(selfRevision) ? parentRevision : selfRevision;
					if (revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
						this.repositoryResources[i].setPegRevision(revision);
					}
				} else {
					this.repositoryResources[i].setPegRevision(self.getRevision());
				}
			}
			if (!this.repositoryResources[i].exists() && this.knownRevisions[i] != ModelBusRevision.INVALID_REVISION_STRING) {
				hasWarning = true;
				ModelBusRevision rev = ModelBusRevision.fromString(this.knownRevisions[i]);
				this.repositoryResources[i].setSelectedRevision(rev);
				this.repositoryResources[i].setPegRevision(rev);
				if (this.msgsOps != null) {
					this.msgsOps[i].setStartRevision(rev);
				}
			}
		}
		if (hasWarning) {
			UIMonitorUtility.getDisplay().syncExec(new Runnable() {
				public void run() {
					boolean one = CorrectRevisionOperation.this.repositoryResources.length == 1;
					MessageDialog dlg = new MessageDialog(UIMonitorUtility.getShell(), CorrectRevisionOperation.this
							.getOperationResource(one ? "Title.Single" : "Title.Multi"), null, CorrectRevisionOperation.this
							.getOperationResource(one ? "Message.Single" : "Message.Multi"), MessageDialog.WARNING, new String[] {
							IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 0);
					if (dlg.open() != 0) {
						monitor.setCanceled(true);
					}
				}
			});
		}
	}

}
