/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.common;

import java.util.Set;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.CommentComposite;
import org.modelbus.team.eclipse.ui.composite.ResourceSelectionComposite;
import org.modelbus.team.eclipse.ui.composite.RevisionComposite;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.event.IResourceSelectionChangeListener;
import org.modelbus.team.eclipse.ui.event.ResourceSelectionChangedEvent;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UserInputHistory;
import org.modelbus.team.eclipse.ui.verifier.AbsolutePathVerifier;
import org.modelbus.team.eclipse.ui.verifier.AbstractVerifier;
import org.modelbus.team.eclipse.ui.verifier.CompositeVerifier;
import org.modelbus.team.eclipse.ui.verifier.NonEmptyFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.URLVerifier;

/**
 * Abstract Branch/Tag panel
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractBranchTagPanel extends AbstractDialogPanel {
	protected Button						startWithCheck;
	protected Button						freezeExternalsCheck;
	protected Combo							destinationCombo;
	protected UserInputHistory				resourceNameHistory;
	protected CommentComposite				comment;
	protected String						destinationUrl;

	protected IRepositoryRoot				root;
	protected String						nationalizationId;
	protected boolean						startsWith;
	protected boolean						freezeExternals;
	protected Set							existingNodesNamesSet;
	protected boolean						considerStructure;
	protected String						historyName;

	// used for revision selection for repository resources
	protected IRepositoryResource[]			selectedRemoteResources;
	protected RevisionComposite				revisionComposite;

	protected ResourceSelectionComposite	resourceSelection;
	protected IResource[]					newResources;
	protected boolean						disableSwitch;

	// public AbstractBranchTagPanel(IRepositoryRoot root, boolean showStartsWith, Set existingNames, String nationalizationId, String historyName) {
	// this(root, showStartsWith, existingNames, nationalizationId, historyName, new IResource[0]);
	// }
	//
	// public AbstractBranchTagPanel(IRepositoryRoot root, boolean showStartsWith, Set existingNames, String nationalizationId, String historyName,
	// IResource[] resources) {
	// super();
	// this.nationalizationId = nationalizationId;
	// this.historyName = historyName;
	//
	// this.newResources = FileUtility.getResourcesRecursive(resources, IStateFilter.SF_NEW, IResource.DEPTH_INFINITE);
	// this.disableSwitch = FileUtility.checkForResourcesPresence(resources, new IStateFilter.AbstractStateFilter() {
	// protected boolean acceptImpl(ILocalResource local, IResource resource, String state, int mask) {
	// return state == IStateFilter.ST_ADDED;
	// }
	// protected boolean allowsRecursionImpl(ILocalResource local, IResource resource, String state, int mask) {
	// return true;
	// }
	// }, IResource.DEPTH_INFINITE);
	//
	// this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Title");
	// this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Description");
	// if (ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
	// ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME)) {
	// this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".MessageAuto");
	// }
	// else {
	// this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Message");
	// }
	//
	// this.existingNodesNamesSet = existingNames;
	// this.root = root;
	// this.startsWith = showStartsWith;
	// this.considerStructure =
	// // root.getRepositoryLocation().isStructureEnabled()
	// // &&
	// ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
	// ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);
	// }

	public AbstractBranchTagPanel(IRepositoryRoot root, boolean showStartsWith, Set existingNames, String nationalizationId, String historyName,
			IRepositoryResource[] selectedRemoteResources) {
		this(root, showStartsWith, existingNames, nationalizationId, historyName, new IResource[0], selectedRemoteResources);
	}

	public AbstractBranchTagPanel(IRepositoryRoot root, boolean showStartsWith, Set existingNames, String nationalizationId, String historyName,
			IResource[] resources, IRepositoryResource[] selectedRemoteResources) {
		super();
		this.nationalizationId = nationalizationId;
		this.historyName = historyName;
		this.selectedRemoteResources = selectedRemoteResources;

		this.newResources = FileUtility.getResourcesRecursive(resources, IStateFilter.SF_NEW, IResource.DEPTH_INFINITE);
		this.disableSwitch = FileUtility.checkForResourcesPresence(resources, new IStateFilter.AbstractStateFilter() {
			protected boolean acceptImpl(ILocalResource local, IResource resource, String state, int mask) {
				return state == IStateFilter.ST_ADDED;
			}

			protected boolean allowsRecursionImpl(ILocalResource local, IResource resource, String state, int mask) {
				return true;
			}
		}, IResource.DEPTH_INFINITE);

		this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Title");
		this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Description");
		if (ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME)) {
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".MessageAuto");
		} else {
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Message");
		}
		// if (!showStartsWith) {
		//			this.defaultMessage += " " + SVNUIMessages.AbstractBranchTagPanel_Message; //$NON-NLS-1$
		// }

		this.existingNodesNamesSet = existingNames;
		this.root = root;
		this.startsWith = showStartsWith;
		this.considerStructure = ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);

		// this.paneParticipantHelper = new PaneParticipantHelper();
	}

	public IResource[] getSelectedResources() {
		return this.resourceSelection == null ? null : this.resourceSelection.getSelectedResources();
	}

	public IResource[] getNotSelectedResources() {
		return this.resourceSelection == null ? null : this.resourceSelection.getNotSelectedResources();
	}

	public boolean isFreezeExternals() {
		return this.freezeExternals;
	}

	public String getMessage() {
		return this.comment.getMessage();
	}

	public IRepositoryResource getDestination() {
		this.destinationUrl = this.destinationUrl.trim();
		while (this.destinationUrl.endsWith("/") || this.destinationUrl.endsWith("\\")) {
			this.destinationUrl = this.destinationUrl.substring(0, this.destinationUrl.length() - 1);
		}
		return ModelBusConnector.asRepositoryContainer(this.destinationUrl, false);
	}

	public boolean isStartWithSelected() {
		return this.startsWith;
	}

	public Point getPrefferedSizeImpl() {
		return new Point(this.newResources != null && this.newResources.length > 0 ? 625 : 525, SWT.DEFAULT);
	}

	public void postInit() {
		super.postInit();
		this.comment.postInit(this.manager);
	}

	public void createControlsImpl(Composite parent) {
		GridData data = null;

		GridLayout layout = new GridLayout();
		Composite select = null;
		String substitutionUppercase = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".NodeName");
		if (this.startsWith) {
			select = new Group(parent, SWT.NULL);
			layout.numColumns = 2;
			((Group) select).setText(this.considerStructure ? substitutionUppercase : ModelBusTeamUIPlugin.instance().getResource(
					this.nationalizationId + ".Location.Group"));
		} else {
			select = new Composite(parent, SWT.NONE);
			layout.marginHeight = 0;
			layout.marginWidth = 0;
			layout.numColumns = 3;
		}
		select.setLayout(layout);
		select.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		if (!this.startsWith) {
			Label description = new Label(select, SWT.NONE);
			data = new GridData();
			description.setLayoutData(data);
			description.setText(this.considerStructure ? substitutionUppercase : ModelBusTeamUIPlugin.instance().getResource(
					this.nationalizationId + ".Location.Field"));
		}
		this.createTopPart(select, substitutionUppercase);

		if (this.startsWith) {
			Composite inner = new Composite(select, SWT.NONE);
			data = new GridData(GridData.FILL_HORIZONTAL);
			inner.setLayoutData(data);
			layout = new GridLayout();
			layout.marginHeight = layout.marginWidth = 0;
			inner.setLayout(layout);

			this.startWithCheck = new Button(inner, SWT.CHECK);
			data = new GridData(GridData.FILL_HORIZONTAL);
			this.startWithCheck.setLayoutData(data);
			this.startWithCheck.setText(ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".StartsWith"));
			this.startWithCheck.setSelection(false);
			this.startWithCheck.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					AbstractBranchTagPanel.this.validateContent();
				}
			});

			this.freezeExternalsCheck = new Button(inner, SWT.CHECK);
			data = new GridData(GridData.FILL_HORIZONTAL);
			this.freezeExternalsCheck.setLayoutData(data);
			this.freezeExternalsCheck.setText(ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".FreezeExternals"));
			this.freezeExternalsCheck.setSelection(false);
		}

		// revision selection
		if (!this.startsWith) {
			IRepositoryResource selectedRemoteResource;
			if (this.selectedRemoteResources.length == 1) {
				selectedRemoteResource = this.selectedRemoteResources[0];
			} else if (this.selectedRemoteResources.length > 1) {
				selectedRemoteResource = this.selectedRemoteResources[0].getRoot();
			} else {
				// selectedRemoteResource = this.root.getRepositoryLocation().getRoot();
				selectedRemoteResource = this.root.getRoot();
				selectedRemoteResource.setPegRevision(this.root.getPegRevision());
				selectedRemoteResource.setSelectedRevision(this.root.getSelectedRevision());
			}
			this.revisionComposite = new RevisionComposite(parent, this, false, null, ModelBusRevision.HEAD, false);
			layout = new GridLayout();
			layout.marginHeight = layout.marginWidth = 0;
			data = new GridData(GridData.FILL_HORIZONTAL);
			this.revisionComposite.setLayout(layout);
			this.revisionComposite.setLayoutData(data);
			this.revisionComposite.setSelectedResource(selectedRemoteResource);
		}

		SashForm splitter = new SashForm(parent, SWT.VERTICAL);
		data = new GridData(GridData.FILL_BOTH);
		splitter.setLayoutData(data);
		layout = new GridLayout();
		layout.marginHeight = layout.marginWidth = 0;
		layout.verticalSpacing = 3;
		splitter.setLayout(layout);

		Group group = new Group(splitter, SWT.NULL);
		group.setLayout(new GridLayout());
		data = new GridData(GridData.FILL_BOTH);
		group.setLayoutData(data);
		group.setText(ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Comment"));

		this.comment = new CommentComposite(group, this);
		data = new GridData(GridData.FILL_BOTH);
		this.comment.setLayoutData(data);

		if (this.startsWith && this.newResources != null && this.newResources.length > 0) {
			this.resourceSelection = new ResourceSelectionComposite(splitter, SWT.NONE, this.newResources, true);
			data = new GridData(GridData.FILL_BOTH);
			this.resourceSelection.setLayoutData(data);
			this.resourceSelection.addResourcesSelectionChangedListener(new IResourceSelectionChangeListener() {
				public void resourcesSelectionChanged(ResourceSelectionChangedEvent event) {
					AbstractBranchTagPanel.this.validateContent();
				}
			});
			this.attachTo(this.resourceSelection, new AbstractVerifier() {
				protected String getWarningMessage(Control input) {
					IResource[] resources = AbstractBranchTagPanel.this.resourceSelection.getSelectedResources();
					if ((resources != null && resources.length != 0 || AbstractBranchTagPanel.this.disableSwitch)
							&& AbstractBranchTagPanel.this.startWithCheck.getSelection()) {
						return AbstractBranchTagPanel.this.defaultMessage + " "
								+ ModelBusTeamUIPlugin.instance().getResource(AbstractBranchTagPanel.this.nationalizationId + ".Warning");
					}
					return null;
				}

				protected String getErrorMessage(Control input) {
					return null;
				}
			});
			splitter.setWeights(new int[] { 1, 1 });
		} else {
			splitter.setWeights(new int[] { 1 });
		}
	}

	protected Composite createTopPart(Composite select, final String substitutionUppercase) {
		this.destinationUrl = this.root.getUrl();

		this.destinationCombo = new Combo(select, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		this.destinationCombo.setLayoutData(data);

		Button browse = new Button(select, SWT.PUSH);
		browse.setText(ModelBusTeamUIPlugin.instance().getResource("Button.Browse"));
		data = new GridData();
		data.widthHint = DefaultDialog.computeButtonWidth(browse);
		browse.setLayoutData(data);

		CompositeVerifier verifier = new CompositeVerifier();

		if (!this.considerStructure) {
			this.resourceNameHistory = new UserInputHistory(this.historyName);
			this.destinationCombo.setText(this.destinationUrl);
			String name = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".Location.Verifier");
			verifier.add(new URLVerifier(name));
			verifier.add(new AbsolutePathVerifier(name));
			verifier.add(new AbstractVerifier() {
				protected String getErrorMessage(Control input) {
					String url = ModelBusConnector.getRepositoryRootUrl();
					// AbstractBranchTagPanel.this.root.getRepositoryLocation().getUrl();
					if (!AbstractBranchTagPanel.this.destinationCombo.getText().startsWith(url)) {
						return ModelBusTeamUIPlugin.instance().getResource(
								AbstractBranchTagPanel.this.nationalizationId + ".Location.Verifier.DoesNotCorresponds",
								new String[] { AbstractBranchTagPanel.this.destinationCombo.getText(), url });
					}
					if (AbstractBranchTagPanel.this.startsWith) {
						if (!AbstractBranchTagPanel.this.destinationCombo.getText().startsWith(AbstractBranchTagPanel.this.root.getUrl())) {
							AbstractBranchTagPanel.this.startWithCheck.setSelection(false);
							AbstractBranchTagPanel.this.startWithCheck.setEnabled(false);
						} else {
							AbstractBranchTagPanel.this.startWithCheck.setEnabled(true);
						}
					}
					if (AbstractBranchTagPanel.this.root.getUrl().equals(
							ModelBusUtility.normalizeURL(AbstractBranchTagPanel.this.destinationCombo.getText()))) {
						return ModelBusTeamUIPlugin.instance().getResource(
								AbstractBranchTagPanel.this.nationalizationId + ".Location.Verifier.NoTagName");
					}
					return null;
				}

				protected String getWarningMessage(Control input) {
					return null;
				}
			});

			browse.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					RepositoryTreePanel panel = new RepositoryTreePanel(ModelBusTeamUIPlugin.instance().getResource(
							AbstractBranchTagPanel.this.nationalizationId + ".SelectionProposal"), ModelBusTeamUIPlugin.instance().getResource(
							"RepositoryBrowsingPanel.Description"), ModelBusTeamUIPlugin.instance().getResource("RepositoryBrowsingPanel.Message"),
							null, true
					// ,
					// AbstractBranchTagPanel.this.root.getRepositoryLocation()
					);
					DefaultDialog browser = new DefaultDialog(AbstractBranchTagPanel.this.manager.getShell(), panel);
					if (browser.open() == 0) {
						IRepositoryResource selected = panel.getSelectedResource();
						if (selected != null) {
							AbstractBranchTagPanel.this.destinationCombo.setText(selected.getUrl());
						}
						AbstractBranchTagPanel.this.validateContent();
					}
				}
			});
		} else {
			this.resourceNameHistory = new UserInputHistory(this.historyName + "Name");

			String name = ModelBusTeamUIPlugin.instance().getResource(this.nationalizationId + ".NodeName.Verifier");
			verifier.add(new NonEmptyFieldVerifier(name) {
				protected String getErrorMessageImpl(Control input) {
					String msg = super.getErrorMessageImpl(input);
					if (msg == null) {
						if (new Path(this.getText(input)).segmentCount() == 0) {
							return NonEmptyFieldVerifier.ERROR_MESSAGE;
						}
					}
					return msg;
				}
			});
			verifier.add(new AbsolutePathVerifier(name));
			verifier.add(new AbstractVerifier() {
				protected String getErrorMessage(Control input) {
					return null;
				}

				protected String getWarningMessage(Control input) {
					String name = AbstractBranchTagPanel.this.destinationCombo.getText();
					if (AbstractBranchTagPanel.this.existingNodesNamesSet != null && AbstractBranchTagPanel.this.existingNodesNamesSet.contains(name)) {
						return ModelBusTeamUIPlugin.instance().getResource(
								AbstractBranchTagPanel.this.nationalizationId + ".NodeName.Verifier.Error.Exists", new String[] { name });
					}
					return null;
				}
			});

			browse.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					RepositoryTreePanel panel = new RepositoryTreePanel(ModelBusTeamUIPlugin.instance().getResource(
							AbstractBranchTagPanel.this.nationalizationId + ".SelectionProposal"), ModelBusTeamUIPlugin.instance().getResource(
							"RepositoryBrowsingPanel.Description"), ModelBusTeamUIPlugin.instance().getResource("RepositoryBrowsingPanel.Message"),
							null, true, AbstractBranchTagPanel.this.root.getRoot());
					DefaultDialog browser = new DefaultDialog(AbstractBranchTagPanel.this.manager.getShell(), panel);
					if (browser.open() == 0) {
						IRepositoryResource selected = panel.getSelectedResource();
						if (selected != null) {
							AbstractBranchTagPanel.this.destinationCombo.setText(selected.getUrl().substring(
									AbstractBranchTagPanel.this.root.getUrl().length() + 1));
						}
						AbstractBranchTagPanel.this.validateContent();
					}
				}
			});
		}
		this.destinationCombo.setVisibleItemCount(this.resourceNameHistory.getDepth());
		this.destinationCombo.setItems(this.resourceNameHistory.getHistory());

		this.attachTo(this.destinationCombo, verifier);

		return select;
	}

	protected void saveChangesImpl() {
		if (!this.considerStructure) {
			this.destinationUrl = this.destinationCombo.getText();
			this.resourceNameHistory.addLine(this.destinationUrl);
		} else {
			this.destinationUrl = this.destinationUrl + "/" + this.destinationCombo.getText();
			this.resourceNameHistory.addLine(this.destinationCombo.getText());
		}

		this.comment.saveChanges();
		if (this.startWithCheck != null) {
			this.startsWith = this.startWithCheck.getSelection();
			this.freezeExternals = this.freezeExternalsCheck.getSelection();
		} else {
			this.startsWith = false;
			this.freezeExternals = false;
		}
	}

	protected void cancelChangesImpl() {
		this.comment.cancelChanges();
	}

	public ModelBusRevision getRevisionForRemoteResources() {
		return this.revisionComposite != null ? this.revisionComposite.getSelectedRevision() : null;
	}

}
