/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.common;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.RevisionComposite;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;

/**
 * Panel to type or select one of existent revisions
 * 
 * @author Sergiy Logvin
 */
public class InputRevisionPanel extends AbstractDialogPanel {

	protected IRepositoryResource	resource;
	protected ModelBusRevision		selectedRevision;
	protected RevisionComposite		revComposite;

	public InputRevisionPanel(IRepositoryResource resource, String dialogTitle) {
		super();
		this.resource = resource;
		this.dialogTitle = dialogTitle;
		this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("InputRevisionPanel.Description");
		this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("InputRevisionPanel.Message");
	}

	public ModelBusRevision getSelectedRevision() {
		return this.selectedRevision;
	}

	protected void createControlsImpl(Composite parent) {
		this.revComposite = new RevisionComposite(parent, this, false, new String[] {
				ModelBusTeamUIPlugin.instance().getResource("InputRevisionPanel.Caption.First"),
				ModelBusTeamUIPlugin.instance().getResource("InputRevisionPanel.Caption.Second") }, ModelBusRevision.HEAD, false);
		this.revComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		this.revComposite.setSelectedResource(this.resource);
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.revisionDialogContext";
	}

	protected void saveChangesImpl() {
		this.resource = this.revComposite.getSelectedResource();
		this.selectedRevision = this.revComposite.getSelectedRevision();
	}

	protected void cancelChangesImpl() {
	}

}
