/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.local;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.BranchTagSelectionComposite;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;

/**
 * Panel for the Replace With Branch/Tag dialog
 * 
 * @author Alexei Goncharov
 */
public class ReplaceBranchTagPanel extends AbstractDialogPanel {
	protected IRepositoryResource			selectedResource;
	protected int							type;
	protected ModelBusRevision			currentRevision;
	protected String						historyKey;
	protected BranchTagSelectionComposite	selectionComposite;

	public ReplaceBranchTagPanel(IRepositoryResource baseResource, ModelBusRevision currentRevision, int type) {
		super();
		this.selectedResource = baseResource;
		this.type = type;
		if (type == BranchTagSelectionComposite.BRANCH_OPERATED) {
			this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource("Replace.Branch.Title");
			this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("Replace.Branch.Description");
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("Replace.Branch.Message");
			this.historyKey = "branchReplace";
		} else {
			this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource("Replace.Tag.Title");
			this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("Replace.Tag.Description");
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("Replace.Tag.Message");
			this.historyKey = "tagReplace";
		}
	}

	protected void createControlsImpl(Composite parent) {
		GridData data = null;
		this.selectionComposite = new BranchTagSelectionComposite(parent, SWT.NONE, this.selectedResource, this.historyKey, this, this.type);
		data = new GridData(GridData.FILL_HORIZONTAL);
		this.selectionComposite.setLayoutData(data);
		this.selectionComposite.setCurrentRevision(this.currentRevision);
	}

	public IRepositoryResource getSelectedResource() {
		return this.selectionComposite.getSelectedResource();
	}

	protected void saveChangesImpl() {
		this.selectionComposite.saveChanges();
	}

	protected void cancelChangesImpl() {
	}

}