/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.remote;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.DepthSelectionComposite;
import org.modelbus.team.eclipse.ui.composite.RevisionComposite;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;
import org.modelbus.team.eclipse.ui.verifier.ExistingResourceVerifier;

/**
 * Export panel
 * 
 * @author Sergiy Logvin
 */
public class ExportPanel extends AbstractDialogPanel {
	protected Text						locationField;
	protected String					location;
	protected RevisionComposite			revisionComposite;
	protected IRepositoryResource		selectedResource;
	protected DepthSelectionComposite	depthSelector;

	public ExportPanel(IRepositoryResource baseResource) {
		super();
		this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource("ExportPanel.Title");
		this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("ExportPanel.Description");
		this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("ExportPanel.Message");
		this.selectedResource = baseResource;
	}

	public ModelBusRevision getSelectedRevision() {
		return this.revisionComposite != null ? this.revisionComposite.getSelectedRevision() : ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
	}

	protected void saveChangesImpl() {
		this.location = this.locationField.getText();
	}

	protected void cancelChangesImpl() {
	}

	public int getDepth() {
		if (this.depthSelector == null) {
			return Depth.INFINITY;
		}
		return this.depthSelector.getDepth();
	}

	public void createControlsImpl(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite folderComposite = new Composite(parent, SWT.NULL);
		layout = new GridLayout();
		layout.numColumns = 3;
		layout.marginHeight = layout.marginWidth = 0;
		folderComposite.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		folderComposite.setLayoutData(data);

		Label label = new Label(folderComposite, SWT.NONE);
		data = new GridData();
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.Folder"));

		this.locationField = new Text(folderComposite, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		this.locationField.setLayoutData(data);
		this.attachTo(this.locationField, new ExistingResourceVerifier(label.getText(), false));

		Button browseButton = new Button(folderComposite, SWT.PUSH);
		browseButton.setText(ModelBusTeamUIPlugin.instance().getResource("Button.Browse"));
		data = new GridData();
		data.widthHint = DefaultDialog.computeButtonWidth(browseButton);
		browseButton.setLayoutData(data);
		browseButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				DirectoryDialog fileDialog = new DirectoryDialog(ExportPanel.this.manager.getShell());
				fileDialog.setText(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder"));
				fileDialog.setMessage(ModelBusTeamUIPlugin.instance().getResource("ExportPanel.ExportFolder.Msg"));
				String path = fileDialog.open();
				if (path != null) {
					ExportPanel.this.locationField.setText(path);
				}
			}
		});

		if (this.selectedResource != null) {
			this.revisionComposite = new RevisionComposite(parent, this, false, null, ModelBusRevision.HEAD, false);
			data = new GridData(GridData.FILL_HORIZONTAL);
			this.revisionComposite.setLayoutData(data);
			this.revisionComposite.setSelectedResource(this.selectedResource);
		}

		if (this.selectedResource instanceof IRepositoryContainer || this.selectedResource == null) {
			Label separator = new Label(parent, SWT.HORIZONTAL | SWT.SEPARATOR);
			separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			separator.setVisible(false);

			this.depthSelector = new DepthSelectionComposite(parent, SWT.NONE);
			data = new GridData(GridData.FILL_HORIZONTAL);
			this.depthSelector.setLayoutData(data);
		}
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.remote_exportDialogContext";
	}

	public String getLocation() {
		return this.location;
	}

}
