/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Dann Martens - [patch] Text decorations 'ascendant' variable
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.preferences;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.RGB;
import org.modelbus.team.eclipse.ui.decorator.TextVariableSetProvider;

/**
 * ModelBus Team plugin preference names
 * 
 * @author Alexander Gurov
 */
public final class ModelBusTeamPreferences {
	public static final String	DECORATION_BASE										= "preference.decoration.";
	public static final String	REPOSITORY_BASE										= "preference.repository.";
	public static final String	SYNCHRONIZE_BASE									= "preference.synchronize.";
	public static final String	HISTORY_BASE										= "preference.history.";
	public static final String	DATE_FORMAT_BASE									= "preference.dateFormat.";
	public static final String	PROPERTIES_BASE										= "preference.properties.";
	public static final String	MAILREPORTER_BASE									= "preference.mailreporter.";
	public static final String	COMMENT_TEMPLATES_BASE								= "preference.templates";
	public static final String	COMMIT_DIALOG_BASE									= "preference.commitDialog.";
	public static final String	MERGE_BASE											= "preference.merge.";
	public static final String	CHECKOUT_BASE										= "preference.checkout.";
	public static final String	CONSOLE_BASE										= "preference.console.";
	public static final String	CORE_BASE											= "preference.core.";
	public static final String	AUTO_PROPERTIES_BASE								= "preference.autoproperties";
	public static final String	CUSTOM_PROPERTIES_BASE								= "preference.customproperties";
	public static final String	BEHAVIOUR_BASE										= "preference.";

	public static final String	BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_NAME			= "resourceSelection.selectNew";
	public static final String	BEHAVIOUR_DO_NOT_SELECT_EXTERNALS_NAME				= "resourceSelection.treatExternalAsLocal";
	public static final String	BEHAVIOUR_ENABLE_AUTO_SHARE_NAME					= "share.enableAuto";
	public static final String	BEHAVIOUR_COMPUTE_KEYWORDS_NAME						= "keywords.computeValues";
	public static final String	BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_NAME		= "sorting.case.insensitive";
	public static final String	BEHAVIOUR_IGNORE_EXTERNALS_NAME						= "externals.ignore";										//$NON-NLS-1$

	public static final boolean	BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_DEFAULT	= true;
	public static final boolean	BEHAVIOUR_COMPUTE_KEYWORDS_DEFAULT					= true;
	public static final boolean	BEHAVIOUR_ENABLE_AUTO_SHARE_DEFAULT					= true;
	public static final boolean	BEHAVIOUR_DO_NOT_SELECT_EXTERNAL_DEFAULT			= true;
	public static final boolean	BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_DEFAULT		= true;
	public static final boolean	BEHAVIOUR_IGNORE_EXTERNALS_DEFAULT					= true;

	public static final String	CONSOLE_AUTOSHOW_TYPE_NAME							= "autoshow";
	public static final String	CONSOLE_ENABLED_NAME								= "enabled";
	public static final String	CONSOLE_HYPERLINKS_ENABLED_NAME						= "hyperlinksEnabled";
	public static final String	CONSOLE_FONT_NAME									= "font";
	public static final String	CONSOLE_WRAP_ENABLED_NAME							= "wrapEnabled";
	public static final String	CONSOLE_WRAP_WIDTH_NAME								= "wrapWidth";
	public static final String	CONSOLE_LIMIT_ENABLED_NAME							= "limitEnabled";
	public static final String	CONSOLE_LIMIT_VALUE_NAME							= "limitRange";

	public static final String	CORE_ModelBusCONNECTOR_NAME							= "modelbusconnector";

	// public static final String CORE_ModelBusCONNECTOR_DEFAULT = IModelBusConnectorFactory.DEFAULT_ID;

	public static final String	CONSOLE_ERR_COLOR_NAME								= "error";
	public static final String	CONSOLE_WRN_COLOR_NAME								= "warning";
	public static final String	CONSOLE_OK_COLOR_NAME								= "ok";
	public static final String	CONSOLE_CMD_COLOR_NAME								= "command";

	public static final int		CONSOLE_AUTOSHOW_TYPE_NEVER							= 0;
	public static final int		CONSOLE_AUTOSHOW_TYPE_ALWAYS						= 1;
	public static final int		CONSOLE_AUTOSHOW_TYPE_ERROR							= 2;
	public static final int		CONSOLE_AUTOSHOW_TYPE_WARNING_ERROR					= 3;
	public static final int		CONSOLE_AUTOSHOW_TYPE_DEFAULT						= ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NEVER;

	public static final boolean	CONSOLE_ENABLED_DEFAULT								= true;
	public static final boolean	CONSOLE_HYPERLINKS_ENABLED_DEFAULT					= true;

	public static final boolean	CONSOLE_WRAP_ENABLED_DEFAULT						= false;
	public static final int		CONSOLE_WRAP_WIDTH_DEFAULT							= 80;
	public static final boolean	CONSOLE_LIMIT_ENABLED_DEFAULT						= true;
	public static final int		CONSOLE_LIMIT_VALUE_DEFAULT							= 500000;

	public static final RGB		CONSOLE_ERR_COLOR_DEFAULT							= new RGB(255, 0, 0);
	public static final RGB		CONSOLE_WRN_COLOR_DEFAULT							= new RGB(128, 0, 0);
	public static final RGB		CONSOLE_OK_COLOR_DEFAULT							= new RGB(0, 0, 255);
	public static final RGB		CONSOLE_CMD_COLOR_DEFAULT							= new RGB(0, 0, 0);

	public static final String	MAILREPORTER_ENABLED_NAME							= "enabled";
	public static final String	MAILREPORTER_ERRORS_ENABLED_NAME					= "errorsEnabled";

	public static final boolean	MAILREPORTER_ENABLED_DEFAULT						= true;
	public static final boolean	MAILREPORTER_ERRORS_ENABLED_DEFAULT					= true;

	public static final String	MERGE_USE_JAVAHL_NAME								= "useJavaHL";
	public static final String	MERGE_INCLUDE_MERGED_NAME							= "includeMerged";

	public static final boolean	MERGE_USE_JAVAHL_DEFAULT							= false;
	public static final boolean	MERGE_INCLUDE_MERGED_DEFAULT						= false;

	public static final String	CHECKOUT_USE_DOT_PROJECT_NAME						= "useDotProject";
	public static final String	CHECKOUT_RESPECT_PROJECT_STRUCTURE_NAME				= "respectProjectStructure";

	public static final boolean	CHECKOUT_USE_DOT_PROJECT_DEFAULT					= true;
	public static final boolean	CHECKOUT_RESPECT_PROJECT_STRUCTURE_DEFAULT			= false;

	public static final String	HISTORY_PAGE_SIZE_NAME								= "pageSize";
	public static final String	HISTORY_PAGING_ENABLE_NAME							= "pagingEnable";
	public static final String	HISTORY_SHOW_MULTILINE_COMMENT_NAME					= "multilineComment";
	public static final String	HISTORY_SHOW_AFFECTED_PATHS_NAME					= "affectedPaths";
	public static final String	HISTORY_GROUPING_TYPE_NAME							= "groupingType";
	public static final String	HISTORY_REVISION_MODE_NAME							= "revisionMode";
	public static final String	HISTORY_HIERARCHICAL_LAYOUT							= "hierarchicalLayout";
	public static final String	HISTORY_COMPARE_MODE								= "compareMode";
	public static final String	HISTORY_CONNECT_TO_COMPARE_WITH_NAME				= "connectToCompareWith";

	public static final int		HISTORY_GROUPING_TYPE_NONE							= 0;
	public static final int		HISTORY_GROUPING_TYPE_DATE							= 1;

	public static final int		HISTORY_REVISION_MODE_BOTH							= 0;
	public static final int		HISTORY_REVISION_MODE_REMOTE						= 1;
	public static final int		HISTORY_REVISION_MODE_LOCAL							= 2;

	public static final int		HISTORY_PAGE_SIZE_DEFAULT							= 25;
	public static final int		HISTORY_GROUPING_TYPE_DEFAULT						= ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NONE;
	public static final int		HISTORY_REVISION_MODE_DEFAULT						= ModelBusTeamPreferences.HISTORY_REVISION_MODE_REMOTE;
	public static final boolean	HISTORY_PAGING_ENABLE_DEFAULT						= true;
	public static final boolean	HISTORY_SHOW_MULTILINE_COMMENT_DEFAULT				= true;
	public static final boolean	HISTORY_SHOW_AFFECTED_PATHS_DEFAULT					= true;
	public static final boolean	HISTORY_HIERARCHICAL_LAYOUT_DEFAULT					= true;
	public static final boolean	HISTORY_COMPARE_MODE_DEFAULT						= false;
	public static final boolean	HISTORY_CONNECT_TO_COMPARE_WITH_DEFAULT				= true;

	public static final String	DATE_FORMAT_NAME									= "dateFormat";
	public static final String	DATE_FORMAT_CUSTOM_NAME								= "dateFormatCustom";

	public static final int		DATE_FORMAT_MODE_SHORT								= 0;
	public static final int		DATE_FORMAT_MODE_MEDIUM								= 1;
	public static final int		DATE_FORMAT_MODE_LONG								= 2;
	public static final int		DATE_FORMAT_MODE_CUSTOM								= 3;

	public static final int		DATE_FORMAT_DEFAULT									= DATE_FORMAT_MODE_SHORT;
	public static final String	DATE_FORMAT_CUSTOM_DEFAULT							= "";

	public static final String	PROPERTY_LINK_WITH_EDITOR_NAME						= "linkWithEditor";

	public static final boolean	PROPERTY_LINK_WITH_EDITOR_DEFAULT					= false;

	public static final String	SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME				= "fastReport";

	public static final boolean	SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_DEFAULT			= true;

	public static final String	REPOSITORY_SHOW_BROWSER_NAME						= "repositoryBrowser";
	public static final boolean	REPOSITORY_SHOW_BROWSER_DEFAULT						= true;
	public static final String	REPOSITORY_FORCE_EXTERNALS_FREEZE_NAME				= "forceExternalsFreeze";
	public static final boolean	REPOSITORY_FORCE_EXTERNALS_FREEZE_DEFAULT			= true;
	public static final String	REPOSITORY_HEAD_NAME								= "head";
	public static final String	REPOSITORY_BRANCHES_NAME							= "branches";
	public static final String	REPOSITORY_TAGS_NAME								= "tags";
	public static final String	REPOSITORY_SHOW_EXTERNALS_NAME						= "showExternals";

	public static final String	REPOSITORY_HEAD_DEFAULT								= "trunk";
	public static final String	REPOSITORY_BRANCHES_DEFAULT							= "branches";
	public static final String	REPOSITORY_TAGS_DEFAULT								= "tags";
	public static final boolean	REPOSITORY_SHOW_EXTERNALS_DEFAULT					= true;

	public static final String	BRANCH_TAG_CONSIDER_STRUCTURE_NAME					= "tagConsideringProjectStructure";
	public static final boolean	BRANCH_TAG_CONSIDER_STRUCTURE_DEFAULT				= true;

	public static final String	DECORATION_FORMAT_FILE_NAME							= "format.file";
	public static final String	DECORATION_FORMAT_FOLDER_NAME						= "format.folder";
	public static final String	DECORATION_FORMAT_PROJECT_NAME						= "format.project";

	// {outgoing_flag} {name} {revision}
	public static final String	DECORATION_FORMAT_FILE_DEFAULT						= "{" + TextVariableSetProvider.NAME_OF_OUTGOING_FLAG + "} {"
																							+ TextVariableSetProvider.NAME_OF_NAME + "} {"
																							+ TextVariableSetProvider.NAME_OF_REVISION + "}";
	// {outgoing_flag} {name} {revision}
	public static final String	DECORATION_FORMAT_FOLDER_DEFAULT					= ModelBusTeamPreferences.DECORATION_FORMAT_FILE_DEFAULT;
	// {outgoing_flag} {name} {revision} [{location_label}{root_prefix}: {fullname}]
	public static final String	DECORATION_FORMAT_PROJECT_DEFAULT					= ModelBusTeamPreferences.DECORATION_FORMAT_FOLDER_DEFAULT
																							+ " [{" + TextVariableSetProvider.NAME_OF_LOCATION_LABEL
																							+ "}{" + TextVariableSetProvider.NAME_OF_ROOT_PREFIX
																							+ "}: {" + TextVariableSetProvider.NAME_OF_ASCENDANT
																							+ "}]";

	public static final String	DECORATION_FLAG_OUTGOING_NAME						= "flag.outgoing";
	public static final String	DECORATION_FLAG_ADDED_NAME							= "flag.added";

	public static final String	DECORATION_FLAG_OUTGOING_DEFAULT					= ">";
	public static final String	DECORATION_FLAG_ADDED_DEFAULT						= "*";

	public static final String	DECORATION_TRUNK_PREFIX_NAME						= "trunk.branch";
	public static final String	DECORATION_BRANCH_PREFIX_NAME						= "prefix.branch";
	public static final String	DECORATION_TAG_PREFIX_NAME							= "prefix.tag";

	public static final String	DECORATION_TRUNK_PREFIX_DEFAULT						= ", Trunk";
	public static final String	DECORATION_BRANCH_PREFIX_DEFAULT					= ", Branch";
	public static final String	DECORATION_TAG_PREFIX_DEFAULT						= ", Tag";

	public static final String	DECORATION_ICON_CONFLICTED_NAME						= "icon.conflicted";
	public static final String	DECORATION_ICON_MODIFIED_NAME						= "icon.modified";
	public static final String	DECORATION_ICON_REMOTE_NAME							= "icon.remote";
	public static final String	DECORATION_ICON_ADDED_NAME							= "icon.added";
	public static final String	DECORATION_ICON_NEW_NAME							= "icon.new";
	public static final String	DECORATION_ICON_LOCKED_NAME							= "icon.locked";
	public static final String	DECORATION_ICON_NEEDS_LOCK_NAME						= "icon.needslock";
	public static final String	DECORATION_ICON_SWITCHED_NAME						= "icon.switched";

	public static final boolean	DECORATION_ICON_CONFLICTED_DEFAULT					= true;
	public static final boolean	DECORATION_ICON_MODIFIED_DEFAULT					= false;
	public static final boolean	DECORATION_ICON_REMOTE_DEFAULT						= true;
	public static final boolean	DECORATION_ICON_ADDED_DEFAULT						= true;
	public static final boolean	DECORATION_ICON_NEW_DEFAULT							= true;
	public static final boolean	DECORATION_ICON_LOCKED_DEFAULT						= true;
	public static final boolean	DECORATION_ICON_NEEDS_LOCK_DEFAULT					= false;
	public static final boolean	DECORATION_ICON_SWITCHED_DEFAULT					= true;

	public static final String	DECORATION_COMPUTE_DEEP_NAME						= "compute.deep";
	public static final String	DECORATION_PRECISE_ENABLEMENTS_NAME					= "precise.enablements";
	public static final String	DECORATION_ENABLE_CACHE_NAME						= "enable.cache";

	public static final boolean	DECORATION_COMPUTE_DEEP_DEFAULT						= true;
	public static final boolean	DECORATION_PRECISE_ENABLEMENTS_DEFAULT				= false;
	public static final boolean	DECORATION_ENABLE_CACHE_DEFAULT						= true;

	public static final String	DECORATION_USE_FONT_COLORS_DECOR_NAME				= "use.fontdecor";
	public static final boolean	DECORATION_USE_FONT_COLORS_DECOR_DEFAULT			= false;

	public static final String	NAME_OF_OUTGOING_FOREGROUND_COLOR					= "outgoing_change_foreground_color";
	public static final String	NAME_OF_OUTGOING_BACKGROUND_COLOR					= "outgoing_change_background_color";
	public static final String	NAME_OF_OUTGOING_FONT								= "outgoing_change_font";
	public static final String	NAME_OF_IGNORED_FOREGROUND_COLOR					= "ignored_resource_foreground_color";
	public static final String	NAME_OF_IGNORED_BACKGROUND_COLOR					= "ignored_resource_background_color";
	public static final String	NAME_OF_IGNORED_FONT								= "ignored_resource_font";
	public static final String	NAME_OF_NOT_RELATED_NODES_FOREGROUND_COLOR			= "not_related_nodes_foreground_color";
	public static final String	NAME_OF_NOT_RELATED_NODES_BACKGROUND_COLOR			= "not_related_nodes_background_color";
	public static final String	NAME_OF_NOT_RELATED_NODES_FONT						= "not_related_nodes_font";
	public static final String	NAME_OF_STRUCTURE_NODES_FOREGROUND_COLOR			= "structure_nodes_foreground_color";
	public static final String	NAME_OF_STRUCTURE_NODES_BACKGROUND_COLOR			= "structure_nodes_background_color";
	public static final String	NAME_OF_STRUCTURE_NODES_FONT						= "structure_nodes_font";
	public static final String	NAME_OF_MERGED_REVISIONS_FOREGROUND_COLOR			= "merged_revision_foreground_color";

	public static final String	COMMENT_TEMPLATES_LIST_NAME							= "comment.templates";
	public static final String	COMMENT_TEMPLATES_LIST_ENABLED_NAME					= "comment.templates.enabled";
	public static final String	COMMENT_LOG_TEMPLATES_ENABLED_NAME					= "comment.logTemplates.enabled";
	public static final String	COMMENT_SAVED_COMMENTS_COUNT_NAME					= "savedCommentsCount";

	public static final String	COMMENT_TEMPLATES_LIST_DEFAULT						= "";
	public static final boolean	COMMENT_TEMPLATES_LIST_ENABLED_DEFAULT				= true;
	public static final boolean	COMMENT_LOG_TEMPLATES_ENABLED_DEFAULT				= true;
	public static final int		COMMENT_SAVED_COMMENTS_COUNT_DEFAULT				= 10;

	public static final String	COMMIT_DIALOG_WEIGHT_NAME							= "CommitPanel.weight";
	public static final int		COMMIT_DIALOG_WEIGHT_DEFAULT						= 50;

	public static final String	AUTO_PROPERTIES_LIST_NAME							= "autoproperties";
	public static final String	AUTO_PROPERTIES_LIST_DEFAULT						= "";

	public static final String	CUSTOM_PROPERTIES_LIST_NAME							= "customproperties";
	public static final String	CUSTOM_PROPERTIES_LIST_DEFAULT						= "";

	public static DateFormat getDateFormat(IPreferenceStore store) {
		int formatMode = ModelBusTeamPreferences.getDateFormatInt(store, ModelBusTeamPreferences.DATE_FORMAT_NAME);
		DateFormat dateTimeFormat = null;
		switch (formatMode) {
			case ModelBusTeamPreferences.DATE_FORMAT_MODE_SHORT:
				dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.getDefault());
				break;
			case ModelBusTeamPreferences.DATE_FORMAT_MODE_MEDIUM:
				dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.getDefault());
				break;
			case ModelBusTeamPreferences.DATE_FORMAT_MODE_LONG:
				dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG, Locale.getDefault());
				break;
			case ModelBusTeamPreferences.DATE_FORMAT_MODE_CUSTOM:
				dateTimeFormat = new SimpleDateFormat(ModelBusTeamPreferences.getDateFormatString(store,
						ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_NAME), Locale.getDefault());
				break;
			default:
				dateTimeFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.getDefault());
				break;
		}
		return dateTimeFormat;
	}

	public static void setDefaultValues(IPreferenceStore store) {
		ModelBusTeamPreferences.setDefaultRepositoryValues(store);
		ModelBusTeamPreferences.setDefaultDecorationValues(store);
		ModelBusTeamPreferences.setDefaultPerformanceValues(store);
		ModelBusTeamPreferences.setDefaultSynchronizeValues(store);
		ModelBusTeamPreferences.setDefaultMailReporterValues(store);
		ModelBusTeamPreferences.setDefaultHistoryValues(store);
		ModelBusTeamPreferences.setDefaultDateFormatValues(store);
		ModelBusTeamPreferences.setDefaultPropertiesValues(store);
		ModelBusTeamPreferences.setDefaultCommentTemplatesValues(store);
		ModelBusTeamPreferences.setDefaultBehaviourValues(store);
		ModelBusTeamPreferences.setDefaultMergeValues(store);
		ModelBusTeamPreferences.setDefaultCheckoutValues(store);
		ModelBusTeamPreferences.setDefaultConsoleValues(store);
		ModelBusTeamPreferences.setDefaultCoreValues(store);
		ModelBusTeamPreferences.setDefaultCommitDialogValues(store);
		ModelBusTeamPreferences.setDefaultAutoPropertiesValues(store);
		ModelBusTeamPreferences.setDefaultCustomPropertiesValues(store);
	}

	public static void setDefaultAutoPropertiesValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullAutoPropertiesName(ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_NAME),
				ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_DEFAULT);
	}

	public static void setDefaultCustomPropertiesValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullCustomPropertiesName(ModelBusTeamPreferences.CUSTOM_PROPERTIES_LIST_NAME),
				ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_DEFAULT);
	}

	public static void setDefaultCommitDialogValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.COMMIT_DIALOG_WEIGHT_NAME, ModelBusTeamPreferences.COMMIT_DIALOG_WEIGHT_DEFAULT);
	}

	public static void setDefaultCheckoutValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullCheckoutName(ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_NAME),
				ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullCheckoutName(ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_NAME),
				ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_DEFAULT);
	}

	public static void setDefaultMergeValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullMergeName(ModelBusTeamPreferences.MERGE_USE_JAVAHL_NAME),
				ModelBusTeamPreferences.MERGE_USE_JAVAHL_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullMergeName(ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME),
				ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_DEFAULT);
	}

	public static void setDefaultHistoryValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_NAME),
				ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME),
				ModelBusTeamPreferences.HISTORY_PAGE_SIZE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME),
				ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_SHOW_MULTILINE_COMMENT_NAME),
				ModelBusTeamPreferences.HISTORY_SHOW_MULTILINE_COMMENT_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_SHOW_AFFECTED_PATHS_NAME),
				ModelBusTeamPreferences.HISTORY_SHOW_AFFECTED_PATHS_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_NAME),
				ModelBusTeamPreferences.HISTORY_GROUPING_TYPE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_REVISION_MODE_NAME),
				ModelBusTeamPreferences.HISTORY_REVISION_MODE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_HIERARCHICAL_LAYOUT),
				ModelBusTeamPreferences.HISTORY_HIERARCHICAL_LAYOUT_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_COMPARE_MODE),
				ModelBusTeamPreferences.HISTORY_COMPARE_MODE_DEFAULT);
	}

	public static void setDefaultDateFormatValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullDateFormatName(ModelBusTeamPreferences.DATE_FORMAT_NAME),
				ModelBusTeamPreferences.DATE_FORMAT_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDateFormatName(ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_NAME),
				ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_DEFAULT);
	}

	public static void setDefaultSynchronizeValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullSynchronizeName(ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME),
				ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_DEFAULT);
	}

	public static void setDefaultPropertiesValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullPropertiesName(ModelBusTeamPreferences.PROPERTY_LINK_WITH_EDITOR_NAME),
				ModelBusTeamPreferences.PROPERTY_LINK_WITH_EDITOR_DEFAULT);
	}

	public static void setDefaultRepositoryValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_HEAD_NAME),
				ModelBusTeamPreferences.REPOSITORY_HEAD_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_BRANCHES_NAME),
				ModelBusTeamPreferences.REPOSITORY_BRANCHES_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_TAGS_NAME),
				ModelBusTeamPreferences.REPOSITORY_TAGS_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_NAME),
				ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_SHOW_BROWSER_NAME),
				ModelBusTeamPreferences.REPOSITORY_SHOW_BROWSER_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_NAME),
				ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_DEFAULT);
	}

	public static void setDefaultDecorationValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_FILE_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_FILE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_FOLDER_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_FOLDER_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_PROJECT_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_PROJECT_DEFAULT);

		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FLAG_OUTGOING_NAME),
				ModelBusTeamPreferences.DECORATION_FLAG_OUTGOING_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FLAG_ADDED_NAME),
				ModelBusTeamPreferences.DECORATION_FLAG_ADDED_DEFAULT);

		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_TRUNK_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_TRUNK_PREFIX_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_BRANCH_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_BRANCH_PREFIX_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_TAG_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_TAG_PREFIX_DEFAULT);

		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_CONFLICTED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_CONFLICTED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_MODIFIED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_MODIFIED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_REMOTE_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_REMOTE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_ADDED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_ADDED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_NEW_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_NEW_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_LOCKED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_LOCKED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_NEEDS_LOCK_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_NEEDS_LOCK_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_SWITCHED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_SWITCHED_DEFAULT);

		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_USE_FONT_COLORS_DECOR_NAME),
				ModelBusTeamPreferences.DECORATION_USE_FONT_COLORS_DECOR_DEFAULT);
	}

	public static void setDefaultPerformanceValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_NAME),
				ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_NAME),
				ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_NAME),
				ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_DEFAULT);
	}

	public static void setDefaultMailReporterValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullMailReporterName(ModelBusTeamPreferences.MAILREPORTER_ENABLED_NAME),
				ModelBusTeamPreferences.MAILREPORTER_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullMailReporterName(ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_NAME),
				ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_DEFAULT);
	}

	public static void setDefaultCommentTemplatesValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_NAME),
				ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_ENABLED_NAME),
				ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_LOG_TEMPLATES_ENABLED_NAME),
				ModelBusTeamPreferences.COMMENT_LOG_TEMPLATES_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_SAVED_COMMENTS_COUNT_NAME),
				ModelBusTeamPreferences.COMMENT_SAVED_COMMENTS_COUNT_DEFAULT);
	}

	public static void setDefaultConsoleValues(final IPreferenceStore store) {
		PreferenceConverter.setDefault(store, ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_ERR_COLOR_NAME),
				ModelBusTeamPreferences.CONSOLE_ERR_COLOR_DEFAULT);
		PreferenceConverter.setDefault(store, ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRN_COLOR_NAME),
				ModelBusTeamPreferences.CONSOLE_WRN_COLOR_DEFAULT);
		PreferenceConverter.setDefault(store, ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_OK_COLOR_NAME),
				ModelBusTeamPreferences.CONSOLE_OK_COLOR_DEFAULT);
		PreferenceConverter.setDefault(store, ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_CMD_COLOR_NAME),
				ModelBusTeamPreferences.CONSOLE_CMD_COLOR_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NAME),
				ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_NAME),
				ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_NAME),
				ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_DEFAULT);
	}

	public static void setDefaultBehaviourValues(IPreferenceStore store) {
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNALS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNAL_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_DEFAULT);
		store.setDefault(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_IGNORE_EXTERNALS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_IGNORE_EXTERNALS_DEFAULT);

	}

	public static void setDefaultCoreValues(IPreferenceStore store) {
		// store.setDefault(ModelBusTeamPreferences.fullCoreName(ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_NAME),
		// ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_DEFAULT);
	}

	public static void resetToDefaultAutoPropsValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullAutoPropertiesName(ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_NAME),
				ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_DEFAULT);
	}

	public static void resetToDefaultCustomoPropsValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullCustomPropertiesName(ModelBusTeamPreferences.CUSTOM_PROPERTIES_LIST_NAME),
				ModelBusTeamPreferences.AUTO_PROPERTIES_LIST_DEFAULT);
	}

	public static void resetToDefaultCoreValues(IPreferenceStore store) {
		// store.setValue(ModelBusTeamPreferences.fullCoreName(ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_NAME),
		// ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_DEFAULT);
	}

	public static void resetToDefaultHistoryValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_NAME),
				ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME),
				ModelBusTeamPreferences.HISTORY_PAGE_SIZE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullHistoryName(ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME),
				ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_DEFAULT);
	}

	public static void resetToDefaultDateFormatValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullDateFormatName(ModelBusTeamPreferences.DATE_FORMAT_NAME),
				ModelBusTeamPreferences.DATE_FORMAT_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDateFormatName(ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_NAME),
				ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_DEFAULT);
	}

	public static void resetToDefaultMailReporterValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullMailReporterName(ModelBusTeamPreferences.MAILREPORTER_ENABLED_NAME),
				ModelBusTeamPreferences.MAILREPORTER_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullMailReporterName(ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_NAME),
				ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_DEFAULT);
	}

	public static void resetToDefaultCheckoutValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullCheckoutName(ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_NAME),
				ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullCheckoutName(ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_NAME),
				ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_DEFAULT);
	}

	public static void resetToDefaultMergeValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullMergeName(ModelBusTeamPreferences.MERGE_USE_JAVAHL_NAME),
				ModelBusTeamPreferences.MERGE_USE_JAVAHL_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullMergeName(ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME),
				ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_DEFAULT);
	}

	public static void resetToDefaultBehaviourValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNALS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNAL_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(ModelBusTeamPreferences.BEHAVIOUR_IGNORE_EXTERNALS_NAME),
				ModelBusTeamPreferences.BEHAVIOUR_IGNORE_EXTERNALS_DEFAULT);

	}

	public static void resetToDefaultSynchronizeValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullSynchronizeName(ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME),
				ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_DEFAULT);
	}

	public static void resetToDefaultRepositoryValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_HEAD_NAME),
				ModelBusTeamPreferences.REPOSITORY_HEAD_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_BRANCHES_NAME),
				ModelBusTeamPreferences.REPOSITORY_BRANCHES_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_TAGS_NAME),
				ModelBusTeamPreferences.REPOSITORY_TAGS_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_NAME),
				ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_SHOW_BROWSER_NAME),
				ModelBusTeamPreferences.REPOSITORY_SHOW_BROWSER_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_NAME),
				ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_DEFAULT);
	}

	public static void resetToDefaultDecorationValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_FILE_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_FILE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_FOLDER_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_FOLDER_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FORMAT_PROJECT_NAME),
				ModelBusTeamPreferences.DECORATION_FORMAT_PROJECT_DEFAULT);

		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FLAG_OUTGOING_NAME),
				ModelBusTeamPreferences.DECORATION_FLAG_OUTGOING_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_FLAG_ADDED_NAME),
				ModelBusTeamPreferences.DECORATION_FLAG_ADDED_DEFAULT);

		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_TRUNK_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_TRUNK_PREFIX_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_BRANCH_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_BRANCH_PREFIX_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_TAG_PREFIX_NAME),
				ModelBusTeamPreferences.DECORATION_TAG_PREFIX_DEFAULT);

		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_CONFLICTED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_CONFLICTED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_MODIFIED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_MODIFIED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_REMOTE_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_REMOTE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_ADDED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_ADDED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_NEW_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_NEW_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_LOCKED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_LOCKED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_NEEDS_LOCK_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_NEEDS_LOCK_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ICON_SWITCHED_NAME),
				ModelBusTeamPreferences.DECORATION_ICON_SWITCHED_DEFAULT);

		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_USE_FONT_COLORS_DECOR_NAME),
				ModelBusTeamPreferences.DECORATION_USE_FONT_COLORS_DECOR_DEFAULT);
	}

	public static void resetToDefaultPerformanceValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_NAME),
				ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_NAME),
				ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullDecorationName(ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_NAME),
				ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_DEFAULT);
	}

	public static void resetToDefaultCommentTemplatesValues(IPreferenceStore store) {
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_NAME),
				ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_ENABLED_NAME),
				ModelBusTeamPreferences.COMMENT_TEMPLATES_LIST_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_LOG_TEMPLATES_ENABLED_NAME),
				ModelBusTeamPreferences.COMMENT_LOG_TEMPLATES_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(ModelBusTeamPreferences.COMMENT_SAVED_COMMENTS_COUNT_NAME),
				ModelBusTeamPreferences.COMMENT_SAVED_COMMENTS_COUNT_DEFAULT);
	}

	public static void resetToDefaultConsoleValues(IPreferenceStore store) {
		ModelBusTeamPreferences.setConsoleRGB(store, ModelBusTeamPreferences.CONSOLE_ERR_COLOR_NAME,
				ModelBusTeamPreferences.CONSOLE_ERR_COLOR_DEFAULT);
		ModelBusTeamPreferences.setConsoleRGB(store, ModelBusTeamPreferences.CONSOLE_WRN_COLOR_NAME,
				ModelBusTeamPreferences.CONSOLE_WRN_COLOR_DEFAULT);
		ModelBusTeamPreferences.setConsoleRGB(store, ModelBusTeamPreferences.CONSOLE_OK_COLOR_NAME, ModelBusTeamPreferences.CONSOLE_OK_COLOR_DEFAULT);
		ModelBusTeamPreferences.setConsoleRGB(store, ModelBusTeamPreferences.CONSOLE_CMD_COLOR_NAME,
				ModelBusTeamPreferences.CONSOLE_CMD_COLOR_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NAME),
				ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_NAME),
				ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_NAME),
				ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_DEFAULT);
		store.setValue(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_NAME),
				ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_DEFAULT);
	}

	public static int getDialogInt(IPreferenceStore store, String name) {
		return store.getInt(name);
	}

	public static boolean getCheckoutBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullCheckoutName(shortName));
	}

	public static boolean getMergeBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullMergeName(shortName));
	}

	public static boolean getSynchronizeBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullSynchronizeName(shortName));
	}

	public static boolean getPropertiesBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullPropertiesName(shortName));
	}

	public static String getRepositoryString(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullRepositoryName(shortName));
	}

	public static boolean getRepositoryBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullRepositoryName(shortName));
	}

	public static String getDecorationString(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullDecorationName(shortName));
	}

	public static int getHistoryInt(IPreferenceStore store, String shortName) {
		return store.getInt(ModelBusTeamPreferences.fullHistoryName(shortName));
	}

	public static boolean getHistoryBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullHistoryName(shortName));
	}

	public static int getDateFormatInt(IPreferenceStore store, String shortName) {
		return store.getInt(ModelBusTeamPreferences.fullDateFormatName(shortName));
	}

	public static String getDateFormatString(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullDateFormatName(shortName));
	}

	public static boolean getBehaviourBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullBehaviourName(shortName));
	}

	public static boolean getDecorationBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullDecorationName(shortName));
	}

	public static String getCommentTemplatesString(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullCommentTemplatesName(shortName));
	}

	public static int getCommentTemplatesInt(IPreferenceStore store, String shortName) {
		return store.getInt(ModelBusTeamPreferences.fullCommentTemplatesName(shortName));
	}

	public static boolean getCommentTemplatesBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullCommentTemplatesName(shortName));
	}

	public static String getAutoPropertiesList(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullAutoPropertiesName(shortName));
	}

	public static String getCustomPropertiesList(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullCustomPropertiesName(shortName));
	}

	public static void setDialogInt(IPreferenceStore store, String name, int value) {
		store.setValue(name, value);
	}

	public static void setCheckoutBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullCheckoutName(shortName), value);
	}

	public static void setMergeBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullMergeName(shortName), value);
	}

	public static void setSynchronizeBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullSynchronizeName(shortName), value);
	}

	public static void setPropertiesBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullPropertiesName(shortName), value);
	}

	public static void setRepositoryString(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(shortName), value);
	}

	public static void setRepositoryBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullRepositoryName(shortName), value);
	}

	public static void setDecorationString(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullDecorationName(shortName), value);
	}

	public static void setDecorationBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullDecorationName(shortName), value);
	}

	public static void setHistoryBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullHistoryName(shortName), value);
	}

	public static void setBehaviourBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullBehaviourName(shortName), value);
	}

	public static void setHistoryInt(IPreferenceStore store, String shortName, int value) {
		store.setValue(ModelBusTeamPreferences.fullHistoryName(shortName), value);
	}

	public static void setDateFormatInt(IPreferenceStore store, String shortName, int value) {
		store.setValue(ModelBusTeamPreferences.fullDateFormatName(shortName), value);
	}

	public static void setDateFormatString(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullDateFormatName(shortName), value);
	}

	public static boolean getMailReporterBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullMailReporterName(shortName));
	}

	public static void setMailReporterBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullMailReporterName(shortName), value);
	}

	public static void setCommentTemplatesInt(IPreferenceStore store, String shortName, int value) {
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(shortName), value);
	}

	public static void setCommentTemplatesString(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(shortName), value);
	}

	public static void setCommentTemplatesBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullCommentTemplatesName(shortName), value);
	}

	public static void setAutoPropertiesList(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullAutoPropertiesName(shortName), value);
	}

	public static void setCustomPropertiesList(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullCustomPropertiesName(shortName), value);
	}

	public static RGB getConsoleRGB(IPreferenceStore store, String shortName) {
		return PreferenceConverter.getColor(store, ModelBusTeamPreferences.fullConsoleName(shortName));
	}

	public static void setConsoleRGB(IPreferenceStore store, String shortName, RGB value) {
		PreferenceConverter.setValue(store, ModelBusTeamPreferences.fullConsoleName(shortName), value);
	}

	public static int getConsoleInt(IPreferenceStore store, String shortName) {
		return store.getInt(ModelBusTeamPreferences.fullConsoleName(shortName));
	}

	public static void setConsoleInt(IPreferenceStore store, String shortName, int value) {
		store.setValue(ModelBusTeamPreferences.fullConsoleName(shortName), value);
	}

	public static boolean getConsoleBoolean(IPreferenceStore store, String shortName) {
		return store.getBoolean(ModelBusTeamPreferences.fullConsoleName(shortName));
	}

	public static void setConsoleBoolean(IPreferenceStore store, String shortName, boolean value) {
		store.setValue(ModelBusTeamPreferences.fullConsoleName(shortName), value);
	}

	public static String getCoreString(IPreferenceStore store, String shortName) {
		return store.getString(ModelBusTeamPreferences.fullCoreName(shortName));
	}

	public static void setCoreString(IPreferenceStore store, String shortName, String value) {
		store.setValue(ModelBusTeamPreferences.fullCoreName(shortName), value);
	}

	public static String fullCommitDialogName(String shortName) {
		return ModelBusTeamPreferences.COMMIT_DIALOG_BASE + shortName;
	}

	public static String fullCheckoutName(String shortName) {
		return ModelBusTeamPreferences.CHECKOUT_BASE + shortName;
	}

	public static String fullMergeName(String shortName) {
		return ModelBusTeamPreferences.MERGE_BASE + shortName;
	}

	public static String fullDecorationName(String shortName) {
		return ModelBusTeamPreferences.DECORATION_BASE + shortName;
	}

	public static String fullRepositoryName(String shortName) {
		return ModelBusTeamPreferences.REPOSITORY_BASE + shortName;
	}

	public static String fullSynchronizeName(String shortName) {
		return ModelBusTeamPreferences.SYNCHRONIZE_BASE + shortName;
	}

	public static String fullPropertiesName(String shortName) {
		return ModelBusTeamPreferences.PROPERTIES_BASE + shortName;
	}

	public static String fullHistoryName(String shortName) {
		return ModelBusTeamPreferences.HISTORY_BASE + shortName;
	}

	public static String fullDateFormatName(String shortName) {
		return ModelBusTeamPreferences.DATE_FORMAT_BASE + shortName;
	}

	public static String fullBehaviourName(String shortName) {
		return ModelBusTeamPreferences.BEHAVIOUR_BASE + shortName;
	}

	public static String fullMailReporterName(String shortName) {
		return ModelBusTeamPreferences.MAILREPORTER_BASE + shortName;
	}

	public static String fullCommitSelectName(String shortName) {
		return ModelBusTeamPreferences.COMMENT_TEMPLATES_BASE + shortName;
	}

	public static String fullCommentTemplatesName(String shortName) {
		return ModelBusTeamPreferences.COMMENT_TEMPLATES_BASE + shortName;
	}

	public static String fullConsoleName(String shortName) {
		return ModelBusTeamPreferences.CONSOLE_BASE + shortName;
	}

	public static String fullCoreName(String shortName) {
		return ModelBusTeamPreferences.CORE_BASE + shortName;
	}

	public static String fullAutoPropertiesName(String shortName) {
		return ModelBusTeamPreferences.AUTO_PROPERTIES_BASE + shortName;
	}

	public static String fullCustomPropertiesName(String shortName) {
		return ModelBusTeamPreferences.CUSTOM_PROPERTIES_BASE + shortName;
	}

	private ModelBusTeamPreferences() {

	}

}
