/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.preferences;

import org.eclipse.compare.internal.TabFolderLayout;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.verifier.AbstractVerifierProxy;
import org.modelbus.team.eclipse.ui.verifier.CompositeVerifier;
import org.modelbus.team.eclipse.ui.verifier.DateFormatVerifier;
import org.modelbus.team.eclipse.ui.verifier.IntegerFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.NonEmptyFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.ResourceNameVerifier;

/**
 * Main ModelBus Team preferences page
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamPreferencesPage extends AbstractModelBusTeamPreferencesPage {
	protected String	head;
	protected String	branches;
	protected String	tags;
	protected boolean	showExternals;
	protected boolean	fastReport;
	protected boolean	pagingEnable;
	protected boolean	connectToCompareWith;
	protected int		pageSize;
	protected int		dateFormat;
	protected String	dateFormatCustom;
	protected boolean	mailReporterEnabled;
	protected boolean	mailReporterErrorsEnabled;
	protected boolean	commitSelectNewResources;
	protected boolean	useSubversionExternalsBehaviour;
	protected String	modelbusConnector;
	// protected IModelBusConnectorFactory []factories;
	protected boolean	useJavaHLMerge;
	protected boolean	includeMergedRevisions;
	protected boolean	checkoutUsingDotProjectName;
	protected boolean	checkoutRespectProjectStructure;
	protected boolean	branchTagConsiderStructure;
	protected boolean	forceExternalsFreeze;
	protected boolean	computeKeywordsValues;
	protected boolean	enableAutoShare;
	protected boolean	caseInsensitiveSorting;

	protected Text		headField;
	protected Text		branchesField;
	protected Text		tagsField;
	protected Button	showExternalsButton;
	protected Combo		modelbusConnectorField;
	protected Button	useInteractiveMergeButton;
	protected Button	includeMergedRevisionsButton;
	protected Button	fastReportButton;
	protected Button	enablePagingButton;
	protected Button	connectToCompareWithButton;
	protected Text		pageSizeField;
	protected Combo		dateFormatField;
	protected Text		dateFormatCustomField;
	protected Button	mailReporterEnabledButton;
	protected Button	mailReporterErrorsEnabledButton;
	protected Button	btnResourceSelectionNew;
	protected Button	btnResourceSelectionExternal;
	protected Button	checkoutUsingDotProjectNameButton;
	protected Button	checkoutRespectProjectStructureButton;
	protected Button	branchTagConsiderStructureButton;
	protected Button	branchTagManualUrlEditButton;
	protected Button	computeKeywordsValuesButton;
	protected Button	enableAutoShareButton;
	protected Button	caseInsensitiveSortingButton;
	protected Button	forceExternalsFreezeButton;

	public ModelBusTeamPreferencesPage() {
		super();
	}

	protected void saveValues(IPreferenceStore store) {
		ModelBusTeamPreferences.setRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_HEAD_NAME, this.head);
		ModelBusTeamPreferences.setRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_BRANCHES_NAME, this.branches);
		ModelBusTeamPreferences.setRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_TAGS_NAME, this.tags);
		ModelBusTeamPreferences.setRepositoryBoolean(store, ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME,
				this.branchTagConsiderStructure);
		ModelBusTeamPreferences
				.setRepositoryBoolean(store, ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_NAME, this.forceExternalsFreeze);
		ModelBusTeamPreferences.setRepositoryBoolean(store, ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_NAME, this.showExternals);

		ModelBusTeamPreferences.setSynchronizeBoolean(store, ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME, this.fastReport);

		ModelBusTeamPreferences.setHistoryInt(store, ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME, this.pageSize);
		ModelBusTeamPreferences.setHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME, this.pagingEnable);
		ModelBusTeamPreferences.setHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_NAME, this.connectToCompareWith);

		ModelBusTeamPreferences.setDateFormatInt(store, ModelBusTeamPreferences.DATE_FORMAT_NAME, this.dateFormat);
		ModelBusTeamPreferences.setDateFormatString(store, ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_NAME, this.dateFormatCustom);

		ModelBusTeamPreferences.setMailReporterBoolean(store, ModelBusTeamPreferences.MAILREPORTER_ENABLED_NAME, this.mailReporterEnabled);
		ModelBusTeamPreferences.setMailReporterBoolean(store, ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_NAME,
				this.mailReporterErrorsEnabled);

		ModelBusTeamPreferences.setBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_NAME,
				this.commitSelectNewResources);
		ModelBusTeamPreferences.setBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNALS_NAME,
				this.useSubversionExternalsBehaviour);
		ModelBusTeamPreferences.setBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_NAME, this.enableAutoShare);
		ModelBusTeamPreferences.setBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_NAME, this.computeKeywordsValues);
		ModelBusTeamPreferences.setBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_NAME,
				this.caseInsensitiveSorting);

		// String oldId = CoreExtensionsManager.instance().getModelBusConnectorFactory().getId();
		// if (!oldId.equals(this.modelbusConnector)) {
		// ModelBusTeamPreferences.setCoreString(store, ModelBusTeamPlugin.CORE_ModelBusCLIENT_NAME, this.modelbusConnector);
		// // destroy all cached proxies
		// ModelBusRemoteStorage.instance().dispose();
		// }

		ModelBusTeamPreferences.setMergeBoolean(store, ModelBusTeamPreferences.MERGE_USE_JAVAHL_NAME, this.useJavaHLMerge);
		ModelBusTeamPreferences.setMergeBoolean(store, ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME, this.includeMergedRevisions);

		ModelBusTeamPreferences.setCheckoutBoolean(store, ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_NAME, this.checkoutUsingDotProjectName);
		ModelBusTeamPreferences.setCheckoutBoolean(store, ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_NAME,
				this.checkoutRespectProjectStructure);
	}

	protected void loadDefaultValues(IPreferenceStore store) {
		this.head = ModelBusTeamPreferences.REPOSITORY_HEAD_DEFAULT;
		this.branches = ModelBusTeamPreferences.REPOSITORY_BRANCHES_DEFAULT;
		this.tags = ModelBusTeamPreferences.REPOSITORY_TAGS_DEFAULT;
		this.showExternals = ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_DEFAULT;

		this.fastReport = ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_DEFAULT;

		this.pagingEnable = ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_DEFAULT;
		this.pageSize = ModelBusTeamPreferences.HISTORY_PAGE_SIZE_DEFAULT;
		this.connectToCompareWith = ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_DEFAULT;

		this.dateFormat = ModelBusTeamPreferences.DATE_FORMAT_DEFAULT;
		this.dateFormatCustom = ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_DEFAULT;

		this.mailReporterEnabled = ModelBusTeamPreferences.MAILREPORTER_ENABLED_DEFAULT;
		this.mailReporterErrorsEnabled = ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_DEFAULT;

		this.commitSelectNewResources = ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_DEFAULT;
		this.useSubversionExternalsBehaviour = ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNAL_DEFAULT;

		this.useJavaHLMerge = ModelBusTeamPreferences.MERGE_USE_JAVAHL_DEFAULT;
		this.includeMergedRevisions = ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_DEFAULT;

		this.checkoutUsingDotProjectName = ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_DEFAULT;
		this.checkoutRespectProjectStructure = ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_DEFAULT;

		this.branchTagConsiderStructure = ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_DEFAULT;
		this.forceExternalsFreeze = ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_DEFAULT;

		this.computeKeywordsValues = ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_DEFAULT;

		this.enableAutoShare = ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_DEFAULT;

		this.caseInsensitiveSorting = ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_DEFAULT;

		// CHECKMWA this.modelbusConnector = ModelBusTeamPreferences.CORE_ModelBusCONNECTOR_DEFAULT;
	}

	protected void loadValues(IPreferenceStore store) {
		this.head = ModelBusTeamPreferences.getRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_HEAD_NAME);
		this.branches = ModelBusTeamPreferences.getRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_BRANCHES_NAME);
		this.tags = ModelBusTeamPreferences.getRepositoryString(store, ModelBusTeamPreferences.REPOSITORY_TAGS_NAME);
		this.showExternals = ModelBusTeamPreferences.getRepositoryBoolean(store, ModelBusTeamPreferences.REPOSITORY_SHOW_EXTERNALS_NAME);

		this.fastReport = ModelBusTeamPreferences.getSynchronizeBoolean(store, ModelBusTeamPreferences.SYNCHRONIZE_SHOW_REPORT_CONTIGUOUS_NAME);

		this.connectToCompareWith = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_CONNECT_TO_COMPARE_WITH_NAME);
		this.pagingEnable = ModelBusTeamPreferences.getHistoryBoolean(store, ModelBusTeamPreferences.HISTORY_PAGING_ENABLE_NAME);
		this.pageSize = ModelBusTeamPreferences.getHistoryInt(store, ModelBusTeamPreferences.HISTORY_PAGE_SIZE_NAME);

		this.dateFormat = ModelBusTeamPreferences.getDateFormatInt(store, ModelBusTeamPreferences.DATE_FORMAT_NAME);
		this.dateFormatCustom = ModelBusTeamPreferences.getDateFormatString(store, ModelBusTeamPreferences.DATE_FORMAT_CUSTOM_NAME);

		this.mailReporterEnabled = ModelBusTeamPreferences.getMailReporterBoolean(store, ModelBusTeamPreferences.MAILREPORTER_ENABLED_NAME);
		this.mailReporterErrorsEnabled = ModelBusTeamPreferences.getMailReporterBoolean(store,
				ModelBusTeamPreferences.MAILREPORTER_ERRORS_ENABLED_NAME);

		this.commitSelectNewResources = ModelBusTeamPreferences.getBehaviourBoolean(store,
				ModelBusTeamPreferences.BEHAVIOUR_COMMIT_SELECT_NEW_RESOURCES_NAME);
		this.useSubversionExternalsBehaviour = ModelBusTeamPreferences.getBehaviourBoolean(store,
				ModelBusTeamPreferences.BEHAVIOUR_DO_NOT_SELECT_EXTERNALS_NAME);
		this.enableAutoShare = ModelBusTeamPreferences.getBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_ENABLE_AUTO_SHARE_NAME);
		this.computeKeywordsValues = ModelBusTeamPreferences.getBehaviourBoolean(store, ModelBusTeamPreferences.BEHAVIOUR_COMPUTE_KEYWORDS_NAME);
		this.caseInsensitiveSorting = ModelBusTeamPreferences.getBehaviourBoolean(store,
				ModelBusTeamPreferences.BEHAVIOUR_CASE_INSENSITIVE_TABLE_SORTING_NAME);

		this.useJavaHLMerge = ModelBusTeamPreferences.getMergeBoolean(store, ModelBusTeamPreferences.MERGE_USE_JAVAHL_NAME);
		this.includeMergedRevisions = ModelBusTeamPreferences.getMergeBoolean(store, ModelBusTeamPreferences.MERGE_INCLUDE_MERGED_NAME);

		this.checkoutUsingDotProjectName = ModelBusTeamPreferences.getCheckoutBoolean(store, ModelBusTeamPreferences.CHECKOUT_USE_DOT_PROJECT_NAME);
		this.checkoutRespectProjectStructure = ModelBusTeamPreferences.getCheckoutBoolean(store,
				ModelBusTeamPreferences.CHECKOUT_RESPECT_PROJECT_STRUCTURE_NAME);

		this.branchTagConsiderStructure = ModelBusTeamPreferences.getRepositoryBoolean(store,
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);
		this.forceExternalsFreeze = ModelBusTeamPreferences.getRepositoryBoolean(store,
				ModelBusTeamPreferences.REPOSITORY_FORCE_EXTERNALS_FREEZE_NAME);

		// Client specified in preferences currently may be uninstalled. So, request real used connector instead of saved.
		// this.modelbusConnector = CoreExtensionsManager.instance().getModelBusConnectorFactory().getId();
	}

	protected void initializeControls() {
		this.headField.setText(this.head);
		this.branchesField.setText(this.branches);
		this.tagsField.setText(this.tags);
		this.showExternalsButton.setSelection(this.showExternals);

		this.fastReportButton.setSelection(this.fastReport);

		this.pageSizeField.setText(String.valueOf(this.pageSize));
		this.enablePagingButton.setSelection(this.pagingEnable);
		this.pageSizeField.setEnabled(this.pagingEnable);
		this.connectToCompareWithButton.setSelection(this.connectToCompareWith);

		this.dateFormatField.select(this.dateFormat);
		this.dateFormatCustomField.setText(this.dateFormatCustom);

		this.mailReporterEnabledButton.setSelection(this.mailReporterEnabled);
		this.mailReporterErrorsEnabledButton.setSelection(this.mailReporterErrorsEnabled);

		this.btnResourceSelectionNew.setSelection(this.commitSelectNewResources);
		this.btnResourceSelectionExternal.setSelection(this.useSubversionExternalsBehaviour);

		// List<IModelBusConnectorFactory> factoriesList = Arrays.asList(this.factories);
		// this.modelbusConnectorField.select(factoriesList.indexOf(CoreExtensionsManager.instance().getModelBusConnectorFactory(this.modelbusConnector)));

		this.initializeClientSettings();

		this.checkoutUsingDotProjectNameButton.setSelection(this.checkoutUsingDotProjectName);
		this.checkoutRespectProjectStructureButton.setSelection(this.checkoutRespectProjectStructure);

		this.branchTagConsiderStructureButton.setSelection(this.branchTagConsiderStructure);
		this.forceExternalsFreezeButton.setSelection(this.forceExternalsFreeze);

		this.branchTagManualUrlEditButton.setSelection(!this.branchTagConsiderStructure);

		this.computeKeywordsValuesButton.setSelection(this.computeKeywordsValues);

		this.enableAutoShareButton.setSelection(this.enableAutoShare);

		this.caseInsensitiveSortingButton.setSelection(this.caseInsensitiveSorting);
	}

	protected void initializeClientSettings() {
		boolean isModelBus15CompatibleConnector = true;// CHECKMWA
														// CoreExtensionsManager.instance().getModelBusConnectorFactory(this.modelbusConnector).getModelBusAPIVersion()
														// >= IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x;
		this.useInteractiveMergeButton.setSelection(!this.useJavaHLMerge);
		this.includeMergedRevisionsButton.setSelection(this.includeMergedRevisions);
		this.includeMergedRevisionsButton.setEnabled(isModelBus15CompatibleConnector);
	}

	protected Control createContentsImpl(Composite parent) {
		TabFolder tabFolder = new TabFolder(parent, SWT.NONE);
		tabFolder.setLayout(new TabFolderLayout());
		tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));

		TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.generalTabName"));
		tabItem.setControl(this.createGeneralPage(tabFolder));

		tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.modelbusConnectorTabName"));
		tabItem.setControl(this.createModelBusConnectorsPage(tabFolder));

		tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.repositoryTabName"));
		tabItem.setControl(this.createRepositorySettingsPage(tabFolder));

		tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.viewSettingsTabName"));
		tabItem.setControl(this.createViewSettingsPage(tabFolder));

		tabItem = new TabItem(tabFolder, SWT.NONE);
		tabItem.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mailReporterGroupName"));
		tabItem.setControl(this.createErrorReportingSettingsPage(tabFolder));

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.ModelBusPreferencesContext");

		return tabFolder;
	}

	protected Control createModelBusConnectorsPage(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite composite = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);

		Label label = new Label(composite, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.modelbusConnectorPrompt"));

		label = new Label(composite, SWT.NONE);
		data = new GridData();
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.modelbusConnector"));

		this.modelbusConnectorField = new Combo(composite, SWT.READ_ONLY);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 100;
		this.modelbusConnectorField.setLayoutData(data);
		// Collection<?> fullSet = CoreExtensionsManager.instance().getAccessibleClients();
		// this.factories = fullSet.toArray(new IModelBusConnectorFactory[fullSet.size()]);
		// Arrays.sort(this.factories, new Comparator<Object>() {
		// public int compare(Object o1, Object o2) {
		// return ((IModelBusConnectorFactory)o1).getName().compareTo(((IModelBusConnectorFactory)o2).getName());
		// }
		// });
		// String []items = new String[fullSet.size()];
		// for (int i = 0; i < items.length; i++) {
		// items[i] = this.factories[i].getName() + " (" + this.factories[i].getClientVersion().replace('\n', ' ') + ")";
		// }
		// this.modelbusConnectorField.setItems(items);
		// this.modelbusConnectorField.setVisibleItemCount(items.length);
		// this.modelbusConnectorField.addSelectionListener(new SelectionAdapter() {
		// public void widgetSelected(SelectionEvent e) {
		// ModelBusTeamPreferencesPage.this.modelbusConnector =
		// ModelBusTeamPreferencesPage.this.factories[ModelBusTeamPreferencesPage.this.modelbusConnectorField.getSelectionIndex()].getId();
		// ModelBusTeamPreferencesPage.this.initializeClientSettings();
		// }
		// });

		// Merge settings group
		Group group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		group.setLayoutData(data);
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mergeGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mergePrompt"));

		this.useInteractiveMergeButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.useInteractiveMergeButton.setLayoutData(data);
		this.useInteractiveMergeButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mergeUseInteractiveMerge"));
		this.useInteractiveMergeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.useJavaHLMerge = !ModelBusTeamPreferencesPage.this.useInteractiveMergeButton.getSelection();
			}
		});

		label = new Label(group, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setVisible(false);

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.includeMergedRevisionsLabel"));

		this.includeMergedRevisionsButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.includeMergedRevisionsButton.setLayoutData(data);
		this.includeMergedRevisionsButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.includeMergedRevisions"));
		this.includeMergedRevisionsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.includeMergedRevisions = ModelBusTeamPreferencesPage.this.includeMergedRevisionsButton
						.getSelection();
			}
		});

		return composite;
	}

	protected Control createGeneralPage(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite composite = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);

		// Commit settings group
		Group group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.resourceSelectionGroupName"));

		Label label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.resourceSelectionPrompt"));

		this.btnResourceSelectionNew = new Button(group, SWT.CHECK);
		data = new GridData();
		this.btnResourceSelectionNew.setLayoutData(data);
		this.btnResourceSelectionNew.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.resourceSelectionNew"));
		this.btnResourceSelectionNew.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.commitSelectNewResources = ModelBusTeamPreferencesPage.this.btnResourceSelectionNew.getSelection();
			}
		});

		this.btnResourceSelectionExternal = new Button(group, SWT.CHECK);
		data = new GridData();
		this.btnResourceSelectionExternal.setLayoutData(data);
		this.btnResourceSelectionExternal.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.resourceSelectionExternal"));
		this.btnResourceSelectionExternal.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.useSubversionExternalsBehaviour = ModelBusTeamPreferencesPage.this.btnResourceSelectionExternal
						.getSelection();
			}
		});

		// Project share settings group
		group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.shareGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.sharePrompt"));

		this.enableAutoShareButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.enableAutoShareButton.setLayoutData(data);
		this.enableAutoShareButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.shareEnableAuto"));
		this.enableAutoShareButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.enableAutoShare = ModelBusTeamPreferencesPage.this.enableAutoShareButton.getSelection();
			}
		});

		// Keywords property settings group
		group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.keywordsGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.keywordsPrompt"));

		this.computeKeywordsValuesButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.computeKeywordsValuesButton.setLayoutData(data);
		this.computeKeywordsValuesButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.keywordsComputeRecursively"));
		this.computeKeywordsValuesButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.computeKeywordsValues = ModelBusTeamPreferencesPage.this.computeKeywordsValuesButton.getSelection();
			}
		});

		// Table sorting settings group
		group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.tableSortingGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.tableSortingPrompt"));

		this.caseInsensitiveSortingButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.caseInsensitiveSortingButton.setLayoutData(data);
		this.caseInsensitiveSortingButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.tableSortingCaseInsensitive"));
		this.caseInsensitiveSortingButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.caseInsensitiveSorting = ModelBusTeamPreferencesPage.this.caseInsensitiveSortingButton
						.getSelection();
			}
		});

		return composite;
	}

	protected Control createViewSettingsPage(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite composite = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);

		Group synchViewGroup = new Group(composite, SWT.NONE);
		layout = new GridLayout();
		synchViewGroup.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		synchViewGroup.setLayoutData(data);

		Group historyViewGroup = new Group(composite, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		historyViewGroup.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		historyViewGroup.setLayoutData(data);

		Group dateFormatGroup = new Group(composite, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		dateFormatGroup.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		dateFormatGroup.setLayoutData(data);

		// Synchronize View group
		synchViewGroup.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.synchronizeGroupName"));

		Label label = new Label(synchViewGroup, SWT.NULL);
		label.setLayoutData(new GridData());
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.synchronizePrompt"));

		this.fastReportButton = new Button(synchViewGroup, SWT.CHECK);
		data = new GridData();
		this.fastReportButton.setLayoutData(data);
		this.fastReportButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.synchronizeFastReportName"));
		this.fastReportButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.fastReport = ModelBusTeamPreferencesPage.this.fastReportButton.getSelection();
			}
		});

		// History View group
		historyViewGroup.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.historyGroupName"));

		label = new Label(historyViewGroup, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.historyPrompt"));

		this.enablePagingButton = new Button(historyViewGroup, SWT.CHECK);
		data = new GridData();
		this.enablePagingButton.setLayoutData(data);
		String labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.historyEnablePaging");
		this.enablePagingButton.setText(labelText);
		this.enablePagingButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (ModelBusTeamPreferencesPage.this.enablePagingButton.getSelection()) {
					ModelBusTeamPreferencesPage.this.pageSizeField.setEnabled(true);
					int tmpPageSize = Integer.parseInt(ModelBusTeamPreferencesPage.this.pageSizeField.getText());
					ModelBusTeamPreferencesPage.this.pageSize = tmpPageSize == 0 ? ModelBusTeamPreferences.HISTORY_PAGE_SIZE_DEFAULT : tmpPageSize;
					ModelBusTeamPreferencesPage.this.pageSizeField.setText(String.valueOf(ModelBusTeamPreferencesPage.this.pageSize));
					ModelBusTeamPreferencesPage.this.pagingEnable = true;
				} else {
					ModelBusTeamPreferencesPage.this.pagingEnable = false;
					ModelBusTeamPreferencesPage.this.pageSizeField.setEnabled(false);
				}
			}
		});

		this.pageSizeField = new Text(historyViewGroup, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.pageSizeField.setLayoutData(data);
		CompositeVerifier verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new IntegerFieldVerifier(labelText, true));
		this.attachTo(this.pageSizeField, verifier);
		this.pageSizeField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					ModelBusTeamPreferencesPage.this.pageSize = Integer.parseInt(ModelBusTeamPreferencesPage.this.pageSizeField.getText());
				} catch (Exception ex) {

				}
			}
		});

		this.connectToCompareWithButton = new Button(historyViewGroup, SWT.CHECK);
		data = new GridData();
		data.horizontalSpan = 2;
		this.connectToCompareWithButton.setLayoutData(data);
		labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.historyConnectToCompareWith");
		this.connectToCompareWithButton.setText(labelText);
		this.connectToCompareWithButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.connectToCompareWith = ((Button) e.widget).getSelection();
			}
		});

		// Date format group
		dateFormatGroup.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.dateFormatGroupName"));

		label = new Label(dateFormatGroup, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.dateFormatPrompt");
		label.setText(labelText);

		this.dateFormatField = new Combo(dateFormatGroup, SWT.DROP_DOWN | SWT.READ_ONLY);
		String[] itemsDateFormat = new String[4];
		itemsDateFormat[ModelBusTeamPreferences.DATE_FORMAT_MODE_SHORT] = ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.dateFormatShort");
		itemsDateFormat[ModelBusTeamPreferences.DATE_FORMAT_MODE_MEDIUM] = ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.dateFormatMedium");
		itemsDateFormat[ModelBusTeamPreferences.DATE_FORMAT_MODE_LONG] = ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.dateFormatLong");
		itemsDateFormat[ModelBusTeamPreferences.DATE_FORMAT_MODE_CUSTOM] = ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.dateFormatCustom");
		this.dateFormatField.setItems(itemsDateFormat);

		data = new GridData();
		data.widthHint = 100;
		this.dateFormatField.setLayoutData(data);
		this.dateFormatField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ModelBusTeamPreferencesPage.this.dateFormat = ModelBusTeamPreferencesPage.this.dateFormatField.getSelectionIndex();
				if (ModelBusTeamPreferencesPage.this.dateFormat == ModelBusTeamPreferences.DATE_FORMAT_MODE_CUSTOM) {
					ModelBusTeamPreferencesPage.this.dateFormatCustomField.setEnabled(true);
					ModelBusTeamPreferencesPage.this.dateFormatCustomField.setFocus();
				} else {
					ModelBusTeamPreferencesPage.this.dateFormatCustomField.setEnabled(false);
				}
				ModelBusTeamPreferencesPage.this.validateContent();
			}
		});

		this.dateFormatCustomField = new Text(dateFormatGroup, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		this.dateFormatCustomField.setLayoutData(data);
		verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new DateFormatVerifier(labelText));
		this.attachTo(this.dateFormatCustomField, new AbstractVerifierProxy(verifier) {
			@Override
			protected boolean isVerificationEnabled(Control input) {
				return ModelBusTeamPreferencesPage.this.dateFormatField.getSelectionIndex() == ModelBusTeamPreferences.DATE_FORMAT_MODE_CUSTOM;
			}
		});
		this.dateFormatCustomField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ModelBusTeamPreferencesPage.this.dateFormatCustom = ModelBusTeamPreferencesPage.this.dateFormatCustomField.getText();
			}
		});

		return composite;
	}

	protected Control createRepositorySettingsPage(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite composite = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);

		Group group = new Group(composite, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		group.setLayout(layout);
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.structureGroupName"));

		Label label = new Label(group, SWT.NULL);
		data = new GridData();
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.repositoryPrompt"));

		label = new Label(group, SWT.NULL);
		data = new GridData();
		label.setLayoutData(data);
		String labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.repositoryHeadName");
		label.setText(labelText);

		this.headField = new Text(group, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.headField.setLayoutData(data);
		CompositeVerifier verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new ResourceNameVerifier(labelText, false));
		this.attachTo(this.headField, verifier);
		this.headField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ModelBusTeamPreferencesPage.this.head = ModelBusTeamPreferencesPage.this.headField.getText();
			}
		});

		label = new Label(group, SWT.NULL);
		label.setLayoutData(new GridData());
		labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.repositoryBranchesName");
		label.setText(labelText);

		this.branchesField = new Text(group, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.branchesField.setLayoutData(data);
		verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new ResourceNameVerifier(labelText, false));
		this.attachTo(this.branchesField, verifier);
		this.branchesField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ModelBusTeamPreferencesPage.this.branches = ModelBusTeamPreferencesPage.this.branchesField.getText();
			}
		});

		label = new Label(group, SWT.NULL);
		label.setLayoutData(new GridData());
		labelText = ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.repositoryTagsName");
		label.setText(labelText);

		this.tagsField = new Text(group, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.tagsField.setLayoutData(data);
		verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new ResourceNameVerifier(labelText, false));
		this.attachTo(this.tagsField, verifier);
		this.tagsField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ModelBusTeamPreferencesPage.this.tags = ModelBusTeamPreferencesPage.this.tagsField.getText();
			}
		});

		this.showExternalsButton = new Button(group, SWT.CHECK);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		this.showExternalsButton.setLayoutData(data);
		this.showExternalsButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.showExternals"));
		this.showExternalsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.showExternals = ((Button) e.widget).getSelection();
			}
		});

		// Checkout settings group
		group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.checkoutGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.checkoutPrompt"));

		this.checkoutUsingDotProjectNameButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.checkoutUsingDotProjectNameButton.setLayoutData(data);
		this.checkoutUsingDotProjectNameButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.checkoutUsingDotProjectName"));
		this.checkoutUsingDotProjectNameButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.checkoutUsingDotProjectName = ModelBusTeamPreferencesPage.this.checkoutUsingDotProjectNameButton
						.getSelection();
			}
		});

		this.checkoutRespectProjectStructureButton = new Button(group, SWT.CHECK);
		data = new GridData();
		this.checkoutRespectProjectStructureButton.setLayoutData(data);
		this.checkoutRespectProjectStructureButton.setText(ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.checkoutRespectProjectStructure"));
		this.checkoutRespectProjectStructureButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.checkoutRespectProjectStructure = ModelBusTeamPreferencesPage.this.checkoutRespectProjectStructureButton
						.getSelection();
			}
		});

		// Branch/tag settings group
		group = new Group(composite, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.branchTagGroupName"));

		label = new Label(group, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.branchTagPrompt"));

		SelectionListener listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.branchTagConsiderStructure = ModelBusTeamPreferencesPage.this.branchTagConsiderStructureButton
						.getSelection();
			}
		};
		this.branchTagConsiderStructureButton = new Button(group, SWT.RADIO);
		this.branchTagConsiderStructureButton.setLayoutData(new GridData());
		this.branchTagConsiderStructureButton.setText(ModelBusTeamUIPlugin.instance().getResource(
				"MainPreferencePage.branchTagConsiderStructureLabel"));
		this.branchTagConsiderStructureButton.addSelectionListener(listener);

		this.branchTagManualUrlEditButton = new Button(group, SWT.RADIO);
		this.branchTagManualUrlEditButton.setLayoutData(new GridData());
		this.branchTagManualUrlEditButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.branchTagManualLabel"));
		this.branchTagManualUrlEditButton.addSelectionListener(listener);

		label = new Label(group, SWT.HORIZONTAL | SWT.SEPARATOR);
		data = new GridData(GridData.FILL_HORIZONTAL);
		label.setLayoutData(data);

		this.forceExternalsFreezeButton = new Button(group, SWT.CHECK);
		data = new GridData(GridData.FILL_HORIZONTAL);
		this.forceExternalsFreezeButton.setLayoutData(data);
		this.forceExternalsFreezeButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.forceFreezeExternals"));
		this.forceExternalsFreezeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.forceExternalsFreeze = ((Button) e.widget).getSelection();
			}
		});

		return composite;
	}

	protected Control createErrorReportingSettingsPage(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite composite = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		composite.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);

		Label label = new Label(composite, SWT.WRAP);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 450;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mailReporterPrompt"));

		this.mailReporterEnabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		this.mailReporterEnabledButton.setLayoutData(data);
		this.mailReporterEnabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mailReporterEnabledName"));
		this.mailReporterEnabledButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.mailReporterEnabled = ModelBusTeamPreferencesPage.this.mailReporterEnabledButton.getSelection();
			}
		});

		this.mailReporterErrorsEnabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		this.mailReporterErrorsEnabledButton.setLayoutData(data);
		this.mailReporterErrorsEnabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("MainPreferencePage.mailReporterErrorsEnabledName"));
		this.mailReporterErrorsEnabledButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamPreferencesPage.this.mailReporterErrorsEnabled = ModelBusTeamPreferencesPage.this.mailReporterErrorsEnabledButton
						.getSelection();
			}
		});

		return composite;
	}

}
