/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *    Thomas Champagne - Bug 217561 : additional date formats for label decorations
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.repository.browser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.DelegatingWrapperItemProvider;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusLock;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFictiveNode;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFictiveWorkingDirectory;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryPending;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryResource;
import org.modelbus.team.eclipse.ui.utility.DateFormatter;

/**
 * Repository browser label provider
 * 
 * @author Sergiy Logvin
 */
public class RepositoryBrowserLabelProvider implements ITableLabelProvider {
	protected Map<ImageDescriptor, Image>	images;
	protected ComposedAdapterFactory		composedAdapterFactory;
	protected AdapterFactoryLabelProvider	adapterFactoryLabelProvider;

	protected static String					hasProps;
	protected static String					noAuthor;
	protected static String					noDate;

	public RepositoryBrowserLabelProvider(RepositoryBrowserTableViewer tableViewer) {
		this.images = new HashMap<ImageDescriptor, Image>();
		RepositoryBrowserLabelProvider.noAuthor = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoAuthor");
		RepositoryBrowserLabelProvider.noDate = ModelBusTeamPlugin.instance().getResource("ModelBusInfo.NoDate");
		RepositoryBrowserLabelProvider.hasProps = ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.Browser.HasProps");

		List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());
		factories.add(new UMLItemProviderAdapterFactory());
		factories.add(new EcoreItemProviderAdapterFactory());
		composedAdapterFactory = new ComposedAdapterFactory(factories);
		adapterFactoryLabelProvider = new AdapterFactoryLabelProvider(composedAdapterFactory);
	}

	public Image getColumnImage(Object element, int columnIndex) {
		if (columnIndex == RepositoryBrowserTableViewer.COLUMN_NAME) {
			ImageDescriptor iDescr = null;
			if (element instanceof RepositoryResource) {
				iDescr = ((RepositoryResource) element).getImageDescriptor(null);
			} else if (element instanceof RepositoryFictiveNode) {
				iDescr = ((RepositoryFictiveNode) element).getImageDescriptor(null);
			}
			if (iDescr != null) {
				Image img = this.images.get(iDescr);
				if (img == null) {
					this.images.put(iDescr, img = iDescr.createImage());
				}
				return img;
			}
			if (iDescr == null && (element instanceof EObject || element instanceof DelegatingWrapperItemProvider)) {
				return adapterFactoryLabelProvider.getColumnImage(element, columnIndex);
			}
		}
		return null;
	}

	public String getColumnText(Object element, int columnIndex) {
		if (element instanceof RepositoryResource) {
			return this.getColumnTextForElement(element, columnIndex);
		} else if (element instanceof RepositoryFictiveNode) {
			if (columnIndex == RepositoryBrowserTableViewer.COLUMN_NAME) {
				return ((RepositoryFictiveNode) element).getLabel(null);
			}
			if (element instanceof RepositoryFictiveWorkingDirectory) {
				return this.getColumnTextForElement(((RepositoryFictiveWorkingDirectory) element).getAssociatedDirectory(), columnIndex);
			}
		} else if (element instanceof EObject) {
			if (columnIndex == 0) {
				if (((EObject) element).eIsProxy()) {
					// ResourceSet resourceSet = new ResourceSetImpl();
					// URI proxy = ((InternalEObject)element).eProxyURI();
					// String fragment = proxy.fragment();
					// URI resourceUri = proxy.trimFragment();
					// Map options = new HashMap();
					// if(resourceUri.toString().endsWith(".emx") || resourceUri.toString().endsWith(".epx")){
					// // resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
					// options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
					// }
					// IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
					//
					// Resource resource = resourceSet.createResource(resourceUri);
					// try {
					// repositoryHelper.checkOutModel(UserSessionHelper.getSession(), resource, /*Collections.EMPTY_MAP*/options);
					// element = resource.getEObject(fragment);
					// } catch (IOException e) {
					// throw new RuntimeException(e.getMessage(), e);
					// } catch (RepositoryAuthentificationException e) {
					// throw new RuntimeException(e.getMessage(), e);
					// } catch (UnresolvedReferencesException e) {
					// throw new RuntimeException(e.getMessage(), e);
					// } catch (NonExistingResourceException e) {
					// throw new RuntimeException(e.getMessage(), e);
					// }
					return "";
				}
				return adapterFactoryLabelProvider.getColumnText(element, columnIndex);
			} else {
				return "";
			}
		} else if (element instanceof DelegatingWrapperItemProvider) {
			if (columnIndex == 0) {
				return adapterFactoryLabelProvider.getColumnText(element, columnIndex);
			} else {
				return "";
			}
		}
		return "";
	}

	public void addListener(ILabelProviderListener listener) {
	}

	public void dispose() {
		for (Image img : this.images.values()) {
			img.dispose();
		}
	}

	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	public void removeListener(ILabelProviderListener listener) {
	}

	protected String getColumnTextForElement(Object element, int columnIndex) {
		if (element instanceof RepositoryResource) {
			IRepositoryResource resource = ((RepositoryResource) element).getRepositoryResource();
			IRepositoryResource.Information resourceInfo = resource.getInfo();
			if (columnIndex == RepositoryBrowserTableViewer.COLUMN_NAME) {
				return ((RepositoryResource) element).getLabel();
			} else if (columnIndex == RepositoryBrowserTableViewer.COLUMN_REVISION) {
				String revision = "";
				try {
					if (resource.isInfoCached()) {
						revision = String.valueOf(((RepositoryResource) element).getRevision());
					} else {
						revision = ModelBusTeamUIPlugin.instance().getResource(RepositoryPending.PENDING);
					}
				} catch (Exception ex) {
					LoggedOperation.reportError(ModelBusTeamUIPlugin.instance().getResource("Error.GetColumnText"), ex);
				}
				return revision;
			} else if (resourceInfo != null) {
				if (columnIndex == RepositoryBrowserTableViewer.COLUMN_LAST_CHANGE_DATE) {
					return (resourceInfo.lastChangedDate != 0) ? DateFormatter.formatDate(resourceInfo.lastChangedDate)
							: RepositoryBrowserLabelProvider.noDate;
				} else if (columnIndex == RepositoryBrowserTableViewer.COLUMN_LAST_CHANGE_AUTHOR) {
					String author = resourceInfo.lastAuthor;
					return (author != null) ? author : RepositoryBrowserLabelProvider.noAuthor;
				} else if (columnIndex == RepositoryBrowserTableViewer.COLUMN_LOCK_OWNER) {
					ModelBusLock lock = resourceInfo.lock;
					String lockOwner = (lock == null) ? "" : lock.owner;
					return lockOwner;
				} else if (columnIndex == RepositoryBrowserTableViewer.COLUMN_SIZE) {
					long size = resourceInfo.fileSize;
					return (resource instanceof IRepositoryFile) ? String.valueOf(size) : "";
				} else if (columnIndex == RepositoryBrowserTableViewer.COLUMN_HAS_PROPS) {
					boolean hasProps = resourceInfo.hasProperties;
					return (hasProps) ? RepositoryBrowserLabelProvider.hasProps : "";
				}
			}
		}
		return "";
	}

}
