package org.modelbus.team.eclipse.ui.status;

import org.modelbus.core.lib.IServicesHelper;
import org.modelbus.core.lib.ModelBusCoreLib;
import org.modelbus.core.lib.types.Status;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.repository.UserSessionHelper;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

public class ModelBusRepositoryPing extends Thread {
	private long			interval;
	private boolean			stop			= false;

	private IServicesHelper	servicesHelper	= null;

	public ModelBusRepositoryPing(long interval) {
		super();

		if (interval < 0) {
			throw new IllegalArgumentException("Value for ping interval has to be non negative.");
		}

		this.interval = interval;
	}

	@Override
	public void run() {
		final ModelBusStatusProvider statusProvider = ModelBusTeamUIPlugin.instance().getStatus();

		while (!this.stop) {
			try {
				if (servicesHelper == null) {
					servicesHelper = ModelBusCoreLib.getServicesHelper();
					// TODO: use ping in CXF mode?
					// servicesHelper.setMode(IModelBusHelper.MODE_CXF);
				}

				final Status status = servicesHelper.getStatus(UserSessionHelper.getSession(), true);

				statusProvider.setStatus(status);

			} catch (RepositoryAuthentificationException e) {
				statusProvider.registerException(e);
				statusProvider.setAvailable(false);
			} catch (Exception e) {
				// e.printStackTrace();
				statusProvider.setAvailable(false);
			}

			try {
				Thread.sleep(this.interval);
			} catch (InterruptedException exc) {
				throw new RuntimeException(exc);
			}

		}
	}

	public void kill() {
		this.stop = true;
	}
}
