package org.modelbus.team.eclipse.ui.status;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.modelbus.core.lib.types.Status;
import org.modelbus.dosgi.repository.descriptor.ServiceStatus;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;

public class ModelBusStatusProvider extends Observable {
	public final static DateFormat	DATE_FORMAT	= DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT);

	private boolean					available	= false;
	private String					repositoryLocation;
	private String					secureRepositoryLocation;
	private long					repositoryStartup;
	private List<ServiceStatus>		services;
	private long					lastResponse;

	private String					statusMessage;

	public ModelBusStatusProvider() {
		this.services = new ArrayList<ServiceStatus>();
	}

	public boolean isAvailable() {
		return available;
	}

	public void setAvailable(boolean available) {
		this.available = available;

		if (available == false) {
			this.reset();
		}

		this.fireChange();
	}

	private void reset() {
		this.repositoryLocation = null;
		this.repositoryStartup = -1l;
		this.services.clear();
	}

	public synchronized void addObserver(Observer o) {
		super.addObserver(o);
		this.fireChange();
	}

	private void fireChange() {
		this.setChanged();
		this.notifyObservers();
		this.clearChanged();
	}

	public String getRepositoryLocation() {
		return repositoryLocation;
	}

	private void setRepositoryLocation(String repositoryLocation) {
		this.repositoryLocation = repositoryLocation;
	}

	public String getSecureRepositoryLocation() {
		return secureRepositoryLocation;
	}

	private void setSecureRepositoryLocation(String repositoryLocation) {
		this.secureRepositoryLocation = repositoryLocation;
	}

	public long getRepositoryStartup() {
		return repositoryStartup;
	}

	public void setRepositoryStartup(long repositoryStartup) {
		this.repositoryStartup = repositoryStartup;
	}

	public List<ServiceStatus> getServices() {
		return services;
	}

	private void setServices(List<ServiceStatus> services) {
		if (services != null) {
			this.services = services;
		} else {
			this.services.clear();
		}
	}

	public String getStatusMessage() {
		return statusMessage;
	}

	private void setStatusMessage(String statusMessage) {
		this.statusMessage = statusMessage;
	}

	public void registerException(final Exception exc) {
		this.setStatusMessage(exc.getMessage());
	}

	public void setStatus(final Status status) {
		if (status == null) {
			this.reset();
			return;
		}

		this.setRepositoryLocation(status.getLocation());
		this.setSecureRepositoryLocation(status.getSecureLocation());
		this.setRepositoryStartup(status.getStartupDate().toGregorianCalendar().getTimeInMillis());

		this.setServices(status.getServiceStatus());

		this.lastResponse = new Date().getTime();

		if (RepositoryLocationHelper.getPropertyUseSSL() == Boolean.FALSE) {
			this.setAvailable(this.getRepositoryLocation() != null);
		} else {
			this.setAvailable(this.getSecureRepositoryLocation() != null);
		}
	}

	public long getLastResponse() {
		return lastResponse;
	}
}
