/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.action;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.team.internal.ui.synchronize.SyncInfoModelElement;
import org.eclipse.team.ui.synchronize.ISynchronizeModelElement;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.compare.PropertyCompareInput;
import org.modelbus.team.eclipse.ui.compare.ThreeWayPropertyCompareInput;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelBusSyncInfo;
import org.modelbus.team.eclipse.ui.synchronize.variant.RemoteResourceVariant;
import org.modelbus.team.eclipse.ui.synchronize.variant.ResourceVariant;

/**
 * Compare properties action for Synchronize View
 * 
 * @author Alexei Goncharov
 */
public class ComparePropertiesAction extends AbstractSynchronizeModelAction {

	public ComparePropertiesAction(String text, ISynchronizePageConfiguration configuration) {
		super(text, configuration);
	}

	protected boolean needsToSaveDirtyEditors() {
		return false;
	}

	protected boolean updateSelection(IStructuredSelection selection) {
		super.updateSelection(selection);
		if (selection.size() == 1 && selection.getFirstElement() instanceof SyncInfoModelElement) {
			ISynchronizeModelElement element = (ISynchronizeModelElement) selection.getFirstElement();
			if (element instanceof SyncInfoModelElement) {
				AbstractModelBusSyncInfo syncInfo = (AbstractModelBusSyncInfo) ((SyncInfoModelElement) selection.getFirstElement()).getSyncInfo();
				ILocalResource incoming = ((ResourceVariant) syncInfo.getRemote()).getResource();
				boolean retVal = IStateFilter.SF_EXCLUDE_DELETED.accept(incoming);
				if (incoming instanceof IResourceChange) {
					retVal &= IStateFilter.ST_DELETED != incoming.getStatus();
				}
				return retVal
						&& ((incoming.getChangeMask() & ILocalResource.PROP_MODIFIED) != 0
								|| (syncInfo.getLocalResource().getChangeMask() & ILocalResource.PROP_MODIFIED) != 0 || incoming.getResource() instanceof IContainer);
			}
		}
		return false;
	}

	protected IActionOperation getOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		IResource resource = this.getSelectedResource();
		ILocalResource baseResource = ModelBusRemoteStorage.instance().asLocalResource(resource);
		IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resource);
		ModelBusEntryRevisionReference baseReference = new ModelBusEntryRevisionReference(FileUtility.getWorkingCopyPath(resource), null,
				ModelBusRevision.BASE);
		ModelBusEntryRevisionReference remoteReference = baseReference;
		ILocalResource change = ((RemoteResourceVariant) this.getSelectedModelBusSyncInfo().getRemote()).getResource();
		if (change instanceof IResourceChange) {
			remote = ((IResourceChange) change).getOriginator();
			remoteReference = new ModelBusEntryRevisionReference(remote.getUrl(), remote.getPegRevision(),
					((IResourceChange) change).getRevision());
		}
		PropertyCompareInput input = new ThreeWayPropertyCompareInput(new CompareConfiguration(), resource, remoteReference, baseReference,
				baseResource.getRevision().getRevision());
		CompareUI.openCompareEditor(input);
		return null;
	}

}
