/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.merge;

import java.util.Collection;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.ISynchronizeScope;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.extension.ExtensionsManager;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelBusParticipant;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelBusSyncInfo;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelbusSubscriber;
import org.modelbus.team.eclipse.ui.synchronize.AbstractSynchronizeActionGroup;
import org.modelbus.team.eclipse.ui.utility.OverlayedImageDescriptor;

/**
 * Merge view participant
 * 
 * @author Alexander Gurov
 */
public class MergeParticipant extends AbstractModelBusParticipant {
	public static final String			PARTICIPANT_ID	= MergeParticipant.class.getName();
	public static final int				SUPPORTED_MODES	= ISynchronizePageConfiguration.ALL_MODES;

	protected IPropertyChangeListener	configurationListener;

	public MergeParticipant() {
		super();
	}

	public MergeParticipant(ISynchronizeScope scope) {
		super(scope);
	}

	public AbstractModelbusSubscriber getMatchingSubscriber() {
		MergeSubscriber subscriber = MergeSubscriber.instance();
		subscriber.setMergeScope((MergeScope) this.getScope());
		return subscriber;
	}

	protected void initializeConfiguration(ISynchronizePageConfiguration configuration) {
		super.initializeConfiguration(configuration);
	}

	public void dispose() {
		super.dispose();
	}

	protected String getParticipantId() {
		return MergeParticipant.PARTICIPANT_ID;
	}

	protected Collection<AbstractSynchronizeActionGroup> getActionGroups() {
		return ExtensionsManager.getInstance().getCurrentSynchronizeActionContributor().getMergeContributions();
	}

	protected int getSupportedModes() {
		return MergeParticipant.SUPPORTED_MODES;
	}

	protected int getDefaultMode() {
		return ISynchronizePageConfiguration.BOTH_MODE;
	}

	protected String getShortTaskName() {
		return ModelBusTeamUIPlugin.instance().getResource("MergeView.TaskName");
	}

	protected ILabelDecorator createLabelDecorator() {
		return new MergeLabelDecorator();
	}

	protected class MergeLabelDecorator extends LabelDecorator {
		public MergeLabelDecorator() {
			super();
		}

		public Image decorateImage(Image image, Object element) {
			AbstractModelBusSyncInfo info = this.getSyncInfo(element);
			if (info != null && (info.getKind() & LabelDecorator.CONFLICTING_REPLACEMENT_MASK) == LabelDecorator.CONFLICTING_REPLACEMENT_MASK) {
				ILocalResource local = info.getLocalResource();
				if (IStateFilter.SF_PREREPLACEDREPLACED.accept(local)) {
					return this.registerImageDescriptor(new OverlayedImageDescriptor(image, AbstractModelBusParticipant.OVR_REPLACED_CONF, new Point(
							22, 16), OverlayedImageDescriptor.RIGHT | OverlayedImageDescriptor.CENTER_V));
				}
			}
			return super.decorateImage(image, element);
		}

	}

}
