/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.variant;

import java.io.ByteArrayInputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.TeamException;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetFileContentOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Remote file revision variant
 * 
 * @author Alexander Gurov
 */
public class RemoteFileVariant extends RemoteResourceVariant {

	public RemoteFileVariant(ILocalResource local) {
		super(local);
	}

	protected void fetchContents(IProgressMonitor monitor) throws TeamException {
		if ((!this.local.isCopied() && this.local.getRevision() == ModelBusRevision.INVALID_REVISION_DESCRIPTOR)
				|| IStateFilter.SF_DELETED.accept(this.local) && !IStateFilter.SF_REPLACED.accept(this.local)) {
			this.setContents(new ByteArrayInputStream(new byte[0]), monitor);
			return;
		}
		IRepositoryResource remote = null;
		if (this.local.isCopied()) {
			// IRepositoryLocation location = ModelBusRemoteStorage.instance().getRepositoryLocation(this.local.getResource());
			ModelBusChangeStatus st = ModelBusUtility.getModelBusInfoForNotConnected(this.local.getResource());
			remote = ModelBusConnector.asRepositoryFile(st.urlCopiedFrom, false);
			remote.setSelectedRevision(st.revisionCopiedFrom);
			remote.setPegRevision(st.revisionCopiedFrom);
		} else {
			remote = ModelBusRemoteStorage.instance().asRepositoryResource(this.local.getResource());
			remote.setSelectedRevision(this.local.getRevision());
			remote.setPegRevision(((IResourceChange) this.local).getPegRevision());
			if (this.local instanceof IResourceChange) {
				IRepositoryResource originator = ((IResourceChange) this.local).getOriginator();
				if (originator != null) {
					remote = originator;
				}
			}
		}

		GetFileContentOperation op = new GetFileContentOperation(remote);
		ProgressMonitorUtility.doTaskExternal(op, monitor);
		if (op.getExecutionState() == IActionOperation.OK) {
			this.setContents(op.getContent(), monitor);
		}
	}

	public boolean isContainer() {
		return false;
	}

}
