/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.verifier;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.swt.widgets.Control;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

/**
 * Property verifier. Checks the property value to match property regExp
 * 
 * @author Alexei Goncharov
 */
public class PropertyVerifier extends AbstractFormattedVerifier {
	private IRepositoryResource	base;
	private Pattern				pattern;
	private String				propName;
	private boolean				toValidate;

	public PropertyVerifier(String fieldName, String regExp, String propName, IRepositoryResource base) {
		super(fieldName);
		if (regExp == null) {
			this.toValidate = false;
			return;
		}
		this.toValidate = true;
		String[] parts = propName.split(":");
		this.propName = "";
		for (int i = 0; i < parts.length; i++) {
			this.propName += parts[i];
		}
		this.pattern = Pattern.compile(regExp);
		this.base = base;
	}

	protected String getErrorMessageImpl(Control input) {
		if (!this.toValidate || this.propName.equals("modelbuslog") || this.propName.equals("modelbusauthor")) {
			return null;
		}
		String inputText = this.getText(input);
		if (this.propName.equals("bugtraqlogregex")) {
			try {
				String[] logs = inputText.split("\r\n");
				for (int i = 0; i < logs.length; i++) {
					Pattern.compile(logs[i]);
				}
			} catch (Exception ex) {
				return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName);
			}
			return null;
		}
		if (this.propName.equals("bugtraqmessage")) {
			if (!inputText.contains("%BUGID%")) {
				return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName);
			}
			return null;
		}
		if (this.propName.equals("modelbusexternals")) {
			try {
				ModelBusUtility.parseModelBusExternalsProperty(inputText, this.base);
			} catch (Exception ex) {
				return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName);
			}
			return null;
		}
		if (this.propName.equals("modelbusdate")) {
			try {
				new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(inputText);
			} catch (Exception ex) {
				return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName,
						new String[] { "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'" });
			}
			return null;
		}
		Matcher matcher = this.pattern.matcher(inputText);
		if (!matcher.matches()) {
			String retVal = ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName);
			if (retVal.equals("PropertyEditPanel.Verifier." + this.propName)) {
				return MessageFormat.format(ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.regExp.Verifier"), this.pattern.pattern());
			}
			return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier." + this.propName);
		}
		return null;
	}

	protected String getWarningMessageImpl(Control input) {
		if (this.propName == null) {
			return null;
		}
		if (this.propName.equals("modelbusauthor") || this.propName.equals("modelbuslog") || this.propName.equals("modelbusdate")) {
			return ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.Verifier.Warning." + this.propName);
		}
		return null;
	}

}
