/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.verifier;

import java.net.URL;

import org.eclipse.swt.widgets.Control;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

/**
 * ModelBus URL verifier
 * 
 * @author Alexander Gurov
 */
public class URLVerifier extends AbstractFormattedVerifier {
	protected static final String	ERROR_REASON	= "$ERROR_REASON$";
	protected static String			ERROR_MESSAGE_SHORT;
	protected static String			ERROR_MESSAGE_FULL;

	public URLVerifier(String fieldName) {
		super(fieldName);
		URLVerifier.ERROR_MESSAGE_SHORT = ModelBusTeamUIPlugin.instance().getResource("Verifier.URL.Short",
				new String[] { AbstractFormattedVerifier.FIELD_NAME });
		URLVerifier.ERROR_MESSAGE_FULL = ModelBusTeamUIPlugin.instance().getResource("Verifier.URL.Full",
				new String[] { AbstractFormattedVerifier.FIELD_NAME, URLVerifier.ERROR_REASON });
	}

	protected String getErrorMessageImpl(Control input) {
		String url = this.getText(input);
		try {
			URL modelbusUrl = ModelBusUtility.getModelBusUrl(url);
			String host = modelbusUrl.getHost();
			if (!host.matches("[a-zA-Z0-9_\\-]+(?:\\.[a-zA-Z0-9_\\-]+)*") && host.length() > 0 || host.length() == 0
					&& !"file".equals(modelbusUrl.getProtocol())) {
				this.setPlaceHolder(URLVerifier.ERROR_REASON, ModelBusTeamUIPlugin.instance().getResource("Verifier.URL.NoHost"));
				return URLVerifier.ERROR_MESSAGE_FULL;
			}
			return null;
		} catch (Exception ex) {
			this.setPlaceHolder(URLVerifier.ERROR_REASON, ex.getMessage());
			return ex.getMessage() == null ? URLVerifier.ERROR_MESSAGE_SHORT : URLVerifier.ERROR_MESSAGE_FULL;
		}
	}

	protected String getWarningMessageImpl(Control input) {
		return null;
	}

}
