/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.extension.CoreExtensionsManager;
import org.modelbus.team.eclipse.core.operation.LoggedOperation;
import org.modelbus.team.eclipse.core.operation.local.management.ReconnectProjectOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * This listener is designed to handle ModelBus team private members
 * 
 * @author Alexander Gurov
 */
public class ModelBusFolderListener implements IResourceChangeListener {

	public ModelBusFolderListener() {

	}

	public void resourceChanged(final IResourceChangeEvent event) {
		try {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(final IProgressMonitor monitor) throws CoreException {
					event.getDelta().accept(new IResourceDeltaVisitor() {
						public boolean visit(IResourceDelta delta) throws CoreException {
							IResource resource = delta.getResource();

							if (!resource.isAccessible()) {
								return false;
							}

							if (resource.getType() == IResource.ROOT) {
								return true;
							}

							if (resource.getType() == IResource.PROJECT && delta.getKind() == IResourceDelta.ADDED
									&& delta.getFlags() == IResourceDelta.OPEN
									&& ModelBusTeamPlugin.instance().getOptionProvider().isAutomaticProjectShareEnabled()
									&& ((IProject) resource).isOpen()) {
								ModelBusChangeStatus info = ModelBusUtility.getModelBusInfoForNotConnected(resource);
								if (info != null && info.url != null) {
									String url = ModelBusUtility.decodeURL(info.url);
									IRepositoryRoot[] roots = ModelBusUtility.findRoots(url, true);
									// IRepositoryLocation location = null;
									if (roots.length == 0) {
										String rootNode = "/" + CoreExtensionsManager.instance().getOptionProvider().getDefaultTrunkName();
										int idx = url.lastIndexOf(rootNode);
										if (idx == -1 || !url.endsWith(rootNode) && url.charAt(idx + rootNode.length()) != '/') {
											rootNode = "/" + CoreExtensionsManager.instance().getOptionProvider().getDefaultBranchesName();
											idx = url.lastIndexOf(rootNode);
											if (idx == -1 || !url.endsWith(rootNode) && url.charAt(idx + rootNode.length()) != '/') {
												rootNode = "/" + CoreExtensionsManager.instance().getOptionProvider().getDefaultTagsName();
												idx = url.lastIndexOf(rootNode);
												if (idx != -1 && !url.endsWith(rootNode) && url.charAt(idx + rootNode.length()) != '/') {
													idx = -1;
												}
											}
										}
										if (idx != -1) {
											url = url.substring(0, idx);
										}
										// location = ModelBusRemoteStorage.instance().newRepositoryLocation();
										// ModelBusUtility.initializeRepositoryLocation(location, url);
										// AddRepositoryLocationOperation mainOp = new AddRepositoryLocationOperation(location);
										// CompositeOperation op = new CompositeOperation(mainOp.getId());
										// op.add(mainOp);
										// op.add(new SaveRepositoryLocationsOperation());
										// important! location doubles when it is added asynchronously and several projects for the same location are
										// imported
										// ProgressMonitorUtility.doTaskExternal(op, monitor);
									} else {
										// location = roots[0].getRepositoryLocation();
									}
									ProgressMonitorUtility.doTaskScheduled(new ReconnectProjectOperation(new IProject[] { (IProject) resource }));
									return false;
								}
							}

							if (delta.getKind() == IResourceDelta.ADDED && (resource instanceof IContainer) && !resource.isTeamPrivateMember()
									&& FileUtility.isModelBusInternals(resource)) {
								FileUtility.findAndMarkModelBusInternals(resource, true);
								return false;
							}

							return true;
						}
					});
				}
			}, null, IWorkspace.AVOID_UPDATE, new NullProgressMonitor());
		} catch (CoreException ex) {
			LoggedOperation.reportError(this.getClass().getName(), ex);
		}
	}

}
