package org.modelbus.team.eclipse.core.connector;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.core.lib.types.IModelBusDataSource;
import org.modelbus.dosgi.repository.descriptor.ConstraintViolationException;
import org.modelbus.dosgi.repository.descriptor.InvalidRevisionException;
import org.modelbus.dosgi.repository.descriptor.InvalidValueException;
import org.modelbus.dosgi.repository.descriptor.LockedException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.RepositoryDirEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryLogEntry;
import org.modelbus.dosgi.repository.descriptor.RepositoryLogPath;
import org.modelbus.dosgi.repository.descriptor.RepositoryNodeChangeType;
import org.modelbus.dosgi.repository.descriptor.RepositoryNodeKind;
import org.modelbus.dosgi.repository.descriptor.RepositoryRuntimeException;
import org.modelbus.dosgi.repository.descriptor.UnresolvedReferencesException;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Kind;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryInfo.ScheduledOperation;
import org.modelbus.team.eclipse.core.connector.ModelBusLogPath.ChangeType;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.StringRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryBranches;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryFile;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryFolder;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryRoot;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryTags;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryTrunk;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

public class ModelBusConnector implements IModelBusConnector {

	private ModelBusConnector() {

	}

	public String retrieveRepoLocationFromPreferenceStore() {
		// return
		// org.modelbus.preferences.ModelBusPreferencePlugin.getDefault().getPreferenceStore().getString(org.modelbus.preferences.PreferenceConstants.REPOSITORY_LOCATION_STRING);
		return RepositoryLocationHelper.getPropertyRepositoryLocation();
	}

	public void add(String path, int depth, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
	}

	public void addToChangeList(String[] paths, String changelist, int depth, String[] changelistNames, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void annotate(ModelBusEntryReference reference, ModelBusRevision revisionStart, ModelBusRevision revisionEnd, long options,
			IModelBusAnnotationCallback callback, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	/**
	 * @deprecated Use ModelBusRepositoryProvider API (Current checkout implementation does not rely on this any more).
	 */
	public long checkout(ModelBusEntryRevisionReference fromReference, String destPath, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {

		if (depth != 3) {
			throw new ModelBusConnectorException("TODOMWA39");
		}
		checkoutFolder(fromReference, destPath, depth, options, monitor);

		return 0;
	}

	/**
	 * @deprecated Use ModelBusRepositoryProvider API (Current checkout implementation does not rely on this any more).
	 * @param fromReference
	 * @param destPath
	 * @param depth
	 * @param options
	 * @param monitor
	 * @return
	 * @throws ModelBusConnectorException
	 */
	public long checkoutFolder(ModelBusEntryRevisionReference fromReference, String destPath, int depth, long options,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {

		File destFolder = new File(destPath);
		if (!destFolder.mkdirs()) {
			throw new ModelBusConnectorException("Could not create dir '" + destPath + "'.");
		}
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		RepositoryDirEntry[] repositoryDirEntries = null;
		try {
			repositoryDirEntries = repositoryHelper.getDirEntries(UserSessionHelper.getSession(), URI.createURI(fromReference.path),
					extractEffectiveRevisionNumber(fromReference));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		} catch (RepositoryAuthentificationException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		} catch (RepositoryRuntimeException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		} catch (InvalidRevisionException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		}

		if (repositoryDirEntries != null) {
			for (int i = 0; i < repositoryDirEntries.length; i++) {
				RepositoryDirEntry repositoryDirEntry = repositoryDirEntries[i];
				if (repositoryDirEntry.getKind() == RepositoryNodeKind.DIR) {
					checkoutFolder(
							new ModelBusEntryRevisionReference(repositoryDirEntry.getUri(), fromReference.pegRevision, fromReference.revision),
							destPath + "/" + repositoryDirEntry.getName(), depth, options, monitor);
				} else {
					checkoutFile(new ModelBusEntryRevisionReference(repositoryDirEntry.getUri(), fromReference.pegRevision, fromReference.revision),
							destPath + "/" + repositoryDirEntry.getName(), depth, options, monitor);
				}
			}
		}
		return 0;
	}

	/**
	 * @deprecated Use ModelBusRepositoryProvider API (Current checkout implementation does not rely on this any more).
	 * @param fromReference
	 * @param destPath
	 * @param depth
	 * @param options
	 * @param monitor
	 * @return
	 * @throws ModelBusConnectorException
	 */
	public long checkoutFile(ModelBusEntryRevisionReference fromReference, String destPath, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();

		try {

			File file = new File(destPath);
			if (!file.exists()) {
				file.createNewFile();
			}
			String effectiveRevNo = extractEffectiveRevisionNumber(fromReference);
			repositoryHelper.checkOutFile(UserSessionHelper.getSession(), URI.createURI(fromReference.path), effectiveRevNo);

		} catch (Exception e) {
			throw new ModelBusConnectorException(e);
		}
		return 0;
	}

	public void cleanup(String path, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public ModelBusRevision[] commit(String[] path, String message, String[] changelistNames, int depth, long options, Map revProps,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public void copy(String[] srcPaths, String destPath, ModelBusRevision revision, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		// System.out
		// .println("TODO: Implement ModelBusConnector#copy(String[] srcPaths, String destPath, ModelBusRevision revision, IModelBusProgressMonitor monitor)");
		throw new ModelBusConnectorException("NYI");

	}

	public void copy(ModelBusEntryRevisionReference[] srcPaths, String destPath, String message, long options, Map revProps,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException, NonExistingResourceException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			repositoryHelper.copy(UserSessionHelper.getSession(), convertModelBusEntryReferences(srcPaths), destPath, message);
		} catch (RepositoryAuthentificationException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		}

	}

	public void diff(ModelBusEntryRevisionReference refPrev, ModelBusEntryRevisionReference refNext, String relativeToDir, String outFileName,
			int depth, long options, String[] changelistNames, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void diff(ModelBusEntryReference reference, ModelBusRevision revPrev, ModelBusRevision revNext, String relativeToDir, String outFileName,
			int depth, long options, String[] changelistNames, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void diffStatus(ModelBusEntryRevisionReference refPrev, ModelBusEntryRevisionReference refNext, int depth, long options,
			String[] changelistNames, IModelBusDiffStatusCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void diffStatus(ModelBusEntryReference reference, ModelBusRevision revPrev, ModelBusRevision revNext, int depth, long options,
			String[] changelistNames, IModelBusDiffStatusCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void dispose() {
		// TODO Auto-generated method stub
		throw new RuntimeException("NYI");

	}

	public long doExport(ModelBusEntryRevisionReference fromReference, String destPath, String nativeEOL, int depth, long options,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return 0;
	}

	public void doImport(String path, String url, String message, int depth, long options, Map revProps, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			if (repositoryHelper.exists(UserSessionHelper.getSession(), URI.createURI(url), IRepositoryHelper.REVISION_HEAD)) // import always works on HEAD
				throw new ModelBusConnectorException("Remote resource " + "'" + url + "'" + " already exists!");
			repositoryHelper.checkInFile(UserSessionHelper.getSession(), URI.createURI(url), new File(path), message);
		} catch (RemoteException remoteException) {
			throw new ModelBusConnectorException(remoteException);
		} catch (IOException ioException) {
			throw new ModelBusConnectorException(ioException);
		} catch (RepositoryAuthentificationException e) {
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			throw new ModelBusConnectorException(e);
		} catch (InvalidValueException e) {
			throw new ModelBusConnectorException(e);
		} catch (ConstraintViolationException e) {
			throw new ModelBusConnectorException(e);
		} catch (LockedException e) {
			throw new ModelBusConnectorException(e);
		} catch (UnresolvedReferencesException e) {
			throw new ModelBusConnectorException(e);
		}
	}

	public long doSwitch(String path, ModelBusEntryRevisionReference toReference, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return 0;
	}

	public void dumpChangeLists(String[] changeLists, String rootPath, int depth, IModelBusChangeListCallback cb, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public String getConfigDirectory() throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public ModelBusMergeInfo getMergeInfo(ModelBusEntryReference reference, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {

		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public void getMergeInfoLog(int logKind, ModelBusEntryReference reference, ModelBusEntryReference mergeSourceReference, String[] revProps,
			long options, IModelBusLogEntryCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	IModelBusNotificationCallback	modelBusNotificationCallback	= null;

	public IModelBusNotificationCallback getNotificationCallback() {
		// TODO Auto-generated method stub
		return modelBusNotificationCallback;
	}

	public IModelBusCredentialsPrompt getPrompt() {
		// TODO Auto-generated method stub
		throw new RuntimeException("NYI");
		// return null;
	}

	public void getProperties(ModelBusEntryRevisionReference reference, int depth, String[] changelistNames, IModelBusPropertyCallback callback,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		callback.next(reference.path, new ModelBusProperty[0]);
	}

	public ModelBusProperty getProperty(ModelBusEntryRevisionReference reference, String name, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		// GDU: if we return null the GetRemoteFolderChildrenOperation does nothing and no error occurs...
		// throw new ModelBusConnectorException("NYI");
		return null;
	}

	public ModelBusProperty[] getRevisionProperties(ModelBusEntryReference reference, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public ModelBusProperty getRevisionProperty(ModelBusEntryReference reference, String name, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public void info(ModelBusEntryRevisionReference reference, int depth, String[] changelistNames, IModelBusEntryInfoCallback cb,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		RepositoryDirEntry dirEntry = null;
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			dirEntry = repositoryHelper.getInfo(UserSessionHelper.getSession(), URI.createURI(reference.path),
					extractEffectiveRevisionNumber(reference));
		} catch (IOException ioException) {
			throw new ModelBusConnectorException(ioException);
		} catch (RepositoryAuthentificationException e) {
			throw new ModelBusConnectorException(e);
		} catch (InvalidRevisionException e) {
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			throw new ModelBusConnectorException(e);
		}
		// TODO: Check all the settings. These are mere defaults.
		if (dirEntry != null) {
			cb.next(new ModelBusEntryInfo(reference.path, // entry local path
					// (TODO: setting
					// not really
					// correct)
					dirEntry.getUri(), // entry remote url
					new ModelBusRevision(dirEntry.getRevision(), dirEntry.getCreationDate().toGregorianCalendar().getTimeInMillis()), // revision
					convertKind(dirEntry.getKind()), // entry node kind
					retrieveRepoLocationFromPreferenceStore(), // repository
					// root url
					"", // repository uuid
					new ModelBusRevision(dirEntry.getRevision(), dirEntry.getCreationDate().toGregorianCalendar().getTimeInMillis()), // last changed revision
					0, // last changed date
					dirEntry.getLastAuthor(), // last changed author
					null, // lock //TODOMES: LockInfo
					false, // this ModelBusEntryinfo includes working copy
					// information
					ScheduledOperation.NORMAL, // @see
					// ModelBusEntryInfo.ScheduledOperation
					"", // copied from URL
					ModelBusRevision.REVISION_0_DESCRIPTOR, // copied from revision
					dirEntry.getCreationDate().toGregorianCalendar().getTimeInMillis(), // content last
					// changed time
					// (nanoseconds)
					// (TODO: setting
					// not
					// really correct)
					0, // properties last changed time (nanoseconds)
					"", // entry checksum
					"", // filename of the base version file (if the entry is in
					// conflicting state)
					"", // filename of the last repository version file (if the
					// entry is in conflicting state)
					""// , // filename of the working copy version file (if the
			// entry is in conflicting state)
			// "" // rejected properties file
			));
		}

	}

	public boolean isCommitMissingFiles() {
		// TODO Auto-generated method stub
		throw new RuntimeException("NYI");
		// return false;
	}

	public boolean isCredentialsCacheEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isSSLCertificateCacheEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isTouchUnresolved() {
		// TODO Auto-generated method stub
		return false;
	}

	public void list(ModelBusEntryRevisionReference reference, int depth, int direntFields, long options, IModelBusEntryCallback cb,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		RepositoryNodeKind targetNodeKind = RepositoryNodeKind.NONE;
		RepositoryDirEntry dirEntries[] = null;
		if (repositoryHelper != null) { // Maybe we just don't have it yet :) Happens to be the case during initialization of the RepositoriesView
			try {
				targetNodeKind = repositoryHelper.checkPath(UserSessionHelper.getSession(), URI.createURI(reference.path),
						extractEffectiveRevisionNumber(reference));

				if (targetNodeKind == RepositoryNodeKind.FILE) {
					dirEntries = new RepositoryDirEntry[] { repositoryHelper.getInfo(UserSessionHelper.getSession(), URI.createURI(reference.path),
							extractEffectiveRevisionNumber(reference)) };
				} else if (targetNodeKind != RepositoryNodeKind.NONE) {// GDU
					dirEntries = repositoryHelper.getDirEntries(UserSessionHelper.getSession(), URI.createURI(reference.path),
							extractEffectiveRevisionNumber(reference));
				}
			} catch (RemoteException remoteException) {
				throw new ModelBusConnectorException(remoteException);
			} catch (IOException ioException) {
				throw new ModelBusConnectorException(ioException);
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				// throw new ModelBusConnectorException(e);
			} catch (RepositoryRuntimeException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			} catch (InvalidRevisionException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			} catch (NonExistingResourceException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			}

			if (dirEntries != null) {
				for (int i = 0; i < dirEntries.length; i++) {
					RepositoryDirEntry repositoryDirEntry = dirEntries[i];
					cb.next(new ModelBusEntry(repositoryDirEntry.getUri(), new ModelBusRevision(repositoryDirEntry.getRevision(), repositoryDirEntry.getCreationDate()
							.toGregorianCalendar().getTimeInMillis()), repositoryDirEntry.getLastAuthor(),
							(repositoryDirEntry.getProperties().isEmpty() ? false : true), convertKind(repositoryDirEntry.getKind()), repositoryDirEntry
									.getSize(), null));
				}
			}
		}
	}

	private int convertKind(RepositoryNodeKind repositoryNodeKind) {
		if (repositoryNodeKind == RepositoryNodeKind.DIR) {
			return Kind.DIR;
		} else if (repositoryNodeKind == RepositoryNodeKind.FILE) {
			return Kind.FILE;
		} else if (repositoryNodeKind == RepositoryNodeKind.MODEL) {
			return Kind.FILE;
		} else if (repositoryNodeKind == RepositoryNodeKind.NONE) {
			return Kind.NONE;
		} else {
			return Kind.UNKNOWN;
		}
	}

	public void lock(String[] path, String comment, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void logEntries(ModelBusEntryReference reference, ModelBusRevision revisionStart, ModelBusRevision revisionEnd, String[] revProps,
			long limit, long options, IModelBusLogEntryCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {

		// For debugging purposes only.
		/*
		 * System.out.println("ModelBusEntryReference: " + reference.toString() + " ModelBusRevision start: " + revisionStart.toString() +
		 * "ModelBusRevision end: " + revisionEnd.toString() + " RevisionProps: " + (revProps != null ? revProps.toString() : "") + " Limit: " + limit
		 * + " Options: " + options);
		 */
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		RepositoryLogEntry[] repositoryLogEntries;
		try {
			repositoryLogEntries = repositoryHelper.getLogEntries(UserSessionHelper.getSession(), new String[] { reference.path },
					getRevisionNumber(revisionStart), getRevisionNumber(revisionEnd), (options & Options.DISCOVER_PATHS) == Options.DISCOVER_PATHS,
					(options & Options.STOP_ON_COPY) == Options.STOP_ON_COPY, limit,
					(options & Options.INCLUDE_MERGED_REVISIONS) == Options.INCLUDE_MERGED_REVISIONS, revProps);
		} catch (RepositoryAuthentificationException e) {
			// TODO Auto-generated catch block
			throw new ModelBusConnectorException(e);
		}

		for (RepositoryLogEntry repositoryLogEntry : repositoryLogEntries) {
			cb.next(convertRepositoryLogEntryToModelBusLogEntry(repositoryLogEntry));
		}
	}

	public void merge(ModelBusEntryRevisionReference reference1, ModelBusEntryRevisionReference reference2, String localPath, int depth,
			long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void merge(ModelBusEntryReference reference, ModelBusRevisionRange[] revisions, String localPath, int depth, long options,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void merge(ModelBusEntryReference reference, String mergePath, ModelBusMergeStatus[] mergeStatus, long options,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void merge(ModelBusEntryReference reference, ModelBusRevisionRange[] revisions, String mergePath, ModelBusMergeStatus[] mergeStatus,
			long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void merge(ModelBusEntryRevisionReference reference1, ModelBusEntryRevisionReference reference2, String mergePath,
			ModelBusMergeStatus[] mergeStatus, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void mergeReintegrate(ModelBusEntryReference reference, String localPath, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void mergeStatus(ModelBusEntryReference reference, String mergePath, long options, IModelBusMergeStatusCallback cb,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void mergeStatus(ModelBusEntryReference reference, ModelBusRevisionRange[] revisions, String path, int depth, long options,
			IModelBusMergeStatusCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void mergeStatus(ModelBusEntryRevisionReference reference1, ModelBusEntryRevisionReference reference2, String path, int depth,
			long options, IModelBusMergeStatusCallback cb, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void mkdir(String[] path, String message, long options, Map revProps, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		for (String currentPath : path) {
			try {
				if (repositoryHelper.exists(UserSessionHelper.getSession(), URI.createURI(currentPath), IRepositoryHelper.REVISION_HEAD))
					throw new ModelBusConnectorException("Remote folder " + "'" + currentPath + "'" + " already exists!");
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			} catch (NonExistingResourceException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			}
			try {
				repositoryHelper.createDir(UserSessionHelper.getSession(), URI.createURI(currentPath), message);
			} catch (RepositoryAuthentificationException e) {
				// TODO Auto-generated catch block
				throw new ModelBusConnectorException(e);
			}
		}

	}

	public void move(String[] srcPaths, String dstPath, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void move(ModelBusEntryReference[] srcPaths, String destPath, String message, long options, Map revProps, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			repositoryHelper.move(UserSessionHelper.getSession(), convertModelBusEntryReferences(srcPaths), destPath, message);
		} catch (RepositoryAuthentificationException e) {
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			throw new ModelBusConnectorException(e);
		}

	}

	private static String[] convertModelBusEntryReferences(ModelBusEntryReference[] references) {
		if (references == null) {
			return new String[0];
		}
		String[] referencesConverted = new String[references.length];
		for (int i = 0; i < references.length; i++) {
			ModelBusEntryReference entry = references[i];
			// TODOMES: Parse entry for [peg]Revision.
			ModelBusRevision pegRevision = entry.pegRevision;
			ModelBusRevision selectedRevision = ModelBusRevision.HEAD;
			if (entry instanceof ModelBusEntryRevisionReference) {
				selectedRevision = ((ModelBusEntryRevisionReference) entry).revision;
			}
			referencesConverted[i] = entry.path + "?pegRevision=" + /* ModelBusRevision.HEAD */pegRevision + "&revision="
					+ /* ModelBusRevision.HEAD */selectedRevision;
		}
		return referencesConverted;
	}

	public void relocate(String from, String to, String path, int depth, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	// TODOMES: Check other parameters. Use monitor to provide cancellation,
	// etc.
	// TODOMWA: Allow transfer of commit message or remove commit dialog for
	// delete.
	public void remove(String[] path, String message, long options, Map revProps, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// use proxy!!! to send notification!
		// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault()
		// .getPreferenceStore()
		// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
		// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
		// .getDefault().getExtensionPreference();
		// RepositoryHelper repositoryHelper = new RepositoryHelper(
		// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		for (String current : path) {
			try {
				repositoryHelper.delete(UserSessionHelper.getSession(), URI.createURI(current), message);
			} catch (RepositoryAuthentificationException e) {
				throw new ModelBusConnectorException(e);
			} catch (LockedException e) {
				throw new ModelBusConnectorException(e);
			} catch (NonExistingResourceException e) {
				throw new ModelBusConnectorException(e);
			}
		}
	}

	public void removeFromChangeLists(String[] paths, int depth, String[] changelistNames, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void removeProperty(String path, String name, int depth, String[] changelistNames, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void removeRevisionProperty(ModelBusEntryReference reference, String name, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void resolve(String path, int conflictResult, int depth, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void revert(String path, int depth, String[] changelistNames, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void setClientSSLCertificate(String certPath, String passphrase) {
		// TODO Auto-generated method stub

	}

	public void setCommitMissingFiles(boolean commitMissingFiles) {
		// TODO Auto-generated method stub

	}

	public void setConfigDirectory(String configDir) throws ModelBusConnectorException {
		// TODO Auto-generated method stub

	}

	public void setConflictResolver(IModelBusConflictResolutionCallback listener) {
		// TODO Auto-generated method stub
		throw new RuntimeException("NYI");

	}

	public void setCredentialsCacheEnabled(boolean cacheCredentials) {
		// TODO Auto-generated method stub

	}

	public void setNotificationCallback(IModelBusNotificationCallback notify) {

		this.modelBusNotificationCallback = notify;
	}

	public void setPassword(String password) {
		// TODO Auto-generated method stub

	}

	public void setPrompt(IModelBusCredentialsPrompt prompt) {
		// TODO Auto-generated method stub

	}

	public void setProperty(String path, String name, String value, int depth, long options, String[] changelistNames,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void setProxy(String host, int port, String userName, String password) {
		// TODO Auto-generated method stub

	}

	public void setRevisionProperty(ModelBusEntryReference reference, String name, String value, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public void setSSHCredentials(String userName, String privateKeyPath, String passphrase, int port) {
		// TODO Auto-generated method stub

	}

	public void setSSHCredentials(String userName, String password, int port) {
		// TODO Auto-generated method stub

	}

	public void setSSLCertificateCacheEnabled(boolean enabled) {
		// TODO Auto-generated method stub

	}

	public void setTouchUnresolved(boolean touchUnresolved) {
		// TODO Auto-generated method stub

	}

	public void setUsername(String username) {
		// TODO Auto-generated method stub

	}

	public void status(String path, int depth, long options, String[] changelistNames, IModelbusEntryStatusCallback callback,
			IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub

		// FileUtility.
		// callback.next(new ModelBusChangeStatus())
	}

	public static String extractEffectiveRevisionNumber(ModelBusEntryRevisionReference reference) {
		String effectiveRevNo = IRepositoryHelper.REVISION_HEAD;
		// Check revision format. See ModelBusRevision.Kind
		switch (reference.revision.revKind) { // pegRevision?
			case ModelBusRevision.Kind.HEAD:
				break; // -1 is fine.
			case ModelBusRevision.Kind.STRING:
				effectiveRevNo = ((StringRevision) reference.revision).revision;
				break;
			default:
				break;
		}
		return effectiveRevNo;
	}

	public void streamFileContent(ModelBusEntryRevisionReference reference, int bufferSize, OutputStream stream, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {

		String effectiveRevNo = extractEffectiveRevisionNumber(reference);

		InputStream istr = null;
		IModelBusDataSource contents;
		try {
			// boolean checkDependencies = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
			// .getDefault()
			// .getPreferenceStore()
			// .getBoolean(org.modelbus.preferences.dependencies.preferences.PreferenceConstants.CHECK_DEPENDENCIES);
			// String[] modelExtensions = org.modelbus.preferences.dependencies.DependenciesPreferencePlugin
			// .getDefault().getExtensionPreference();
			// RepositoryHelper repositoryHelper = new RepositoryHelper(
			// retrieveRepoLocationFromPreferenceStore(),checkDependencies,modelExtensions);
			IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			contents = repositoryHelper.checkOutFileAsDataSource(UserSessionHelper.getSession(), URI.createURI(reference.path), effectiveRevNo);
			if (contents == null)
				return;
			istr = contents.getInputStream();
			if (istr == null)
				return;
			byte[] buffer = new byte[bufferSize];
			int len;
			while ((len = istr.read(buffer)) >= 0) {
				stream.write(buffer, 0, len);
			}
			istr.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String[] suggestMergeSources(ModelBusEntryReference reference, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public void unlock(String[] path, long options, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");

	}

	public long[] update(String[] path, ModelBusRevision revision, int depth, long options, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		throw new ModelBusConnectorException("NYI");
		// return null;
	}

	public void control(String fragmentURI, String fragmentFileLocation, final String message, IModelBusProgressMonitor monitor)
			throws ModelBusConnectorException {
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			repositoryHelper.control(UserSessionHelper.getSession(), URI.createURI(fragmentURI), fragmentFileLocation, message);
		} catch (RepositoryAuthentificationException e) {
			throw new ModelBusConnectorException(e);
		} catch (LockedException e) {
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			throw new ModelBusConnectorException(e);
		}

	}

	public void uncontrol(String resourceUri, final String message, IModelBusProgressMonitor monitor) throws ModelBusConnectorException {
		// TODO Auto-generated method stub
		IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		try {
			repositoryHelper.uncontrol(UserSessionHelper.getSession(), URI.createURI(resourceUri), message);
		} catch (RepositoryAuthentificationException e) {
			throw new ModelBusConnectorException(e);
		} catch (LockedException e) {
			throw new ModelBusConnectorException(e);
		} catch (NonExistingResourceException e) {
			throw new ModelBusConnectorException(e);
		}

	}

	private static ModelBusConnector	modelBusConnector	= null;

	public static IModelBusConnector getModelBusConnector() {
		if (modelBusConnector == null) {
			modelBusConnector = new ModelBusConnector();
		}
		return modelBusConnector;
	}

	public static boolean isSupportAtomicCommit() {
		return true;
	}

	public static String getUserName() {
		return UserSessionHelper.getPropertyUserName();
	}

	public static String getRepositoryRootUrl() {
		return "http://";
	}

	public static IRepositoryFile asRepositoryFile(String url, boolean selectedRevision) {
		// TODOMWA
		// CHECKMWA
		return new ModelBusRepositoryFile(url, ModelBusRevision.HEAD);
	}

	public static IRepositoryContainer asRepositoryContainer(String url, boolean selectedRevision) {
		// TODOMWA
		// CHECKMWA
		// MES: Check, if root was requested ( url == "http:/", @See
		// ModelBusRepositoryResource#getParent() )
		if (getRepositoryRootUrl().startsWith(url)) {
			return getRepositoryRoot();
		}

		if (url.endsWith("/trunk")) {
			return new ModelBusRepositoryTrunk(url, ModelBusRevision.HEAD);
		}

		if (url.endsWith("/tags")) {
			return new ModelBusRepositoryTags(url, ModelBusRevision.HEAD);
		}

		if (url.endsWith("/branches")) {
			return new ModelBusRepositoryBranches(url, ModelBusRevision.HEAD);
		}

		return new ModelBusRepositoryFolder(url, ModelBusRevision.HEAD);
	}

	public static IRepositoryRoot getRepositoryRoot() {
		return new ModelBusRepositoryRoot();
	}

	/* MES: Using UserSessionHelper instead */
	/*
	 * private static Session session = null;
	 * 
	 * private static String oldName = "";
	 * 
	 * private static String oldPassword = "";
	 * 
	 * public static Session getSession() { if (oldName == null) { oldName = ""; } if (oldPassword == null) { oldPassword = ""; }
	 * 
	 * if (session == null // || !oldName.equals(getPropertyUserName()) || // !oldPassword.equals(getPropertyUserPassword()) ) { session = new
	 * Session(); session.setId(EcoreUtil.generateUUID()); Property propertyUserName = new Property(); propertyUserName.setKey("username"); // MES
	 * propertyUserName.setValue("Admin"); // propertyUserName.setValue(UserSessionHelper.getPropertyUserName());
	 * 
	 * Property propertyPassword = new Property(); propertyPassword.setKey("password"); // MES propertyPassword.setValue("ModelBus"); //
	 * propertyPassword.setValue(UserSessionHelper.getPropertyUserPassword());
	 * 
	 * session.addProperties(propertyUserName); session.addProperties(propertyPassword); } return session; }
	 */

	/*
	 * Currently revisions may only be referenced by their revision number.
	 * 
	 * @see ConversionUtility#convert()
	 */
	private static String getRevisionNumber(ModelBusRevision modelBusRevision) {
		if (modelBusRevision == null) {
			return ModelBusRevision.INVALID_REVISION_STRING_2; // not valid
		}
		switch (modelBusRevision.getKind()) {
			case ModelBusRevision.Kind.STRING:
				return ((ModelBusRevision.StringRevision) modelBusRevision).getRevision();
			default:
				return ModelBusRevision.INVALID_REVISION_STRING_2; // not valid
		}
	}

	private static ModelBusLogEntry convertRepositoryLogEntryToModelBusLogEntry(RepositoryLogEntry repositoryLogEntry) {

		List<ModelBusLogPath> modelBusLogPaths = new ArrayList<ModelBusLogPath>();
		List<RepositoryLogPath> repositoryLogPaths = repositoryLogEntry.getChangedPaths();

		for (RepositoryLogPath repositoryLogPath : repositoryLogPaths) {
			//TODO date
			final ModelBusRevision revision = new ModelBusRevision(repositoryLogPath.getCopiedFromRevision(), null);
			
			modelBusLogPaths.add(new ModelBusLogPath(repositoryLogPath.getPath(), convertNodeChangeTypeToChangeType(repositoryLogPath.getAction()),
					repositoryLogPath.getCopiedFromPath(), revision));
		}

		ModelBusLogEntry modelBusLogEntry = new ModelBusLogEntry(repositoryLogEntry.getRevision(), repositoryLogEntry.getDate().toGregorianCalendar()
				.getTimeInMillis(), repositoryLogEntry.getAuthor(), repositoryLogEntry.getMessage(),
				modelBusLogPaths.toArray(new ModelBusLogPath[] {}), repositoryLogEntry.isHasChildren());

		return modelBusLogEntry;
	}

	private static char convertNodeChangeTypeToChangeType(RepositoryNodeChangeType nodeChangeType) {
		if (nodeChangeType == RepositoryNodeChangeType.ADDED) {
			return ChangeType.ADDED;
		} else if (nodeChangeType == RepositoryNodeChangeType.DELETED) {
			return ChangeType.DELETED;
		} else if (nodeChangeType == RepositoryNodeChangeType.MODIFIED) {
			return ChangeType.MODIFIED;
		} else if (nodeChangeType == RepositoryNodeChangeType.REPLACED) {
			return ChangeType.REPLACED;
		} else { // n is for none. Should never come to this.
			return 'n';
		}
	}
}
