/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.connector;

/**
 * ModelBus connector unresolved conflict exception
 * 
 * The JavaHL API's is the only way to interact between ModelBus and Java-based tools. At the same time JavaHL connector library is not EPL compatible
 * and we won't to pin plug-in with concrete connector implementation. So, the only way to do this is providing our own connector interface which will
 * be covered by concrete connector implementation.
 * 
 * @author Alexander Gurov
 */
public class ModelBusConnectorUnresolvedConflictException extends ModelBusConnectorException {
	private static final long	serialVersionUID	= 7591147418116040719L;

	public ModelBusConnectorUnresolvedConflictException(String message) {
		super(message, ModelBusErrorCodes.fsConflict, null);
	}

	public ModelBusConnectorUnresolvedConflictException(Throwable cause) {
		super(cause == null ? null : cause.getMessage(), ModelBusErrorCodes.fsConflict, cause);
	}

	public ModelBusConnectorUnresolvedConflictException(String message, Throwable cause) {
		super(message, ModelBusErrorCodes.fsConflict, cause);
	}

}
