/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.history;

import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.history.IFileHistory;
import org.eclipse.team.core.history.IFileHistoryProvider;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.core.history.provider.FileHistoryProvider;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.GetLogMessagesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * ModelBus file history provider implementation
 * 
 * @author Alexander Gurov
 */
public class ModelBusFileHistoryProvider extends FileHistoryProvider {

	public IFileHistory getFileHistoryFor(IResource resource, int flags, IProgressMonitor monitor) {
		IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resource);
		GetLogMessagesOperation logOp = new GetLogMessagesOperation(remote);
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(resource);
		if ((flags & IFileHistoryProvider.SINGLE_REVISION) != 0) {
			if (local.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
				remote.setSelectedRevision(local.getRevision());
			}
			logOp.setLimit(1);
		} else if ((flags & IFileHistoryProvider.SINGLE_LINE_OF_DESCENT) != 0) {
			if (local.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
				remote.setSelectedRevision(local.getRevision());
			}
			logOp.setLimit(2);
		}
		ProgressMonitorUtility.doTaskExternal(logOp, monitor);
		if (logOp.getExecutionState() == IActionOperation.OK && logOp.getMessages() != null) {
			return new ModelBusResourceHistory(local, remote, logOp.getMessages(), flags == 0);
		}
		return null;
	}

	public IFileRevision getWorkspaceFileRevision(IResource resource) {
		return new ModelBusLocalResourceRevision(ModelBusRemoteStorage.instance().asLocalResource(resource), ModelBusRevision.WORKING);
	}

	public IFileHistory getFileHistoryFor(IFileStore store, int flags, IProgressMonitor monitor) {
		return null;
	}

}
