/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import org.eclipse.core.resources.IResource;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.ICommentProvider;
import org.modelbus.team.eclipse.core.resource.IFolderChange;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;

/**
 * Folder change descriptor
 * 
 * @author Alexander Gurov
 */
public class ModelBusFolderChange extends ModelBusLocalFolder implements IFolderChange {
	protected ModelBusRevision		pegRevision;
	protected IRepositoryResource	originator;
	protected String				comment;
	protected ICommentProvider		provider;

	public ModelBusFolderChange(IResource resource, ModelBusRevision revision, String status, int changeMask, String author, long lastCommitDate,
			ModelBusRevision pegRevision, String comment) {
		super(resource, revision, revision, status, changeMask, author, lastCommitDate);
		this.comment = comment;
		this.pegRevision = pegRevision;
	}

	public void treatAsReplacement() {
		this.status = IStateFilter.ST_REPLACED;
	}

	public ModelBusRevision getPegRevision() {
		return this.pegRevision == null ? (this.revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? this.revision
				: ModelBusRevision.INVALID_REVISION_DESCRIPTOR) : this.pegRevision;
	}

	public void setPegRevision(ModelBusRevision pegRevision) {
		this.pegRevision = pegRevision;
	}

	public ILocalResource[] getChildren() {
		return new ILocalResource[0];
	}

	public IRepositoryResource getOriginator() {
		if (this.originator == null && this.getRevision() != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(this.resource);
			remote.setPegRevision(this.getPegRevision());
			remote.setSelectedRevision(this.getRevision());
			return remote;
		}
		return this.originator;
	}

	public void setOriginator(IRepositoryResource originator) {
		this.originator = originator;
	}

	public synchronized String getComment() {
		if (this.comment == null && this.provider != null) {
			ModelBusRevision rev = this.getRevision();
			this.comment = this.provider.getComment(this.getResource(),
					rev == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? ModelBusRevision.INVALID_REVISION_DESCRIPTOR : rev,
					this.getPegRevision());
			this.provider = null;
		}
		return this.comment;
	}

	public void setCommentProvider(ICommentProvider provider) {
		this.provider = provider;
	}

}
