/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.local.GetAllResourcesOperation;
import org.modelbus.team.eclipse.core.resource.ILocalFolder;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Working Copy folder representation
 * 
 * @author Alexander Gurov
 */
public class ModelBusLocalFolder extends ModelBusLocalResource implements ILocalFolder {
	public ModelBusLocalFolder(IResource resource, ModelBusRevision revision, ModelBusRevision baseRevision, String status, int changeMask, String author, long lastCommitDate) {
		super(resource, revision, baseRevision, status, changeMask, author, lastCommitDate);
	}

	public ILocalResource[] getChildren() {
		IContainer root = (IContainer) this.resource;
		List<ILocalResource> members = new ArrayList<ILocalResource>();

		GetAllResourcesOperation op = new GetAllResourcesOperation(root);
		ProgressMonitorUtility.doTaskExternalDefault(op, new NullProgressMonitor());
		IResource[] resources = op.getChildren();
		for (int i = 0; i < resources.length; i++) {
			ILocalResource local = ModelBusRemoteStorage.instance().asLocalResource(resources[i]);
			if (!IStateFilter.SF_INTERNAL_INVALID.accept(local) && local.getStatus() != IStateFilter.ST_NOTEXISTS) {
				members.add(local);
			}
		}

		return members.toArray(new ILocalResource[members.size()]);
	}

}
