/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.modelbusstorage;

import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusEntry.Fields;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryFile;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * ModelBus based representation of IRepositoryFile
 * 
 * @author Alexander Gurov
 */
public class ModelBusRepositoryFile extends ModelBusRepositoryResource implements IRepositoryFile {
	private static final long	serialVersionUID	= 6042328067024796901L;

	public ModelBusRepositoryFile(String url, ModelBusRevision selectedRevision) {
		super(url, selectedRevision);
	}

	protected void getRevisionImpl(IModelBusConnector proxy) throws ModelBusConnectorException {
		ModelBusEntry[] entries = ModelBusUtility.list(proxy, ModelBusUtility.getEntryRevisionReference(this), Depth.EMPTY, Fields.ALL,
				IModelBusConnector.Options.FETCH_LOCKS, new ModelBusNullProgressMonitor());
		if (entries != null && entries.length > 0 && entries[0].revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {// FIXME -1 for ModelBus Kit
																														// 1.2.0
																														// if resource is not exists
			this.lastRevision = entries[0].revision;
			this.setInfo(new IRepositoryResource.Information(entries[0].lock, entries[0].size, entries[0].author, entries[0].revision.getDate(),
					entries[0].hasProperties));
		}
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof IRepositoryFile)) {
			return false;
		}
		return super.equals(obj);
	}

}
