/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelbusEntryStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;

/**
 * Abstract status reporting operation
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractStatusOperation extends AbstractFileOperation {
	protected ModelBusChangeStatus[]	statuses;
	protected boolean					recursive;

	public AbstractStatusOperation(String operationName, File[] files, boolean recursive) {
		super(operationName, files);
		this.recursive = recursive;
	}

	public AbstractStatusOperation(String operationName, IFileProvider provider, boolean recursive) {
		super(operationName, provider);
		this.recursive = recursive;
	}

	public ModelBusChangeStatus[] getStatuses() {
		return this.statuses;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		final List<ModelBusChangeStatus> result = new ArrayList<ModelBusChangeStatus>();
		IModelbusEntryStatusCallback cb = new IModelbusEntryStatusCallback() {
			public void next(ModelBusChangeStatus status) {
				result.add(status);
			}
		};
		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(files[i], false);

			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.reportStatuses(proxy, cb, files[i], monitor, files.length);
		}
		this.statuses = result.toArray(new ModelBusChangeStatus[result.size()]);
	}

	protected void reportStatuses(final IModelBusConnector proxy, final IModelbusEntryStatusCallback cb, final File current,
			IProgressMonitor monitor, int tasks) {
		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				proxy.status(current.getAbsolutePath(), Depth.infinityOrImmediates(AbstractStatusOperation.this.recursive),
						AbstractStatusOperation.this.isRemote() ? IModelBusConnector.Options.SERVER_SIDE : IModelBusConnector.Options.NONE, null, cb,
						new ModelBusProgressMonitor(AbstractStatusOperation.this, monitor, null, false));
			}
		}, monitor, tasks);
	}

	protected abstract boolean isRemote();

}
