/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Checkout specified resource from repository
 * 
 * @author Alexander Gurov
 */
public class CheckoutAsOperation extends AbstractFileOperation {
	protected IRepositoryResource	resource;
	protected int					recureDepth;
	protected boolean				ignoreExternals;
	protected boolean				override;

	public CheckoutAsOperation(File to, IRepositoryResource resource, int recureDepth, boolean ignoreExternals, boolean override) {
		super("Operation.CheckoutAsFile", new File[] { to });
		this.resource = resource;
		this.recureDepth = recureDepth;
		this.ignoreExternals = ignoreExternals;
		this.override = override;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File to = this.operableData()[0];
		if (this.override) {
			FileUtility.deleteRecursive(to, monitor);
		}

		to.mkdirs();

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			String path = to.getAbsolutePath();
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus checkout \"" + this.resource.getUrl() + "@" + this.resource.getPegRevision() +
			// "\" -r " + this.resource.getSelectedRevision() + (this.recursive ? "" : " -N") + " --ignore-externals \"" +
			// FileUtility.normalizePath(path) +
			// "\"" + FileUtility.getUsernameParam(location.getUsername()) + "\n");
			proxy.checkout(ModelBusUtility.getEntryRevisionReference(this.resource), path, this.recureDepth,
					this.ignoreExternals ? IModelBusConnector.Options.IGNORE_EXTERNALS : IModelBusConnector.Options.NONE,
					new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

}
