/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.io.FileFilter;
import java.util.Arrays;
import java.util.HashSet;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Get all children for the specified folder (including non-committed deletions)
 * 
 * @author Alexander Gurov
 */
public class GetAllFilesOperation extends AbstractFileOperation implements IFileProvider {
	protected File[]	children;

	public GetAllFilesOperation(File file) {
		super("Operation.GetAllFiles", new File[] { file });
	}

	public File[] getFiles() {
		return this.children;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.children = new File[0];

		File file = this.operableData()[0];
		if (!file.exists() || file.isFile()) {
			return;
		}

		final HashSet<File> allFiles = new HashSet<File>();

		File[] children = file.listFiles(new FileFilter() {
			public boolean accept(File arg0) {
				return !arg0.getName().equals(ModelBusUtility.getModelBusFolderName());
			}
		});
		if (children != null) {
			allFiles.addAll(Arrays.asList(children));
		}

		// IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(file, true);
		// if (remote != null) {
		// IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// try {
		// proxy.status(file.getAbsolutePath(), Depth.IMMEDIATES, IModelBusConnector.Options.INCLUDE_UNCHANGED |
		// IModelBusConnector.Options.INCLUDE_IGNORED, null, new IModelbusEntryStatusCallback() {
		// public void next(ModelBusChangeStatus status) {
		// allFiles.add(new File(status.path));
		// }
		// }, new ModelBusProgressMonitor(this, monitor, null));
		// }
		// finally {
		// }
		// }

		// this.children = allFiles.toArray(new File[allFiles.size()]);
		throw new RuntimeException("NYI");
	}

}
