/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.io.OutputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * This operation retrieve a content of the file in selected revision
 * 
 * @author Alexander Gurov
 */
public class GetFileContentOperation extends AbstractFileOperation {
	public static final int		DEFAULT_BUFFER_SIZE	= 2048;

	protected ModelBusRevision	revision;
	protected ModelBusRevision	pegRevision;
	protected int				bufferSize;
	protected OutputStream		target;

	public GetFileContentOperation(File file, ModelBusRevision revision, ModelBusRevision pegRevision, OutputStream target) {
		this(file, revision, pegRevision, GetFileContentOperation.DEFAULT_BUFFER_SIZE, target);
	}

	public GetFileContentOperation(File file, ModelBusRevision revision, ModelBusRevision pegRevision, int bufferSize, OutputStream target) {
		super("Operation.GetFileContent", new File[] { file });
		this.revision = revision;
		this.pegRevision = pegRevision;
		this.bufferSize = bufferSize;
		this.target = target;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File file = this.operableData()[0];

		IRepositoryResource remote = ModelBusFileStorage.instance().asRepositoryResource(file, false);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			int kind = this.revision.getKind();
			if (kind == Kind.BASE || kind == Kind.WORKING) {
				proxy.streamFileContent(new ModelBusEntryRevisionReference(file.getAbsolutePath(), null, this.revision), this.bufferSize,
						this.target, new ModelBusProgressMonitor(this, monitor, null));
			} else {
				proxy.streamFileContent(ModelBusUtility.getEntryRevisionReference(remote), this.bufferSize, this.target, new ModelBusProgressMonitor(
						this, monitor, null));
			}
		} finally {
		}
	}

}
