/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.IModelbusEntryStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * This operation fetch remote resource statuses
 * 
 * @author Alexander Gurov
 */
public class RemoteStatusOperation extends AbstractStatusOperation implements IModelBusNotificationCallback {
	protected Map<String, ModelBusRevision>	pegRevisions	= new HashMap<String, ModelBusRevision>();

	public RemoteStatusOperation(File[] files, boolean recursive) {
		super("Operation.UpdateStatusFile", files, recursive);
	}

	public RemoteStatusOperation(IFileProvider provider, boolean recursive) {
		super("Operation.UpdateStatusFile", provider, recursive);
	}

	public ModelBusRevision getPegRevision(File change) {
		IPath resourcePath = new Path(change.getAbsolutePath());
		for (Iterator<?> it = this.pegRevisions.entrySet().iterator(); it.hasNext();) {
			Map.Entry entry = (Map.Entry) it.next();
			IPath rootPath = new Path((String) entry.getKey());
			if (rootPath.isPrefixOf(resourcePath)) {
				return (ModelBusRevision) entry.getValue();
			}
		}
		return null;
	}

	public void notify(ModelBusNotification info) {
		if (info.revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
			this.pegRevisions.put(info.path, info.revision);
		}
	}

	protected void reportStatuses(final IModelBusConnector proxy, final IModelbusEntryStatusCallback cb, final File current,
			IProgressMonitor monitor, int tasks) {
		ModelBusUtility.addModelBusNotifyListener(proxy, this);
		super.reportStatuses(proxy, cb, current, monitor, tasks);
		ModelBusUtility.removeModelBusNotifyListener(proxy, this);
	}

	protected boolean isRemote() {
		return true;
	}

}
