/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * The operation switches working copy base url
 * 
 * @author Alexander Gurov
 */
public class SwitchOperation extends AbstractFileOperation {
	protected IRepositoryResource	destination;

	public SwitchOperation(File file, IRepositoryResource destination) {
		super("Operation.SwitchFile", new File[] { file });
		this.destination = destination;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File file = this.operableData()[0];

		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		this.writeToConsole(IConsoleStream.LEVEL_CMD,
				"modelbus switch \"" + this.destination.getUrl() + "\" \"" + FileUtility.normalizePath(file.getAbsolutePath()) + "\" -r "
						+ this.destination.getSelectedRevision() + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
		try {
			proxy.doSwitch(file.getAbsolutePath(), ModelBusUtility.getEntryRevisionReference(this.destination), Depth.infinityOrFiles(true),
					IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

}
