/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.file.management;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.file.AbstractFileOperation;
import org.modelbus.team.eclipse.core.operation.file.IFileProvider;
import org.modelbus.team.eclipse.core.operation.file.IFileStorage;
import org.modelbus.team.eclipse.core.operation.file.ModelBusFileStorage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Cleanup working copy after failure operation implementation
 * 
 * @author Alexander Gurov
 */
public class CleanupOperation extends AbstractFileOperation {
	public CleanupOperation(File[] files) {
		super("Operation.CleanupFile", files);
	}

	public CleanupOperation(IFileProvider provider) {
		super("Operation.CleanupFile", provider);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		File[] files = this.operableData();

		IFileStorage storage = ModelBusFileStorage.instance();

		for (int i = 0; i < files.length && !monitor.isCanceled(); i++) {
			IRepositoryResource remote = storage.asRepositoryResource(files[i], false);

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			final String current = files[i].getAbsolutePath();

			ProgressMonitorUtility.setTaskInfo(monitor, this, current);

			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus cleanup \"" + FileUtility.normalizePath(current) + "\"\n");

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.cleanup(current, new ModelBusProgressMonitor(CleanupOperation.this, monitor, null));
				}
			}, monitor, files.length);
		}
	}

}
