/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.StringTokenizer;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty.BuiltIn;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IModelBusStorage;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * "Add to modelbus::ignore" operation implementation
 * 
 * @author Alexander Gurov
 */
public class AddToModelBusIgnoreOperation extends AbstractWorkingCopyOperation {
	protected int		ignoreType;
	protected String	pattern;

	public AddToModelBusIgnoreOperation(IResource[] resources, int ignoreType, String pattern) {
		super("Operation.AddToModelBusIgnore", resources);

		this.ignoreType = ignoreType;
		this.pattern = pattern;
	}

	public AddToModelBusIgnoreOperation(IResourceProvider provider, int ignoreType, String pattern) {
		super("Operation.AddToModelBusIgnore", provider);

		this.ignoreType = ignoreType;
		this.pattern = pattern;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		final IRemoteStorage storage = ModelBusRemoteStorage.instance();

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource current = resources[i];

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					AddToModelBusIgnoreOperation.this.handleResource(storage, current);
				}
			}, monitor, resources.length);
		}
	}

	protected void handleResource(IRemoteStorage storage, IResource current) throws Exception {
		IResource parent = current.getParent();
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			AddToModelBusIgnoreOperation.changeIgnoreProperty(proxy, this.ignoreType, this.pattern, FileUtility.getWorkingCopyPath(parent),
					current.getName());
		} finally {
		}
	}

	public static void changeIgnoreProperty(IModelBusConnector proxy, int ignoreType, String pattern, String path, String name) throws Exception {
		ModelBusProperty data = proxy.getProperty(new ModelBusEntryRevisionReference(path), BuiltIn.IGNORE, new ModelBusNullProgressMonitor());
		String ignoreValue = data == null ? "" : data.value;
		String mask = null;
		switch (ignoreType) {
			case IModelBusStorage.IGNORE_NAME: {
				mask = name;
				break;
			}
			case IModelBusStorage.IGNORE_EXTENSION: {
				String extension = new Path(path + "/" + name).getFileExtension();
				if (extension != null) {
					mask = "*." + extension;
				}
				break;
			}
			case IModelBusStorage.IGNORE_PATTERN: {
				mask = pattern;
				break;
			}
		}
		ignoreValue = AddToModelBusIgnoreOperation.addMask(ignoreValue, mask);
		proxy.setProperty(path, BuiltIn.IGNORE, ignoreValue, Depth.EMPTY, IModelBusConnector.Options.NONE, null, new ModelBusNullProgressMonitor());
	}

	protected static String addMask(String ignore, String mask) {
		if (mask == null || mask.length() == 0) {
			return ignore;
		}
		StringTokenizer tok = new StringTokenizer(ignore, "\n", false);
		boolean found = false;
		while (tok.hasMoreTokens()) {
			if (tok.nextToken().equals(mask)) {
				found = true;
				break;
			}
		}

		return found ? ignore : (ignore + (ignore.length() > 0 ? "\n" : "") + mask);
	}

}
