/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.StringTokenizer;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty.BuiltIn;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusNullProgressMonitor;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Add to version control operation
 * 
 * @author Alexander Gurov
 */
public class AddToModelBusOperation extends AbstractWorkingCopyOperation {
	protected boolean	isRecursive;

	public AddToModelBusOperation(IResource[] resources) {
		this(resources, false);
	}

	public AddToModelBusOperation(IResource[] resources, boolean isRecursive) {
		super("Operation.AddToModelBus", resources);
		this.isRecursive = isRecursive;
	}

	public AddToModelBusOperation(IResourceProvider provider) {
		this(provider, false);
	}

	public AddToModelBusOperation(IResourceProvider provider, boolean isRecursive) {
		super("Operation.AddToModelBus", provider);
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		if (this.isRecursive) {
			this.isRecursive = !FileUtility.checkForResourcesPresenceRecursive(resources, IStateFilter.SF_IGNORED);
		}
		if (this.isRecursive) {
			resources = FileUtility.shrinkChildNodes(resources);
		} else {
			FileUtility.reorder(resources, true);
		}

		final IRemoteStorage storage = ModelBusRemoteStorage.instance();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource current = resources[i];
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					AddToModelBusOperation.this.doAdd(current, proxy, monitor);
				}
			}, monitor, resources.length);
		}
	}

	public static void removeFromParentIgnore(IModelBusConnector proxy, String parentPath, String name) throws Exception {
		ModelBusProperty data = proxy.getProperty(new ModelBusEntryRevisionReference(parentPath), BuiltIn.IGNORE, new ModelBusNullProgressMonitor());
		String ignoreValue = data == null ? "" : data.value;

		StringTokenizer tok = new StringTokenizer(ignoreValue, "\n", true);
		ignoreValue = "";
		boolean skipToken = false;
		while (tok.hasMoreTokens()) {
			String oneOf = tok.nextToken();

			if (!oneOf.equals(name) && !skipToken) {
				ignoreValue += oneOf;
			} else {
				skipToken = !skipToken;
			}
		}

		if (ignoreValue.length() > 0) {
			proxy.setProperty(parentPath, BuiltIn.IGNORE, ignoreValue, Depth.EMPTY, IModelBusConnector.Options.NONE, null,
					new ModelBusNullProgressMonitor());
		} else {
			proxy.removeProperty(parentPath, BuiltIn.IGNORE, Depth.EMPTY, null, new ModelBusNullProgressMonitor());
		}
	}

	protected void doAdd(IResource current, IModelBusConnector proxy, IProgressMonitor monitor) throws Exception {
		String wcPath = FileUtility.getWorkingCopyPath(current);

		AddToModelBusOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus add \"" + FileUtility.normalizePath(wcPath) + "\""
				+ (AddToModelBusOperation.this.isRecursive ? "" : " -N") + "\n");

		IResource parent = current.getParent();
		if (parent != null) {
			AddToModelBusOperation.removeFromParentIgnore(proxy, FileUtility.getWorkingCopyPath(parent), current.getName());
		}

		proxy.add(wcPath, Depth.infinityOrEmpty(AddToModelBusOperation.this.isRecursive), IModelBusConnector.Options.FORCE
				| IModelBusConnector.Options.INCLUDE_PARENTS, new ModelBusProgressMonitor(AddToModelBusOperation.this, monitor, null));
	}

}
