/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.Team;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnectorException;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty.BuiltIn;
import org.modelbus.team.eclipse.core.extension.CoreExtensionsManager;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Set content properties operation
 * 
 * @author Sergiy Logvin
 */
public class AddToModelBusWithPropertiesOperation extends AddToModelBusOperation {
	protected static final String	BINARY_FILE	= "application/octet-stream";
	protected static final String	TEXT_FILE	= "text/plain";

	public AddToModelBusWithPropertiesOperation(IResource[] resources) {
		this(resources, false);
	}

	public AddToModelBusWithPropertiesOperation(IResource[] resources, boolean isRecursive) {
		super(resources, isRecursive);
	}

	public AddToModelBusWithPropertiesOperation(IResourceProvider provider, boolean isRecursive) {
		super(provider, isRecursive);
	}

	protected void doAdd(IResource current, final IModelBusConnector proxy, final IProgressMonitor monitor) throws Exception {
		super.doAdd(current, proxy, monitor);

		if (!this.isRecursive) {
			this.processResource(current, proxy, monitor);
		} else {
			FileUtility.visitNodes(current, new IResourceVisitor() {

				public boolean visit(IResource resource) throws CoreException {
					if (monitor.isCanceled()) {
						return false;
					}

					try {
						AddToModelBusWithPropertiesOperation.this.processResource(resource, proxy, monitor);
					} catch (ModelBusConnectorException cwe) {
						AddToModelBusWithPropertiesOperation.this.reportError(cwe);
						return false;
					}
					return true;
				}

			}, IResource.DEPTH_INFINITE);
		}
	}

	protected void processResource(IResource resource, IModelBusConnector proxy, IProgressMonitor monitor) throws ModelBusConnectorException {
		String path = FileUtility.getWorkingCopyPath(resource);
		ModelBusProperty[] properties = CoreExtensionsManager.instance().getOptionProvider().getAutomaticProperties(resource.getName());
		for (int pCount = 0; pCount < properties.length; pCount++) {
			proxy.setProperty(path, properties[pCount].name, properties[pCount].value, Depth.EMPTY, IModelBusConnector.Options.NONE, null,
					new ModelBusProgressMonitor(this, monitor, null));
		}
		if (resource.getType() == IResource.FILE) {
			this.processFile(resource, proxy, monitor);
		}
	}

	protected void processFile(IResource resource, IModelBusConnector proxy, IProgressMonitor monitor) throws ModelBusConnectorException {
		String path = FileUtility.getWorkingCopyPath(resource);
		int type = FileUtility.getMIMEType(resource);
		ModelBusProperty data = proxy.getProperty(new ModelBusEntryRevisionReference(path), BuiltIn.MIME_TYPE, new ModelBusProgressMonitor(this,
				monitor, null));
		if (data == null) {
			if (type == Team.BINARY) {
				proxy.setProperty(path, BuiltIn.MIME_TYPE, AddToModelBusWithPropertiesOperation.BINARY_FILE, Depth.EMPTY,
						IModelBusConnector.Options.NONE, null, new ModelBusProgressMonitor(this, monitor, null));
			} else if (type == Team.TEXT) {
				proxy.setProperty(path, BuiltIn.MIME_TYPE, AddToModelBusWithPropertiesOperation.TEXT_FILE, Depth.EMPTY,
						IModelBusConnector.Options.NONE, null, new ModelBusProgressMonitor(this, monitor, null));
			}
		}
	}

}
