/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    Alexey Mikoyan - Initial implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Branch and Tag from working copy operation implementation
 * 
 * @author Alexey Mikoyan
 * 
 */
public class BranchTagOperation extends AbstractWorkingCopyOperation {
	protected IRepositoryResource	destination;
	protected String				message;

	public BranchTagOperation(String operationName, IResource[] resources, IRepositoryResource destination, String message) {
		super("Operation." + operationName, resources);
		this.destination = destination;
		this.message = message;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		ProgressMonitorUtility.setTaskInfo(monitor, this, FileUtility.getNamesListAsString(resources));
		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			final String destinationUrl = ModelBusUtility.encodeURL(this.destination.getUrl());
			for (int i = 0; i < resources.length; i++) {
				final String wcPath = FileUtility.getWorkingCopyPath(resources[i]);

				IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
					public void notify(ModelBusNotification info) {
						if (!"-1".equals(info.revision)) {
							String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
									new String[] { String.valueOf(info.revision) });
							BranchTagOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
						}
					}
				};
				ModelBusUtility.addModelBusNotifyListener(proxy, notify);

				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						BranchTagOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD,
								"modelbus copy \"" + wcPath + "\" \"" + destinationUrl + "\" -r " + ModelBusRevision.WORKING + " -m \""
										+ BranchTagOperation.this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName())
										+ "\n");
						ModelBusEntryRevisionReference[] src = new ModelBusEntryRevisionReference[] { new ModelBusEntryRevisionReference(wcPath,
								null, ModelBusRevision.WORKING) };
						proxy.copy(src, destinationUrl, BranchTagOperation.this.message, IModelBusConnector.Options.INCLUDE_PARENTS, null,
								new ModelBusProgressMonitor(BranchTagOperation.this, monitor, null));
					}
				}, monitor, resources.length);

				ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
			}
		} finally {
		}
	}

}
