/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * Export local resources
 * 
 * @author Alexander Gurov
 */
public class ExportOperation extends AbstractWorkingCopyOperation {
	protected ModelBusRevision	revision;
	protected String			path;

	public ExportOperation(IResource[] resources, String path, ModelBusRevision revision) {
		super("Operation.ExportRevision", resources);
		this.revision = revision;
		this.path = path;
	}

	public ExportOperation(IResourceProvider provider, String path, ModelBusRevision revision) {
		super("Operation.ExportRevision", provider);
		this.revision = revision;
		this.path = path;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource current = resources[i];
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					String wcPath = FileUtility.getWorkingCopyPath(current);
					String targetPath = ExportOperation.this.path + "/" + current.getName();
					ExportOperation.this.writeToConsole(
							IConsoleStream.LEVEL_CMD,
							"modelbus export \"" + wcPath + "\" -r " + ExportOperation.this.revision.toString() + " \""
									+ FileUtility.normalizePath(targetPath) + "\" --force"
									+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
					proxy.doExport(new ModelBusEntryRevisionReference(wcPath, null, ExportOperation.this.revision), targetPath, null, Depth.INFINITY,
							IModelBusConnector.Options.FORCE, new ModelBusProgressMonitor(ExportOperation.this, monitor, null));
				}
			}, monitor, resources.length);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { FileUtility.getNamesListAsString(this.operableData()) });
	}

}
