/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.io.File;
import java.io.FileOutputStream;
import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.AbstractGetFileContentOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.utility.FileUtility;

/**
 * This operation retrieve a local file content of selected revision: BASE or WORKING
 * 
 * @author Alexander Gurov
 */
public class GetLocalFileContentOperation extends AbstractGetFileContentOperation {
	protected IResource			resource;
	protected ModelBusRevision	revision;

	public GetLocalFileContentOperation(IResource resource, int revisionKind) {
		super("Local");
		this.resource = resource;
		this.revision = revisionKind == Kind.BASE ? ModelBusRevision.BASE : ModelBusRevision.WORKING;
	}

	public int getOperationWeight() {
		return 0;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		if (this.revision.getKind() == Kind.BASE) {
			IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			FileOutputStream stream = null;
			try {
				this.tmpFile = this.createTempFile();
				stream = new FileOutputStream(this.tmpFile);
				proxy.streamFileContent(new ModelBusEntryRevisionReference(FileUtility.getWorkingCopyPath(this.resource), null, this.revision), 2048,
						stream, new ModelBusProgressMonitor(GetLocalFileContentOperation.this, monitor, null));
			} finally {
				if (stream != null) {
					try {
						stream.close();
					} catch (Exception ex) {
					}
				}
			}
		} else {
			this.tmpFile = new File(FileUtility.getWorkingCopyPath(this.resource));
		}
	}

	protected String getExtension() {
		return this.resource.getFileExtension();
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.resource.getName() });
	}

}
