/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Lock resources operation implementation
 * 
 * @author Alexander Gurov
 */
public class LockOperation extends AbstractWorkingCopyOperation {
	protected String	message;
	protected boolean	force;

	public LockOperation(IResource[] resources, String message, boolean force) {
		super("Operation.Lock", resources);
		this.message = message;
		this.force = force;
	}

	public LockOperation(IResourceProvider provider, String message, boolean force) {
		super("Operation.Lock", provider);
		this.message = message;
		this.force = force;
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		Map<?, ?> wc2Resources = ModelBusUtility.splitWorkingCopies(resources);
		for (Iterator<?> it = wc2Resources.entrySet().iterator(); it.hasNext() && !monitor.isCanceled();) {
			Map.Entry entry = (Map.Entry) it.next();
			final String[] paths = FileUtility.asPathArray(((List<?>) entry.getValue()).toArray(new IResource[0]));

			this.complexWriteToConsole(new Runnable() {
				public void run() {
					LockOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus lock");
					for (int i = 0; i < paths.length && !monitor.isCanceled(); i++) {
						LockOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, " \"" + paths[i] + "\"");
					}
					LockOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, (LockOperation.this.force ? " --force" : "") + " -m \""
							+ LockOperation.this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
				}
			});

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.lock(paths, LockOperation.this.message, LockOperation.this.force ? IModelBusConnector.Options.FORCE
							: IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(LockOperation.this, monitor, null));
				}
			}, monitor, wc2Resources.size());
		}
	}

}
