/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusMergeStatusCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusMergeStatus;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Merge status operation implementation
 * 
 * @author Alexander Gurov
 */
public class MergeStatusOperation extends AbstractWorkingCopyOperation implements IRemoteStatusOperation {
	protected AbstractMergeSet		info;
	protected ModelBusMergeStatus[]	retVal;

	public MergeStatusOperation(AbstractMergeSet info, IResource[] resources) {
		super("Operation.MergeStatus", resources == null ? info.to : resources);
		this.info = info;
	}

	public IResource[] getScope() {
		return this.info.to;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final ArrayList<ModelBusMergeStatus> st = new ArrayList<ModelBusMergeStatus>();

		HashSet<IResource> resources = new HashSet<IResource>(Arrays.asList(this.operableData()));

		final IModelBusMergeStatusCallback cb = new IModelBusMergeStatusCallback() {
			public void next(ModelBusMergeStatus status) {
				st.add(status);
			}
		};

		for (int i = 0; i < this.info.to.length && !monitor.isCanceled(); i++) {
			if (resources.contains(this.info.to[i])) {
				ProgressMonitorUtility.setTaskInfo(monitor, this, this.info.to[i].getFullPath().toString());
				final int idx = i;
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						if (MergeStatusOperation.this.info instanceof MergeSet1URL) {
							MergeStatusOperation.this.doMerge1URL(idx, cb, monitor);
						} else if (MergeStatusOperation.this.info instanceof MergeSet2URL) {
							MergeStatusOperation.this.doMerge2URL(idx, cb, monitor);
						} else {
							MergeStatusOperation.this.doMergeReintegrate(idx, cb, monitor);
						}
					}
				}, monitor, this.info.to.length);
			}
		}
		this.info.addStatuses(this.retVal = st.toArray(new ModelBusMergeStatus[st.size()]));
	}

	public ModelBusEntryStatus[] getStatuses() {
		return this.retVal;
	}

	public void setPegRevision(IResourceChange change) {

	}

	protected void doMerge1URL(int idx, IModelBusMergeStatusCallback cb, IProgressMonitor monitor) throws Exception {
		MergeSet1URL info = (MergeSet1URL) this.info;
		ModelBusEntryRevisionReference mergeRef = ModelBusUtility.getEntryRevisionReference(info.from[idx]);
		String wcPath = FileUtility.getWorkingCopyPath(info.to[idx]);
		long options = info.ignoreAncestry ? (IModelBusConnector.Options.IGNORE_ANCESTRY | IModelBusConnector.Options.FORCE)
				: IModelBusConnector.Options.FORCE;
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			proxy.mergeStatus(mergeRef, info.revisions, wcPath, info.depth, options, cb, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	protected void doMerge2URL(int idx, IModelBusMergeStatusCallback cb, IProgressMonitor monitor) throws Exception {
		MergeSet2URL info = (MergeSet2URL) this.info;
		ModelBusEntryRevisionReference startRef = ModelBusUtility.getEntryRevisionReference(info.fromStart[idx]);
		ModelBusEntryRevisionReference endRef = ModelBusUtility.getEntryRevisionReference(info.fromEnd[idx]);
		String wcPath = FileUtility.getWorkingCopyPath(info.to[idx]);
		long options = info.ignoreAncestry ? (IModelBusConnector.Options.IGNORE_ANCESTRY | IModelBusConnector.Options.FORCE)
				: IModelBusConnector.Options.FORCE;
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			proxy.mergeStatus(startRef, endRef, wcPath, info.depth, options, cb, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	protected void doMergeReintegrate(int idx, IModelBusMergeStatusCallback cb, IProgressMonitor monitor) throws Exception {
		MergeSetReintegrate info = (MergeSetReintegrate) this.info;
		ModelBusEntryRevisionReference mergeRef = ModelBusUtility.getEntryRevisionReference(info.from[idx]);
		String wcPath = FileUtility.getWorkingCopyPath(info.to[idx]);
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			proxy.mergeStatus(mergeRef, wcPath, IModelBusConnector.Options.NONE, cb, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

}
