/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local;

import java.util.HashSet;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.core.runtime.jobs.MultiRule;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.IConnectedProjectInformation;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.remote.AbstractRepositoryOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Switch working copy base url operation implementation
 * 
 * @author Alexander Gurov
 */
public class SwitchOperation extends AbstractRepositoryOperation {
	protected IResource[]	resources;
	protected int			depth;

	public SwitchOperation(IResource[] resources, IRepositoryResourceProvider destination, int depth) {
		super("Operation.Switch", destination);
		this.resources = resources;
		this.depth = depth;
	}

	public SwitchOperation(IResource[] resources, IRepositoryResource[] destination, int depth) {
		super("Operation.Switch", destination);
		this.resources = resources;
		this.depth = depth;
	}

	public int getOperationWeight() {
		return 19;
	}

	public ISchedulingRule getSchedulingRule() {
		HashSet<IResource> ruleSet = new HashSet<IResource>();
		for (int i = 0; i < this.resources.length; i++) {
			ruleSet.add(this.resources[i] instanceof IProject ? this.resources[i] : this.resources[i].getParent());
		}
		return new MultiRule(ruleSet.toArray(new IResource[ruleSet.size()]));
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource[] destinations = this.operableData();
		for (int i = 0; i < this.resources.length; i++) {
			final IResource resource = this.resources[i];
			final IRepositoryResource destination = destinations[i];
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					String wcPath = FileUtility.getWorkingCopyPath(resource);
					SwitchOperation.this.writeToConsole(
							IConsoleStream.LEVEL_CMD,
							"modelbus switch \"" + destination.getUrl() + "\" \"" + FileUtility.normalizePath(wcPath) + "\" -r "
									+ destination.getSelectedRevision() + ModelBusUtility.getDepthArg(SwitchOperation.this.depth)
									+ FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
					proxy.doSwitch(wcPath, ModelBusUtility.getEntryRevisionReference(destination), SwitchOperation.this.depth,
							IModelBusConnector.Options.NONE, new ModelBusProgressMonitor(SwitchOperation.this, monitor, null));

					if (resource instanceof IProject) {
						IConnectedProjectInformation provider = (IConnectedProjectInformation) RepositoryProvider.getProvider((IProject) resource);
						provider.switchResource(destination);
					}
				}
			}, monitor, this.resources.length);

		}
	}

}
