/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.management;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Cleanup working copy after failure operation implementation
 * 
 * @author Alexander Gurov
 */
public class CleanupOperation extends AbstractWorkingCopyOperation {
	public CleanupOperation(IResource[] resources) {
		super("Operation.CleanupResources", resources);
	}

	public CleanupOperation(IResourceProvider provider) {
		super("Operation.CleanupResources", provider);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		IRemoteStorage storage = ModelBusRemoteStorage.instance();

		for (int i = 0; i < resources.length; i++) {
			final String wcPath = FileUtility.getWorkingCopyPath(resources[i]);
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			ProgressMonitorUtility.setTaskInfo(monitor, this, resources[i].getName());

			this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus cleanup \"" + FileUtility.normalizePath(wcPath) + "\"\n");

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.cleanup(wcPath, new ModelBusProgressMonitor(CleanupOperation.this, monitor, null));
				}
			}, monitor, resources.length);
			ProgressMonitorUtility.progress(monitor, i, resources.length);
		}
	}

}
