/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.management;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.IConnectedProjectInformation;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;

/**
 * Finds projects related to the specified repository location
 * 
 * @author Alexander Gurov
 */
public class FindRelatedProjectsOperation extends AbstractActionOperation implements IResourceProvider {
	protected List<IProject>	resources;
	protected Set<IProject>		exceptProjects;

	public FindRelatedProjectsOperation() {
		this(null);
	}

	public FindRelatedProjectsOperation(IProject[] exceptProjects) {
		super("Operation.FindRelatedProjects");
		if (exceptProjects != null) {
			this.exceptProjects = new HashSet<IProject>(Arrays.asList(exceptProjects));
		}
	}

	public IResource[] getResources() {
		return this.resources == null ? new IProject[0] : this.resources.toArray(new IProject[this.resources.size()]);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.resources = new ArrayList<IProject>();
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length && !monitor.isCanceled(); i++) {
			final IProject current = projects[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					IConnectedProjectInformation provider = (IConnectedProjectInformation) RepositoryProvider.getProvider(current,
							ModelBusTeamPlugin.NATURE_ID);
					if (provider != null
							&& (FindRelatedProjectsOperation.this.exceptProjects == null || !FindRelatedProjectsOperation.this.exceptProjects
									.contains(current))) {
						FindRelatedProjectsOperation.this.resources.add(current);
					}
				}
			}, monitor, projects.length);
		}
	}

}
