/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.management;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.team.eclipse.core.IConnectedProjectInformation;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusChangeStatus;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * This operation relocate any projects associated with the selected repository
 * 
 * @author Alexander Gurov
 */
public class RelocateWorkingCopyOperation extends AbstractWorkingCopyOperation implements IResourceProvider {
	protected List<IProject>	resources;

	public RelocateWorkingCopyOperation(IResource[] resources) {
		super("Operation.RelocateResources", resources);
	}

	public RelocateWorkingCopyOperation(IResourceProvider provider) {
		super("Operation.RelocateResources", provider);
	}

	public IResource[] getResources() {
		return this.resources == null ? new IResource[0] : this.resources.toArray(new IResource[this.resources.size()]);
	}

	public ISchedulingRule getSchedulingRule() {
		return ResourcesPlugin.getWorkspace().getRoot();
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.resources = new ArrayList<IProject>();
		IResource[] projects = this.operableData();
		if (projects.length == 0) {
			return;
		}

		try {
			final IRepositoryResource[] children = ModelBusConnector.getRepositoryRoot().getChildren();
			final String rootUrl = ModelBusConnector.getRepositoryRootUrl();

			for (int i = 0; i < projects.length && !monitor.isCanceled(); i++) {
				final IProject current = (IProject) projects[i];
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						IConnectedProjectInformation provider = (IConnectedProjectInformation) RepositoryProvider.getProvider(current,
								ModelBusTeamPlugin.NATURE_ID);
						IPath fsLocation = current.getLocation();
						if (fsLocation != null) {
							String path = fsLocation.toString();
							ModelBusChangeStatus st = ModelBusUtility.getModelBusInfoForNotConnected(current);
							if (st != null) {
								String url = ModelBusUtility.decodeURL(st.url);
								String oldRoot = ModelBusUtility.getOldRoot(url, children);
								if (oldRoot != null) {
									RelocateWorkingCopyOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD,
											"modelbus switch --relocate \"" + oldRoot + "\" \"" + rootUrl + "\" \"" + FileUtility.normalizePath(path)
													+ "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");

									ModelBusConnector.getModelBusConnector().relocate(oldRoot, rootUrl, path, Depth.INFINITY,
											new ModelBusProgressMonitor(RelocateWorkingCopyOperation.this, monitor, null));
									provider.relocateResource();
									RelocateWorkingCopyOperation.this.resources.add(current);
								}
							}
						}
					}
				}, monitor, projects.length);
			}
		} finally {
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { ModelBusConnector.getRepositoryRootUrl() });
	}

}
