/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.property;

import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Commit operation implementation
 * 
 * @author Sergiy Logvin
 */
public class GetPropertiesOperation extends AbstractActionOperation implements IResourcePropertyProvider {
	protected ModelBusProperty[]	properties;
	protected IResource				resource;
	protected ModelBusRevision		revision;

	public GetPropertiesOperation(IResource resource) {
		this(resource, ModelBusRevision.WORKING);
	}

	public GetPropertiesOperation(IResource resource, ModelBusRevision revision) {
		super("Operation.GetProperties");
		this.resource = resource;
		this.revision = revision;
	}

	public ModelBusProperty[] getProperties() {
		return this.properties;
	}

	public boolean isEditAllowed() {
		return true;
	}

	public void refresh() {
		this.run(new NullProgressMonitor());
	}

	public IResource getLocal() {
		return this.resource;
	}

	public IRepositoryResource getRemote() {
		return ModelBusRemoteStorage.instance().asRepositoryResource(this.resource);
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		this.properties = null;
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus proplist \"" + local.getWorkingCopyPath() + "\"\n");
			this.properties = ModelBusUtility.properties(proxy, new ModelBusEntryRevisionReference(FileUtility.getWorkingCopyPath(this.resource),
					null, this.revision), new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.resource.getName() });
	}

}
