/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.property;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Commit operation implementation
 * 
 * @author Sergiy Logvin
 */
public class RemovePropertiesOperation extends AbstractWorkingCopyOperation {
	protected ModelBusProperty[]	data;
	protected boolean				isRecursive;

	public RemovePropertiesOperation(IResource[] resources, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.RemoveProperties", resources);
		this.data = data;
		this.isRecursive = isRecursive;
	}

	public RemovePropertiesOperation(IResourceProvider resourceProvider, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.RemoveProperties", resourceProvider);
		this.data = data;
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		if (this.isRecursive) {
			resources = FileUtility.shrinkChildNodes(resources);
		}

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource resource = resources[i];

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					RemovePropertiesOperation.this.processResource(proxy, resource, monitor);
				}
			}, monitor, resources.length);

		}
	}

	protected void processResource(final IModelBusConnector proxy, IResource resource, IProgressMonitor monitor) {
		ProgressMonitorUtility.setTaskInfo(monitor, this, resource.getFullPath().toString());
		final String wcPath = FileUtility.getWorkingCopyPath(resource);

		for (int i = 0; i < this.data.length && !monitor.isCanceled(); i++) {
			final ModelBusProperty current = this.data[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.removeProperty(wcPath, current.name, RemovePropertiesOperation.this.isRecursive ? Depth.INFINITY : Depth.EMPTY, null,
							new ModelBusProgressMonitor(RemovePropertiesOperation.this, monitor, null));
				}
			}, monitor, this.data.length);
		}
	}

}
