/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.property;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Operation to set the properties on multiple resources
 * 
 * @author Sergiy Logvin
 */
public class SetMultiPropertiesOperation extends AbstractWorkingCopyOperation {
	protected IPropertyProvider	propertyProvider;
	protected int				depth;
	protected IStateFilter		filter;

	public SetMultiPropertiesOperation(IResourceProvider resourceProvider, IPropertyProvider propertyProvider, IStateFilter filter, int depth) {
		super("Operation.SetMultiProperties", resourceProvider);
		this.propertyProvider = propertyProvider;
		this.depth = depth;
		this.filter = filter != null ? filter : IStateFilter.SF_VERSIONED;
	}

	public SetMultiPropertiesOperation(IResource[] resources, IPropertyProvider propertyProvider, IStateFilter filter, int depth) {
		super("Operation.SetMultiProperties", resources);
		this.propertyProvider = propertyProvider;
		this.depth = depth;
		this.filter = filter != null ? filter : IStateFilter.SF_VERSIONED;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource current = resources[i];

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			try {
				this.protectStep(new IUnprotectedOperation() {
					public void run(final IProgressMonitor monitor) throws Exception {
						FileUtility.visitNodes(current, new IResourceVisitor() {
							public boolean visit(IResource resource) throws CoreException {
								if (monitor.isCanceled()) {
									return false;
								}
								ILocalResource local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resource);
								if (SetMultiPropertiesOperation.this.filter.accept(local)) {
									ModelBusProperty[] properties = SetMultiPropertiesOperation.this.propertyProvider.getProperties(resource);
									if (properties != null) {
										SetMultiPropertiesOperation.this.processResource(proxy, resource, properties, monitor);
									}
								}
								return SetMultiPropertiesOperation.this.filter.allowsRecursion(local);
							}
						}, SetMultiPropertiesOperation.this.depth);
					}
				}, monitor, resources.length);
			} finally {
			}
		}

	}

	protected void processResource(final IModelBusConnector proxy, IResource current, ModelBusProperty[] properties, IProgressMonitor monitor) {
		ProgressMonitorUtility.setTaskInfo(monitor, this, current.getFullPath().toString());
		final String wcPath = FileUtility.getWorkingCopyPath(current);
		for (int i = 0; i < properties.length && !monitor.isCanceled(); i++) {
			final ModelBusProperty property = properties[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.setProperty(wcPath, property.name, property.value, Depth.EMPTY, IModelBusConnector.Options.NONE, null,
							new ModelBusProgressMonitor(SetMultiPropertiesOperation.this, monitor, null));
				}
			}, monitor, properties.length);
		}
	}

}
