/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.local.property;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.IResourceProvider;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;

/**
 * Set resource property operation
 * 
 * @author Sergiy Logvin
 */
public class SetPropertiesOperation extends AbstractWorkingCopyOperation {
	protected ModelBusProperty[]		propertyData;
	protected boolean					isRecursive;
	protected IResourcePropertyProvider	propertyProvider;

	public SetPropertiesOperation(IResource[] resources, String name, byte[] data, boolean isRecursive) {
		this(resources, new ModelBusProperty[] { new ModelBusProperty(name, new String(data)) }, isRecursive);
	}

	public SetPropertiesOperation(IResource[] resources, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.SetProperties", resources);
		this.propertyData = data;
		this.isRecursive = isRecursive;
	}

	public SetPropertiesOperation(IResourceProvider resourceProvider, ModelBusProperty[] data, boolean isRecursive) {
		super("Operation.SetProperties", resourceProvider);
		this.propertyData = data;
		this.isRecursive = isRecursive;
	}

	public SetPropertiesOperation(IResource[] resources, IResourcePropertyProvider propertyProvider, boolean isRecursive) {
		super("Operation.SetProperties", resources);
		this.propertyProvider = propertyProvider;
		this.isRecursive = isRecursive;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IResource[] resources = this.operableData();

		if (this.isRecursive) {
			resources = FileUtility.shrinkChildNodes(resources);
		}

		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IResource resource = resources[i];

			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();

			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					SetPropertiesOperation.this.processResource(proxy, resource, monitor);
				}
			}, monitor, resources.length);

		}
	}

	protected void processResource(final IModelBusConnector proxy, IResource resource, IProgressMonitor monitor) {
		ProgressMonitorUtility.setTaskInfo(monitor, this, resource.getFullPath().toString());
		final String wcPath = FileUtility.getWorkingCopyPath(resource);

		ModelBusProperty[] properties = this.getOperableProperties();

		for (int i = 0; i < properties.length && !monitor.isCanceled(); i++) {
			final ModelBusProperty current = properties[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.setProperty(wcPath, current.name, current.value, Depth.infinityOrEmpty(SetPropertiesOperation.this.isRecursive),
							IModelBusConnector.Options.NONE, null, new ModelBusProgressMonitor(SetPropertiesOperation.this, monitor, null));
				}
			}, monitor, properties.length);
		}
	}

	protected ModelBusProperty[] getOperableProperties() {
		return this.propertyData == null ? this.propertyProvider.getProperties() : this.propertyData;
	}

}
