/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.util.ArrayList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Base implementation for copy and move repository resources operations
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractCopyMoveResourcesOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected IRepositoryResource		destinationResource;
	protected String					message;
	protected String					resName;
	protected ArrayList<RevisionPair>	revisionsPairs;

	public AbstractCopyMoveResourcesOperation(String operationName, IRepositoryResource destinationResource, IRepositoryResource[] selectedResources,
			String message, String resName) {
		super(operationName, selectedResources);
		this.destinationResource = destinationResource;
		this.message = message;
		this.resName = resName;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionsPairs == null ? null : this.revisionsPairs.toArray(new RevisionPair[this.revisionsPairs.size()]);
	}

	protected void runImpl(final IProgressMonitor monitor) throws Exception {
		this.revisionsPairs = new ArrayList<RevisionPair>();
		IRepositoryResource[] selectedResources = this.operableData();
		final ModelBusEntryRevisionReference[] refs = new ModelBusEntryRevisionReference[selectedResources.length];
		for (int i = 0; i < selectedResources.length; i++) {
			refs[i] = ModelBusUtility.getEntryRevisionReference(selectedResources[i]);
		}
		final String dstUrl = this.destinationResource.getUrl()
				+ (this.resName != null && this.resName.length() > 0 ? "/" + this.resName : (selectedResources.length > 1 ? "" : "/"
						+ selectedResources[0].getName()));
		IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
			private int	i	= 0;

			public void notify(ModelBusNotification info) {
				if (this.i == refs.length) {
					return;
				}
				String[] paths = AbstractCopyMoveResourcesOperation.this.getRevisionPaths(refs[this.i].path, dstUrl);
				AbstractCopyMoveResourcesOperation.this.revisionsPairs.add(new RevisionPair(info.revision, paths));
				String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
						new String[] { String.valueOf(info.revision) });
				AbstractCopyMoveResourcesOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
				this.i++;
			}
		};
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		// NOTE NPE in ModelBus Kit if parents exists and MAKE_PARENTS is specified
		// NOTE JavaHL is crashed when empty folder is copied independently from MAKE_PARENTS option
		ModelBusUtility.addModelBusNotifyListener(proxy, notify);
		try {
			this.runCopyMove(proxy, refs, ModelBusUtility.encodeURL(dstUrl), monitor);
		} finally {
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
		}
	}

	protected abstract String[] getRevisionPaths(String srcUrl, String dstUrl);

	protected abstract void runCopyMove(IModelBusConnector proxy, ModelBusEntryRevisionReference[] source, String destinationUrl,
			IProgressMonitor monitor) throws Exception;
}
