/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Create new remote file by importing it from local file system
 * 
 * @author Sergiy Logvin
 */
public class CreateFileOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String			path;
	protected String			message;
	protected String[]			fileNames;
	protected RevisionPair[]	revisionPair;

	public CreateFileOperation(IRepositoryResource resource, String path, String message, String[] fileNames) {
		super("Operation.CreateFile", new IRepositoryResource[] { resource });
		this.path = path;
		this.message = message;
		this.fileNames = fileNames;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionPair;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final IRepositoryResource resource = this.operableData()[0];
		this.revisionPair = new RevisionPair[1];
		final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
			public void notify(ModelBusNotification info) {
				if (info.revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
					String[] path = new String[] { resource.getUrl() };
					CreateFileOperation.this.revisionPair[0] = new RevisionPair(info.revision, path);
					String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
							new String[] { String.valueOf(info.revision) });
					CreateFileOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
				}
			}
		};
		try {
			ModelBusUtility.addModelBusNotifyListener(proxy, notify);
			for (int i = 0; i < this.fileNames.length; i++) {
				final String[] currentFile = new String[] { this.fileNames[i] };
				this.protectStep(new IUnprotectedOperation() {
					public void run(IProgressMonitor monitor) throws Exception {
						String path = FileUtility.normalizePath(CreateFileOperation.this.path + "/" + currentFile[0]);
						String url = resource.getUrl() + "/" + currentFile[0];
						CreateFileOperation.this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus import \"" + path + "\" \"" + url + "\" -m \""
								+ CreateFileOperation.this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + " -N\n");
						proxy.doImport(path, ModelBusUtility.encodeURL(url), CreateFileOperation.this.message, Depth.FILES,
								IModelBusConnector.Options.INCLUDE_IGNORED | IModelBusConnector.Options.IGNORE_UNKNOWN_NODE_TYPES, null,
								new ModelBusProgressMonitor(CreateFileOperation.this, monitor, null));
					}
				}, monitor, this.fileNames.length);
			}
		} finally {
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
		}
	}

}