/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Get repository resource LogMessage's opearation implementation
 * 
 * @author Alexander Gurov
 */
public class GetLogMessagesOperation extends AbstractRepositoryOperation {
	protected ModelBusLogEntry[]	msg;
	protected boolean				stopOnCopy;
	protected boolean				discoverPaths;
	protected ModelBusRevision		startRevision;
	protected ModelBusRevision		endRevision;
	protected long					limit;
	protected boolean				includeMerged;

	public GetLogMessagesOperation(IRepositoryResource resource) {
		this(resource, false);
	}

	public GetLogMessagesOperation(IRepositoryResource resource, boolean stopOnCopy) {
		super("Operation.GetLogMessages", new IRepositoryResource[] { resource });
		this.stopOnCopy = stopOnCopy;
		this.includeMerged = false;
		this.discoverPaths = true;
		this.limit = 0;
		this.endRevision = ModelBusRevision.fromString("0");
	}

	public boolean getIncludeMerged() {
		return this.includeMerged;
	}

	public void setIncludeMerged(boolean includeMerged) {
		this.includeMerged = includeMerged;
	}

	public boolean getStopOnCopy() {
		return this.stopOnCopy;
	}

	public void setStopOnCopy(boolean stopOnCopy) {
		this.stopOnCopy = stopOnCopy;
	}

	public boolean getDiscoverPaths() {
		return this.discoverPaths;
	}

	public void setDiscoverPaths(boolean discoverPaths) {
		this.discoverPaths = discoverPaths;
	}

	public long getLimit() {
		return this.limit;
	}

	public void setLimit(long limit) {
		this.limit = limit;
	}

	public void setStartRevision(ModelBusRevision revision) {
		this.startRevision = revision;
	}

	public void setEndRevision(ModelBusRevision revision) {
		this.endRevision = revision;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource resource = this.operableData()[0];
		if (this.startRevision == null) {
			this.startRevision = resource.getSelectedRevision();
		}
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus log " + ModelBusUtility.encodeURL(this.resource.getUrl()) + (this.limit != 0 ?
			// (" --limit " + this.limit) : "") + (this.stopOnCopy ? " --stop-on-copy" : "") + " -r " + this.selectedRevision + ":0 --username \"" +
			// location.getUsername() + "\"\n");
			long options = this.discoverPaths ? IModelBusConnector.Options.DISCOVER_PATHS : IModelBusConnector.Options.NONE;
			options |= this.stopOnCopy ? IModelBusConnector.Options.STOP_ON_COPY : IModelBusConnector.Options.NONE;
			options |= this.includeMerged ? IModelBusConnector.Options.INCLUDE_MERGED_REVISIONS : IModelBusConnector.Options.NONE;
			this.msg = ModelBusUtility.logEntries(proxy, ModelBusUtility.getEntryReference(resource), this.startRevision, this.endRevision, options,
					IModelBusConnector.DEFAULT_LOG_ENTRY_PROPS, this.limit, new ModelBusProgressMonitor(this, monitor, null));
		} finally {
		}
	}

	public ModelBusLogEntry[] getMessages() {
		return this.msg;
	}

	public IRepositoryResource getResource() {
		return this.operableData()[0];
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getUrl() });
	}

}
