/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.operation.IResourcePropertyProvider;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * This operation allows us to get properties of the concrete revision of the versioned resource
 * 
 * @author Alexander Gurov
 */
public class GetRemotePropertiesOperation extends AbstractRepositoryOperation implements IResourcePropertyProvider {
	protected ModelBusProperty[]	properties;

	public GetRemotePropertiesOperation(IRepositoryResource resource) {
		super("Operation.GetRevisionProperties", new IRepositoryResource[] { resource });
	}

	public GetRemotePropertiesOperation(IRepositoryResourceProvider provider) {
		super("Operation.GetRevisionProperties", provider);
	}

	public ModelBusProperty[] getProperties() {
		return this.properties;
	}

	public boolean isEditAllowed() {
		return false;
	}

	public void refresh() {

	}

	public IResource getLocal() {
		return null;
	}

	public IRepositoryResource getRemote() {
		return this.operableData()[0];
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource resource = this.operableData()[0];
		this.properties = null;
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			// this.writeToConsole(IConsoleStream.LEVEL_CMD, "modelbus proplist " + url + "@" + resource.getPegRevision() + " --revprop -r " +
			// resource.getSelectedRevision() + " --username \"" + location.getUsername() + "\"\n");
			this.properties = ModelBusUtility.properties(proxy, ModelBusUtility.getEntryRevisionReference(resource), new ModelBusProgressMonitor(
					this, monitor, null));
		} finally {
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getUrl() });
	}

}
