/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusNotificationCallback;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusNotification;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Operation organize import repository resources to file sistem
 * 
 * @author Sergiy Logvin
 */
public class ImportOperation extends AbstractRepositoryOperation implements IRevisionProvider {
	protected String			path;
	protected String			message;
	protected int				depth;
	protected RevisionPair[]	revisionPair;

	public ImportOperation(IRepositoryResource resource, String path, String message, int depth) {
		super("Operation.Import", new IRepositoryResource[] { resource });
		this.path = path;
		this.message = message;
		this.depth = depth;
	}

	public RevisionPair[] getRevisions() {
		return this.revisionPair;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final IRepositoryResource resource = this.operableData()[0];
		this.revisionPair = new RevisionPair[1];
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		IModelBusNotificationCallback notify = new IModelBusNotificationCallback() {
			public void notify(ModelBusNotification info) {
				if (info.revision != ModelBusRevision.INVALID_REVISION_DESCRIPTOR) {
					String[] path = new String[] { resource.getUrl() };
					ImportOperation.this.revisionPair[0] = new RevisionPair(info.revision, path);
					String message = ModelBusTeamPlugin.instance().getResource("Console.CommittedRevision",
							new String[] { String.valueOf(info.revision) });
					ImportOperation.this.writeToConsole(IConsoleStream.LEVEL_OK, message);
				}
			}
		};
		try {
			ModelBusUtility.addModelBusNotifyListener(proxy, notify);
			this.writeToConsole(IConsoleStream.LEVEL_CMD,
					"modelbus import \"" + FileUtility.normalizePath(this.path) + "\" \"" + ModelBusUtility.getDepthArg(this.depth) + " -m \""
							+ this.message + "\"" + FileUtility.getUsernameParam(ModelBusConnector.getUserName()) + "\n");
			proxy.doImport(this.path, ModelBusUtility.encodeURL(resource.getUrl()), this.message, this.depth,
					IModelBusConnector.Options.INCLUDE_IGNORED | IModelBusConnector.Options.IGNORE_UNKNOWN_NODE_TYPES, null,
					new ModelBusProgressMonitor(this, monitor, null));
		} finally {
			ModelBusUtility.removeModelBusNotifyListener(proxy, notify);
		}
	}

	protected String getShortErrorMessage(Throwable t) {
		return MessageFormat.format(super.getShortErrorMessage(t), new Object[] { this.operableData()[0].getUrl() });
	}

}
