/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider;
import org.modelbus.team.eclipse.core.operation.IRevisionProvider.RevisionPair;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;

/**
 * Set revision author name operation implementation
 * 
 * @author Alexei Goncharov
 */
public class SetRevisionAuthorNameOperation extends AbstractActionOperation {

	protected IRevisionProvider	provider;
	protected long				options;

	public SetRevisionAuthorNameOperation(final RevisionPair[] revisions, long options) {
		this(new IRevisionProvider() {
			public RevisionPair[] getRevisions() {
				return revisions;
			}
		}, options);

	}

	public SetRevisionAuthorNameOperation(IRevisionProvider provider, long options) {
		super("Operation.SetRevisionAuthorName");
		this.provider = provider;
		this.options = options;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final RevisionPair[] revisions = this.provider.getRevisions();
		if (revisions == null) {
			return;
		}
		for (int i = 0; i < revisions.length; i++) {
			if (revisions[i] == null) {
				continue;
			}
			final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
			final ModelBusRevision rev = revisions[i].revision;
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					proxy.setRevisionProperty(new ModelBusEntryReference(ModelBusConnector.getRepositoryRootUrl(), rev), "modelbus:author",
							ModelBusConnector.getUserName(), SetRevisionAuthorNameOperation.this.options, new ModelBusProgressMonitor(
									SetRevisionAuthorNameOperation.this, monitor, null));
				}
			}, monitor, 1);
		}
	}

}
