/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Options;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryReference;
import org.modelbus.team.eclipse.core.connector.ModelBusProperty;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.operation.local.property.IRevisionPropertiesProvider;

/**
 * Set revision property operation implementation
 * 
 * @author Alexei Goncharov
 */
public class SetRevisionPropertyOperation extends AbstractActionOperation {

	protected ModelBusRevision				revision;
	protected IRevisionPropertiesProvider	provider;

	/**
	 * Creates an instance of SetRevisionPropertyOperation depending on the repository location.
	 * 
	 * @param location
	 *            - repository location to set property to
	 * @param revision
	 *            - revision to apply property to
	 * @param revProp
	 *            - the revision property to set
	 * 
	 * @author Alexei Goncharov
	 */
	public SetRevisionPropertyOperation(ModelBusRevision revision, final ModelBusProperty revProp) {
		this(revision, new IRevisionPropertiesProvider() {
			public ModelBusProperty[] getRevisionProperties() {
				return new ModelBusProperty[] { revProp };
			}
		});
	}

	/**
	 * Creates an instance of SetRevisionPropertyOperation depending on the repository location.
	 * 
	 * @param location
	 *            - repository location to set property to
	 * @param revision
	 *            - revision to apply property to
	 * @param revPropProvider
	 *            - the revision property to set provider
	 * 
	 * @author Alexei Goncharov
	 */
	public SetRevisionPropertyOperation(ModelBusRevision revision, IRevisionPropertiesProvider revPropProvider) {
		super("Operation.SetRevisionProperty");
		this.revision = revision;
		this.provider = revPropProvider;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		ModelBusProperty toSet = this.provider.getRevisionProperties()[0];
		IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
		try {
			// CHECKMWA root = ""
			proxy.setRevisionProperty(new ModelBusEntryReference("", this.revision), toSet.name, toSet.value, Options.FORCE,
					new ModelBusProgressMonitor(SetRevisionPropertyOperation.this, monitor, null));
		} finally {
		}
	}

}
