/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.core.operation.remote.management;

import org.eclipse.core.runtime.IProgressMonitor;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.remote.AbstractRepositoryOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResourceProvider;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;

/**
 * Add revision link to repository location
 * 
 * @author Alexander Gurov
 */
public class AddRevisionLinkOperation extends AbstractRepositoryOperation {
	protected ModelBusRevision	revision;

	public AddRevisionLinkOperation(IRepositoryResource resource, String revision) {
		this(resource, ModelBusRevision.fromString(revision));
	}

	public AddRevisionLinkOperation(IRepositoryResource resource, ModelBusRevision revision) {
		super("Operation.AddRevisionLink", new IRepositoryResource[] { resource });
		this.revision = revision;
	}

	public AddRevisionLinkOperation(IRepositoryResourceProvider provider, String revision) {
		this(provider, ModelBusRevision.fromString(revision));
	}

	public AddRevisionLinkOperation(IRepositoryResourceProvider provider, ModelBusRevision revision) {
		super("Operation.AddRevisionLink", provider);
		this.revision = revision;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		IRepositoryResource[] resources = this.operableData();
		for (int i = 0; i < resources.length && !monitor.isCanceled(); i++) {
			final IRepositoryResource source = resources[i];
			this.protectStep(new IUnprotectedOperation() {
				public void run(IProgressMonitor monitor) throws Exception {
					IRepositoryResource target = ModelBusUtility.copyOf(source);

					ModelBusRevision selectedRevision = AddRevisionLinkOperation.this.revision == null ? source.getSelectedRevision()
							: AddRevisionLinkOperation.this.revision;
					if (selectedRevision.equals(ModelBusRevision.HEAD)) {
						selectedRevision = source.getRevision();
					}

					ModelBusRevision pegRevision = source.getPegRevision();
					// if (pegRevision.equals(ModelBusRevision.HEAD)) {
					// CHECKMWA pegRevision = ModelBusRevision.fromNumber(location.getRepositoryRoot().getRevision());
					// }

					target.setSelectedRevision(selectedRevision);
					target.setPegRevision(pegRevision);

					// location.addRevisionLink(target);
				}
			}, monitor, resources.length);
		}
	}

}
