/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.model.mapping;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.mapping.ResourceMapping;
import org.modelbus.team.eclipse.model.ModelContainer;
import org.modelbus.team.eclipse.model.ModelObject;
import org.modelbus.team.eclipse.model.ModelObjectDefinitionFile;
import org.modelbus.team.eclipse.model.ModelObjectElementFile;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public abstract class ModelResourceMapping extends ResourceMapping {

	private final ModelObject	object;

	public static ResourceMapping create(ModelObject object) {
		if (object instanceof ModelContainer) {
			return new ModelContainerResourceMapping((ModelContainer) object);
		}
		if (object instanceof ModelObjectDefinitionFile) {
			return new ModResourceMapping((ModelObjectDefinitionFile) object);
		}
		if (object instanceof ModelObjectElementFile) {
			return new MoeResourceMapping((ModelObjectElementFile) object);
		}
		return null;
	}

	protected ModelResourceMapping(ModelObject object) {
		this.object = object;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.mapping.ResourceMapping#getModelObject()
	 */
	public Object getModelObject() {
		return object;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.mapping.ResourceMapping#getModelProviderId()
	 */
	public String getModelProviderId() {
		return ExampleModelProvider.ID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.resources.mapping.ResourceMapping#getProjects()
	 */
	public IProject[] getProjects() {
		return new IProject[] { (IProject) object.getProject().getResource() };
	}

}
