/**
 *  ModelBus is the means to interact with models in a distributed
 * consistent way.
 * 
 *  Copyright (C) 2008  Tom Ritter, Christian Hein, Michael Wagner
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.modelbus.team.eclipse.model.ui;

import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.team.core.mapping.IResourceMappingMerger;
import org.eclipse.team.ui.history.IHistoryPageSource;
import org.eclipse.team.ui.mapping.ISynchronizationCompareAdapter;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.modelbus.team.eclipse.model.ModelObject;
import org.modelbus.team.eclipse.model.mapping.ExampleModelProvider;
import org.modelbus.team.eclipse.model.mapping.ModelMerger;
import org.modelbus.team.eclipse.model.mapping.ModelResourceMapping;
import org.modelbus.team.eclipse.model.ui.mapping.CompareAdapter;
import org.modelbus.team.eclipse.repository.ui.ModelBusRepositoryHistoryPageSource;

/**
 * TODO comment
 * 
 * @author Michael Wagner
 * 
 */

public class AdapterFactory implements IAdapterFactory {

	private IWorkbenchAdapter	modelAdapter		= new ModelWorkbenchAdapter();
	private ModelMerger			modelMerger;
	private CompareAdapter		compareAdapter;
	private static Object		historyPageSource	= new ModelBusRepositoryHistoryPageSource();

	public Object getAdapter(Object adaptableObject, Class adapterType) {
		if (adapterType == IWorkbenchAdapter.class && adaptableObject instanceof ModelObject)
			return modelAdapter;
		if (adapterType == ResourceMapping.class && adaptableObject instanceof ModelObject)
			return ModelResourceMapping.create((ModelObject) adaptableObject);
		if (adapterType == IResourceMappingMerger.class && adaptableObject instanceof ExampleModelProvider) {
			if (modelMerger == null) {
				modelMerger = new ModelMerger((ExampleModelProvider) adaptableObject);
			}
			return modelMerger;
		}
		if (adapterType == ISynchronizationCompareAdapter.class && adaptableObject instanceof ExampleModelProvider) {
			if (compareAdapter == null) {
				compareAdapter = new CompareAdapter((ExampleModelProvider) adaptableObject);
			}
			return compareAdapter;
		}

		if (adapterType == IHistoryPageSource.class) {
			return historyPageSource;
		}

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdapterFactory#getAdapterList()
	 */
	public Class[] getAdapterList() {
		return new Class[] { IWorkbenchAdapter.class, ResourceMapping.class, IResourceMappingMerger.class, ISynchronizationCompareAdapter.class };
	}

}
