/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.subscriber;

import java.io.UnsupportedEncodingException;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.team.core.diff.IDiff;
import org.eclipse.team.core.diff.IThreeWayDiff;
import org.eclipse.team.core.mapping.ISynchronizationScopeManager;
import org.eclipse.team.core.mapping.provider.ResourceDiffTree;
import org.eclipse.team.core.subscribers.SubscriberMergeContext;
import org.eclipse.team.internal.core.Policy;

/**
 * A merge context for merging file system changes.
 */
public class ModelBusRepositoryMergeContext extends SubscriberMergeContext {

	/**
	 * Create the file system merge context for the given scope manager.
	 * 
	 * @param manager
	 *            the scope manager
	 */
	public ModelBusRepositoryMergeContext(ISynchronizationScopeManager manager) {
		super(ModelBusRepositorySubscriber.getInstance(), manager);
		initialize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.mapping.provider.MergeContext#makeInSync(org.eclipse.team.core.diff.IDiff,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void makeInSync(IDiff diff, IProgressMonitor monitor) throws CoreException {
		IResource resource = ResourceDiffTree.getResourceFor(diff);
		try {
			ModelBusRepositorySubscriber.getInstance().makeInSync(resource);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.mapping.IMergeContext#markAsMerged(org.eclipse.team.core.diff.IDiff, boolean,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void markAsMerged(IDiff diff, boolean inSyncHint, IProgressMonitor monitor) throws CoreException {
		// TODO if inSyncHint is true, we should test to see if the contents match
		IResource resource = ResourceDiffTree.getResourceFor(diff);
		try {
			ModelBusRepositorySubscriber.getInstance().markAsMerged(resource, monitor);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.mapping.IMergeContext#reject(org.eclipse.team.core.diff.IDiff, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void reject(IDiff diff, IProgressMonitor monitor) throws CoreException {
		markAsMerged(diff, false, monitor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.mapping.provider.MergeContext#getMergeRule(org.eclipse.team.core.diff.IDiff)
	 */
	public ISchedulingRule getMergeRule(IDiff node) {
		return ResourceDiffTree.getResourceFor(node).getProject();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.core.mapping.IMergeContext#merge(org.eclipse.team.core.diff.IDiffNode, boolean,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 * 
	 * MES: Very ugly hack to be able to correctly handle incoming deletions when updating via the Synchronize View. If not present, update won't do
	 * anything, (unless triggered via the Package Explorer, which does not call this code).
	 * 
	 * TODOMES: The source of the problem may lie in an incorrect conversion from the SyncInfo to the ThreeWayDiff. Take a look at
	 * SyncInfoToDiffConverter.
	 */
	public IStatus merge(IDiff diff, boolean ignoreLocalChanges, IProgressMonitor monitor) throws CoreException {
		Policy.checkCanceled(monitor);
		IResource resource = getDiffTree().getResource(diff);
		if (resource.getType() != IResource.FILE) {
			if (diff instanceof IThreeWayDiff) {
				IThreeWayDiff twd = (IThreeWayDiff) diff;
				// System.out.println(diff.toDiffString());
				if (resource.getType() == IResource.FOLDER && twd.getDirection() == IThreeWayDiff.INCOMING && twd.getKind() == IDiff.REMOVE) {
					((IFolder) resource).delete(false, monitor);
					makeInSync(diff, monitor);
					return Status.OK_STATUS;
				}
			}
		}
		return super.merge(diff, ignoreLocalChanges, monitor);
	}
}
