package org.modelbus.team.eclipse.repository.ui;

import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.ui.progress.UIJob;

import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.core.lib.notification.INotificationListener;
import org.modelbus.core.lib.notification.NotificationListenerManager;
import org.modelbus.dosgi.repository.descriptor.LockInfo;
import org.modelbus.dosgi.repository.descriptor.LockedException;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

public class LockHelper implements INotificationListener, Observer {

	private static LockInfo[]			lockInfos;
	private static IRepositoryHelper	repositoryHelper		= null;

	private static boolean				repositoryIsAvailable	= false;

	private LockHelper() throws RemoteException {

	}

	private static LockHelper					lockHelper	= null;
	private static NotificationListenerManager	nManager	= null;

	public static synchronized LockHelper getLockHelper() {

		if (nManager == null) {
			try {
				nManager = NotificationListenerManager.getNotificationListenerManager();
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
		if (nManager != null && lockHelper == null) {

			try {
				lockHelper = new LockHelper();
				ModelBusRepositoryPlugin.getPlugin().getRepositoryStatus().addObserver(lockHelper);
				if (nManager != null) {

					if (lockHelper != null) {
						nManager.addNotificationListener(lockHelper);
					}
				}
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		return lockHelper;
	}

	/*
	 * public void setModelEditPart(AbstractEditPart editpart) { modelEditPart = editpart; System.out.println();
	 * 
	 * }
	 */

	private void setRepositoryStatus(boolean isAvailable) {
		LockHelper.repositoryIsAvailable = isAvailable;
	}

	public static void resetLockHelper() {
		nManager.removeNotificationListener(lockHelper);
		lockHelper = null;
		lockInfos = null;
		UIJob go = new UIJob("DecoratorNotificationListener") {

			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				// TODO Auto-generated method stub
				try {
					LockDecorator.refreshDecoration();
				} catch (Exception e) {
					e.printStackTrace();
				}

				return Status.OK_STATUS;
			}
		};
		go.setSystem(true);
		go.setUser(false);
		go.schedule();

	}

	public LockInfo[] getLocks() throws RepositoryAuthentificationException, LockedException, NonExistingResourceException {
		repositoryHelper = getRepositoryHelper();
		if (repositoryHelper != null) {
			if (lockInfos == null) {
				if (repositoryIsAvailable) {
					lockInfos = repositoryHelper.getLocks(UserSessionHelper.getSession(), "");
				} else {
					return new LockInfo[0];
				}
			}
			return lockInfos;
		} else {
			return new LockInfo[0];
		}
	}

	private IRepositoryHelper getRepositoryHelper() {
		if (repositoryHelper == null) {
			repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
		}
		return repositoryHelper;
	}

	public String getObjectLocation(URI resourceURI, String fragment) {

		String objectLocation = null;
		String resourceLocation = resourceURI.toString();
		URI projectURI = null;
		if (resourceURI.segmentCount() > 1) {
			IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();

			IProject project = null;
			for (IProject currentProject : projects) {
				if (resourceURI.isPlatformResource()) {
					projectURI = URI.createPlatformResourceURI(currentProject.getName(), true);
				} else if (resourceURI.isFile()) {
					projectURI = URI.createFileURI(currentProject.getLocation().toString());
				}
				if (projectURI != null) {
					if (projectURI != null && resourceLocation.startsWith(projectURI.toString() + "/")
							|| resourceLocation.equals(projectURI.toString())) {
						project = currentProject;
						break;
					}
				}

			}
			if (project != null) {
				IResource resource = project.getFile(URLDecoder.decode(resourceLocation.replace(projectURI.toString() + "/", "")));
				RepositoryProvider repositoryProvider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);

				if (repositoryProvider != null) {
					ModelBusRepositoryProvider modelBusRepositoryProvider = (ModelBusRepositoryProvider) repositoryProvider;

					objectLocation = modelBusRepositoryProvider.getRepositoryPath(resource);
					URI uri = URI.createURI(objectLocation);
					if (fragment != null) {
						uri = uri.appendFragment(fragment);
					}
					objectLocation = uri.toString();
				}
			}
		} else {
			if (fragment != null) {
				objectLocation = resourceURI.appendFragment(fragment).toString();
			} else {
				objectLocation = resourceLocation;
			}
		}
		return objectLocation;
	}

	public boolean isDirectLocked(Object object) {
		if (object == null) {
			return false;
		}
		String objectLocation = null;
		if (object instanceof IFile) {
			objectLocation = getObjectLocation(URI.createURI(((IFile) object).getLocationURI().toString()), null);
		} else if (object instanceof EObject) {
			EObject eObject = (EObject) object;
			Resource objectResource = eObject.eResource();
			if (objectResource == null) {
				return false;
			}
			String objectUri = objectResource.getURIFragment(eObject);
			objectLocation = getObjectLocation(objectResource.getURI(), objectUri);

		} else if (object instanceof Resource) {
			objectLocation = getObjectLocation(((Resource) object).getURI(), null);
		}

		try {

			LockInfo[] lockInfos = getLocks();
			for (int i = 0; i < lockInfos.length; i++) {
				LockInfo lockInfo = lockInfos[i];
				if (lockInfo != null) {
					String objectUrl = lockInfo.getUrl();
					if (objectLocation != null && objectLocation.equals(objectUrl)) {
						return true;
					}
				}
			}

			// }
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			return false;
		}

		return false;
	}

	public String objectLockedBy(Object object) {
		String objectLocation = null;
		if (object instanceof IFile) {
			objectLocation = getObjectLocation(URI.createURI(((IFile) object).getLocationURI().toString()), null);
		} else if (object instanceof EObject) {
			EObject eObject = (EObject) object;
			Resource objectResource = eObject.eResource();
			if (objectResource == null) {
				return null;
			}
			String objectUri = objectResource.getURIFragment(eObject);
			objectLocation = getObjectLocation(objectResource.getURI(), objectUri);

		} else if (object instanceof Resource) {
			objectLocation = getObjectLocation(((Resource) object).getURI(), null);
		}

		try {

			LockInfo[] lockInfos = getLocks();
			for (int i = 0; i < lockInfos.length; i++) {
				LockInfo lockInfo = lockInfos[i];
				if (lockInfo != null) {
					String objectUrl = lockInfo.getUrl();
					String user = lockInfo.getUserName();
					if (objectLocation != null && objectLocation.equals(objectUrl)) {
						return user;
					}
				}
			}

			// }
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			return null;
		}

		if (object instanceof EObject) {
			Object container = ((EObject) object).eContainer();
			if (container == null) {
				container = ((EObject) object).eResource();
			}
			if (container != null) {
				return objectLockedBy(container);
			}
		}

		return null;
	}

	@Override
	public void notification(String url, String mode, String user, String sessionID) {
		if ("http://www.modelbus.org/system/model/ModelBus.lock".equals(url)) {
			if (repositoryIsAvailable) {
				resetLockHelper();
			}
		}

	}

	@Override
	public void commitChangeModelNotification(String modelUri, String messageID, String changeModelContent) {
	}

	@Override
	public void update(Observable o, Object arg) {
		setRepositoryStatus(ModelBusRepositoryPlugin.getPlugin().getRepositoryStatus().isAvailable());
	}

}
