/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.modelbus.model.user.User;
import org.modelbus.model.user.UserModel;
import org.modelbus.model.user.UserPackage;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.Policy;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

/**
 * Action for checking in the selected resources
 */
public class PutAction extends ModelBusRepositoryAction {

	/*
	 * Method declared on IActionDelegate.
	 */
	public void execute(IAction action) {
		try {
			Shell shell = ModelBusRepositoryPlugin.getPlugin().getWorkbench().getActiveWorkbenchWindow().getShell();
			IResource[] selectedResources = getSelectedResources();

			IResource selectedElement = selectedResources[0];
			IResource[] allChildren = getAllChildren(selectedElement, new ArrayList<IResource>());

			for (int j = 0; j < allChildren.length; j++) {
				IResource child = allChildren[j];
				String childName = child.getName();
				if (childName.equals("ModelBus.user")) {

					ResourceSet resourceSet = new ResourceSetImpl();
					resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("user", new XMIResourceFactoryImpl());
					resourceSet.getPackageRegistry().put(UserPackage.eNS_URI, UserPackage.eINSTANCE);
					Resource res = resourceSet.getResource(URI.createFileURI(child.getLocation().toString()), true);
					UserModel userModel = (UserModel) res.getContents().get(0);
					EList<User> ownedUser = userModel.getOwnedUser();
					for (User user : ownedUser) {
						if (UserSessionHelper.getPropertyUserName().equals(user.getName())
								&& !UserSessionHelper.getPropertyUserPassword().equals(user.getPassword())) {

							MessageDialog messageDialog = new MessageDialog(
									shell,
									"Changing current password",
									null,
									"You are about to change your password!\nPlease change your password in the ModelBus Preferences Page after committing the file.",
									MessageDialog.INFORMATION, new String[] { "OK", "CANCEL" }, 0);
							if (messageDialog.open() == 1) {
								return;
							}
							break;
						}
					}
					break;
				}
			}

			MultiLineInputDialog inputDialog = new MultiLineInputDialog(shell, "Commit Message Dialog");

			if (inputDialog.open() == Window.OK) {
				String commitMessage = inputDialog.getValue();
				PutOperation operation = new PutOperation(getTargetPart(), ModelBusRepositoryOperation.createScopeManager(
						Policy.bind("PutAction.working"), getSelectedMappings()), commitMessage); //$NON-NLS-1$
				operation.setOverwriteIncoming(isOverrideIncoming());
				operation.run();
			}
			// PutOperation operation = new PutOperation(getTargetPart(),
			//								ModelBusRepositoryOperation.createScopeManager(Policy.bind("PutAction.working"), getSelectedMappings()),commitMessage); //$NON-NLS-1$
			// operation.setOverwriteIncoming(isOverrideIncoming());
			// operation.run();
		} catch (InvocationTargetException e) {
			handle(e, null, Policy.bind("PutAction.problemMessage")); //$NON-NLS-1$
		} catch (InterruptedException e) {
			// Ignore
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Indicate whether the put should override incoming changes.
	 * 
	 * @return whether the put should override incoming changes.
	 */
	protected boolean isOverrideIncoming() {
		return false;
	}

	private IResource[] getAllChildren(IResource resource, ArrayList<IResource> allChildren) throws CoreException {

		if (!allChildren.contains(resource)) {
			allChildren.add(resource);
		}
		if (resource instanceof IContainer) {
			IResource[] members = ((IContainer) resource).members();
			if (!(members.length == 0)) {
				// allChildren.addAll(Arrays.asList(members));
				for (int i = 0; i < members.length; i++) {
					IResource member = members[i];
					getAllChildren(member, allChildren);

				}
			}
		}

		return allChildren.toArray(new IResource[allChildren.size()]);
	}
}
