/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.modelbus.team.eclipse.repository.ui;

import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.jface.action.IAction;
import org.eclipse.team.core.subscribers.SubscriberScopeManager;
import org.eclipse.team.ui.TeamUI;
import org.eclipse.team.ui.synchronize.ISynchronizeParticipant;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositoryMergeContext;
import org.modelbus.team.eclipse.repository.subscriber.ModelBusRepositorySubscriber;

/**
 * Action to synchronize the selected resources. This results in a file-system participant being added to the synchronize view.
 */
public class SynchronizeAction extends ModelBusRepositoryAction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void execute(IAction action) {
		ResourceMapping[] mappings = getSelectedMappings();
		if (mappings.length == 0)
			return;
		SubscriberScopeManager manager = ModelBusRepositoryOperation.createScopeManager(ModelBusRepositorySubscriber.getInstance().getName(),
				mappings);
		ModelBusRepositoryMergeContext context = new ModelBusRepositoryMergeContext(manager);
		ModelBusRepositorySynchronizeParticipant participant = new ModelBusRepositorySynchronizeParticipant(context);
		TeamUI.getSynchronizeManager().addSynchronizeParticipants(new ISynchronizeParticipant[] { participant });
		participant.run(getTargetPart());
	}

}
