/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Shell;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractNonRecursiveTeamAction;
import org.modelbus.team.eclipse.ui.dialog.OperationErrorDialog;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;

/**
 * Branch or tag action
 * 
 * @author Sergiy Logvin
 */
public class BranchTagAction extends AbstractNonRecursiveTeamAction {
	public static final int	BRANCH_ACTION	= 0;
	public static final int	TAG_ACTION		= 1;

	protected int			actionType;

	public BranchTagAction(int actionType) {
		this.actionType = actionType;
	}

	public void runImpl(IAction action) {
		IResource[] resources = this.getSelectedResources(IStateFilter.SF_EXCLUDE_DELETED);

		IActionOperation op = BranchTagAction.getBranchTagOperation(this.getShell(), this.actionType, resources);
		if (op != null) {
			this.runScheduled(op);
		}
	}

	public static IActionOperation getBranchTagOperation(Shell shell, int actionType, IResource[] resources) {
		IRepositoryResource[] remoteResources = new IRepositoryResource[resources.length];

		for (int i = 0; i < resources.length; i++) {
			remoteResources[i] = ModelBusRemoteStorage.instance().asRepositoryResource(resources[i]);
		}

		if (!OperationErrorDialog.isAcceptableAtOnce(
				remoteResources,
				ModelBusTeamUIPlugin.instance().getResource(
						actionType == BRANCH_ACTION ? "BranchTagAction.Error.Branch" : "BranchTagAction.Error.Tag"), shell)) {
			return null;
		}

		Set<String> nodeNames = new HashSet<String>();
		boolean isStructureEnabled =
		// CHECKMWA remoteResources[0].getRepositoryLocation().isStructureEnabled()&&
		ModelBusTeamPreferences.getRepositoryBoolean(ModelBusTeamUIPlugin.instance().getPreferenceStore(),
				ModelBusTeamPreferences.BRANCH_TAG_CONSIDER_STRUCTURE_NAME);
		// if (true) {
		//
		// // nodeNames = org.modelbus.team.eclipse.ui.action.remote.BranchTagAction.getExistingNodeNames(actionType == BranchTagAction.BRANCH_ACTION
		// ?
		// ModelBusUtility.getBranchesLocation(remoteResources[0]) : ModelBusUtility.getTagsLocation(remoteResources[0]));
		// }
		//
		// AbstractBranchTagPanel panel =
		// actionType == BranchTagAction.BRANCH_ACTION ?
		// new BranchPanel(ModelBusUtility.getBranchesLocation(remoteResources[0]), true, nodeNames, resources) :
		// new TagPanel(ModelBusUtility.getTagsLocation(remoteResources[0]), true, nodeNames, resources);
		// DefaultDialog dialog = new DefaultDialog(shell, panel);
		//
		// if (dialog.open() == 0) {
		// IRepositoryResource destination = panel.getDestination();
		//
		// boolean forceCreate =
		// isStructureEnabled && resources.length == 1 && resources[0].getType() == IResource.PROJECT &&
		// !org.modelbus.team.eclipse.ui.action.remote.BranchTagAction.isSingleProjectLayout(remoteResources[0]);
		// PreparedBranchTagOperation mainOp = new PreparedBranchTagOperation((actionType == BRANCH_ACTION ? "Branch" : "Tag"), resources,
		// remoteResources, destination, panel.getMessage(), forceCreate);
		//
		// CompositeOperation op = new CompositeOperation(mainOp.getId());
		//
		// IResource []newResources = panel.getSelectedResources();
		// if (newResources != null && newResources.length > 0) {
		// op.add(new AddToModelBusOperation(newResources, false));
		// }
		//
		// if (panel.isFreezeExternals()) {
		// FreezeExternalsOperation freezeOp = new FreezeExternalsOperation(resources);
		// op.add(freezeOp);
		// op.add(mainOp, new IActionOperation[] {freezeOp});
		// op.add(new RestoreExternalsOperation(freezeOp));
		// }
		// else {
		// op.add(mainOp);
		// }
		//
		// if (newResources != null && newResources.length > 0) {
		// op.add(new RevertOperation(newResources, false));
		// }
		//
		// if (panel.isStartWithSelected()) {
		// SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
		// op.add(saveOp);
		// op.add(new SwitchOperation(resources, mainOp, Depth.INFINITY), new IActionOperation[] {mainOp});
		// op.add(new RestoreProjectMetaOperation(saveOp));
		// op.add(new RefreshResourcesOperation(resources));
		// }
		//
		// return op;
		throw new RuntimeException("TODOMWA35");
		// }
		// return null;
	}

	public boolean isEnabled() {
		return this.checkForResourcesPresence(IStateFilter.SF_EXCLUDE_DELETED);
	}

	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

}
