/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.internal.CompareEditor;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.eclipse.team.core.RepositoryProvider;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.StringRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryFile;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRepositoryFolder;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryPlugin;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryProvider;
import org.modelbus.team.eclipse.ui.action.AbstractWorkingCopyAction;
import org.modelbus.team.eclipse.ui.compare.ModelBusCompareEditorInput;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.panel.remote.ComparePanel;

/**
 * Compare menu "compare with selected revision" action implementation
 * 
 * @author Alexander Gurov
 */
public class CompareWithRevisionAction extends AbstractWorkingCopyAction {

	public CompareWithRevisionAction() {
		super();
	}

	public void runImpl(IAction action) {
		IResource resource = this.getSelectedResources()[0];
		IRepositoryResource remote = null;

		String remotePath = null;

		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject(), ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider instanceof ModelBusRepositoryProvider) {
			remotePath = ((ModelBusRepositoryProvider) provider).getRepositoryPath(resource);
			if (resource.getType() == IResource.FILE) {
				remote = new ModelBusRepositoryFile(remotePath, ModelBusRevision.HEAD);
			} else if (resource.getType() == IResource.FOLDER) {
				remote = new ModelBusRepositoryFolder(remotePath, ModelBusRevision.HEAD);
			}
		}

		ComparePanel panel = new ComparePanel(remote);
		DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);
		if (dialog.open() == 0) {
			remote = panel.getSelectedResource();
			ModelBusRevision revision = remote.getSelectedRevision();
			String revisionString = IRepositoryHelper.REVISION_HEAD;
			if (revision instanceof StringRevision) {
				revisionString = ((StringRevision) revision).getRevision();
			}
			CompareConfiguration configuration = new CompareConfiguration();
			configuration.setProperty(CompareEditor.CONFIRM_SAVE_PROPERTY, Boolean.TRUE);
			ModelBusCompareEditorInput compare = new ModelBusCompareEditorInput(configuration, provider, resource,/* revision */revisionString);
			CompareUI.openCompareEditor(compare);
		}
	}

	public boolean isEnabled() {
		boolean isCompareFoldersAllowed = true; // CHECKMWA CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion() ==
												// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x;
		if (this.getSelectedResources().length != 1) {
			return false;
		}
		RepositoryProvider provider = RepositoryProvider.getProvider(this.getSelectedResources()[0].getProject(),
				ModelBusRepositoryPlugin.PROVIDER_ID);
		if (provider instanceof ModelBusRepositoryProvider && (isCompareFoldersAllowed || this.getSelectedResources()[0].getType() == IResource.FILE)) {
			return true;
		}
		return false;
		// return true;
		// boolean isCompareFoldersAllowed = true; // CHECKMWA CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion()
		// ==
		// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x;
		// return
		// this.getSelectedResources().length == 1 &&
		// (isCompareFoldersAllowed || this.getSelectedResources()[0].getType() == IResource.FILE);// &&
		// this.checkForResourcesPresence(CompareWithWorkingCopyAction.COMPARE_FILTER);
	}

	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

}
