/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import java.util.HashMap;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Shell;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.GetRemoteContentsOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.action.AbstractWorkingCopyAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.dialog.ReplaceWarningDialog;
import org.modelbus.team.eclipse.ui.panel.local.ReplaceBranchTagPanel;

/**
 * Replace with branch/tag action implementation
 * 
 * @author Alexei Goncharov
 */
public class ReplaceWithBranchTagAction extends AbstractWorkingCopyAction {
	protected int	type;

	public ReplaceWithBranchTagAction(int type) {
		super();
		this.type = type;
	}

	public boolean isEnabled() {
		return this.getSelectedResources().length == 1 && this.checkForResourcesPresence(IStateFilter.SF_ONREPOSITORY);
	}

	public void runImpl(IAction action) {
		IResource[] resources = this.getSelectedResources(IStateFilter.SF_ONREPOSITORY);
		IActionOperation op = ReplaceWithBranchTagAction.getReplaceOperation(resources, this.getShell(), this.type);
		if (op != null) {
			this.runScheduled(op);
		}
	}

	public static IActionOperation getReplaceOperation(IResource[] resources, Shell shell, int type) {
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resources[0]);
		IRepositoryResource remote = local.isCopied() ? ModelBusUtility.getCopiedFrom(resources[0]) : ModelBusRemoteStorage.instance()
				.asRepositoryResource(resources[0]);
		ReplaceBranchTagPanel panel = new ReplaceBranchTagPanel(remote, local.getRevision(), type);
		DefaultDialog dlg = new DefaultDialog(shell, panel);
		if (dlg.open() == 0) {
			ReplaceWarningDialog dialog = new ReplaceWarningDialog(shell);
			if (dialog.open() == 0) {
				IRepositoryResource selected = panel.getSelectedResource();
				HashMap<String, String> remote2local = new HashMap<String, String>();
				remote2local.put(ModelBusUtility.encodeURL(selected.getUrl()), FileUtility.getWorkingCopyPath(resources[0]));
				CompositeOperation op = new CompositeOperation("Operation.ReplaceWithRevision");
				SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
				op.add(saveOp);
				op.add(new GetRemoteContentsOperation(resources, new IRepositoryResource[] { selected }, remote2local));
				op.add(new RestoreProjectMetaOperation(saveOp));
				op.add(new RefreshResourcesOperation(resources));
				return op;
			}
		}
		return null;
	}

}
