/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.local;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SwitchOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractNonRecursiveTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.dialog.OperationErrorDialog;
import org.modelbus.team.eclipse.ui.panel.local.SwitchPanel;

/**
 * Switch working copy to the new URL action implementation
 * 
 * @author Alexander Gurov
 */
public class SwitchAction extends AbstractNonRecursiveTeamAction {

	public SwitchAction() {
		super();
	}

	public void runImpl(IAction action) {
		IResource[] resources = this.getSelectedResources(IStateFilter.SF_EXCLUDE_DELETED);

		if (!OperationErrorDialog.isAcceptableAtOnce(resources, ModelBusTeamUIPlugin.instance().getResource("SwitchAction.Error"), this.getShell())) {
			return;
		}

		// in case of multiple switch selected repository resource should be recognized as branch/tag/trunk (using copied from). If "copied from"
		// inaccessible multi switch will fails, single switch for project should be performed like for folders

		// FIXME peg revision, revision limitation ???
		SwitchPanel panel;
		if (resources.length > 1) {
			boolean containFolders = false;
			for (IResource current : resources) {
				if (current instanceof IContainer) {
					containFolders = true;
					break;
				}
			}
			IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resources[0]);
			// remote = ModelBusUtility.getTrunkLocation(remote);
			System.out.println("CHECKMWA36");
			panel = new SwitchPanel(remote, ModelBusRevision.INVALID_REVISION_DESCRIPTOR, containFolders);
		} else {
			IResource resource = resources[0];
			IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resource);
			ILocalResource local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resource);
			panel = new SwitchPanel(remote, local.getRevision(), resources[0] instanceof IContainer);
		}

		DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);
		if (dialog.open() == 0) {
			IRepositoryResource[] destinations = panel.getSelection(resources);

			SwitchOperation mainOp = new SwitchOperation(resources, destinations, panel.getDepth());

			CompositeOperation op = new CompositeOperation(mainOp.getId());

			SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
			op.add(saveOp);
			op.add(mainOp);
			op.add(new RestoreProjectMetaOperation(saveOp));
			op.add(new RefreshResourcesOperation(resources));

			this.runScheduled(op);
		}
	}

	public boolean isEnabled() {
		return this.checkForResourcesPresence(IStateFilter.SF_EXCLUDE_DELETED);
	}

	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

}
