/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Options;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision.Kind;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.remote.DeleteResourcesOperation;
import org.modelbus.team.eclipse.core.operation.remote.SetRevisionAuthorNameOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;
import org.modelbus.team.eclipse.ui.panel.common.CommentPanel;
import org.modelbus.team.eclipse.ui.repository.RepositoriesView;

/**
 * Delete remote resource action
 * 
 * @author Alexander Gurov
 */
public class DeleteAction extends AbstractRepositoryTeamAction {

	public DeleteAction() {
		super();
	}

	public void runImpl(IAction action) {
		CommentPanel commentPanel = new CommentPanel(ModelBusTeamUIPlugin.instance().getResource("DeleteAction.Comment.Title"));
		DefaultDialog dialog = new DefaultDialog(this.getShell(), commentPanel);
		if (dialog.open() == 0) {
			IRepositoryResource[] resources = this.getSelectedRepositoryResources();
			IRepositoryResource[] commonParents = ModelBusUtility.getCommonParents(resources);

			DeleteResourcesOperation mainOp = new DeleteResourcesOperation(resources, commentPanel.getMessage());
			CompositeOperation op = new CompositeOperation(mainOp.getId());

			op.add(mainOp);
			op.add(new RefreshRemoteResourcesOperation(commonParents));
			op.add(new AbstractActionOperation("") {
				protected void runImpl(IProgressMonitor monitor) throws Exception {
					DeleteAction.this.getShell().getDisplay().syncExec(new Runnable() {
						public void run() {
							RepositoriesView view = RepositoriesView.instance();
							if (view == null) {
								return;
							}

							view.getRepositoryTree().fireEmptySelectionEvent();
						}
					});
				}

				public ISchedulingRule getSchedulingRule() {
					return null;
				}
			});
			op.add(new SetRevisionAuthorNameOperation(mainOp, Options.FORCE));

			this.runScheduled(op);
		}
	}

	public boolean isEnabled() {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		for (int i = 0; i < resources.length; i++) {
			// CHECKMWA "" for root
			if (resources[i].getUrl().equals("")
					|| resources[i].getSelectedRevision().getKind() != Kind.HEAD
					|| resources[i] instanceof IRepositoryRoot
					&& (((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_ROOT || ((IRepositoryRoot) resources[i]).getKind() == IRepositoryRoot.KIND_LOCATION_ROOT)) {
				return false;
			}
		}
		return resources.length > 0;
	}

}
