/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote.management;

import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.ModelBusTeamPlugin;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.CreateFolderOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryContainer;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryRoot;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.operation.RefreshRemoteResourcesOperation;
import org.modelbus.team.eclipse.ui.panel.remote.CreateProjectStructurePanel;

/**
 * Base implementation for CreateXXXLocationAction's
 * 
 * @author Alexander Gurov
 */
public class CreateProjectStructureAction extends AbstractRepositoryTeamAction {

	private boolean	toolbarAction;

	public CreateProjectStructureAction() {
		super();
		this.toolbarAction = false;

	}

	public CreateProjectStructureAction(boolean toolbarAction) {
		super();
		this.toolbarAction = toolbarAction;

	}

	public void runImpl(IAction action) {
		CreateProjectStructurePanel panel = new CreateProjectStructurePanel();
		DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);
		if (dialog.open() == 0) {
			String name = panel.getResourceName();
			IRepositoryResource[] parent = this.getSelectedRepositoryResources();
			if (this.toolbarAction) {
				parent = new IRepositoryResource[] { ModelBusConnector.getRepositoryRoot() };
			}
			// IRepositoryLocation location = parent[0].getRepositoryLocation();
			IRepositoryRoot root = (IRepositoryRoot) parent[0].getRoot();
			String trunk = ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTrunkName();
			String branches = ModelBusTeamPlugin.instance().getOptionProvider().getDefaultBranchesName();
			String tags = ModelBusTeamPlugin.instance().getOptionProvider().getDefaultTagsName();
			// String trunk = ShareProjectOperation.getTrunkName(location);
			// String branches = ShareProjectOperation.getBranchesName(location);
			// String tags = ShareProjectOperation.getTagsName(location);
			String[] folders = name.length() == 0 ? new String[] { trunk, branches, tags } : new String[] { name + "/" + trunk,
					name + "/" + branches, name + "/" + tags };

			CreateFolderOperation mainOp = new CreateFolderOperation(parent[0], folders, panel.getMessage());
			CompositeOperation op = new CompositeOperation(mainOp.getId());
			op.add(mainOp);
			op.add(new RefreshRemoteResourcesOperation(parent), new IActionOperation[] { mainOp });

			this.runScheduled(op);
			// throw new RuntimeException("TODOMWA");
		}
	}

	public boolean isEnabled() {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		return this.toolbarAction || (resources.length == 1 && resources[0] instanceof IRepositoryContainer);
		// MES: Option 'New.. > Project Structure' in the RepositoriesView pop-up menu is always disabled.
		// return false;
	}

}
