/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote.management;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DiscardConfirmationDialog;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryRevision;

/**
 * Discard revision links on the repository location
 * 
 * @author Alexander Gurov
 */
public class DiscardRevisionLinksAction extends AbstractRepositoryTeamAction {

	public DiscardRevisionLinksAction() {
		super();
	}

	public void runImpl(IAction action) {
		final RepositoryRevision[] revisions = ((RepositoryRevision[]) this.getAdaptedSelection(RepositoryRevision.class));
		DiscardConfirmationDialog dialog = new DiscardConfirmationDialog(this.getShell(), revisions.length == 1, DiscardConfirmationDialog.MSG_LINK);
		if (dialog.open() == 0) {
			AbstractActionOperation mainOp = new AbstractActionOperation("Operation.RemoveRevisionLinks") {
				protected void runImpl(IProgressMonitor monitor) throws Exception {
					for (int i = 0; i < revisions.length; i++) {
						final IRepositoryResource[] links = revisions[i].getRepositoryResources();
						this.protectStep(new IUnprotectedOperation() {
							public void run(IProgressMonitor monitor) throws Exception {
								for (int i = 0; i < links.length; i++) {
									// TODOMWA
									// CHECKMWA
								}
							}
						}, monitor, revisions.length);
					}
				}
			};
			CompositeOperation op = new CompositeOperation(mainOp.getId());
			op.add(mainOp);
			this.runBusy(op);
		}
	}

	public boolean isEnabled() {
		return this.getAdaptedSelection(RepositoryRevision.class).length > 0;
	}

}
