/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.action.remote.management;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.Dialog;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.remote.LocateResourceURLInHistoryOperation;
import org.modelbus.team.eclipse.core.operation.remote.management.AddRevisionLinkOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractRepositoryTeamAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.panel.common.InputRevisionPanel;

/**
 * Select revision for any repository resource
 * 
 * @author Alexander Gurov
 */
public class SelectResourceRevisionAction extends AbstractRepositoryTeamAction {
	public SelectResourceRevisionAction() {
		super();
	}

	public void runImpl(IAction action) {
		IRepositoryResource[] resources = this.getSelectedRepositoryResources();
		this.runImpl(resources);
	}

	protected void runImpl(IRepositoryResource[] resources) {
		ModelBusRevision selectedRevision = null;

		if (resources.length == 1) {
			InputRevisionPanel panel = new InputRevisionPanel(resources[0], ModelBusTeamUIPlugin.instance().getResource(
					"SelectResourceRevisionAction.InputRevisionPanel.Title"));
			DefaultDialog dialog = new DefaultDialog(this.getShell(), panel);
			if (dialog.open() == Dialog.OK) {
				selectedRevision = panel.getSelectedRevision();
				resources[0] = ModelBusUtility.copyOf(resources[0]);
				resources[0].setSelectedRevision(selectedRevision);
			} else {
				return;
			}
		}

		LocateResourceURLInHistoryOperation locateOp = new LocateResourceURLInHistoryOperation(resources);
		AbstractActionOperation mainOp = new AddRevisionLinkOperation(locateOp, selectedRevision);
		CompositeOperation op = new CompositeOperation(mainOp.getId());
		op.add(locateOp);
		op.add(mainOp, new IActionOperation[] { locateOp });
		// op.add(new SaveRepositoryLocationsOperation());
		// HashSet<IRepositoryLocation> locations = new HashSet<IRepositoryLocation>();
		// for (IRepositoryResource resource : resources) {
		// locations.add(resource.getRepositoryLocation());
		// }
		// op.add(new RefreshRepositoryLocationsOperation(locations.toArray(new IRepositoryLocation[locations.size()]), true));
		// TODOMWA CHECKMWA
		this.runScheduled(op);
	}

	public boolean isEnabled() {
		// MES: Always disabled.
		// return this.getSelectedRepositoryResources().length > 0;
		return false;
	}

}
