/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.compare;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusEntryRevisionReference;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.ModelBusProgressMonitor;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.local.SetPropertyAction;
import org.modelbus.team.eclipse.ui.dialog.DefaultDialog;
import org.modelbus.team.eclipse.ui.properties.ResourcePropertyEditPanel;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Compare input for comparison of local and remote ModelBus properties.
 * 
 * @author Alexei Goncharov
 */
public class ThreeWayPropertyCompareInput extends PropertyCompareInput {

	protected String	baseRevision;
	protected IResource	leftResource;

	public ThreeWayPropertyCompareInput(CompareConfiguration configuration, IResource left, ModelBusEntryRevisionReference right,
			ModelBusEntryRevisionReference ancestor,
			// IRepositoryLocation location,
			String baseRevisionNumber) {
		super(configuration, new ModelBusEntryRevisionReference(FileUtility.getWorkingCopyPath(left), null, ModelBusRevision.WORKING), right,
				ancestor
		// ,
		// location
		);
		this.baseRevision = baseRevisionNumber;
		this.leftResource = left;
	}

	protected void fillMenu(IMenuManager manager, TreeSelection selection) {
		final PropertyCompareNode selectedNode = (PropertyCompareNode) selection.getFirstElement();
		Action tAction = null;
		manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("SetPropertyAction.label")) {
			public void run() {
				IResource[] resources = new IResource[] { ThreeWayPropertyCompareInput.this.leftResource };
				ResourcePropertyEditPanel panel = new ResourcePropertyEditPanel(null, resources, false);
				DefaultDialog dialog = new DefaultDialog(UIMonitorUtility.getShell(), panel);
				if (dialog.open() == Dialog.OK) {
					SetPropertyAction.doSetProperty(resources, panel, null);
					boolean notContained = true;
					RootCompareNode root = (RootCompareNode) ThreeWayPropertyCompareInput.this.getCompareResult();
					IDiffElement[] nodes = root.getChildren();
					for (int i = 0; i < nodes.length; i++) {
						PropertyCompareNode current = (PropertyCompareNode) nodes[i];
						if (current.getName().equals(panel.getPropertyName())) {
							notContained = false;
							((PropertyElement) current.getLeft()).setValue(panel.getPropertyValue());
							current.setKind(ThreeWayPropertyCompareInput.this.calculateDifference(panel.getPropertyValue(),
									((PropertyElement) current.getRight()).getValue(), ((PropertyElement) current.getAncestor()).getValue()));
							current.fireChange();
						}
					}
					if (notContained) {
						new PropertyCompareNode(root, ThreeWayPropertyCompareInput.this.calculateDifference(panel.getPropertyValue(), null, null),
								new PropertyElement(panel.getPropertyName(), null, false), new PropertyElement(panel.getPropertyName(), panel
										.getPropertyValue(), true), new PropertyElement(panel.getPropertyName(), null, false));
					}
					ThreeWayPropertyCompareInput.this.viewer.refresh();
				}
			}
		});
		tAction.setEnabled(true);

		manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("RemovePropertyAction.label")) {
			public void run() {
				ThreeWayPropertyCompareInput.this.removeProperty(selectedNode);
			}
		});
		tAction.setEnabled(((PropertyElement) selectedNode.getLeft()).getValue() != null);
	}

	protected void removeProperty(PropertyCompareNode currentNode) {
		// perform property removing
		final String propName = currentNode.getName();
		IActionOperation op = null;
		UIMonitorUtility.doTaskNowDefault(op = new AbstractActionOperation("Operation.RemoveProperties") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				final IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
				// ThreeWayPropertyCompareInput.this.location.acquireModelBusProxy();
				try {
					proxy.removeProperty(ThreeWayPropertyCompareInput.this.left.path, propName, Depth.EMPTY, null, new ModelBusProgressMonitor(this,
							monitor, null));
				} finally {
					// ThreeWayPropertyCompareInput.this.location.releaseModelBusProxy(proxy);
				}
			}
		}, false);

		if (op.getExecutionState() != IActionOperation.OK) {
			return;
		}
		// refresh UI
		if ((currentNode.getKind() & Differencer.CHANGE_TYPE_MASK) == Differencer.ADDITION
				&& (currentNode.getKind() & Differencer.DIRECTION_MASK) == Differencer.LEFT) {
			((RootCompareNode) this.getCompareResult()).remove(currentNode);
		} else {
			((PropertyElement) currentNode.getLeft()).setValue(null);
			currentNode.setKind(this.calculateDifference(null, ((PropertyElement) currentNode.getRight()).getValue(),
					((PropertyElement) currentNode.getAncestor()).getValue()));
		}
		currentNode.fireChange();
		this.viewer.refresh();
	}

	public String getTitle() {
		return ModelBusTeamUIPlugin.instance().getResource(
				"PropertyCompareInput.Title3",
				new String[] { this.left.path.substring(this.left.path.lastIndexOf("/") + 1) + " [" + this.getRevisionPart(this.left),
						this.getRevisionPart(this.ancestor), this.getRevisionPart(this.right) + "] " });
	}

	protected String getRevisionPart(ModelBusEntryRevisionReference reference) {
		if (reference.revision == ModelBusRevision.WORKING) {
			return ModelBusTeamUIPlugin.instance().getResource("ResourceCompareInput.LocalSign");
		} else if (reference.revision == ModelBusRevision.BASE) {
			if (this.ancestor == null) {
				return ModelBusTeamUIPlugin.instance().getResource("ResourceCompareInput.ResourceIsNotAvailable");
			}
			return ModelBusTeamUIPlugin.instance().getResource("ResourceCompareInput.BaseSign",
					new String[] { String.valueOf(this.baseRevision) });
		}
		return ModelBusTeamUIPlugin.instance().getResource("ResourceCompareInput.RevisionSign", new String[] { String.valueOf(reference.revision) });
	}

}
