/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.console;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.operation.IConsoleStream;
import org.modelbus.team.eclipse.core.utility.ProgressMonitorUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * ModelBus Console implementation
 * 
 * @author Alexander Gurov
 */
public class ModelBusConsole extends MessageConsole implements IPropertyChangeListener {
	public static final String		ModelBus_CONSOLE_TYPE	= "org.modelbus.team.eclipse.ui.console.ModelBusConsole";

	protected MessageConsoleStream	cmdStream;
	protected MessageConsoleStream	okStream;
	protected MessageConsoleStream	warningStream;
	protected MessageConsoleStream	errorStream;

	protected int					autoshow;
	protected boolean				enabled;

	public ModelBusConsole() {
		super(ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Name"), ModelBusTeamUIPlugin.instance().getImageDescriptor(
				"icons/views/console.gif"));

		super.setType(ModelBusConsole.ModelBus_CONSOLE_TYPE);

		super.init();

		this.setTabWidth(4);

		this.cmdStream = this.newMessageStream();
		this.okStream = this.newMessageStream();
		this.warningStream = this.newMessageStream();
		this.errorStream = this.newMessageStream();

		this.loadPreferences();

		JFaceResources.getFontRegistry().addListener(this);
		ModelBusTeamUIPlugin.instance().getPreferenceStore().addPropertyChangeListener(this);
	}

	public IConsoleStream getConsoleStream() {
		return new ModelBusConsoleStream();
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().startsWith(ModelBusTeamPreferences.CONSOLE_BASE)) {
			this.loadPreferences();
		}
	}

	public void shutdown() {
		super.dispose();

		ModelBusTeamUIPlugin.instance().getPreferenceStore().removePropertyChangeListener(this);
		JFaceResources.getFontRegistry().removeListener(this);

		this.enabled = false;

		ConsolePlugin.getDefault().getConsoleManager().removeConsoles(new IConsole[] { this });

		Color tmp1 = this.cmdStream.getColor();
		Color tmp2 = this.okStream.getColor();
		Color tmp3 = this.warningStream.getColor();
		Color tmp4 = this.errorStream.getColor();

		// unsupported in Eclipse IDE 3.0
		try {
			this.cmdStream.close();
		} catch (Exception ex) {
		}
		try {
			this.okStream.close();
		} catch (Exception ex) {
		}
		try {
			this.warningStream.close();
		} catch (Exception ex) {
		}
		try {
			this.errorStream.close();
		} catch (Exception ex) {
		}

		if (tmp1 != null) {
			tmp1.dispose();
		}
		if (tmp1 != null) {
			tmp2.dispose();
		}
		if (tmp1 != null) {
			tmp3.dispose();
		}
		if (tmp1 != null) {
			tmp4.dispose();
		}
	}

	protected void init() {

	}

	protected void dispose() {

	}

	protected void loadPreferences() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();

		this.enabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_ENABLED_NAME);

		Color tmp = this.cmdStream.getColor();
		this.cmdStream.setColor(new Color(UIMonitorUtility.getDisplay(), ModelBusTeamPreferences.getConsoleRGB(store,
				ModelBusTeamPreferences.CONSOLE_CMD_COLOR_NAME)));
		if (tmp != null && !tmp.equals(this.cmdStream.getColor())) {
			tmp.dispose();
		}
		tmp = this.okStream.getColor();
		this.okStream.setColor(new Color(UIMonitorUtility.getDisplay(), ModelBusTeamPreferences.getConsoleRGB(store,
				ModelBusTeamPreferences.CONSOLE_OK_COLOR_NAME)));
		if (tmp != null && !tmp.equals(this.okStream.getColor())) {
			tmp.dispose();
		}
		tmp = this.warningStream.getColor();
		this.warningStream.setColor(new Color(UIMonitorUtility.getDisplay(), ModelBusTeamPreferences.getConsoleRGB(store,
				ModelBusTeamPreferences.CONSOLE_WRN_COLOR_NAME)));
		if (tmp != null && !tmp.equals(this.warningStream.getColor())) {
			tmp.dispose();
		}
		tmp = this.errorStream.getColor();
		this.errorStream.setColor(new Color(UIMonitorUtility.getDisplay(), ModelBusTeamPreferences.getConsoleRGB(store,
				ModelBusTeamPreferences.CONSOLE_ERR_COLOR_NAME)));
		if (tmp != null && !tmp.equals(this.errorStream.getColor())) {
			tmp.dispose();
		}

		if (ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_NAME)) {
			this.setConsoleWidth(ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_NAME));
		} else {
			this.setConsoleWidth(-1);
		}

		this.autoshow = ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NAME);

		if (ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_NAME)) {
			int limit = ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_NAME);
			this.setWaterMarks(1000 < limit ? 1000 : limit - 1, limit);
		} else {
			this.setWaterMarks(-1, 0);
		}

		UIMonitorUtility.getDisplay().asyncExec(new Runnable() {
			public void run() {
				ModelBusConsole.this.setFont(PlatformUI.getWorkbench().getThemeManager().getCurrentTheme().getFontRegistry()
						.get(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_FONT_NAME)));
			}
		});

		if (this.enabled) {
			ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[] { this });
		} else {
			ConsolePlugin.getDefault().getConsoleManager().removeConsoles(new IConsole[] { this });
		}
	}

	protected class ModelBusConsoleStream implements IConsoleStream {
		protected long		start;
		protected String	buffer;
		protected boolean	outputStarted;
		protected boolean	hasError;
		protected boolean	hasWarning;
		protected boolean	activated;
		protected boolean	cancelled;

		public ModelBusConsoleStream() {

		}

		public void markEnd() {
			if (this.outputStarted) {
				this.write(IConsoleStream.LEVEL_CMD, "*** ");
				if (this.hasError) {
					this.write(IConsoleStream.LEVEL_ERROR, ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Error"));
				} else if (this.hasWarning) {
					this.write(IConsoleStream.LEVEL_WARNING, ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Warning"));
				} else {
					this.write(IConsoleStream.LEVEL_CMD, this.cancelled ? ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Cancelled")
							: ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Ok"));
				}
				this.write(
						IConsoleStream.LEVEL_CMD,
						" "
								+ ModelBusTeamUIPlugin.instance().getResource("ModelBusConsole.Took",
										new String[] { new SimpleDateFormat("mm:ss.SSS").format(new Date(System.currentTimeMillis() - this.start)) })
								+ "\n\n");
			}
		}

		public void markStart(String data) {
			this.start = System.currentTimeMillis();
			this.buffer = data;
		}

		public void doComplexWrite(Runnable runnable) {
			this.flushBuffer();
			if (ModelBusConsole.this.enabled) {
				runnable.run();
			}
		}

		public void write(int severity, String data) {
			this.flushBuffer();
			if (ModelBusConsole.this.enabled) {
				if (!this.activated
						&& (ModelBusConsole.this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ALWAYS
								|| ModelBusConsole.this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ERROR
								&& severity == IConsoleStream.LEVEL_ERROR || ModelBusConsole.this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_WARNING_ERROR
								&& (severity == IConsoleStream.LEVEL_ERROR || severity == IConsoleStream.LEVEL_WARNING))) {
					this.activated = true;
					ConsolePlugin.getDefault().getConsoleManager().showConsoleView(ModelBusConsole.this);
				}
				switch (severity) {
					case IConsoleStream.LEVEL_CMD: {
						this.print(ModelBusConsole.this.cmdStream, data);
						break;
					}
					case IConsoleStream.LEVEL_OK: {
						this.print(ModelBusConsole.this.okStream, data);
						break;
					}
					case IConsoleStream.LEVEL_WARNING: {
						this.hasWarning = true;
						this.print(ModelBusConsole.this.warningStream, data);
						break;
					}
					case IConsoleStream.LEVEL_ERROR:
					default: {
						this.hasError = true;
						this.print(ModelBusConsole.this.errorStream, data);
						break;
					}
				}
			}
		}

		public void markCancelled() {
			this.cancelled = true;
		}

		protected void print(final MessageConsoleStream stream, final String data) {
			// workaround for the Eclipse issue #136943
			if (UIMonitorUtility.getDisplay().getThread() == Thread.currentThread()) {
				ProgressMonitorUtility.doTaskScheduledDefault(new AbstractActionOperation("Operation.WriteToConsoleResources") {
					protected void runImpl(IProgressMonitor monitor) throws Exception {
						stream.print(data);
					}
				}, true);
			} else {
				stream.print(data);
			}
		}

		protected void flushBuffer() {
			this.outputStarted = true;
			if (this.buffer != null) {
				String tmp = this.buffer;
				this.buffer = null;
				this.write(IConsoleStream.LEVEL_CMD, "*** ");
				this.write(IConsoleStream.LEVEL_CMD, tmp);
				this.write(IConsoleStream.LEVEL_CMD, "\n");
			}
		}

	}

}
