/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.extension.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.extension.factory.IPredefinedPropertySet;
import org.modelbus.team.eclipse.ui.extension.factory.PredefinedProperty;

/**
 * IPropertyProvider implementation
 * 
 * @author Sergiy Logvin
 */
public class PredefinedPropertySet implements IPredefinedPropertySet {

	public List<PredefinedProperty> getPredefinedProperties() {

		List<PredefinedProperty> properties = new ArrayList<PredefinedProperty>();

		properties.add(new PredefinedProperty(ModelBusTeamUIPlugin.instance().getResource("AbstractPropertyEditPanel.modelbus_description"), "", ""));
		properties.add(new PredefinedProperty("modelbus:eol-style", this.getDescription("ModelBus.EOL"), ""));
		properties.add(new PredefinedProperty("modelbus:executable", this.getDescription("ModelBus.Executable"), ""));
		properties.add(new PredefinedProperty("modelbus:externals", this.getDescription("ModelBus.Externals"), ""));
		properties.add(new PredefinedProperty("modelbus:ignore", this.getDescription("ModelBus.Ignore"), ""));
		properties.add(new PredefinedProperty("modelbus:keywords", this.getDescription("ModelBus.Keywords"), ""));
		properties.add(new PredefinedProperty("modelbus:needs-lock", this.getDescription("ModelBus.NeedsLock"), ""));
		properties.add(new PredefinedProperty("modelbus:mime-type", this.getDescription("ModelBus.Mimetype"), ""));
		this.getBugtrackProperties(properties);
		properties.add(new PredefinedProperty(ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.tmodelbus_description"), "", ""));
		properties.add(new PredefinedProperty("tmodelbus:logtemplate", this.getDescription("TModelBus.LogTemplate"), ""));
		properties.add(new PredefinedProperty("tmodelbus:logwidthmarker", this.getDescription("TModelBus.LogWidthMarker"), ""));
		properties.add(new PredefinedProperty("tmodelbus:logminsize", this.getDescription("TModelBus.LogMinSize"), ""));
		properties.add(new PredefinedProperty("tmodelbus:lockmsgminsize", this.getDescription("TModelBus.LockMsgMinSize"), ""));
		properties.add(new PredefinedProperty("tmodelbus:logfilelistenglish", this.getDescription("TModelBus.LogFileListEnglish"), ""));
		properties.add(new PredefinedProperty("tmodelbus:projectlanguage", this.getDescription("TModelBus.ProjectLanguage"), ""));

		return properties;
	}

	public Map<String, String> getPredefinedPropertiesRegexps() {

		HashMap<String, String> regexpmap = new HashMap<String, String>();

		regexpmap.put("modelbus:eol-style", "((native)|(LF)|(CR)|(CRLF))");
		regexpmap.put("modelbus:executable", null);
		regexpmap.put("modelbus:externals", "");
		regexpmap.put("modelbus:ignore", "([^\\\\/\\:])+");
		regexpmap.put("modelbus:keywords",
				"((Date)|(Revision)|(Author)|(HeadURL)|(Id)|(LastChangedDate)|(Rev)|(LastChangedRevision)|(LastChangedBy)|(URL)|(\\s))+");
		regexpmap.put("modelbus:needs-lock", null);
		regexpmap.put("modelbus:mime-type", null);
		this.getBugtrackRegExps(regexpmap);
		regexpmap.put("tmodelbus:logtemplate", null);
		regexpmap.put("tmodelbus:logwidthmarker", "(\\d+)");
		regexpmap.put("tmodelbus:logminsize", "(\\d+)");
		regexpmap.put("tmodelbus:lockmsgminsize", "(\\d+)");
		regexpmap.put("tmodelbus:logfilelistenglish", "((true)|(false))");
		regexpmap.put("tmodelbus:projectlanguage", null);

		return regexpmap;
	}

	/**
	 * Allow to define custom bugtraq properties, clients should override this method.
	 * 
	 * @param properties
	 */
	protected void getBugtrackProperties(List<PredefinedProperty> properties) {
		properties.add(new PredefinedProperty(ModelBusTeamUIPlugin.instance().getResource("PropertyEditPanel.bugtraq_description"), "", ""));
		properties.add(new PredefinedProperty("bugtraq:url", this.getDescription("Bugtraq.URL"), "%BUGID%"));
		properties.add(new PredefinedProperty("bugtraq:logregex", this.getDescription("Bugtraq.LogRegex"), ""));
		properties.add(new PredefinedProperty("bugtraq:label", this.getDescription("Bugtraq.Label"), ""));
		properties.add(new PredefinedProperty("bugtraq:message", this.getDescription("Bugtraq.Message"), "%BUGID%"));
		properties.add(new PredefinedProperty("bugtraq:number", this.getDescription("Bugtraq.Number"), ""));
		properties.add(new PredefinedProperty("bugtraq:warnifnoissue", this.getDescription("Bugtraq.WarnIfNoIssue"), ""));
		properties.add(new PredefinedProperty("bugtraq:append", this.getDescription("Bugtraq.Append"), ""));
	}

	/**
	 * Allow to define custom bugtraq properties, clients should override this method.
	 * 
	 * @param name2regexp
	 *            map
	 */
	protected void getBugtrackRegExps(HashMap<String, String> regexpmap) {
		regexpmap.put("bugtraq:url", "((http:\\/\\/)|(https:\\/\\/))(\\S+)?((\\%BUGID\\%))(\\S+)?");
		regexpmap.put("bugtraq:logregex", "");
		regexpmap.put("bugtraq:label", null);
		regexpmap.put("bugtraq:message", "");
		regexpmap.put("bugtraq:number", "((true)|(false))");
		regexpmap.put("bugtraq:warnifnoissue", "((true)|(false))");
		regexpmap.put("bugtraq:append", "((true)|(false))");
	}

	protected String getDescription(String id) {
		return ModelBusTeamUIPlugin.instance().getResource("Property." + id);
	}

}
