/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.extension.impl.synchronize;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.synchronize.AbstractSynchronizeActionGroup;
import org.modelbus.team.eclipse.ui.synchronize.action.CleanUpAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ComparePropertiesAction;
import org.modelbus.team.eclipse.ui.synchronize.action.CreateBranchAction;
import org.modelbus.team.eclipse.ui.synchronize.action.CreatePatchFileAction;
import org.modelbus.team.eclipse.ui.synchronize.action.EditConflictsAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ExpandAllAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ExtractIncomingToAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ExtractOutgoingToAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ExtractToAction;
import org.modelbus.team.eclipse.ui.synchronize.action.RevertAction;
import org.modelbus.team.eclipse.ui.synchronize.action.SetKeywordsAction;
import org.modelbus.team.eclipse.ui.synchronize.action.SetPropertyAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ShowHistoryAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ShowIncomingAnnotationAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ShowIncomingPropertiesAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ShowOutgoingAnnotationAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ShowOutgoingPropertiesAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.AddToModelBusAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.AddToModelBusIgnoreAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.CommitAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.LockAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.MarkAsMergedAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.OverrideAndCommitAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.OverrideAndUpdateAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.UnlockAction;
import org.modelbus.team.eclipse.ui.synchronize.update.action.UpdateAction;

/**
 * Synchronize view update action set
 * 
 * @author Alexander Gurov
 */
public class UpdateActionGroup extends AbstractSynchronizeActionGroup {
	public static final String	GROUP_SYNC_NORMAL		= "syncIncomingOutgoing";
	public static final String	GROUP_SYNC_CONFLICTS	= "syncConflicting";

	public void configureMenuGroups(ISynchronizePageConfiguration configuration) {
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_NORMAL);
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS);
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, AbstractSynchronizeActionGroup.GROUP_MANAGE_LOCALS);
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_TEAM);
		configuration.addMenuGroup(ISynchronizePageConfiguration.P_TOOLBAR_MENU, AbstractSynchronizeActionGroup.GROUP_PROCESS_ALL);
	}

	protected void configureActions(ISynchronizePageConfiguration configuration) {
		UpdateAction updateAllAction = new UpdateAction(ModelBusTeamUIPlugin.instance()
				.getResource("SynchronizeActionGroup.UpdateAllIncomingChanges"), configuration, this.getVisibleRootsSelectionProvider());
		updateAllAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/update.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_TOOLBAR_MENU, UpdateActionGroup.GROUP_PROCESS_ALL, updateAllAction);
		CommitAction commitAllAction = new CommitAction(ModelBusTeamUIPlugin.instance().getResource("UpdateActionGroup.CommitAllOutgoingChanges"),
				configuration, this.getVisibleRootsSelectionProvider());
		commitAllAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/commit.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_TOOLBAR_MENU, UpdateActionGroup.GROUP_PROCESS_ALL, commitAllAction);
		ExpandAllAction expandAllAction = new ExpandAllAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.ExpandAll"),
				configuration, this.getVisibleRootsSelectionProvider());
		expandAllAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/expandall.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_TOOLBAR_MENU, ISynchronizePageConfiguration.MODE_GROUP, expandAllAction);

		UpdateAction updateAction = new UpdateAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.Update"), configuration);
		updateAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/update.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_NORMAL, updateAction);
		CommitAction commitAction = new CommitAction(ModelBusTeamUIPlugin.instance().getResource("UpdateActionGroup.Commit"), configuration);
		commitAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/commit.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_NORMAL, commitAction);

		OverrideAndUpdateAction overrideUpdateAction = new OverrideAndUpdateAction(ModelBusTeamUIPlugin.instance().getResource(
				"SynchronizeActionGroup.OverrideAndUpdate"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS, overrideUpdateAction);
		OverrideAndCommitAction overrideCommitAction = new OverrideAndCommitAction(ModelBusTeamUIPlugin.instance().getResource(
				"UpdateActionGroup.OverrideAndCommit"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS, overrideCommitAction);
		EditConflictsAction editConflictsAction = new EditConflictsAction(ModelBusTeamUIPlugin.instance().getResource(
				"UpdateActionGroup.EditConflicts"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS, editConflictsAction);
		ComparePropertiesAction comparePropsAction = new ComparePropertiesAction(ModelBusTeamUIPlugin.instance().getResource(
				"SynchronizeActionGroup.CompareProperties"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS, comparePropsAction);
		MarkAsMergedAction markMergedAction = new MarkAsMergedAction(ModelBusTeamUIPlugin.instance().getResource(
				"SynchronizeActionGroup.MarkAsMerged"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_SYNC_CONFLICTS, markMergedAction);

		RevertAction revertAction = new RevertAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.Revert"), configuration);
		revertAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/revert.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_MANAGE_LOCALS, revertAction);
		ShowHistoryAction showHistoryAction = new ShowHistoryAction(ModelBusTeamUIPlugin.instance().getResource(
				"SynchronizeActionGroup.ShowResourceHistory"), configuration);
		showHistoryAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/showhistory.gif"));
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_MANAGE_LOCALS, showHistoryAction);
		AddToModelBusAction addToModelBusAction = new AddToModelBusAction(ModelBusTeamUIPlugin.instance().getResource(
				"UpdateActionGroup.AddToVersionControl"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_MANAGE_LOCALS, addToModelBusAction);
		AddToModelBusIgnoreAction addToModelBusIgnoreAction = new AddToModelBusIgnoreAction(ModelBusTeamUIPlugin.instance().getResource(
				"UpdateActionGroup.AddToIgnore"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_MANAGE_LOCALS, addToModelBusIgnoreAction);
		ExtractToAction extractTo = new ExtractToAction(ModelBusTeamUIPlugin.instance().getResource("ExtractAllToAction.Label"), configuration);
		this.appendToGroup(ISynchronizePageConfiguration.P_CONTEXT_MENU, UpdateActionGroup.GROUP_MANAGE_LOCALS, extractTo);

		this.addSpecificActions(extractTo, configuration);
	}

	protected void addLocalActions(IMenuManager manager, ISynchronizePageConfiguration configuration) {
		ShowOutgoingPropertiesAction showPropertiesAction = new ShowOutgoingPropertiesAction(ModelBusTeamUIPlugin.instance().getResource(
				"ShowPropertiesAction.label"), configuration);
		showPropertiesAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/propertiesedit.gif"));
		manager.add(showPropertiesAction);
		SetPropertyAction setPropAction = new SetPropertyAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.SetProperty"),
				configuration);
		manager.add(setPropAction);
		SetKeywordsAction setKeywordsAction = new SetKeywordsAction(
				ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.SetKeywords"), configuration);
		manager.add(setKeywordsAction);
		ShowOutgoingAnnotationAction showAnnotationAction = new ShowOutgoingAnnotationAction(ModelBusTeamUIPlugin.instance().getResource(
				"ShowAnnotationCommand.label"), configuration);
		manager.add(showAnnotationAction);

		manager.add(new Separator());

		LockAction lockAction = new LockAction(ModelBusTeamUIPlugin.instance().getResource("UpdateActionGroup.Lock"), configuration);
		lockAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/lock.gif"));
		manager.add(lockAction);
		UnlockAction unlockAction = new UnlockAction(ModelBusTeamUIPlugin.instance().getResource("UpdateActionGroup.Unlock"), configuration);
		unlockAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/unlock.gif"));
		manager.add(unlockAction);

		manager.add(new Separator());

		CreatePatchFileAction patchAction = new CreatePatchFileAction(ModelBusTeamUIPlugin.instance().getResource("CreatePatchCommand.label"),
				configuration);
		manager.add(patchAction);
		CreateBranchAction branchAction = new CreateBranchAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.Branch"),
				configuration);
		branchAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/actions/branch.gif"));
		manager.add(branchAction);
		ExtractOutgoingToAction extractActionOutgoing = new ExtractOutgoingToAction(ModelBusTeamUIPlugin.instance().getResource(
				"ExtractToAction.Label"), configuration);
		manager.add(extractActionOutgoing);

		manager.add(new Separator());

		CleanUpAction cleanUpAction = new CleanUpAction(ModelBusTeamUIPlugin.instance().getResource("SynchronizeActionGroup.Cleanup"), configuration);
		manager.add(cleanUpAction);
	}

	protected void addRemoteActions(IMenuManager manager, ISynchronizePageConfiguration configuration) {
		ShowIncomingPropertiesAction showIncomingPropertiesAction = new ShowIncomingPropertiesAction(ModelBusTeamUIPlugin.instance().getResource(
				"ShowPropertiesAction.label"), configuration);
		showIncomingPropertiesAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/propertiesedit.gif"));
		manager.add(showIncomingPropertiesAction);
		ShowIncomingAnnotationAction showIncomingAnnotationAction = new ShowIncomingAnnotationAction(ModelBusTeamUIPlugin.instance().getResource(
				"ShowAnnotationAction.label"), configuration);
		manager.add(showIncomingAnnotationAction);

		manager.add(new Separator());

		ExtractIncomingToAction extractActionIncoming = new ExtractIncomingToAction(ModelBusTeamUIPlugin.instance().getResource(
				"ExtractToAction.Label"), configuration);
		manager.add(extractActionIncoming);
	}

}
