/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.history.model;

import org.eclipse.jface.resource.ImageDescriptor;
import org.modelbus.team.eclipse.core.connector.ModelBusLogEntry;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.history.data.HistoryCategory;
import org.modelbus.team.eclipse.ui.history.data.ModelBusLocalFileRevision;
import org.modelbus.team.eclipse.ui.history.data.RootHistoryCategory;

/**
 * History View category UI node
 * 
 * @author Alexander Gurov
 */
public class CategoryLogNode extends AbstractLogNode {
	protected HistoryCategory	category;

	public CategoryLogNode(RootHistoryCategory category) {
		this(category, null);
	}

	public CategoryLogNode(HistoryCategory category, ILogNode parent) {
		super(parent);
		this.category = category;
	}

	public Object getAdapter(Class adapter) {
		if (adapter.equals(HistoryCategory.class)) {
			return this.category;
		}
		return null;
	}

	public ILogNode[] getChildren() {
		Object[] entries = this.category.getEntries();
		ILogNode[] children = new ILogNode[entries.length];
		for (int i = 0; i < entries.length; i++) {
			if (entries[i] instanceof ModelBusLogEntry) {
				children[i] = new ModelBusLogNode((ModelBusLogEntry) entries[i], this);
			} else if (entries[i] instanceof ModelBusLocalFileRevision) {
				children[i] = new LocalLogNode((ModelBusLocalFileRevision) entries[i], this);
			} else if (entries[i] instanceof HistoryCategory) {
				children[i] = new CategoryLogNode((HistoryCategory) entries[i], this);
			} else if (entries[i] instanceof String) {
				children[i] = new PlainTextLogNode((String) entries[i], this);
			}
		}
		return children;
	}

	public Object getEntity() {
		return this.category;
	}

	public boolean requiresBoldFont(ModelBusRevision currentRevision) {
		return true;
	}

	public ImageDescriptor getImageDescriptor() {
		return ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/history/group_by_date.gif");
	}

	public String getLabel(int columnIndex, int labelType, ModelBusRevision currentRevision) {
		if (columnIndex == ILogNode.COLUMN_REVISION) {
			return this.category.getName();
		}
		return "";
	}

	public int getType() {
		return ILogNode.TYPE_CATEGORY;
	}

	public boolean hasChildren() {
		return true;
	}

	public String getAuthor() {
		return "";
	}

	public int getChangesCount() {
		return 0;
	}

	public String getComment() {
		return "";
	}

	public ModelBusRevision getRevision() {
		return ModelBusRevision.INVALID_REVISION_DESCRIPTOR;
	}

	public long getTimeStamp() {
		return 0;
	}

	public int hashCode() {
		return this.category.hashCode();
	}

	public boolean equals(Object obj) {
		if (obj instanceof CategoryLogNode) {
			return this.category.equals(((CategoryLogNode) obj).category);
		}
		return false;
	}

}
