/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.operation;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IWorkbenchPage;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.operation.IUnprotectedOperation;
import org.modelbus.team.eclipse.core.operation.local.AbstractWorkingCopyOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.annotate.BuiltInAnnotate;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * The operation shows annotation for the local resource.
 * 
 * @author Alexander Gurov
 */
public class LocalShowAnnotationOperation extends AbstractWorkingCopyOperation {
	protected ModelBusRevision	revision;

	public LocalShowAnnotationOperation(IResource resource) {
		this(resource, null);
	}

	public LocalShowAnnotationOperation(IResource resource, ModelBusRevision revision) {
		super("Operation.ShowAnnotation", new IResource[] { resource });
		this.revision = revision;
	}

	protected void runImpl(IProgressMonitor monitor) throws Exception {
		final IResource resource = this.operableData()[0];
		ILocalResource local = ModelBusRemoteStorage.instance().asLocalResourceAccessible(resource);
		boolean notExists = IStateFilter.SF_NOTEXISTS.accept(local);
		final ModelBusRevision revision = this.revision != null ? this.revision : (notExists
				|| local.getRevision() == ModelBusRevision.INVALID_REVISION_DESCRIPTOR ? ModelBusRevision.HEAD : local
				.getRevision());

		final IRepositoryResource remote = ModelBusRemoteStorage.instance().asRepositoryResource(resource);
		remote.setSelectedRevision(revision);
		final CorrectRevisionOperation correctOp = new CorrectRevisionOperation(null, remote, local.getRevision(), resource);

		this.protectStep(new IUnprotectedOperation() {
			public void run(IProgressMonitor monitor) throws Exception {
				correctOp.run(monitor);
			}
		}, monitor, 1);

		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = UIMonitorUtility.getActivePage();
				if (page != null) {
					new BuiltInAnnotate().open(page, remote, (IFile) resource);
				}
			}
		});
	}

}
