/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.local;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.modelbusstorage.ResourcesParentsProvider;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.local.AddToModelBusIgnoreOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.refactor.DeleteResourceOperation;
import org.modelbus.team.eclipse.core.resource.IRemoteStorage;
import org.modelbus.team.eclipse.core.resource.events.IResourceStatesListener;
import org.modelbus.team.eclipse.core.resource.events.ResourceStatesChangedEvent;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.dialog.DiscardConfirmationDialog;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Add resources panel implementation
 * 
 * @author Alexander Gurov
 */
public class AddToModelBusPanel extends AbstractResourceSelectionPanel {

	protected boolean					actionTookEffect;
	protected IResourceStatesListener	resourceStatesListener;

	public AddToModelBusPanel(IResource[] resources) {
		this(resources, null);
	}

	public AddToModelBusPanel(IResource[] resources, IResource[] userSelectedResources) {
		super(resources, userSelectedResources, new String[] { IDialogConstants.OK_LABEL, IDialogConstants.CANCEL_LABEL });
		this.actionTookEffect = false;
		this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Title");
		this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Description");
		if (resources.length == 1) {
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Message.Single");
		} else {
			this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Message.Multi",
					new String[] { String.valueOf(resources.length) });
		}
	}

	protected void addContextMenu() {
		final TableViewer tableViewer = this.selectionComposite.getTableViewer();
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(tableViewer.getTable());
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				final IStructuredSelection tSelection = (IStructuredSelection) tableViewer.getSelection();
				final IResource[] selectedResources = (IResource[]) tSelection.toList().toArray(new IResource[tSelection.size()]);
				Action tAction = null;
				if (selectedResources.length == 1) {
					manager.add(tAction = new Action(MessageFormat.format(
							ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Ignore.Single"), selectedResources[0].getName())) {
						public void run() {
							CompositeOperation op = new CompositeOperation("AddToIgnore");
							op.add(new AddToModelBusIgnoreOperation(selectedResources, IRemoteStorage.IGNORE_NAME, null));
							op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(selectedResources), IResource.DEPTH_INFINITE,
									RefreshResourcesOperation.REFRESH_ALL));
							UIMonitorUtility.doTaskNowDefault(op, true);
						}
					});
					tAction.setEnabled(true);
					String name = selectedResources[0].getName();
					String[] parts = name.split("\\.");
					if ((parts.length != 0)) {
						manager.add(tAction = new Action(MessageFormat.format(
								ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Ignore.Single"), "*." + parts[parts.length - 1])) {
							public void run() {
								CompositeOperation op = new CompositeOperation("AddToIgnore");
								op.add(new AddToModelBusIgnoreOperation(selectedResources, IRemoteStorage.IGNORE_EXTENSION, null));
								op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(selectedResources), IResource.DEPTH_INFINITE,
										RefreshResourcesOperation.REFRESH_ALL));
								UIMonitorUtility.doTaskNowDefault(op, true);
							}
						});
						tAction.setEnabled(true);
					}
				} else {
					manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.IgnoreByNames.Multiple")) {
						public void run() {
							CompositeOperation op = new CompositeOperation("AddToIgnore");
							op.add(new AddToModelBusIgnoreOperation(selectedResources, IRemoteStorage.IGNORE_NAME, null));
							op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(selectedResources), IResource.DEPTH_INFINITE,
									RefreshResourcesOperation.REFRESH_ALL));
							UIMonitorUtility.doTaskNowDefault(op, true);
						}
					});
					tAction.setEnabled(tSelection.size() > 0);
					manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.IgnoreByExtension.Multiple")) {
						public void run() {
							CompositeOperation op = new CompositeOperation("AddToIgnore");
							op.add(new AddToModelBusIgnoreOperation(selectedResources, IRemoteStorage.IGNORE_EXTENSION, null));
							op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(selectedResources), IResource.DEPTH_INFINITE,
									RefreshResourcesOperation.REFRESH_ALL));
							UIMonitorUtility.doTaskNowDefault(op, true);
						}
					});
					tAction.setEnabled(tSelection.size() > 0);
				}
				manager.add(new Separator());
				manager.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("AddToModelBusPanel.Delete.Action.Lable")) {
					public void run() {
						DiscardConfirmationDialog dialog = new DiscardConfirmationDialog(UIMonitorUtility.getShell(), selectedResources.length == 1,
								DiscardConfirmationDialog.MSG_RESOURCE);
						if (dialog.open() == 0) {
							DeleteResourceOperation deleteOperation = new DeleteResourceOperation(selectedResources);
							CompositeOperation op = new CompositeOperation(deleteOperation.getId());
							SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(selectedResources);
							RestoreProjectMetaOperation restoreOp = new RestoreProjectMetaOperation(saveOp);
							op.add(saveOp);
							op.add(deleteOperation);
							op.add(restoreOp);
							op.add(new RefreshResourcesOperation(new ResourcesParentsProvider(selectedResources), IResource.DEPTH_INFINITE,
									RefreshResourcesOperation.REFRESH_ALL));
							UIMonitorUtility.doTaskNowDefault(op, true);
						}
					}
				});
				tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/delete.gif"));
				tAction.setEnabled(tSelection.size() > 0);
			}
		});
		menuMgr.setRemoveAllWhenShown(true);
		tableViewer.getTable().setMenu(menu);
	}

	public void postInit() {
		super.postInit();
		this.resourceStatesListener = new IResourceStatesListener() {
			public void resourcesStateChanged(ResourceStatesChangedEvent event) {
				AddToModelBusPanel.this.updateResources();
			}
		};
		ModelBusRemoteStorage.instance().addResourceStatesListener(ResourceStatesChangedEvent.class, AddToModelBusPanel.this.resourceStatesListener);
	}

	public boolean ifActionTookEffect() {
		return this.actionTookEffect;
	}

	public void updateResources() {
		final TableViewer tableViewer = this.selectionComposite.getTableViewer();
		HashSet<IResource> toDeleteSet = new HashSet<IResource>();
		toDeleteSet.addAll(Arrays.asList(this.resources));
		HashSet<IResource> newResourcesSet = new HashSet<IResource>();
		newResourcesSet.addAll(Arrays.asList(FileUtility.getResourcesRecursive(this.resources, IStateFilter.SF_UNVERSIONED, IResource.DEPTH_ZERO)));
		List<IResource> ignored = Arrays.asList(FileUtility.getResourcesRecursive(this.resources, IStateFilter.SF_IGNORED, IResource.DEPTH_ZERO));
		if (ignored.size() != 0) {
			this.actionTookEffect = true;
		}
		newResourcesSet.removeAll(Arrays.asList(FileUtility.getResourcesRecursive(this.resources, IStateFilter.SF_IGNORED, IResource.DEPTH_ZERO)));
		for (int i = 0; i < this.resources.length; i++) {
			if (!this.resources[i].exists()) {
				newResourcesSet.remove(this.resources[i]);
				this.actionTookEffect = true;
			}
		}
		final IResource[] newResources = newResourcesSet.toArray(new IResource[newResourcesSet.size()]);
		toDeleteSet.removeAll(newResourcesSet);
		final IResource[] toDeleteResources = toDeleteSet.toArray(new IResource[toDeleteSet.size()]);
		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				AddToModelBusPanel.this.selectionComposite.setResources(newResources);
				if (!tableViewer.getTable().isDisposed()) {
					tableViewer.remove(toDeleteResources);
					tableViewer.refresh();
					AddToModelBusPanel.this.selectionComposite.fireSelectionChanged();
				}
			}
		});
		this.resources = newResources;
	}

	public void dispose() {
		super.dispose();
		ModelBusRemoteStorage.instance().removeResourceStatesListener(ResourceStatesChangedEvent.class, this.resourceStatesListener);
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.addToVCDialogContext";
	}

}
