/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.local;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector.Depth;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.DepthSelectionComposite;
import org.modelbus.team.eclipse.ui.composite.RepositoryResourceSelectionComposite;
import org.modelbus.team.eclipse.ui.panel.common.AbstractRepositoryResourceSelectionPanel;

/**
 * Switch panel implementation
 * 
 * @author Alexander Gurov
 */
public class SwitchPanel extends AbstractRepositoryResourceSelectionPanel {

	protected boolean					containFolders;
	protected DepthSelectionComposite	depthSelector;

	public SwitchPanel(IRepositoryResource baseResource, ModelBusRevision currentRevision, boolean containFolders) {
		super(baseResource, currentRevision, ModelBusTeamUIPlugin.instance().getResource("SwitchPanel.Title"), ModelBusTeamUIPlugin.instance()
				.getResource("SwitchPanel.Description"), "SwitchPanel.URL_HISTORY_NAME", ModelBusTeamUIPlugin.instance().getResource(
				"SwitchPanel.Selection.Title"), ModelBusTeamUIPlugin.instance().getResource("SwitchPanel.Selection.Description"),
				RepositoryResourceSelectionComposite.TEXT_NONE);
		this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("SwitchPanel.Message");
		this.containFolders = containFolders;
	}

	public void createControlsImpl(Composite parent) {
		super.createControlsImpl(parent);
		if (this.containFolders) {
			Label separator = new Label(parent, SWT.HORIZONTAL | SWT.SEPARATOR);
			separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			separator.setVisible(false);

			this.depthSelector = new DepthSelectionComposite(parent, SWT.NONE);
			this.depthSelector.addAndSelectWorkingCopyDepth();
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			this.depthSelector.setLayoutData(data);
		}
	}

	public int getDepth() {
		if (this.depthSelector == null) {
			return Depth.INFINITY;
		}
		return this.depthSelector.getDepth();
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.switchDialogContext";
	}

}
