/*******************************************************************************
 * Copyright (c) 2005-2006 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.panel.view;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.modelbus.team.eclipse.core.connector.IModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.connector.ModelBusRevision;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.RevisionComposite;
import org.modelbus.team.eclipse.ui.panel.AbstractDialogPanel;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Allows to define the history fetch range
 * 
 * @author Alexander Gurov
 */
public class HistoryRangePanel extends AbstractDialogPanel {
	protected IRepositoryResource	resource;
	protected IRepositoryResource	initStartResource;
	protected IRepositoryResource	initStopResource;

	protected RevisionComposite		startComposite;
	protected RevisionComposite		stopComposite;

	protected boolean				reversed;

	public HistoryRangePanel(IRepositoryResource resource, ModelBusRevision initStartRevision, ModelBusRevision initStopRevision) {
		this.dialogTitle = ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.Title");
		this.dialogDescription = ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.Description");
		this.defaultMessage = ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.Message");

		this.resource = resource;
		this.initStartResource = ModelBusUtility.copyOf(resource);
		this.initStartResource.setSelectedRevision(initStartRevision);
		this.initStopResource = ModelBusUtility.copyOf(resource);
		this.initStopResource.setSelectedRevision(initStopRevision);
	}

	protected Point getPrefferedSizeImpl() {
		return new Point(715, SWT.DEFAULT);
	}

	public String getHelpId() {
		return "org.modelbus.team.eclipse.help.historyRangeDialogContext";
	}

	public ModelBusRevision getStartRevision() {
		return (this.reversed ? this.stopComposite : this.startComposite).getSelectedRevision();
	}

	public ModelBusRevision getStopRevision() {
		return (this.reversed ? this.startComposite : this.stopComposite).getSelectedRevision();
	}

	protected void createControlsImpl(Composite parent) {
		GridLayout layout = null;
		GridData data = null;

		Composite cmp = new Composite(parent, SWT.NONE);
		layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = 0;
		cmp.setLayout(layout);
		data = new GridData(GridData.FILL_BOTH);
		cmp.setLayoutData(data);

		String defaultRevision = ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.Default");
		this.startComposite = new RevisionComposite(cmp, this, true, new String[] {
				ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.StartRevision"), defaultRevision }, null, false);
		this.startComposite.setBaseResource(this.resource);
		this.startComposite.setSelectedResource(this.initStartResource);
		this.stopComposite = new RevisionComposite(cmp, this, true, new String[] {
				ModelBusTeamUIPlugin.instance().getResource("HistoryRangePanel.StopRevision"), defaultRevision }, null, false);
		this.stopComposite.setBaseResource(this.resource);
		this.stopComposite.setSelectedResource(this.initStopResource);
	}

	protected void cancelChangesImpl() {

	}

	protected void saveChangesImpl() {
		if (this.getStartRevision() == null || this.getStopRevision() == null) {
			return;
		}
		this.initStartResource.setSelectedRevision(this.getStartRevision());
		this.initStopResource.setSelectedRevision(this.getStopRevision());

		UIMonitorUtility.doTaskNowDefault(new AbstractActionOperation("Operation.CheckRevisions") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				IModelBusConnector proxy = ModelBusConnector.getModelBusConnector();
				// HistoryRangePanel.this.initStartResource.getRepositoryLocation().acquireModelBusProxy();
				try {
					HistoryRangePanel.this.reversed = ModelBusUtility.compareRevisions(
							HistoryRangePanel.this.initStartResource.getSelectedRevision(),
							HistoryRangePanel.this.initStopResource.getSelectedRevision(),
							ModelBusUtility.getEntryRevisionReference(HistoryRangePanel.this.initStartResource),
							ModelBusUtility.getEntryRevisionReference(HistoryRangePanel.this.initStopResource), proxy) == -1;
				} finally {
					// HistoryRangePanel.this.initStartResource.getRepositoryLocation().releaseModelBusProxy(proxy);
				}
			}
		}, false);
	}

}
