/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.preferences;

import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.verifier.AbstractVerificationKeyListener;
import org.modelbus.team.eclipse.ui.verifier.AbstractVerifier;
import org.modelbus.team.eclipse.ui.verifier.IValidationManager;

/**
 * Abstract preferences page provides validation support.
 * 
 * @author Alexander Gurov
 */
public abstract class AbstractModelBusTeamPreferencesPage extends PreferencePage implements IWorkbenchPreferencePage, IValidationManager {
	private VerificationKeyListener	changeListener;

	public AbstractModelBusTeamPreferencesPage() {
		super();
		this.changeListener = new VerificationKeyListener();
	}

	public void init(IWorkbench workbench) {

	}

	public boolean performOk() {
		this.saveValues(this.getPreferenceStore());

		ModelBusTeamUIPlugin.instance().savePluginPreferences();

		return true;
	}

	protected void performDefaults() {
		super.performDefaults();
		this.loadDefaultValues(this.getPreferenceStore());
		this.initializeControls();

		this.validateContent();
	}

	protected Control createContents(Composite parent) {
		Control retVal = this.createContentsImpl(parent);

		this.loadValues(this.getPreferenceStore());
		this.initializeControls();

		this.addListeners();

		return retVal;
	}

	protected IPreferenceStore doGetPreferenceStore() {
		return ModelBusTeamUIPlugin.instance().getPreferenceStore();
	}

	protected abstract void loadDefaultValues(IPreferenceStore store);

	protected abstract void loadValues(IPreferenceStore store);

	protected abstract void saveValues(IPreferenceStore store);

	protected abstract void initializeControls();

	protected abstract Control createContentsImpl(Composite parent);

	public boolean isFilledRight() {
		return this.changeListener.isFilledRight();
	}

	public void attachTo(Control cmp, AbstractVerifier verifier) {
		this.changeListener.attachTo(cmp, verifier);
	}

	public void addListeners() {
		this.changeListener.addListeners();
		this.validateContent();
		this.setMessage(this.getTitle(), IMessageProvider.NONE);
	}

	public void detachFrom(Control cmp) {
		this.changeListener.detachFrom(cmp);
	}

	public void detachAll() {
		this.changeListener.detachAll();
	}

	public void validateContent() {
		this.changeListener.validateContent();
	}

	public boolean validateControl(Control cmp) {
		return this.changeListener.validateControl(cmp);
	}

	protected class VerificationKeyListener extends AbstractVerificationKeyListener {
		public VerificationKeyListener() {
			super();
		}

		public void hasError(String errorReason) {
			AbstractModelBusTeamPreferencesPage.this.setMessage(errorReason, IMessageProvider.ERROR);
			this.handleButtons();
		}

		public void hasWarning(String warningReason) {
			AbstractModelBusTeamPreferencesPage.this.setMessage(warningReason, IMessageProvider.WARNING);
			this.handleButtons();
		}

		public void hasNoError() {
			AbstractModelBusTeamPreferencesPage.this.setMessage(AbstractModelBusTeamPreferencesPage.this.getTitle(), IMessageProvider.NONE);
			this.handleButtons();
		}

		protected void handleButtons() {
			AbstractModelBusTeamPreferencesPage.this.setValid(this.isFilledRight());
		}

	}

}
