/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.preferences;

import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.verifier.CompositeVerifier;
import org.modelbus.team.eclipse.ui.verifier.IntegerFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.NonEmptyFieldVerifier;

/**
 * Console preferences page
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamConsolePreferencesPage extends AbstractModelBusTeamPreferencesPage {
	protected ColorFieldEditor	cmdEditor;
	protected ColorFieldEditor	okEditor;
	protected ColorFieldEditor	wrnEditor;
	protected ColorFieldEditor	errEditor;

	protected boolean			enabled;
	protected boolean			hyperlinksEnabled;
	protected int				autoshow;
	protected boolean			wrapEnabled;
	protected boolean			limitEnabled;
	protected int				wrapWidth;
	protected int				limitValue;

	protected Button			enabledButton;
	protected Button			hyperlinksEnabledButton;
	protected Button			showNeverButton;
	protected Button			showAlwaysButton;
	protected Button			showErrorButton;
	protected Button			showWarningErrorButton;
	protected Button			wrapEnabledButton;
	protected Button			limitEnabledButton;
	protected Text				wrapWidthText;
	protected Text				limitValueText;

	public ModelBusTeamConsolePreferencesPage() {
		super();
	}

	protected void saveValues(IPreferenceStore store) {
		this.cmdEditor.store();
		this.okEditor.store();
		this.wrnEditor.store();
		this.errEditor.store();

		ModelBusTeamPreferences.setConsoleInt(store, ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NAME, this.autoshow);

		ModelBusTeamPreferences.setConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_ENABLED_NAME, this.enabled);
		ModelBusTeamPreferences.setConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_NAME, this.hyperlinksEnabled);

		ModelBusTeamPreferences.setConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_NAME, this.wrapEnabled);
		ModelBusTeamPreferences.setConsoleInt(store, ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_NAME, this.wrapWidth);

		ModelBusTeamPreferences.setConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_NAME, this.limitEnabled);
		ModelBusTeamPreferences.setConsoleInt(store, ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_NAME, this.limitValue);
	}

	protected void loadDefaultValues(IPreferenceStore store) {
		this.autoshow = ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_DEFAULT;
		this.enabled = ModelBusTeamPreferences.CONSOLE_ENABLED_DEFAULT;
		this.hyperlinksEnabled = ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_DEFAULT;
		this.wrapEnabled = ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_DEFAULT;
		this.wrapWidth = ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_DEFAULT;
		this.limitEnabled = ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_DEFAULT;
		this.limitValue = ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_DEFAULT;

		this.cmdEditor.loadDefault();
		this.okEditor.loadDefault();
		this.wrnEditor.loadDefault();
		this.errEditor.loadDefault();
	}

	protected void loadValues(IPreferenceStore store) {
		this.autoshow = ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NAME);
		this.enabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_ENABLED_NAME);
		this.hyperlinksEnabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_HYPERLINKS_ENABLED_NAME);
		this.wrapEnabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_WRAP_ENABLED_NAME);
		this.wrapWidth = ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_WRAP_WIDTH_NAME);
		this.limitEnabled = ModelBusTeamPreferences.getConsoleBoolean(store, ModelBusTeamPreferences.CONSOLE_LIMIT_ENABLED_NAME);
		this.limitValue = ModelBusTeamPreferences.getConsoleInt(store, ModelBusTeamPreferences.CONSOLE_LIMIT_VALUE_NAME);

		this.cmdEditor.load();
		this.okEditor.load();
		this.wrnEditor.load();
		this.errEditor.load();
	}

	protected void initializeControls() {
		this.showNeverButton.setSelection(false);
		this.showAlwaysButton.setSelection(false);
		this.showErrorButton.setSelection(false);
		if (this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NEVER) {
			this.showNeverButton.setSelection(true);
		} else if (this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ALWAYS) {
			this.showAlwaysButton.setSelection(true);
		} else if (this.autoshow == ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ERROR) {
			this.showErrorButton.setSelection(true);
		} else {
			this.showWarningErrorButton.setSelection(true);
		}

		this.enabledButton.setSelection(this.enabled);
		this.hyperlinksEnabledButton.setSelection(this.hyperlinksEnabled);

		this.wrapEnabledButton.setSelection(this.wrapEnabled);
		this.wrapWidthText.setEnabled(this.wrapEnabled);
		this.wrapWidthText.setText(String.valueOf(this.wrapWidth));

		this.limitEnabledButton.setSelection(this.limitEnabled);
		this.limitValueText.setEnabled(this.limitEnabled);
		this.limitValueText.setText(String.valueOf(this.limitValue));
	}

	protected Control createContentsImpl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);
		GridLayout layout = new GridLayout();
		layout.marginHeight = layout.marginWidth = 0;
		layout.numColumns = 2;
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.grabExcessVerticalSpace = false;
		composite.setLayoutData(data);

		this.enabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		data.horizontalSpan = 2;
		this.enabledButton.setLayoutData(data);
		this.enabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textIsEnabled"));
		this.enabledButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamConsolePreferencesPage.this.enabled = ModelBusTeamConsolePreferencesPage.this.enabledButton.getSelection();
			}
		});

		this.hyperlinksEnabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		data.horizontalSpan = 2;
		this.hyperlinksEnabledButton.setLayoutData(data);
		this.hyperlinksEnabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.hyperlinksIsEnabled"));
		this.hyperlinksEnabledButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamConsolePreferencesPage.this.hyperlinksEnabled = ModelBusTeamConsolePreferencesPage.this.hyperlinksEnabledButton
						.getSelection();
			}
		});

		Group showType = new Group(composite, SWT.FILL);
		showType.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textShowOnGroup"));
		layout = new GridLayout();
		layout.numColumns = 4;
		layout.horizontalSpacing = 10;
		showType.setLayout(layout);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		data.grabExcessVerticalSpace = false;
		showType.setLayoutData(data);

		this.showNeverButton = new Button(showType, SWT.RADIO);
		data = new GridData();
		this.showNeverButton.setLayoutData(data);
		this.showNeverButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textShowNever"));
		this.showNeverButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (ModelBusTeamConsolePreferencesPage.this.showNeverButton.getSelection()) {
					ModelBusTeamConsolePreferencesPage.this.autoshow = ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_NEVER;
				}
			}
		});

		this.showAlwaysButton = new Button(showType, SWT.RADIO);
		data = new GridData();
		this.showAlwaysButton.setLayoutData(data);
		this.showAlwaysButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textShowAlways"));
		this.showAlwaysButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (ModelBusTeamConsolePreferencesPage.this.showAlwaysButton.getSelection()) {
					ModelBusTeamConsolePreferencesPage.this.autoshow = ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ALWAYS;
				}
			}
		});

		this.showErrorButton = new Button(showType, SWT.RADIO);
		data = new GridData();
		this.showErrorButton.setLayoutData(data);
		this.showErrorButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textShowError"));
		this.showErrorButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (ModelBusTeamConsolePreferencesPage.this.showErrorButton.getSelection()) {
					ModelBusTeamConsolePreferencesPage.this.autoshow = ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_ERROR;
				}
			}
		});

		this.showWarningErrorButton = new Button(showType, SWT.RADIO);
		data = new GridData();
		this.showWarningErrorButton.setLayoutData(data);
		this.showWarningErrorButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textShowWarningError"));
		this.showWarningErrorButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (ModelBusTeamConsolePreferencesPage.this.showWarningErrorButton.getSelection()) {
					ModelBusTeamConsolePreferencesPage.this.autoshow = ModelBusTeamPreferences.CONSOLE_AUTOSHOW_TYPE_WARNING_ERROR;
				}
			}
		});

		Label separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		separator.setLayoutData(data);
		separator.setVisible(false);

		this.wrapEnabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		data.horizontalSpan = 2;
		this.wrapEnabledButton.setLayoutData(data);
		this.wrapEnabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textWrapEnabled"));
		this.wrapEnabledButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamConsolePreferencesPage.this.wrapEnabled = ModelBusTeamConsolePreferencesPage.this.wrapEnabledButton.getSelection();
				ModelBusTeamConsolePreferencesPage.this.wrapWidthText.setEnabled(ModelBusTeamConsolePreferencesPage.this.wrapEnabled);
			}
		});

		Label label = new Label(composite, SWT.NULL);
		data = new GridData();
		label.setLayoutData(data);
		String labelText = ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textWrapWidth");
		label.setText(labelText);

		this.wrapWidthText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.wrapWidthText.setLayoutData(data);
		CompositeVerifier verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new IntegerFieldVerifier(labelText, true));
		this.attachTo(this.wrapWidthText, verifier);
		this.wrapWidthText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					ModelBusTeamConsolePreferencesPage.this.wrapWidth = Integer.parseInt(ModelBusTeamConsolePreferencesPage.this.wrapWidthText
							.getText());
				} catch (Exception ex) {
				}
			}
		});

		this.limitEnabledButton = new Button(composite, SWT.CHECK);
		data = new GridData();
		data.horizontalSpan = 2;
		this.limitEnabledButton.setLayoutData(data);
		this.limitEnabledButton.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textLimitEnabled"));
		this.limitEnabledButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ModelBusTeamConsolePreferencesPage.this.limitEnabled = ModelBusTeamConsolePreferencesPage.this.limitEnabledButton.getSelection();
				ModelBusTeamConsolePreferencesPage.this.limitValueText.setEnabled(ModelBusTeamConsolePreferencesPage.this.limitEnabled);
			}
		});

		label = new Label(composite, SWT.NULL);
		data = new GridData();
		label.setLayoutData(data);
		labelText = ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textLimitValue");
		label.setText(labelText);

		this.limitValueText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.grabExcessHorizontalSpace = true;
		this.limitValueText.setLayoutData(data);
		verifier = new CompositeVerifier();
		verifier.add(new NonEmptyFieldVerifier(labelText));
		verifier.add(new IntegerFieldVerifier(labelText, true));
		this.attachTo(this.limitValueText, verifier);
		this.limitValueText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					ModelBusTeamConsolePreferencesPage.this.limitValue = Integer.parseInt(ModelBusTeamConsolePreferencesPage.this.limitValueText
							.getText());
				} catch (Exception ex) {
				}
			}
		});

		separator = new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		separator.setLayoutData(data);
		separator.setVisible(false);

		label = new Label(composite, SWT.NULL);
		data = new GridData();
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		label.setText(ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textColorsGroup"));

		this.cmdEditor = new ColorFieldEditor(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_CMD_COLOR_NAME),
				ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textCmdMessage"), composite);
		this.cmdEditor.setPage(this);
		this.cmdEditor.setPreferenceStore(this.getPreferenceStore());

		this.okEditor = new ColorFieldEditor(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_OK_COLOR_NAME),
				ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textOkMessage"), composite);
		this.okEditor.setPage(this);
		this.okEditor.setPreferenceStore(this.getPreferenceStore());

		this.wrnEditor = new ColorFieldEditor(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_WRN_COLOR_NAME),
				ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textWrnMessage"), composite);
		this.wrnEditor.setPage(this);
		this.wrnEditor.setPreferenceStore(this.getPreferenceStore());

		this.errEditor = new ColorFieldEditor(ModelBusTeamPreferences.fullConsoleName(ModelBusTeamPreferences.CONSOLE_ERR_COLOR_NAME),
				ModelBusTeamUIPlugin.instance().getResource("ConsolePreferencePage.textErrMessage"), composite);
		this.errEditor.setPage(this);
		this.errEditor.setPreferenceStore(this.getPreferenceStore());

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.consolePreferencesContext");

		return composite;
	}

}
