/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.preferences;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

/**
 * Performance options
 * 
 * @author Alexander Gurov
 */
public class ModelBusTeamPerformancePage extends AbstractModelBusTeamPreferencesPage {
	protected Button	computeDeepButton;
	protected Button	preciseEnablementsButton;
	protected Button	enableCacheButton;

	protected boolean	computeDeep;
	protected boolean	preciseEnablements;
	protected boolean	enableCache;

	public ModelBusTeamPerformancePage() {
		super();
	}

	public void init(IWorkbench workbench) {
		setDescription(ModelBusTeamUIPlugin.instance().getResource("PerformancePreferencePage.optionsDesc"));
	}

	protected void saveValues(IPreferenceStore store) {
		ModelBusTeamPreferences.setDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_NAME, this.computeDeep);
		ModelBusTeamPreferences.setDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_NAME, this.preciseEnablements);
		ModelBusTeamPreferences
				.setDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_NAME, this.computeDeep | this.enableCache);
	}

	protected void loadDefaultValues(IPreferenceStore store) {
		this.computeDeep = ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_DEFAULT;
		this.preciseEnablements = ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_DEFAULT;
		this.enableCache = ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_DEFAULT;
	}

	protected void loadValues(IPreferenceStore store) {
		this.computeDeep = ModelBusTeamPreferences.getDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_COMPUTE_DEEP_NAME);
		this.preciseEnablements = ModelBusTeamPreferences.getDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_PRECISE_ENABLEMENTS_NAME);
		this.enableCache = ModelBusTeamPreferences.getDecorationBoolean(store, ModelBusTeamPreferences.DECORATION_ENABLE_CACHE_NAME);
	}

	protected void initializeControls() {
		this.computeDeepButton.setSelection(this.computeDeep);
		this.preciseEnablementsButton.setSelection(this.preciseEnablements);
		this.enableCacheButton.setSelection(this.enableCache);
		if (this.computeDeep || this.preciseEnablements) {
			this.enableCacheButton.setEnabled(false);
		} else if (!this.enableCache) {
			this.computeDeepButton.setEnabled(false);
			this.preciseEnablementsButton.setEnabled(false);
		}
	}

	protected Control createContentsImpl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);
		GridLayout layout = new GridLayout();
		layout.marginHeight = layout.marginWidth = 0;
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.grabExcessVerticalSpace = false;
		composite.setLayoutData(data);

		Composite noteComposite = new Composite(composite, SWT.FILL);
		layout = new GridLayout();
		layout.marginWidth = 0;
		noteComposite.setLayout(layout);
		noteComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label separator = new Label(noteComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		data = new GridData(GridData.FILL_HORIZONTAL);
		separator.setLayoutData(data);

		this.computeDeepButton = new Button(composite, SWT.CHECK);
		this.computeDeepButton.setLayoutData(new GridData());
		this.computeDeepButton.setText(ModelBusTeamUIPlugin.instance().getResource("PerformancePreferencePage.computeDeep"));
		this.computeDeepButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamPerformancePage.this.computeDeep = ModelBusTeamPerformancePage.this.computeDeepButton.getSelection();
				ModelBusTeamPerformancePage.this.enableCacheButton
						.setEnabled(!(ModelBusTeamPerformancePage.this.computeDeep | ModelBusTeamPerformancePage.this.preciseEnablements));
			}
		});

		this.preciseEnablementsButton = new Button(composite, SWT.CHECK);
		this.preciseEnablementsButton.setLayoutData(new GridData());
		this.preciseEnablementsButton.setText(ModelBusTeamUIPlugin.instance().getResource("PerformancePreferencePage.preciseEnablements"));
		this.preciseEnablementsButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamPerformancePage.this.preciseEnablements = ModelBusTeamPerformancePage.this.preciseEnablementsButton.getSelection();
				ModelBusTeamPerformancePage.this.enableCacheButton
						.setEnabled(!(ModelBusTeamPerformancePage.this.computeDeep | ModelBusTeamPerformancePage.this.preciseEnablements));
			}
		});

		this.enableCacheButton = new Button(composite, SWT.CHECK);
		this.enableCacheButton.setLayoutData(new GridData());
		this.enableCacheButton.setText(ModelBusTeamUIPlugin.instance().getResource("PerformancePreferencePage.enableCache"));
		this.enableCacheButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				ModelBusTeamPerformancePage.this.enableCache = ModelBusTeamPerformancePage.this.enableCacheButton.getSelection();
				ModelBusTeamPerformancePage.this.computeDeepButton.setEnabled(ModelBusTeamPerformancePage.this.enableCache);
				ModelBusTeamPerformancePage.this.preciseEnablementsButton.setEnabled(ModelBusTeamPerformancePage.this.enableCache);
			}
		});

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.performancePreferencesContext");

		return composite;
	}

}
