/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.repository;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;
import org.modelbus.team.eclipse.core.modelbusstorage.ModelBusRemoteStorage;
import org.modelbus.team.eclipse.core.modelbusstorage.events.IRepositoriesStateChangedListener;
import org.modelbus.team.eclipse.core.modelbusstorage.events.RepositoriesStateChangedEvent;
import org.modelbus.team.eclipse.core.operation.AbstractActionOperation;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.action.AbstractModelBusTeamAction;
import org.modelbus.team.eclipse.ui.action.remote.CreateFolderAction;
import org.modelbus.team.eclipse.ui.action.remote.DeleteAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileAction;
import org.modelbus.team.eclipse.ui.action.remote.OpenFileWithAction;
import org.modelbus.team.eclipse.ui.action.remote.RefreshAction;
import org.modelbus.team.eclipse.ui.action.remote.management.CreateProjectStructureAction;
import org.modelbus.team.eclipse.ui.action.remote.management.DiscardRevisionLinksAction;
import org.modelbus.team.eclipse.ui.preferences.ModelBusTeamPreferences;
import org.modelbus.team.eclipse.ui.repository.browser.RepositoryBrowser;
import org.modelbus.team.eclipse.ui.repository.model.RepositoriesRoot;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryContentProvider;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryLabelProvider;
import org.modelbus.team.eclipse.ui.utility.UIMonitorUtility;

/**
 * Repository view implementation
 * 
 * @author Alexander Gurov
 */
public class RepositoriesView extends ViewPart {
	public static final String		VIEW_ID	= RepositoriesView.class.getName();

	protected RepositoryTreeViewer	repositoryTree;
	protected RepositoriesRoot		root;
	protected DrillDownAdapter		ddAdapter;
	protected Action				showBrowserAction;
	protected IPartListener2		partListener;

	public RepositoriesView() {
		super();
	}

	public static MenuManager newMenuInstance(final ISelectionProvider provider) {
		MenuManager menuMgr = new MenuManager();
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager manager) {
				MenuManager sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.New"), "addMenu");
				sub.add(new Separator("mainGroup"));
				sub.add(new Separator("managementGroup"));
				sub.add(new Separator("repositoryGroup"));
				sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

				// Action newRepositoryLocation = new Action(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.RepositoryLocation")) {
				// public void run() {
				// new NewRepositoryLocationAction().run(this);
				// }
				// };
				// newRepositoryLocation.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/objects/repository.gif"));
				// sub.add(newRepositoryLocation);
				manager.add(sub);

				manager.add(new Separator("checkoutGroup"));

				sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.OpenWith"), "openWithMenu");
				sub.add(new Separator("dynamicGroup"));
				IStructuredSelection selection = (IStructuredSelection) provider.getSelection();
				if (selection.size() == 1) {
					Object item = selection.getFirstElement();
					if (item instanceof RepositoryFile) {
						String name = ((RepositoryFile) item).getRepositoryResource().getName();
						IEditorDescriptor[] editors = ModelBusTeamUIPlugin.instance().getWorkbench().getEditorRegistry().getEditors(name);
						for (int i = 0; i < editors.length; i++) {
							if (!editors[i].getId().equals(EditorsUI.DEFAULT_TEXT_EDITOR_ID)) {
								final OpenFileWithAction openAction = new OpenFileWithAction(editors[i].getId(), false);
								Action wrapper = new Action(editors[i].getLabel()) {
									public void run() {
										openAction.run(this);
									}
								};
								openAction.selectionChanged(wrapper, selection);
								sub.add(wrapper);
							}
						}
					}
				}
				sub.add(new Separator("fixedGroup"));
				sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				manager.add(sub);

				manager.add(new Separator("miscGroup"));

				sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.Refactor"), "refactorMenu");
				sub.add(new Separator("mainGroup"));
				sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				manager.add(sub);

				manager.add(new Separator("locationGroup"));

				manager.add(new Separator("propertiesGroup"));

				manager.add(new Separator("importExportGroup"));

				sub = new MenuManager(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.CompareWith"), "compareMenu");
				sub.add(new Separator("mainGroup"));
				sub.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				manager.prependToGroup("importExportGroup", sub);

				manager.add(new Separator("refreshGroup"));

				manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
			}

		});
		menuMgr.setRemoveAllWhenShown(true);
		return menuMgr;
	}

	public void createPartControl(Composite parent) {

		this.repositoryTree = new RepositoryTreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		this.repositoryTree.setContentProvider(new RepositoryContentProvider(this.repositoryTree));
		this.repositoryTree.setLabelProvider(new RepositoryLabelProvider());// (new WorkbenchLabelProvider());
		this.getSite().setSelectionProvider(this.repositoryTree);

		try {
			this.root = new RepositoriesRoot();
			this.repositoryTree.setInput(this.root);
		} catch (Exception e1) {
			this.repositoryTree.setInput(null);
			this.root = null;
		}

		// this.repositoryTree.setSorter(new ViewSorter())

		this.ddAdapter = new DrillDownAdapter(this.repositoryTree);

		// popup menu
		Tree tree = this.repositoryTree.getTree();
		MenuManager menuMgr = RepositoriesView.newMenuInstance(this.repositoryTree);
		tree.setMenu(menuMgr.createContextMenu(tree));
		this.getSite().registerContextMenu(menuMgr, this.repositoryTree);

		// toolbar
		IActionBars actionBars = getViewSite().getActionBars();
		IToolBarManager tbm = actionBars.getToolBarManager();
		this.ddAdapter.addNavigationActions(tbm);
		Action tAction = null;
		tbm.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.Refresh.Label")) {
			public void run() {
				if (RepositoriesView.this.repositoryTree.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection) RepositoriesView.this.repositoryTree.getSelection();
					RepositoriesView.this.handleRefresh(selection);
				}
			}
		});
		tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/refresh.gif"));
		tAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("ModelBusView.Refresh.ToolTip"));

		tbm.add(new Separator("collapseAllGroup"));

		tbm.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.CollapseAll.Label")) {
			public void run() {
				RepositoriesView.this.repositoryTree.collapseAll();
			}
		});
		tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/common/collapseall.gif"));
		tAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.CollapseAll.ToolTip"));

		tbm.add(new Separator("repositoryGroup"));

		// gdu
		tbm.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("CreateRemoteFolderAction.label")) {
			public void run() {
				new CreateFolderAction(true).run(this);
			}
		});
		tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/repositories/newfolder.gif"));
		tAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("CreateRemoteFolderAction.tooltip"));

		tbm.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("CreateProjectStructureAction.label")) {
			public void run() {
				new CreateProjectStructureAction(true).run(this);
			}
		});
		tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/objects/root.gif"));
		tAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("CreateProjectStructureAction.tooltip"));

		// tbm.add(tAction = new Action(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.NewLocation.Label")) {
		// public void run() {
		// new NewRepositoryLocationAction().run(this);
		// }
		// });
		// tAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/repositories/new_location.gif"));
		// tAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.NewLocation.ToolTip"));

		tbm.add(this.showBrowserAction = new Action(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.ShowBrowser.Label"),
				Action.AS_CHECK_BOX) {
			public void run() {
				if (this.isChecked()) {
					RepositoriesView.this.showRepositoryBrowser(true);
				} else {
					RepositoriesView.this.hideRepositoryBrowser();
				}
			}
		});
		this.showBrowserAction.setImageDescriptor(ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/views/repositories/browser.gif"));
		this.showBrowserAction.setToolTipText(ModelBusTeamUIPlugin.instance().getResource("RepositoriesView.ShowBrowser.ToolTip"));

		this.repositoryTree.getControl().addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent event) {
				if (RepositoriesView.this.repositoryTree.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection) RepositoriesView.this.repositoryTree.getSelection();
					if (event.keyCode == SWT.F5) {
						RepositoriesView.this.handleRefresh(selection);
					} else if (event.keyCode == SWT.DEL) {
						RepositoriesView.this.handleDeleteKey(selection);
					}
				}
			}
		});

		this.repositoryTree.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent e) {
				ISelection selection = e.getSelection();
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection structured = (IStructuredSelection) selection;
					if (structured.size() == 1) {
						RepositoriesView.this.handleDoubleClick(structured);
					}
				}
			}
		});

		this.partListener = new IPartListener2() {
			public void partVisible(IWorkbenchPartReference partRef) {
				if (partRef.getId().equals(RepositoriesView.VIEW_ID)) {
					try {
						RepositoriesView.this.refreshRepositoriesImpl(false);
					} catch (Exception e) {
						try {
							PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(IPageLayout.ID_PROBLEM_VIEW);
						} catch (PartInitException e1) {
							e1.printStackTrace();
						}

						ModelBusTeamUIPlugin
								.instance()
								.getLog()
								.log(new Status(IStatus.ERROR, ModelBusTeamUIPlugin.instance().getDescriptor().getUniqueIdentifier(), IStatus.ERROR,
										e.getMessage(), e));
					}
				}
			}

			public void partHidden(IWorkbenchPartReference partRef) {
			}

			public void partInputChanged(IWorkbenchPartReference partRef) {
			}

			public void partOpened(IWorkbenchPartReference partRef) {
			}

			public void partDeactivated(IWorkbenchPartReference partRef) {
			}

			public void partClosed(IWorkbenchPartReference partRef) {
				if (partRef.getId().equals(RepositoryBrowser.VIEW_ID)) {
					RepositoriesView.this.getViewSite().getPage().removePartListener(this);
				}
			}

			public void partBroughtToTop(IWorkbenchPartReference partRef) {
			}

			public void partActivated(IWorkbenchPartReference partRef) {
			}
		};

		this.getViewSite().getPage().addPartListener(this.partListener);

		ModelBusRemoteStorage.instance().addRepositoriesStateChangedListener(new IRepositoriesStateChangedListener() {
			public void repositoriesStateChanged(RepositoriesStateChangedEvent event) {
				if (event.getAction() == RepositoriesStateChangedEvent.ADDED) {
					RepositoriesView.refreshRepositories(false);
				}
			}
		});

		// Setting context help
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, "org.modelbus.team.eclipse.help.repositoryViewContext");
	}

	public void dispose() {
		super.dispose();
		this.getViewSite().getPage().removePartListener(this.partListener);
	}

	public void setFocus() {
		this.repositoryTree.getControl().setFocus();
	}

	public static void refresh(Object where) {
		RepositoriesView.refresh(where, null);
	}

	public static void refresh(Object where, RepositoryTreeViewer.IRefreshVisitor visitor) {
		RepositoriesView instance = RepositoriesView.instance();
		if (instance != null) {
			instance.repositoryTree.refresh(where, visitor, false);
		}
	}

	public static void refreshRepositories(boolean deep) {
		RepositoriesView instance = RepositoriesView.instance();
		if (instance != null) {
			instance.refreshRepositoriesImpl(deep);
		}
	}

	public RepositoryTreeViewer getRepositoryTree() {
		return this.repositoryTree;
	}

	public void refreshButtonsState() {
		IPreferenceStore store = ModelBusTeamUIPlugin.instance().getPreferenceStore();
		boolean isBrowserVisible = ModelBusTeamPreferences.getRepositoryBoolean(store, ModelBusTeamPreferences.REPOSITORY_SHOW_BROWSER_NAME);

		if (this.showBrowserAction != null) {
			this.showBrowserAction.setChecked(isBrowserVisible);
		}
	}

	public static RepositoriesView instance() {
		final RepositoriesView[] view = new RepositoriesView[1];
		UIMonitorUtility.getDisplay().syncExec(new Runnable() {
			public void run() {
				IWorkbenchWindow window = ModelBusTeamUIPlugin.instance().getWorkbench().getActiveWorkbenchWindow();
				if (window != null && window.getActivePage() != null) {
					view[0] = (RepositoriesView) window.getActivePage().findView(RepositoriesView.VIEW_ID);
				}
			}
		});
		return view[0];
	}

	protected void refreshRepositoriesImpl(boolean deep) {
		if (this.root == null) {
			try {
				this.root = new RepositoriesRoot();
				this.repositoryTree.setInput(this.root);
			} catch (Exception e1) {
				// this.repositoryTree.setInput(null);
				this.root = null;
			}
		}

		if (this.root == null) {
			return;
		}

		if (deep) {
			this.root.refresh();
		} else {
			this.root.softRefresh();
		}
		this.repositoryTree.refresh();
	}

	protected void showRepositoryBrowser(final boolean force) {
		final IWorkbenchPage page = this.getSite().getPage();
		UIMonitorUtility.doTaskBusyDefault(new AbstractActionOperation("Operation.ShowBrowser") {
			protected void runImpl(IProgressMonitor monitor) throws Exception {
				RepositoryBrowser browser = (RepositoryBrowser) page.showView(RepositoryBrowser.VIEW_ID);
				ISelection selection = RepositoriesView.this.repositoryTree.getSelection();
				browser.selectionChanged(new SelectionChangedEvent(RepositoriesView.this.repositoryTree, selection));
			}
		});
	}

	protected void hideRepositoryBrowser() {
		IWorkbenchPage page = this.getSite().getPage();
		IViewPart part = page.findView(RepositoryBrowser.VIEW_ID);
		if (part != null) {
			page.hideView(part);
		}
	}

	protected void handleRefresh(IStructuredSelection selection) {
		Action tmp = new Action() {
		};
		AbstractModelBusTeamAction action = null;

		action = new RefreshAction();

		// MES
		if (selection instanceof TreeSelection && selection.isEmpty()) {
			final RepositoriesRoot input = (RepositoriesRoot) getRepositoryTree().getInput();

			if (input != null) {
				action.selectionChanged(tmp, new StructuredSelection(input.getChildren(null)));
			}
		} else {
			action.selectionChanged(tmp, selection);
		}
		action.setActivePart(tmp, RepositoriesView.this);
		if (tmp.isEnabled()) {
			action.run(tmp);
		}

		// action = new RefreshRepositoryLocationAction();
		// action.selectionChanged(tmp, selection);
		// action.setActivePart(tmp, RepositoriesView.this);
		// if (tmp.isEnabled()) {
		// action.run(tmp);
		// }
	}

	protected void handleDeleteKey(IStructuredSelection selection) {
		Action tmp = new Action() {
		};
		AbstractModelBusTeamAction action = new DeleteAction();
		action.selectionChanged(tmp, selection);
		action.setActivePart(tmp, RepositoriesView.this);
		if (tmp.isEnabled()) {
			action.run(tmp);
		} else {
			action = new DiscardRevisionLinksAction();
			action.selectionChanged(tmp, selection);
			action.setActivePart(tmp, RepositoriesView.this);
			if (tmp.isEnabled()) {
				action.run(tmp);
			} else {
				// action = new DiscardRepositoryLocationAction();
				// action.selectionChanged(tmp, selection);
				// action.setActivePart(tmp, RepositoriesView.this);
				// if (tmp.isEnabled()) {
				// action.run(tmp);
				// }
			}
		}
	}

	protected void handleDoubleClick(IStructuredSelection selection) {
		Action tmp = new Action() {
		};
		AbstractModelBusTeamAction action = new OpenFileAction();
		action.selectionChanged(tmp, selection);
		action.setActivePart(tmp, RepositoriesView.this);
		if (tmp.isEnabled()) {
			action.run(tmp);
		}
	}

	public boolean canGoBack() {
		return this.ddAdapter.canGoBack();
	}

	public void goBack() {
		this.ddAdapter.goBack();
	}

}
