/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Sergiy Logvin - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.repository.browser;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.BasicResourceHandler;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.DelegatingWrapperItemProvider;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.ui.model.BaseWorkbenchContentProvider;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.UnresolvedReferencesException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;
import org.modelbus.team.eclipse.ui.repository.model.IParentTreeNode;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryError;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFictiveWorkingDirectory;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFolder;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryPending;
import org.modelbus.team.eclipse.ui.repository.preferences.PreferenceInitializer;

/**
 * Repository browser content provider
 * 
 * @author Sergiy Logvin
 */
public class RepositoryBrowserContentProvider extends BaseWorkbenchContentProvider {

	protected ComposedAdapterFactory		composedAdapterFactory;
	protected AdapterFactoryContentProvider	adapterFactoryContentProvider;

	public RepositoryBrowserContentProvider() {
		super();
		List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());
		factories.add(new UMLItemProviderAdapterFactory());
		factories.add(new EcoreItemProviderAdapterFactory());
		composedAdapterFactory = new ComposedAdapterFactory(factories);
		adapterFactoryContentProvider = new AdapterFactoryContentProvider(composedAdapterFactory);
	}

	private boolean isModelToBrowse(String path) {
		List<String> modelExtensions = Arrays.asList(getModelExtensions());
		URI uri = URI.createURI(path);
		String fileExtension = uri.fileExtension();
		for (String modelExtension : modelExtensions) {
			if (modelExtension.startsWith("*.")) {
				modelExtension = modelExtension.substring(1);
			}
			if (modelExtension.startsWith(".") && path.endsWith(modelExtension)) {
				return true;
			} else if (modelExtension.equals(fileExtension)) {
				return true;
			}
		}
		return false;
	}

	private String[] getModelExtensions() {
		String[] modelExtensions = new String[0];
		modelExtensions = PreferenceInitializer.getExtensionPreference();
		return modelExtensions;
	}

	public boolean hasChildren(Object element) {
		IWorkbenchAdapter adapter = this.getAdapter(element);
		if (adapter instanceof IParentTreeNode) {
			return ((IParentTreeNode) adapter).hasChildren();
		} else if (adapter instanceof RepositoryFile) {
			String path = ((RepositoryFile) adapter).getRepositoryResource().getUrl();
			if (isModelToBrowse(path)/*
									 * modelExtensions.contains(fileExtension)path.endsWith(".uml")|| path.endsWith(".ecore") || path.endsWith(".emx")
									 * || path.endsWith(".epx") || path.endsWith(".simulink")
									 */) {
				return true;
			}

		} else if (adapter == null && element instanceof EObject) {
			if (((EObject) element).eIsProxy()) {
				return true;
			}
			return adapterFactoryContentProvider.hasChildren(element);
		} else if (adapter == null && element instanceof DelegatingWrapperItemProvider) {
			boolean hasChildren = adapterFactoryContentProvider.hasChildren(element);
			return adapterFactoryContentProvider.hasChildren(element);
		}
		return false;
	}

	public Object[] getChildren(Object parentElement) {
		IWorkbenchAdapter adapter = this.getAdapter(parentElement);

		if (adapter instanceof RepositoryFolder) {
			RepositoryFolder folder = (RepositoryFolder) adapter;
			Object[] children = folder.peekChildren(parentElement);
			if (folder.getParent() == null) {
				return children;
			}
			if (children.length != 0 && children[0] != null && (children[0] instanceof RepositoryError || children[0] instanceof RepositoryPending)) {
				return children;
			}
			Object[] childrenWithParent = new Object[children.length + 1];
			childrenWithParent[0] = new RepositoryFictiveWorkingDirectory(parentElement);
			System.arraycopy(children, 0, childrenWithParent, 1, children.length);
			return childrenWithParent;
		} else if (adapter instanceof RepositoryFile) {
			// RepositoryFile modelFile = (RepositoryFile) adapter;
			// String resourcePath = modelFile.getRepositoryResource().getUrl();
			// ResourceSet resourceSet = new ResourceSetImpl();
			// Map options = new HashMap();
			// if(resourcePath.endsWith(".emx") || resourcePath.endsWith(".epx")){
			// // resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
			// options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
			// }
			// IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
			// try {
			// Resource modelResource = resourceSet.createResource(URI.createURI(resourcePath));
			// try {
			// repositoryHelper.checkOutModel(UserSessionHelper.getSession(), modelResource, /*Collections.EMPTY_MAP*/options);
			// } catch (RepositoryAuthentificationException e) {
			// throw new RuntimeException(e);
			// } catch (UnresolvedReferencesException e) {
			// throw new RuntimeException(e);
			// } catch (NonExistingResourceException e) {
			// throw new RuntimeException(e);
			// }
			// EList<EObject> contents = modelResource.getContents();
			// if(contents != null && !contents.isEmpty()){
			// EObject content = contents.get(0);
			// Resource tmp = resourceSet.createResource(URI.createURI(resourcePath));
			// tmp.getContents().add(content);
			// return adapterFactoryContentProvider.getChildren(tmp);
			// }
			// }catch(IOException e){
			// e.printStackTrace();
			// throw new RuntimeException(e.getMessage(), e);
			// }
			return new Object[0];
		} else if (adapter == null && parentElement instanceof EObject) {
			EObject eObject = (EObject) parentElement;
			if (eObject.eIsProxy()) {
				ResourceSet resourceSet = new ResourceSetImpl();
				URI proxyURI = ((InternalEObject) eObject).eProxyURI();
				String resourceUri = proxyURI.trimFragment().toString();
				String fragment = proxyURI.fragment();
				Map options = new HashMap();
				if (resourceUri.endsWith(".emx") || resourceUri.endsWith(".epx")) {
					// resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
					options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
				}

				Resource resource = resourceSet.createResource(URI.createURI(resourceUri));
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();
				try {
					try {
						repositoryHelper.checkOutModel(UserSessionHelper.getSession(), resource,/* Collections.EMPTY_MAP */options);
					} catch (RepositoryAuthentificationException e) {
						throw new RuntimeException(e);
					} catch (UnresolvedReferencesException e) {
						throw new RuntimeException(e);
					} catch (NonExistingResourceException e) {
						throw new RuntimeException(e);
					}
					parentElement = resource.getEObject(fragment);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new RuntimeException(e.getMessage(), e);
				}
			}

			return adapterFactoryContentProvider.getChildren((EObject) parentElement);
		} else if (adapter == null && parentElement instanceof DelegatingWrapperItemProvider) {

			return adapterFactoryContentProvider.getChildren(parentElement);
		}
		return super.getChildren(parentElement);
	}

}
