package org.modelbus.team.eclipse.ui.repository.model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.BasicResourceHandler;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.DelegatingWrapperItemProvider;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.modelbus.core.lib.IRepositoryHelper;
import org.modelbus.dosgi.repository.descriptor.NonExistingResourceException;
import org.modelbus.dosgi.repository.descriptor.RepositoryAuthentificationException;
import org.modelbus.dosgi.repository.descriptor.UnresolvedReferencesException;
import org.modelbus.team.eclipse.repository.ModelBusRepositoryHelper;
import org.modelbus.team.eclipse.repository.UserSessionHelper;

public class RepositoryLabelProvider extends LabelProvider {

	protected ComposedAdapterFactory		composedAdapterFactory;
	protected AdapterFactoryLabelProvider	adapterFactoryLabelProvider;
	protected WorkbenchLabelProvider		workbenchLabelProvider	= new WorkbenchLabelProvider();

	public RepositoryLabelProvider() {
		List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());
		factories.add(new UMLItemProviderAdapterFactory());
		factories.add(new EcoreItemProviderAdapterFactory());
		composedAdapterFactory = new ComposedAdapterFactory(factories);
		adapterFactoryLabelProvider = new AdapterFactoryLabelProvider(composedAdapterFactory);
	}

	public Image getImage(Object element) {
		// TODO Auto-generated method stub
		if (element instanceof EObject || element instanceof DelegatingWrapperItemProvider) {
			return adapterFactoryLabelProvider.getImage(element);
		}
		return workbenchLabelProvider.getImage(element);
	}

	public String getText(Object element) {
		// TODO Auto-generated method stub
		if (element instanceof EObject) {
			if (((EObject) element).eIsProxy()) {
				ResourceSet resourceSet = new ResourceSetImpl();
				URI proxy = ((InternalEObject) element).eProxyURI();
				String fragment = proxy.fragment();
				URI resourceUri = proxy.trimFragment();

				Map options = new HashMap();
				if (resourceUri.toString().endsWith(".emx") || resourceUri.toString().endsWith(".epx")) {
					// resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
					options.put(XMIResource.OPTION_RESOURCE_HANDLER, new BasicResourceHandler());
				}
				IRepositoryHelper repositoryHelper = ModelBusRepositoryHelper.getRepositoryHelper();

				Resource resource = resourceSet.createResource(resourceUri);
				try {
					try {
						repositoryHelper.checkOutModel(UserSessionHelper.getSession(), resource, /* Collections.EMPTY_MAP */options);
					} catch (RepositoryAuthentificationException e) {
						throw new RuntimeException(e);
					} catch (UnresolvedReferencesException e) {
						throw new RuntimeException(e);
					} catch (NonExistingResourceException e) {
						throw new RuntimeException(e);
					}
					element = resource.getEObject(fragment);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new RuntimeException(e.getMessage(), e);
				}
			}
			return adapterFactoryLabelProvider.getText(element);
		} else if (element instanceof DelegatingWrapperItemProvider) {
			return adapterFactoryLabelProvider.getText(element);
		}
		return workbenchLabelProvider.getText(element);
	}

}
