package org.modelbus.team.eclipse.ui.status;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

public class ModelBusStatusLineContributionItem extends ContributionItem implements Observer {
	private final static String	AV_ICON_PATH	= "/icons/status/modelbus_av.gif";
	private final static String	UNAV_ICON_PATH	= "/icons/status/modelbus_unav.gif";
	private final static String	AV_LABEL		= "ModelBus available";
	private final static String	UNAV_LABEL		= "ModelBus unavailable";

	private Image				statusImage;
	private Composite			parent;
	private Canvas				canvas;

	public ModelBusStatusLineContributionItem(final String id) {
		super(id);
	}

	public void fill(final Composite parent) {
		this.parent = parent;

		final Composite canvasComposite = new Composite(this.parent, SWT.NONE);
		canvasComposite.setLayout(new GridLayout(1, false));
		
		this.canvas = new Canvas(canvasComposite, SWT.NONE);
		this.canvas.setLayoutData(new GridData(32, 16));
		
		this.statusImage = ModelBusTeamUIPlugin.instance().getImageDescriptor(UNAV_ICON_PATH).createImage();

//		this.canvas.setBackground(new Color(parent.getDisplay(), 100, 100, 100));
		this.canvas.addPaintListener(new PaintListener() {
			public void paintControl(final PaintEvent e) {
				if (ModelBusStatusLineContributionItem.this.statusImage != null && !ModelBusStatusLineContributionItem.this.statusImage.isDisposed()) {
					e.gc.drawImage(ModelBusStatusLineContributionItem.this.statusImage, 0, 0);
					ModelBusStatusLineContributionItem.this.statusImage.dispose();
				}
			}
		});

		this.canvas.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(final MouseEvent e) {
				try {
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(StatusView.VIEW_ID);
				} catch (final PartInitException e1) {
					e1.printStackTrace();
				}
			}
		});

		this.canvas.setToolTipText(UNAV_LABEL);

		this.updateControls();
	}

	private void updateControls() {
		if (this.parent == null) {
			return;
		}

		this.parent.getDisplay().syncExec(new Runnable() {
			public void run() {
				if (ModelBusStatusLineContributionItem.this.canvas.isDisposed()) {
					return;
				}

				if (ModelBusTeamUIPlugin.instance().getStatus().isAvailable()) {
					ModelBusStatusLineContributionItem.this.statusImage = ModelBusTeamUIPlugin.instance().getImageDescriptor(AV_ICON_PATH)
							.createImage();
					ModelBusStatusLineContributionItem.this.canvas.setToolTipText(AV_LABEL + " (Last response: "
							+ ModelBusStatusProvider.DATE_FORMAT.format(ModelBusTeamUIPlugin.instance().getStatus().getLastResponse()) + ")");
				} else {
					ModelBusStatusLineContributionItem.this.statusImage = ModelBusTeamUIPlugin.instance().getImageDescriptor(UNAV_ICON_PATH)
							.createImage();
					ModelBusStatusLineContributionItem.this.canvas.setToolTipText(ModelBusTeamUIPlugin.instance().getStatus().getStatusMessage());
				}
				
//				ModelBusStatusLineContributionItem.this.canvas.setSize(new Point(32, 16));
				ModelBusStatusLineContributionItem.this.canvas.redraw();
			}
		});
	}

	public void update(final Observable o, final Object arg) {
		this.updateControls();
	}
}
