package org.modelbus.team.eclipse.ui.status;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.modelbus.core.lib.util.AbstractLocationsUtil;
import org.modelbus.dosgi.repository.descriptor.ServiceStatus;
import org.modelbus.team.eclipse.repository.RepositoryLocationHelper;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

public class StatusView extends ViewPart implements Observer {
	public final static String	VIEW_ID				= "org.modelbus.team.eclipse.ui.status.StatusView";
	private final static String	MON_ICON_PATH		= "/icons/status/monitor_view.gif";

	private final static String	NOT_AVAILABLE_LABEL	= "n/a";

	private Label				locationLabel;
	private Label				secureLocationLabel;
	private Label				startupLabel;
	// private Label statusLabel;
	private Composite			parent;

	private TableViewer			servicesTableViewer;

	public StatusView() {
		// TODO Auto-generated constructor stub
	}

	@Override
	public void createPartControl(Composite parent) {
		this.parent = parent;

		parent.setLayout(new GridLayout(2, false));

		final GridData fillHorizontalData = new GridData();
		fillHorizontalData.grabExcessHorizontalSpace = true;
		fillHorizontalData.horizontalAlignment = SWT.FILL;

		// final Label statusLabelCaption = new Label(parent, SWT.NONE);
		// statusLabelCaption.setText("Repository available:");
		//
		// this.statusLabel = new Label(parent, SWT.NONE);
		// this.statusLabel.setLayoutData(fillHorizontalData);

		final Label startupLabelCaption = new Label(parent, SWT.NONE);
		startupLabelCaption.setText("Startup time:");

		this.startupLabel = new Label(parent, SWT.NONE);
		this.startupLabel.setLayoutData(fillHorizontalData);

		final Label locationLabelCaption = new Label(parent, SWT.NONE);
		locationLabelCaption.setText("Repository service location:");

		this.locationLabel = new Label(parent, SWT.NONE);
		this.locationLabel.setLayoutData(fillHorizontalData);

		final Label secureLocationLabelCaption = new Label(parent, SWT.NONE);
		secureLocationLabelCaption.setText("Repository service location (HTTPS):");

		this.secureLocationLabel = new Label(parent, SWT.NONE);
		this.secureLocationLabel.setLayoutData(fillHorizontalData);

		final Label servicesLabel = new Label(parent, SWT.NONE);
		servicesLabel.setText("Registered ModelBus services:");

		final GridData fillHorizontal3ColumnsData = new GridData();
		fillHorizontal3ColumnsData.grabExcessHorizontalSpace = true;
		fillHorizontal3ColumnsData.horizontalAlignment = SWT.FILL;
		fillHorizontal3ColumnsData.horizontalSpan = 2;

		servicesLabel.setLayoutData(fillHorizontal3ColumnsData);

		this.servicesTableViewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);

		final String[] titles = { "Name", "Location", "Startup" };
		final int[] bounds = { 200, 350, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(this.servicesTableViewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}

		final Table table = this.servicesTableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		table.addMouseListener(new MouseListener() {

			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseDown(MouseEvent e) {

			}

			public void mouseDoubleClick(MouseEvent e) {
				final Table t = (Table) e.getSource();

				final int index = t.getSelectionIndex();

				if (index < 0) {
					return;
				}

				final List<ServiceStatus> serviceStatus = ModelBusTeamUIPlugin.instance().getStatus().getServices();

				if (serviceStatus.isEmpty()) {
					return;
				}

				if (index > serviceStatus.size() - 1) {
					return;
				}

				final ServiceStatus status = ModelBusTeamUIPlugin.instance().getStatus().getServices().get(index);

				try {
					PlatformUI.getWorkbench().getBrowserSupport().getExternalBrowser().openURL(new URL(status.getLocation() + "?wsdl"));
				} catch (PartInitException e1) {
					throw new RuntimeException(e1);
				} catch (MalformedURLException e1) {
					throw new RuntimeException(e1);
				}
			}
		});

		this.servicesTableViewer.setLabelProvider(new ServiceStatusLabelProvider());
		this.servicesTableViewer.setContentProvider(new ServiceStatusContentProvider());

		final GridData tableViewerLData = new GridData();
		tableViewerLData.horizontalAlignment = GridData.FILL;
		tableViewerLData.verticalAlignment = GridData.FILL;
		tableViewerLData.grabExcessHorizontalSpace = true;
		tableViewerLData.grabExcessVerticalSpace = true;
		tableViewerLData.horizontalSpan = 2;

		this.servicesTableViewer.getControl().setLayoutData(tableViewerLData);

		ModelBusTeamUIPlugin.instance().getStatus().addObserver(this);
	}

	class ServiceStatusContentProvider implements IStructuredContentProvider {
		private List<ServiceStatus>	services;

		public Object[] getElements(final Object inputElement) {
			if (this.services == null) {
				return new ServiceStatus[] {};
			}

			return this.services.toArray();
		}

		public void dispose() {
			// TODO Auto-generated method stub

		}

		@SuppressWarnings("unchecked")
		public void inputChanged(final Viewer viewer, final Object oldInput, final Object newInput) {
			this.services = (List<ServiceStatus>) newInput;
		}
	}

	class ServiceStatusLabelProvider extends LabelProvider implements ITableLabelProvider, IColorProvider {
		private int	index;

		@Override
		public void addListener(final ILabelProviderListener listener) {

		}

		@Override
		public void dispose() {

		}

		@Override
		public boolean isLabelProperty(final Object element, final String property) {
			return false;
		}

		@Override
		public void removeListener(final ILabelProviderListener listener) {

		}

		public Image getColumnImage(final Object element, final int columnIndex) {
			if (columnIndex == 1) {
				return ModelBusTeamUIPlugin.instance().getImageDescriptor(MON_ICON_PATH).createImage();
			}

			return null;
		}

		public String getColumnText(final Object element, final int columnIndex) {
			this.index = columnIndex;

			final ServiceStatus serviceStatus = (ServiceStatus) element;

			switch (columnIndex) {
				case 0: {
					return serviceStatus.getName();
				}
				case 1: {
					return serviceStatus.getLocation();
				}
				case 2: {
					return ModelBusStatusProvider.DATE_FORMAT.format(serviceStatus.getStartupDate().toGregorianCalendar().getTime());
				}
				default: {
					return "?";
				}
			}
		}

		public Color getForeground(Object element) {
			if (this.index == 0) {
				return null;
			}

			if (this.index == 1) {
				return Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
			}

			return null;
		}

		public Color getBackground(Object element) {
			// TODO Auto-generated method stub
			return null;
		}
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
	}

	public void update(Observable o, Object arg) {
		this.parent.getDisplay().syncExec(new Runnable() {

			public void run() {
				final ModelBusStatusProvider status = ModelBusTeamUIPlugin.instance().getStatus();

				String dateString = null;

				if (status.getRepositoryStartup() != -1) {
					final Date date = new Date(status.getRepositoryStartup());
					dateString = ModelBusStatusProvider.DATE_FORMAT.format(date);
				} else {
					dateString = NOT_AVAILABLE_LABEL;
				}

				StatusView.this.startupLabel.setText(dateString);

				final Boolean useSecure = RepositoryLocationHelper.getPropertyUseSSL();

				if (status.getRepositoryLocation() != null) {
					String repoLocationValue = status.getRepositoryLocation();

					repoLocationValue = AbstractLocationsUtil.getExternalLocation(repoLocationValue);

					if (!useSecure) {
						repoLocationValue += " (active)";
					}

					StatusView.this.locationLabel.setText(repoLocationValue);
				} else {
					StatusView.this.locationLabel.setText(NOT_AVAILABLE_LABEL);
				}

				if (status.getSecureRepositoryLocation() != null) {
					String secureRepoLocationValue = status.getSecureRepositoryLocation();

					secureRepoLocationValue = AbstractLocationsUtil.getExternalLocation(secureRepoLocationValue);

					if (useSecure) {
						secureRepoLocationValue += " (active)";
					}

					StatusView.this.secureLocationLabel.setText(secureRepoLocationValue);
				} else {
					StatusView.this.secureLocationLabel.setText(NOT_AVAILABLE_LABEL);
				}

				StatusView.this.servicesTableViewer.setInput(status.getServices());
				StatusView.this.servicesTableViewer.refresh();
			}
		});
	}

}
