/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.merge;

import org.eclipse.core.resources.IResource;
import org.eclipse.team.ui.synchronize.AbstractSynchronizeScope;
import org.eclipse.ui.IMemento;
import org.modelbus.team.eclipse.core.operation.local.AbstractMergeSet;
import org.modelbus.team.eclipse.core.operation.local.MergeSet1URL;
import org.modelbus.team.eclipse.core.operation.local.MergeSet2URL;
import org.modelbus.team.eclipse.core.operation.local.MergeSetReintegrate;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;

/**
 * Merge resources scope. Non-persistent.
 * 
 * @author Alexander Gurov
 */
public class MergeScope extends AbstractSynchronizeScope {
	protected AbstractMergeSet	info;

	public MergeScope(AbstractMergeSet info) {
		super();
		this.info = info;
	}

	public MergeScope(IMemento memento) {
		super(memento);
	}

	public String getName() {
		if (this.info.to == null) {
			return "";
		}
		String url = null;
		if (this.info instanceof MergeSet1URL) {
			MergeSet1URL info = (MergeSet1URL) this.info;
			url = (info.from.length > 1 ? info.from[0].getRoot() : info.from[0]).getUrl();
		} else if (this.info instanceof MergeSet2URL) {
			MergeSet2URL info = (MergeSet2URL) this.info;
			url = (info.fromEnd.length > 1 ? info.fromEnd[0].getRoot() : info.fromEnd[0]).getUrl();
		} else {
			MergeSetReintegrate info = (MergeSetReintegrate) this.info;
			url = (info.from.length > 1 ? info.from[0].getRoot() : info.from[0]).getUrl();
		}
		String names = null;
		for (int i = 0; i < this.info.to.length; i++) {
			String path = this.info.to[i].getFullPath().toString().substring(1);
			names = names == null ? path : (names + ", " + path);
		}
		return ModelBusTeamUIPlugin.instance().getResource("MergeScope.Name", new String[] { url, names });
	}

	public IResource[] getRoots() {
		return this.info.to;
	}

	public void setMergeSet(AbstractMergeSet info) {
		this.info = info;
		this.fireRootsChanges();
	}

	public AbstractMergeSet getMergeSet() {
		return this.info;
	}

}
