/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.synchronize.merge.action;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.compare.structuremergeviewer.IDiffElement;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.team.core.synchronize.FastSyncInfoFilter;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.modelbus.team.eclipse.core.IStateFilter;
import org.modelbus.team.eclipse.core.operation.CompositeOperation;
import org.modelbus.team.eclipse.core.operation.IActionOperation;
import org.modelbus.team.eclipse.core.operation.local.GetRemoteContentsOperation;
import org.modelbus.team.eclipse.core.operation.local.RefreshResourcesOperation;
import org.modelbus.team.eclipse.core.operation.local.RestoreProjectMetaOperation;
import org.modelbus.team.eclipse.core.operation.local.SaveProjectMetaOperation;
import org.modelbus.team.eclipse.core.resource.ILocalResource;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.core.resource.IResourceChange;
import org.modelbus.team.eclipse.core.utility.FileUtility;
import org.modelbus.team.eclipse.core.utility.ModelBusUtility;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.operation.ClearMergeStatusesOperation;
import org.modelbus.team.eclipse.ui.synchronize.AbstractModelBusSyncInfo;
import org.modelbus.team.eclipse.ui.synchronize.action.AbstractSynchronizeModelAction;
import org.modelbus.team.eclipse.ui.synchronize.action.ISyncStateFilter;
import org.modelbus.team.eclipse.ui.synchronize.variant.RemoteResourceVariant;
import org.modelbus.team.eclipse.ui.synchronize.variant.ResourceVariant;

/**
 * Merge update action implementation
 * 
 * @author Alexander Gurov
 */
public class UpdateAction extends AbstractSynchronizeModelAction {
	protected boolean	advancedMode;

	public UpdateAction(String text, ISynchronizePageConfiguration configuration) {
		super(text, configuration);
		this.advancedMode = false;
	}

	public UpdateAction(String text, ISynchronizePageConfiguration configuration, ISelectionProvider selectionProvider) {
		super(text, configuration, selectionProvider);
		this.advancedMode = true;
	}

	protected FastSyncInfoFilter getSyncInfoFilter() {
		return new FastSyncInfoFilter.SyncInfoDirectionFilter(new int[] { SyncInfo.INCOMING, SyncInfo.OUTGOING }) {
			public boolean select(SyncInfo info) {
				return super.select(info) && !IStateFilter.SF_OBSTRUCTED.accept(((AbstractModelBusSyncInfo) info).getLocalResource());
			}
		};
	}

	protected IActionOperation getOperation(ISynchronizePageConfiguration configuration, IDiffElement[] elements) {
		IResource[] allResources = this.syncInfoSelector.getSelectedResources();
		if (this.advancedMode) {
			String message;
			if (allResources.length == 1) {
				message = ModelBusTeamUIPlugin.instance().getResource("AcceptAll.Message.Single");
			} else {
				message = ModelBusTeamUIPlugin.instance()
						.getResource("AcceptAll.Message.Multi", new String[] { String.valueOf(allResources.length) });
			}
			MessageDialog dlg = new MessageDialog(configuration.getSite().getShell(), ModelBusTeamUIPlugin.instance().getResource("AcceptAll.Title"),
					null, message, MessageDialog.QUESTION, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL }, 0);
			if (dlg.open() != 0) {
				return null;
			}
		}

		AbstractModelBusSyncInfo[] infos = this.getModelBusSyncInfos();
		HashMap<String, String> remote2local = new HashMap<String, String>();
		ArrayList<IRepositoryResource> remoteSet = new ArrayList<IRepositoryResource>();
		ArrayList<IResource> localSet = new ArrayList<IResource>();
		for (int i = 0; i < infos.length; i++) {
			ILocalResource remote = ((ResourceVariant) infos[i].getRemote()).getResource();
			if (remote instanceof IResourceChange
					&& ISyncStateFilter.SF_ONREPOSITORY.acceptRemote(remote.getResource(), remote.getStatus(), remote.getChangeMask())) {
				IResource resource = infos[i].getLocal();
				localSet.add(resource);
				IRepositoryResource remoteResource = ((IResourceChange) ((RemoteResourceVariant) infos[i].getRemote()).getResource()).getOriginator();
				remoteSet.add(remoteResource);
				remote2local.put(ModelBusUtility.encodeURL(remoteResource.getUrl()), FileUtility.getWorkingCopyPath(resource));
			}
		}

		IResource[] resources = localSet.toArray(new IResource[localSet.size()]);
		if (resources.length > 0) {
			GetRemoteContentsOperation mainOp = new GetRemoteContentsOperation(resources,
					remoteSet.toArray(new IRepositoryResource[remoteSet.size()]), remote2local);
			CompositeOperation op = new CompositeOperation(mainOp.getId());
			SaveProjectMetaOperation saveOp = new SaveProjectMetaOperation(resources);
			op.add(saveOp);
			op.add(mainOp);
			op.add(new RestoreProjectMetaOperation(saveOp));
			op.add(new RefreshResourcesOperation(resources));
			op.add(new ClearMergeStatusesOperation(allResources));
			return op;
		}
		return new ClearMergeStatusesOperation(allResources);
	}

}
