package org.modelbus.team.eclipse.ui.wizard;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.modelbus.team.eclipse.core.connector.ModelBusConnector;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.composite.RepositoryTreeComposite;
import org.modelbus.team.eclipse.ui.panel.common.RepositoryTreePanel;
import org.modelbus.team.eclipse.ui.panel.common.RepositoryTreePanel.ProjectRoot;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFile;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryFolder;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryLocationFilter;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryResource;
import org.modelbus.team.eclipse.ui.repository.model.RepositoryRevisions;
import org.modelbus.team.eclipse.ui.verifier.CompositeVerifier;
import org.modelbus.team.eclipse.ui.verifier.NonEmptyFieldVerifier;
import org.modelbus.team.eclipse.ui.verifier.ResourceNameVerifier;

public class SelectFragmentDestinationPage extends AbstractVerifiedWizardPage {

	protected IRepositoryResource		resource;
	protected IRepositoryResource[]		resources;
	protected EObject					fragment;
	protected RepositoryTreeComposite	repositoryTree;
	protected Text						newResourceName;

	public SelectFragmentDestinationPage(IRepositoryResource resource, EObject fragment) {
		super(SelectFragmentDestinationPage.class.getName(), ModelBusTeamUIPlugin.instance().getResource("RepositoryTreePanel.Description"),
				ModelBusTeamUIPlugin.instance().getImageDescriptor("icons/wizards/newconnect.png"));
		URI fragmentUri = fragment.eResource().getURI();
		String urifragment = fragment.eResource().getURIFragment(fragment);
		this.setDescription("Operation will be applied to resource: " + fragmentUri.appendFragment(urifragment).toString());
		this.resource = resource;
		this.resources = new IRepositoryResource[] { this.resource };
		this.fragment = fragment;
	}

	protected Composite createControlImpl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginHeight = 5;
		layout.marginWidth = 5;
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(data);
		data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 2;
		this.repositoryTree = new RepositoryTreeComposite(composite, SWT.BORDER, false, new RepositoryTreePanel.ProjectRoot(this.resources[0]));
		ProjectRoot root = (ProjectRoot) this.repositoryTree.getRepositoryTreeViewer().getInput();
		this.repositoryTree.getRepositoryTreeViewer().setExpandedElements(new Object[] { root.getChildren(null)[0] });
		String url = ModelBusConnector.getRepositoryRootUrl();
		// this.resources[0].getRepositoryLocation().getRepositoryRootUrl();
		this.repositoryTree.setFilter(new RepositoryLocationFilter(url) {
			public boolean accept(Object obj) {
				if (// obj instanceof RepositoryFile ||
				obj instanceof RepositoryRevisions || obj instanceof RepositoryFolder
						&& this.acceptYourself(((RepositoryFolder) obj).getRepositoryResource())) {
					return false;
				}
				return super.accept(obj);
			}

			private boolean acceptYourself(IRepositoryResource resource) {
				for (int i = 0; i < SelectFragmentDestinationPage.this.resources.length; i++) {
					if (resource.equals(SelectFragmentDestinationPage.this.resources[i])) {
						return true;
					}
				}
				return false;
			}
		});
		this.repositoryTree.setLayoutData(data);
		this.repositoryTree.getRepositoryTreeViewer().addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				ISelection selection = event.getSelection();
				if (selection instanceof IStructuredSelection) {
					IStructuredSelection structuredSelection = ((IStructuredSelection) selection);
					Object obj = structuredSelection.getFirstElement();
					if (obj instanceof RepositoryFile) {
						SelectFragmentDestinationPage.this.newResourceName.setText(((RepositoryFile) obj).getRepositoryResource().getName());
					}
				}
				SelectFragmentDestinationPage.this.validateContent();
			}
		});

		data = new GridData();
		Label name = new Label(composite, SWT.NONE);
		name.setText("Fragment File:");
		name.setLayoutData(data);
		data = new GridData();
		data.horizontalAlignment = SWT.FILL;
		this.newResourceName = new Text(composite, SWT.BORDER);
		this.newResourceName.setLayoutData(data);
		CompositeVerifier verifier = new CompositeVerifier();
		if (this.resources.length == 1) {
			verifier.add(new NonEmptyFieldVerifier(name.getText()));
			this.newResourceName.setText(this.resource.getName());
		}
		verifier.add(new ResourceNameVerifier(name.getText(), true // CHECKMWA
		// CoreExtensionsManager.instance().getModelBusConnectorFactory().getModelBusAPIVersion() ==
		// IModelBusConnectorFactory.APICompatibility.ModelBusAPI_1_5_x
		));
		this.attachTo(this.newResourceName, verifier);

		return composite;
	}

	public boolean isPageComplete() {
		return super.isPageComplete() && !this.repositoryTree.getRepositoryTreeViewer().getSelection().isEmpty();
	}

	public IRepositoryResource getDestination() {
		IStructuredSelection sel = (IStructuredSelection) this.repositoryTree.getRepositoryTreeViewer().getSelection();
		RepositoryResource repositoryResource = (RepositoryResource) sel.getFirstElement();
		if (repositoryResource instanceof RepositoryFile) {
			return repositoryResource.getParent().getRepositoryResource();
		}
		return ((RepositoryResource) sel.getFirstElement()).getRepositoryResource();
	}

	public String getNewResourceName() {
		return this.newResourceName.getText();
	}

	public RepositoryTreeComposite getRepositoryTree() {
		return this.repositoryTree;
	}

}
