/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexei Goncharov (Polarion Software) - initial API and implementation
 *******************************************************************************/

package org.modelbus.team.eclipse.ui.wizard.copymove;

import org.eclipse.jface.wizard.IWizardPage;
import org.modelbus.team.eclipse.core.resource.IRepositoryResource;
import org.modelbus.team.eclipse.ui.ModelBusTeamUIPlugin;
import org.modelbus.team.eclipse.ui.wizard.AbstractModelBusWizard;

/**
 * Wizard for 'Copy To...' and 'Move To...' refactoring actions on repository.
 * 
 * @author Alexei Goncharov
 */
public class CopyMoveWizard extends AbstractModelBusWizard {
	protected SelectDestinationPage	destinationPage;
	protected CommentWizardPage		commentPage;

	protected IRepositoryResource[]	selectedResources;
	protected boolean				isMove;

	protected IRepositoryResource	destination;
	protected String				comment;
	protected String				newName;

	public CopyMoveWizard(IRepositoryResource[] selectedResources, boolean isMove) {
		super();
		this.selectedResources = selectedResources;
		this.setWindowTitle((this.isMove = isMove) ? ModelBusTeamUIPlugin.instance().getResource("MoveToAction.Select.Title") : ModelBusTeamUIPlugin
				.instance().getResource("CopyToAction.Select.Title"));
	}

	public void addPages() {
		this.addPage(this.destinationPage = new SelectDestinationPage(this.selectedResources));
		this.addPage(this.commentPage = new CommentWizardPage(this.isMove));
	}

	public boolean canFinish() {
		IWizardPage currentPage = this.getContainer().getCurrentPage();
		if (currentPage instanceof SelectDestinationPage) {
			return false;
		}
		return super.canFinish();
	}

	public boolean performFinish() {
		this.destination = this.destinationPage.getDestination();
		this.newName = this.destinationPage.getNewResourceName();
		this.comment = this.commentPage.getComment();
		return true;
	}

	public String getComment() {
		return this.comment;
	}

	public String getNewName() {
		return this.newName;
	}

	public IRepositoryResource getDestination() {
		return this.destination;
	}

}
